import sys
import os
import shutil             #file utility commands
import wx
import logging
import copy
import wx.lib.wordwrap as ww

import FirestormDialogs
import GlobalVariables as gv    # globally used variables
import export                   # export data files as xml files
import dataStruct

#import wx.lib.inspection

VERSION = "1.3.2"
image_wildcard = "Images (*.png; *.jpg)|*.png;*.jpg"


########################################################################
class PrintOptionsBase(FirestormDialogs.PrintOptionsCore):
  """"""

  #----------------------------------------------------------------------
  def __init__(self, parent):
    """Constructor"""
    FirestormDialogs.PrintOptionsCore.__init__(self, parent)
    gv.Report_DR_Color = self.cpkDRcolor.GetColour()
    self.cpkCRcolor.SetColour(gv.Report_CR_Color)
    self.cpkHPcolor.SetColour(gv.Report_HP_Color)
    self.cpkMVcolor.SetColour(gv.Report_MV_Color)
    self.cpkTLcolor.SetColour(gv.Report_TL_Color)
    self.cpkCPcolor.SetColour(gv.Report_CP_Color)
    self.cpkAPcolor.SetColour(gv.Report_AP_Color)
    self.cpkPDcolor.SetColour(gv.Report_PD_Color)
    self.cpkSRcolor.SetColour(gv.Report_SR_Color)
    self.cpkMNcolor.SetColour(gv.Report_MN_Color)
    self.cpkWCcolor.SetColour(gv.Report_WC_Color)
  
    self.cpkDirectFireHeader.SetColour(gv.Report_DirectFireHeader_Color)
    self.cpkDirectFireEntry.SetColour(gv.Report_DirectFire_Color)
    self.cpkIndirectFireHeader.SetColour(gv.Report_IndirectFireHeader_Color)
    self.cpkIndirectFireEntry.SetColour(gv.Report_IndirectFire_Color)
    self.cpkSrsEntry.SetColour(gv.Report_SRS_Color)
    self.cpkHardpointEntry.SetColour(gv.Report_Hardpoint_Color)
    self.cpkUpgradeEntry.SetColour(gv.Report_Upgrade_Color)
    self.cpkMarEntry.SetColour(gv.Report_MAR_Color)
  
    self.cbPrintBW.SetValue(gv.PrintBW)
    if (gv.PrintMAR == 'Detailed'):
      self.rbDetailedMAR.SetValue(True)
    elif (gv.PrintMAR == 'Summary'):
      self.rbSummaryMAR.SetValue(True)
    

    return
    
  def OnOk(self, event):
    self.EndModal(True)
    return
  
  def OnCancel(self, event):
    self.EndModal(False)
    return
  
########################################################################
class FleetPrintout(wx.Printout):
  def __init__(self, fleet):
    wx.Printout.__init__(self, fleet.fleetName)
    self.fleet = fleet
    
    return
    
  def OnBeginDocument(self, start, end):
    return super(FleetPrintout, self).OnBeginDocument(start, end)
    
  def OnEndDocument(self):
    super(FleetPrintout, self).OnEndDocument()
    return
    
  def OnBeginPrinting(self):
    super(FleetPrintout, self).OnBeginPrinting()
    return
    
  def OnEndPrinting(self):
    super(FleetPrintout, self).OnEndPrinting()
    return
    
  def OnPreparePrinting(self):
    super(FleetPrintout, self).OnPreparePrinting()
    return
    
  def HasPage(self, page):
    if page <= 2:
        return True
    else:
        return False
    
  def GetPageInfo(self):
    return (1, 2, 1, 2)
    
  def OnPrintPage(self, page):
    if (0):
      self.fleet = dataStruct.FleetRecord() 
      dc = wx.DC()
    
    resolutionDPI,x = self.GetPPIPrinter()
    
    leftMargin = resolutionDPI/2     # half inch margin
    rightMargin = resolutionDPI/2
    topMargin = resolutionDPI/4      # quarter inch margin
    bottomMargin = resolutionDPI/4
    
    #set this dynamically
    pixelWidth, pixelHeight = self.GetPageSizePixels()
    pageWidth = pixelWidth/resolutionDPI
    pageHeight = pixelHeight/resolutionDPI
    
    printAreaWidth = pixelWidth - (leftMargin + rightMargin)
    printAreaHeight = pixelHeight - (topMargin + bottomMargin)
    
    pageList = []   #list of all pages to be printed
    #dc = self.GetDC()    # this gets a dc for the printer page
    
    #Draw the ship cards here.
    outlinePen = wx.BLACK_PEN
    fillBrush = wx.WHITE_BRUSH
    fontSmall = wx.Font(50,wx.FONTFAMILY_SWISS, wx.FONTSTYLE_NORMAL,wx.FONTWEIGHT_NORMAL, False, "Calibri")
    fontLarge = wx.Font(60,wx.FONTFAMILY_SWISS, wx.FONTSTYLE_NORMAL,wx.FONTWEIGHT_NORMAL, False, "Calibri")
    fontBoldSmall = wx.Font(50,wx.FONTFAMILY_SWISS, wx.FONTSTYLE_NORMAL,wx.FONTWEIGHT_BOLD, False, "Calibri")
    fontBoldLarge = wx.Font(60,wx.FONTFAMILY_SWISS, wx.FONTSTYLE_NORMAL,wx.FONTWEIGHT_BOLD, False, "Calibri")

    #run through each squadron to determine the print size per squadron.
    xpos = 0
    ypos = 0
    
    numSteps = 2*len(self.fleet.squadronList)
    currentStep = 0
    progressDlg = wx.ProgressDialog("Printing Progress", "Preparing Print Job", numSteps)
    progressDlg.Show()
    

    for squadron in self.fleet.squadronList:
      printBitmap = wx.EmptyBitmap(pixelWidth, pixelHeight)
      dc = wx.MemoryDC(printBitmap)
      dc.SetMapMode(wx.MM_TEXT)
      dc.SetFont(fontSmall)
      dc.SetTextBackground(wx.WHITE)
      dc.SetTextForeground(wx.BLACK)
      
      # Provide a white background
      dc.SetPen(wx.WHITE_PEN)
      dc.SetBrush(wx.WHITE_BRUSH)
      dc.DrawRectangle(0,0,pixelWidth, pixelHeight)
      
      Xmargin = 20
      Ymargin = 20
      if (0):
        squadron = dataStruct.squadronRecord()
        squadron.ship = dataStruct.ShipRecord()
      dc.SetDeviceOrigin(leftMargin, topMargin)
      
      fillBrush = wx.WHITE_BRUSH
      dc.SetBrush(fillBrush)
      dc.SetPen(outlinePen)
      
      #faction image
      xpos_IMG = int(5.5*resolutionDPI)
      ypos_IMG = ypos
      
      imageX = int(1.5*resolutionDPI)
      imageY = int(1.5*resolutionDPI)
      
      dc.DrawRectangle(xpos_IMG, ypos_IMG,imageX, imageY)
      source = self.SetFactionImage(squadron.ship.faction)
      image = source.Scale(imageX, imageY, wx.IMAGE_QUALITY_HIGH)
      bitmap = wx.BitmapFromImage(image)
      dc.DrawBitmap(bitmap,xpos_IMG, ypos_IMG)
      
      ypos_IMG += imageY
      
      #ship image
      dc.DrawRectangle(xpos_IMG, ypos_IMG, imageX, imageY)
      if (squadron.ship.imageCustomFilename):
        source = wx.Image(os.path.join(gv.ship_custom_images_folder, squadron.ship.imageCustomFilename), wx.BITMAP_TYPE_ANY)
      elif (squadron.ship.imageFilename):
        source = wx.Image(os.path.join(gv.ship_images_folder, squadron.ship.imageFilename), wx.BITMAP_TYPE_ANY)
      else:
        source = wx.Image(os.path.join(gv.ship_images_folder, gv.defaultImage), wx.BITMAP_TYPE_PNG)
      image = source.Scale(imageX, imageY, wx.IMAGE_QUALITY_HIGH)
      bitmap = wx.BitmapFromImage(image)
      dc.DrawBitmap(bitmap,xpos_IMG, ypos_IMG)
      width, height = dc.GetTextExtent(squadron.ship.className)
      dc.DrawRotatedText(squadron.ship.className, xpos_IMG+imageX+height, ypos_IMG, -90)
      ypos_IMG += imageY  #move pointer to the next image position
      
      # heavy cruiser ship image, if applicable
      if (squadron.heavyCruiser):
        dc.DrawRectangle(xpos_IMG, ypos_IMG, imageX, imageY)
        if (squadron.heavyCruiser.imageFilename):
          source = wx.Image(os.path.join(gv.ship_images_folder, squadron.heavyCruiser.imageFilename), wx.BITMAP_TYPE_ANY)
        else:
          source = wx.Image(os.path.join(gv.ship_images_folder, gv.defaultImage), wx.BITMAP_TYPE_PNG)
        image = source.Scale(imageX, imageY, wx.IMAGE_QUALITY_HIGH)
        bitmap = wx.BitmapFromImage(image)
        dc.DrawBitmap(bitmap,xpos_IMG, ypos_IMG)
        width, height = dc.GetTextExtent(squadron.ship.className)
        dc.DrawRotatedText(squadron.heavyCruiser.className, xpos_IMG+imageX+height, ypos_IMG, -90)
        ypos_IMG += imageY  #move pointer to the next image position

      # accompaniment ship image, if applicable
      if (squadron.ship.currentAccompaniment):
        dc.DrawRectangle(xpos_IMG, ypos_IMG, imageX, imageY)
        if (squadron.ship.currentAccompaniment.ship.imageFilename):
          source = wx.Image(os.path.join(gv.ship_images_folder, squadron.ship.currentAccompaniment.ship.imageFilename), wx.BITMAP_TYPE_ANY)
        else:
          source = wx.Image(os.path.join(gv.ship_images_folder, gv.defaultImage), wx.BITMAP_TYPE_PNG)
        image = source.Scale(imageX, imageY, wx.IMAGE_QUALITY_HIGH)
        bitmap = wx.BitmapFromImage(image)
        dc.DrawBitmap(bitmap,xpos_IMG, ypos_IMG)
        width, height = dc.GetTextExtent(squadron.ship.className)
        dc.DrawRotatedText(squadron.ship.currentAccompaniment.ship.className, xpos_IMG+imageX+height, ypos_IMG, -90)
        ypos_IMG += imageY  #move pointer to the next image position

      #Print the SRS block (if any) below the faction and ship images
      if (squadron.ship.currentSRS != []):
        if (squadron.ship.currentSRS[0].srsType != 'None') or (squadron.ship.currentSRS[1].srsType != 'None'):
          textBlock = "{0} class {1}\nShort Range Spacecraft (SRS)\n".format(squadron.ship.className, squadron.ship.designation)
          for srs in squadron.ship.currentSRS:
            if (srs.srsType == 'None'):
              continue
            else:
              textBlock +="{0} squadron, {1} Wings\n    {2}\n".format(srs.srsType, srs.numWings, dataStruct.srsTypes[srs.srsType])
          width,height,lineHeight = dc.GetMultiLineTextExtent(textBlock)
          width += 10 #add a little buffer because we are printing the first line in bold
          dc.SetBrush(wx.Brush(gv.Report_WC_Color))
          dc.DrawRectangle(xpos_IMG, ypos_IMG, width, height)
          textBlock = textBlock.rstrip('\n')
          textList = textBlock.split('\n')
          firstLine = True
          for line in textList:
            if (firstLine):
              dc.SetFont(fontBoldSmall)
              dc.DrawText(line, xpos_IMG, ypos_IMG)
              dc.SetFont(fontSmall) 
              firstLine = False
            else:
              dc.DrawText(line, xpos_IMG, ypos_IMG)
            ypos_IMG += lineHeight
          ypos_IMG += lineHeight #move down for next entry
        
      #Print the SRS block for Accompaniment (if any) below the faction and ship images
      if (squadron.ship.currentAccompaniment) and (squadron.ship.currentAccompaniment.ship.currentSRS != []):
        textBlock = "{0} class {1}\nShort Range Spacecraft (SRS)\n".format(squadron.ship.currentAccompaniment.ship.className, squadron.ship.currentAccompaniment.ship.designation)
        for srs in squadron.ship.currentAccompaniment.ship.currentSRS:
          textBlock +="{0} squadron, {1} Wings\n    {2}\n".format(srs.srsType, srs.numWings, dataStruct.srsTypes[srs.srsType])
        width,height,lineHeight = dc.GetMultiLineTextExtent(textBlock)
        width += 10 #add a little buffer because we are printing the first line in bold
        dc.SetBrush(wx.Brush(gv.Report_WC_Color))
        dc.DrawRectangle(xpos_IMG, ypos_IMG, width, height)
        textBlock = textBlock.rstrip('\n')
        textList = textBlock.split('\n')
        firstLine = True
        for line in textList:
          if (firstLine):
            dc.SetFont(fontBoldSmall)
            dc.DrawText(line, xpos_IMG, ypos_IMG)
            dc.SetFont(fontSmall) 
            firstLine = False
          else:
            dc.DrawText(line, xpos_IMG, ypos_IMG)
          ypos_IMG += lineHeight
        ypos_IMG += lineHeight #move down for next entry
        
      #Print the SRS block for the heavy cruiser (if any) below the faction and ship images
      if (squadron.heavyCruiser) and (squadron.heavyCruiser.currentSRS != []):
        if (squadron.heavyCruiser.currentSRS[0].srsType != 'None') or (squadron.heavyCruiser.currentSRS[1].srsType != 'None'):
          textBlock = "{0} class {1}\nShort Range Spacecraft (SRS)\n".format(squadron.heavyCruiser.className, squadron.heavyCruiser.designation)
          for srs in squadron.heavyCruiser.currentSRS:
            if (srs.srsType == 'None'):
              continue
            else:
              textBlock +="{0} squadron, {1} Wings\n    {2}\n".format(srs.srsType, srs.numWings, dataStruct.srsTypes[srs.srsType])
          width,height,lineHeight = dc.GetMultiLineTextExtent(textBlock)
          width += 10 #add a little buffer because we are printing the first line in bold
          dc.SetBrush(wx.Brush(gv.Report_WC_Color))
          dc.DrawRectangle(xpos_IMG, ypos_IMG, width, height)
          textBlock = textBlock.rstrip('\n')
          textList = textBlock.split('\n')
          firstLine = True
          for line in textList:
            if (firstLine):
              dc.SetFont(fontBoldSmall)
              dc.DrawText(line, xpos_IMG, ypos_IMG)
              dc.SetFont(fontSmall) 
              firstLine = False
            else:
              dc.DrawText(line, xpos_IMG, ypos_IMG)
            ypos_IMG += lineHeight
          ypos_IMG += lineHeight #move down for next entry
        
      #squadron name
      dc.SetBrush(wx.WHITE_BRUSH)
      dc.DrawRectangle(xpos,ypos, 5.5*resolutionDPI, 0.25*resolutionDPI)
      xpos = Xmargin
      ypos = Ymargin
      nameField = ""
      if (squadron.name != ""):
        nameField = squadron.name + ": "
      nameField += squadron.ship.className+" Class "+squadron.ship.designation + " squadron"
      if (squadron.ship.source != 'Official'):
        nameField += ' (Custom Ship)'
      dc.SetFont(fontBoldLarge)
      dc.DrawText(nameField, xpos, ypos)
      ypos += 0.25*resolutionDPI
      
      #number of points/tier/number of ships
      dc.DrawRectangle(xpos,ypos, 5.5*resolutionDPI, 0.25*resolutionDPI)
      if (squadron.ship.currentAccompaniment):
        text = "{0} points / {1} Capital ship / {2} {4} Escorts/ Tier {3} Squadron".format(squadron.ship.currentCost*squadron.size, squadron.size, squadron.ship.currentAccompaniment.numSelected,squadron.tier, squadron.ship.currentAccompaniment.ship.className)
      else:
        if (squadron.size == 1):
          text = "{0} points / {1} ship / Tier {2} Squadron".format(squadron.ship.currentCost*squadron.size, squadron.size, squadron.tier)
        elif (squadron.heavyCruiser):
          text = "{0} points / {1} ships (includes {3} Heavy Cruiser) / Tier {2} Squadron".format((squadron.heavyCruiser.currentCost+squadron.ship.currentCost*squadron.size), squadron.size+1, squadron.tier, squadron.heavyCruiser.className)
        else:
          text = "{0} points / {1} ships / Tier {2} Squadron".format(squadron.ship.currentCost*squadron.size, squadron.size, squadron.tier)
      xpos = Xmargin
      ypos += Ymargin
      dc.SetFont(fontLarge)
      dc.DrawText(text, xpos, ypos)
      ypos += 0.25*resolutionDPI
      
      #main Stat block
      dc.SetFont(fontLarge)
      if (squadron.ship.currentAccompaniment) or (squadron.heavyCruiser):  #print the capital ship class name
        dc.SetFont(fontBoldSmall)
        dc.DrawText("{0}:".format(squadron.ship.className), xpos, ypos)
        ypos += 0.15*resolutionDPI
      
      blockWidth = 0.35*resolutionDPI
      if (True):  #print color stat block backgrounds for main ship
        
        fillBrush = wx.Brush(gv.Report_DR_Color)
        dc.SetBrush(fillBrush)
        dc.DrawRectangle(xpos,ypos,blockWidth, blockWidth)
        xpos += blockWidth
  
        fillBrush = wx.Brush(gv.Report_CR_Color)
        dc.SetBrush(fillBrush)
        dc.DrawRectangle(xpos,ypos,blockWidth, blockWidth)
        xpos += blockWidth
  
        fillBrush = wx.Brush(gv.Report_HP_Color)
        dc.SetBrush(fillBrush)
        dc.DrawRectangle(xpos,ypos,blockWidth, blockWidth)
        xpos += blockWidth
  
        fillBrush = wx.Brush(gv.Report_MV_Color)
        dc.SetBrush(fillBrush)
        dc.DrawRectangle(xpos,ypos,blockWidth, blockWidth)
        xpos += blockWidth
  
        fillBrush = wx.Brush(gv.Report_TL_Color)
        dc.SetBrush(fillBrush)
        dc.DrawRectangle(xpos,ypos,blockWidth, blockWidth)
        xpos += blockWidth
  
        fillBrush = wx.Brush(gv.Report_CP_Color)
        dc.SetBrush(fillBrush)
        dc.DrawRectangle(xpos,ypos,blockWidth, blockWidth)
        xpos += blockWidth
  
        fillBrush = wx.Brush(gv.Report_AP_Color)
        dc.SetBrush(fillBrush)
        dc.DrawRectangle(xpos,ypos,blockWidth, blockWidth)
        xpos += blockWidth
  
        fillBrush = wx.Brush(gv.Report_PD_Color)
        dc.SetBrush(fillBrush)
        dc.DrawRectangle(xpos,ypos,blockWidth, blockWidth)
        xpos += blockWidth
  
        fillBrush = wx.Brush(gv.Report_SR_Color)
        dc.SetBrush(fillBrush)
        dc.DrawRectangle(xpos,ypos,blockWidth, blockWidth)
        xpos += blockWidth
  
        fillBrush = wx.Brush(gv.Report_MN_Color)
        dc.SetBrush(fillBrush)
        dc.DrawRectangle(xpos,ypos,blockWidth, blockWidth)
        xpos += blockWidth
  
        fillBrush = wx.Brush(gv.Report_WC_Color)
        dc.SetBrush(fillBrush)
        dc.DrawRectangle(xpos,ypos,blockWidth, blockWidth)
      
      if (True):  #print stat header (DR, CR, HP, etc.) for main ship
        dc.SetFont(fontBoldSmall)
        ypos += 0.0*resolutionDPI
        xpos = 0.05*resolutionDPI
        dc.DrawText("DR", xpos, ypos)
        xpos += blockWidth
        dc.DrawText("CR", xpos, ypos)
        xpos += blockWidth
        dc.DrawText("HP", xpos, ypos)
        xpos += blockWidth
        dc.DrawText("MV", xpos, ypos)
        xpos += blockWidth
        dc.DrawText("TL", xpos, ypos)
        xpos += blockWidth
        dc.DrawText("CP", xpos, ypos)
        xpos += blockWidth
        dc.DrawText("AP", xpos, ypos)
        xpos += blockWidth
        dc.DrawText("PD", xpos, ypos)
        xpos += blockWidth
        if (squadron.ship.cloakingField):
          dc.DrawText("Cloak", xpos, ypos)
        else:
          dc.DrawText("SR", xpos, ypos)
        xpos += blockWidth
        dc.DrawText("MN", xpos, ypos)
        xpos += blockWidth
        dc.DrawText("WC", xpos, ypos)
      
      if (True):  #print stats for main ship class
        xpos = 0.05*resolutionDPI
        ypos += 0.15*resolutionDPI
        dc.SetFont(fontBoldSmall)
        dc.DrawText(str(squadron.ship.DR), xpos, ypos)
        xpos += blockWidth
        dc.DrawText(str(squadron.ship.CR), xpos, ypos)
        xpos += blockWidth
        dc.DrawText(str(squadron.ship.HP), xpos, ypos)
        xpos += blockWidth
        dc.DrawText(str(squadron.ship.MV), xpos, ypos)
        xpos += blockWidth
        dc.DrawText(str(squadron.ship.TL), xpos, ypos)
        xpos += blockWidth
        dc.DrawText(str(squadron.ship.CP), xpos, ypos)
        xpos += blockWidth
        dc.DrawText(str(squadron.ship.AP), xpos, ypos)
        xpos += blockWidth
        dc.DrawText(str(squadron.ship.PD), xpos, ypos)
        xpos += blockWidth
        if (squadron.ship.cloakingField):
          dc.DrawText("Field",xpos, ypos)
        else:
          dc.DrawText(str(squadron.ship.SR), xpos, ypos)
        xpos += blockWidth
        dc.DrawText(str(squadron.ship.MN), xpos, ypos)
        xpos += blockWidth
        dc.DrawText(str(squadron.ship.WC), xpos, ypos)
      
      if (squadron.heavyCruiser):  #stat block for heavy cruiser ships, if any
        xpos = 0
        ypos += 0.2*resolutionDPI
        dc.SetFont(fontBoldSmall)
        dc.DrawText("{0}:".format(squadron.heavyCruiser.className), xpos, ypos)
        ypos += 0.15*resolutionDPI
        
        fillBrush = wx.Brush(gv.Report_DR_Color)
        dc.SetBrush(fillBrush)
        dc.DrawRectangle(xpos,ypos,blockWidth, blockWidth)
        xpos += blockWidth
  
        fillBrush = wx.Brush(gv.Report_CR_Color)
        dc.SetBrush(fillBrush)
        dc.DrawRectangle(xpos,ypos,blockWidth, blockWidth)
        xpos += blockWidth
  
        fillBrush = wx.Brush(gv.Report_HP_Color)
        dc.SetBrush(fillBrush)
        dc.DrawRectangle(xpos,ypos,blockWidth, blockWidth)
        xpos += blockWidth
  
        fillBrush = wx.Brush(gv.Report_MV_Color)
        dc.SetBrush(fillBrush)
        dc.DrawRectangle(xpos,ypos,blockWidth, blockWidth)
        xpos += blockWidth
  
        fillBrush = wx.Brush(gv.Report_TL_Color)
        dc.SetBrush(fillBrush)
        dc.DrawRectangle(xpos,ypos,blockWidth, blockWidth)
        xpos += blockWidth
  
        fillBrush = wx.Brush(gv.Report_CP_Color)
        dc.SetBrush(fillBrush)
        dc.DrawRectangle(xpos,ypos,blockWidth, blockWidth)
        xpos += blockWidth
  
        fillBrush = wx.Brush(gv.Report_AP_Color)
        dc.SetBrush(fillBrush)
        dc.DrawRectangle(xpos,ypos,blockWidth, blockWidth)
        xpos += blockWidth
  
        fillBrush = wx.Brush(gv.Report_PD_Color)
        dc.SetBrush(fillBrush)
        dc.DrawRectangle(xpos,ypos,blockWidth, blockWidth)
        xpos += blockWidth
  
        fillBrush = wx.Brush(gv.Report_SR_Color)
        dc.SetBrush(fillBrush)
        dc.DrawRectangle(xpos,ypos,blockWidth, blockWidth)
        xpos += blockWidth
  
        fillBrush = wx.Brush(gv.Report_MN_Color)
        dc.SetBrush(fillBrush)
        dc.DrawRectangle(xpos,ypos,blockWidth, blockWidth)
        xpos += blockWidth
  
        fillBrush = wx.Brush(gv.Report_WC_Color)
        dc.SetBrush(fillBrush)
        dc.DrawRectangle(xpos,ypos,blockWidth, blockWidth)
        
        ypos += 0.0*resolutionDPI
        xpos = 0.05*resolutionDPI
        dc.DrawText("DR", xpos, ypos)
        xpos += blockWidth
        dc.DrawText("CR", xpos, ypos)
        xpos += blockWidth
        dc.DrawText("HP", xpos, ypos)
        xpos += blockWidth
        dc.DrawText("MV", xpos, ypos)
        xpos += blockWidth
        dc.DrawText("TL", xpos, ypos)
        xpos += blockWidth
        dc.DrawText("CP", xpos, ypos)
        xpos += blockWidth
        dc.DrawText("AP", xpos, ypos)
        xpos += blockWidth
        dc.DrawText("PD", xpos, ypos)
        xpos += blockWidth
        dc.DrawText("SR", xpos, ypos)
        xpos += blockWidth
        dc.DrawText("MN", xpos, ypos)
        xpos += blockWidth
        dc.DrawText("WC", xpos, ypos)
        
        xpos = 0.05*resolutionDPI
        ypos += 0.15*resolutionDPI
        dc.DrawText(str(squadron.heavyCruiser.DR), xpos, ypos)
        xpos += blockWidth
        dc.DrawText(str(squadron.heavyCruiser.CR), xpos, ypos)
        xpos += blockWidth
        dc.DrawText(str(squadron.heavyCruiser.HP), xpos, ypos)
        xpos += blockWidth
        dc.DrawText(str(squadron.heavyCruiser.MV), xpos, ypos)
        xpos += blockWidth
        dc.DrawText(str(squadron.heavyCruiser.TL), xpos, ypos)
        xpos += blockWidth
        dc.DrawText(str(squadron.heavyCruiser.CP), xpos, ypos)
        xpos += blockWidth
        dc.DrawText(str(squadron.heavyCruiser.AP), xpos, ypos)
        xpos += blockWidth
        dc.DrawText(str(squadron.heavyCruiser.PD), xpos, ypos)
        xpos += blockWidth
        dc.DrawText(str(squadron.heavyCruiser.SR), xpos, ypos)
        xpos += blockWidth
        dc.DrawText(str(squadron.heavyCruiser.MN), xpos, ypos)
        xpos += blockWidth
        dc.DrawText(str(squadron.heavyCruiser.WC), xpos, ypos)

      if (squadron.ship.currentAccompaniment):  #stat block for accompaniment ships, if any
        xpos = 0
        ypos += 0.2*resolutionDPI
        dc.SetFont(fontBoldSmall)
        dc.DrawText("{0}:".format(squadron.ship.currentAccompaniment.ship.className), xpos, ypos)
        ypos += 0.15*resolutionDPI
        
        fillBrush = wx.Brush(gv.Report_DR_Color)
        dc.SetBrush(fillBrush)
        dc.DrawRectangle(xpos,ypos,blockWidth, blockWidth)
        xpos += blockWidth
  
        fillBrush = wx.Brush(gv.Report_CR_Color)
        dc.SetBrush(fillBrush)
        dc.DrawRectangle(xpos,ypos,blockWidth, blockWidth)
        xpos += blockWidth
  
        fillBrush = wx.Brush(gv.Report_HP_Color)
        dc.SetBrush(fillBrush)
        dc.DrawRectangle(xpos,ypos,blockWidth, blockWidth)
        xpos += blockWidth
  
        fillBrush = wx.Brush(gv.Report_MV_Color)
        dc.SetBrush(fillBrush)
        dc.DrawRectangle(xpos,ypos,blockWidth, blockWidth)
        xpos += blockWidth
  
        fillBrush = wx.Brush(gv.Report_TL_Color)
        dc.SetBrush(fillBrush)
        dc.DrawRectangle(xpos,ypos,blockWidth, blockWidth)
        xpos += blockWidth
  
        fillBrush = wx.Brush(gv.Report_CP_Color)
        dc.SetBrush(fillBrush)
        dc.DrawRectangle(xpos,ypos,blockWidth, blockWidth)
        xpos += blockWidth
  
        fillBrush = wx.Brush(gv.Report_AP_Color)
        dc.SetBrush(fillBrush)
        dc.DrawRectangle(xpos,ypos,blockWidth, blockWidth)
        xpos += blockWidth
  
        fillBrush = wx.Brush(gv.Report_PD_Color)
        dc.SetBrush(fillBrush)
        dc.DrawRectangle(xpos,ypos,blockWidth, blockWidth)
        xpos += blockWidth
  
        fillBrush = wx.Brush(gv.Report_SR_Color)
        dc.SetBrush(fillBrush)
        dc.DrawRectangle(xpos,ypos,blockWidth, blockWidth)
        xpos += blockWidth
  
        fillBrush = wx.Brush(gv.Report_MN_Color)
        dc.SetBrush(fillBrush)
        dc.DrawRectangle(xpos,ypos,blockWidth, blockWidth)
        xpos += blockWidth
  
        fillBrush = wx.Brush(gv.Report_WC_Color)
        dc.SetBrush(fillBrush)
        dc.DrawRectangle(xpos,ypos,blockWidth, blockWidth)
        
        ypos += 0.0*resolutionDPI
        xpos = 0.05*resolutionDPI
        dc.DrawText("DR", xpos, ypos)
        xpos += blockWidth
        dc.DrawText("CR", xpos, ypos)
        xpos += blockWidth
        dc.DrawText("HP", xpos, ypos)
        xpos += blockWidth
        dc.DrawText("MV", xpos, ypos)
        xpos += blockWidth
        dc.DrawText("TL", xpos, ypos)
        xpos += blockWidth
        dc.DrawText("CP", xpos, ypos)
        xpos += blockWidth
        dc.DrawText("AP", xpos, ypos)
        xpos += blockWidth
        dc.DrawText("PD", xpos, ypos)
        xpos += blockWidth
        dc.DrawText("SR", xpos, ypos)
        xpos += blockWidth
        dc.DrawText("MN", xpos, ypos)
        xpos += blockWidth
        dc.DrawText("WC", xpos, ypos)
        
        xpos = 0.05*resolutionDPI
        ypos += 0.15*resolutionDPI
        dc.DrawText(str(squadron.ship.currentAccompaniment.ship.DR), xpos, ypos)
        xpos += blockWidth
        dc.DrawText(str(squadron.ship.currentAccompaniment.ship.CR), xpos, ypos)
        xpos += blockWidth
        dc.DrawText(str(squadron.ship.currentAccompaniment.ship.HP), xpos, ypos)
        xpos += blockWidth
        dc.DrawText(str(squadron.ship.currentAccompaniment.ship.MV), xpos, ypos)
        xpos += blockWidth
        dc.DrawText(str(squadron.ship.currentAccompaniment.ship.TL), xpos, ypos)
        xpos += blockWidth
        dc.DrawText(str(squadron.ship.currentAccompaniment.ship.CP), xpos, ypos)
        xpos += blockWidth
        dc.DrawText(str(squadron.ship.currentAccompaniment.ship.AP), xpos, ypos)
        xpos += blockWidth
        dc.DrawText(str(squadron.ship.currentAccompaniment.ship.PD), xpos, ypos)
        xpos += blockWidth
        dc.DrawText(str(squadron.ship.currentAccompaniment.ship.SR), xpos, ypos)
        xpos += blockWidth
        dc.DrawText(str(squadron.ship.currentAccompaniment.ship.MN), xpos, ypos)
        xpos += blockWidth
        dc.DrawText(str(squadron.ship.currentAccompaniment.ship.WC), xpos, ypos)

      dc.SetFont(fontSmall)
      #section for direct fire weapons, if any
      directFire = False
      indirectFire = False
      for weapon in squadron.ship.currentWeapons:  #which weapon types need to be printed?
        if (weapon.category == 'Direct'):
          directFire = True
        if (weapon.category == 'Indirect'):
          indirectFire = True

      if (squadron.ship.currentAccompaniment):  #which weapon types need to be printed?
        for weapon in squadron.ship.currentWeapons:
          if (weapon.category == 'Direct'):
            directFire = True
          if (weapon.category == 'Indirect'):
            indirectFire = True

      if (squadron.heavyCruiser):  #which weapon types need to be printed?
        for weapon in squadron.heavyCruiser.currentWeapons:
          if (weapon.category == 'Direct'):
            directFire = True
          if (weapon.category == 'Indirect'):
            indirectFire = True                
  
      fillBrush = wx.Brush(gv.Report_DirectFireHeader_Color)
      dc.SetBrush(fillBrush)
      xpos = 0
      ypos += 0.25*resolutionDPI
      dc.DrawRectangle(xpos, ypos ,5.5*resolutionDPI, 0.20*resolutionDPI)
      if (not directFire):
        dc.DrawText("No Direct Fire Weapons Available", xpos, ypos)
        ypos += 0.20*resolutionDPI
      else:
        dc.DrawText( "Direct Fire Weapons",xpos+0.05*resolutionDPI, ypos)
        dc.DrawText("RB1",xpos+4.0*resolutionDPI, ypos)
        dc.DrawText("RB2",xpos+4.4*resolutionDPI, ypos)
        dc.DrawText("RB3",xpos+4.8*resolutionDPI, ypos)
        dc.DrawText("RB4",xpos+5.2*resolutionDPI, ypos)
        xpos += 0.10*resolutionDPI  #offset entries under Direct Fire Weapons
        ypos += 0.20*resolutionDPI

        for weaponType in dataStruct.weaponTypes:
          printBlock = False
          for weapon in squadron.ship.currentWeapons:
            if (weaponType in weapon.weaponType):
              printBlock = True
              break
          if (not printBlock):  #continue to the next weapon type if there are no matching weapons
            continue
          needHeader = True
          for weapon in squadron.ship.currentWeapons:  #print stats for individual Direct fire weapons
            if (weapon.category != 'Direct'):
              continue
            if (needHeader):  #print the Ship|Weapon Type|Weapon Arc|Range Increments header
              needHeader = False
              fillBrush = wx.Brush(gv.Report_DirectFireHeader_Color)
              dc.SetBrush(fillBrush)
              dc.DrawRectangle(xpos, ypos, 5.4*resolutionDPI, 0.20*resolutionDPI)
              dc.DrawText("Ship",xpos+0.05*resolutionDPI, ypos)
              dc.DrawText("Weapon Type",xpos+1.1*resolutionDPI, ypos)
              dc.DrawText("Weapon Arc",xpos+2.2*resolutionDPI, ypos)
              if (weaponType == 'Beam') and (weapon.category == 'Direct'):
                dc.DrawText('<10"',xpos+3.9*resolutionDPI, ypos)
                dc.DrawText('<20"',xpos+4.3*resolutionDPI, ypos)
                dc.DrawText('<30"',xpos+4.7*resolutionDPI, ypos)
                dc.DrawText('<40"',xpos+5.1*resolutionDPI, ypos)
              elif ((weaponType == 'Gun Rack') or (weaponType == 'Primary')) and (weapon.category == 'Direct'):
                dc.DrawText('<8"',xpos+3.9*resolutionDPI, ypos)
                dc.DrawText('<16"',xpos+4.3*resolutionDPI, ypos)
                dc.DrawText('<24"',xpos+4.7*resolutionDPI, ypos)
                dc.DrawText('<32"',xpos+5.1*resolutionDPI, ypos)
              elif (weaponType == 'Kinetic') and (weapon.category == 'Direct'):
                dc.DrawText('<12"',xpos+3.9*resolutionDPI, ypos)
                dc.DrawText('<24"',xpos+4.3*resolutionDPI, ypos)
                dc.DrawText('<36"',xpos+4.7*resolutionDPI, ypos)
                dc.DrawText('<48"',xpos+5.1*resolutionDPI, ypos)
              elif (weaponType == 'Scatter') and (weapon.category == 'Direct'):
                dc.DrawText('<8"',xpos+3.9*resolutionDPI, ypos)
                dc.DrawText('<16"',xpos+4.3*resolutionDPI, ypos)
                dc.DrawText('<24"',xpos+4.7*resolutionDPI, ypos)
                dc.DrawText('<32"',xpos+5.1*resolutionDPI, ypos)
              elif (weaponType == 'Nuclear') and (weapon.category == 'Direct'):
                dc.DrawText('<8"',xpos+3.9*resolutionDPI, ypos)
                dc.DrawText('<16"',xpos+4.3*resolutionDPI, ypos)
                dc.DrawText('<24"',xpos+4.7*resolutionDPI, ypos)
                dc.DrawText('<32"',xpos+5.1*resolutionDPI, ypos)
              ypos += 0.20*resolutionDPI  #move the 'pointer' down ready for the next entry.
            if (weaponType in weapon.weaponType) and (weapon.category == 'Direct'):
              fillBrush = wx.Brush(gv.Report_DirectFire_Color)
              dc.SetBrush(fillBrush)
              dc.DrawRectangle(xpos, ypos, 5.4*resolutionDPI, 0.20*resolutionDPI)
              dc.DrawText("{0}".format(squadron.ship.className),xpos+0.05*resolutionDPI, ypos)
              dc.DrawText("{0} {1}".format(weapon.weaponType, weapon.weaponUpgrade),xpos+1.1*resolutionDPI, ypos)
              dc.DrawText("{0} {1}".format(weapon.arcDirection, weapon.arcRestriction),xpos+2.2*resolutionDPI, ypos)
              dc.DrawText("{0}".format(weapon.rb1),xpos+3.9*resolutionDPI, ypos)
              dc.DrawText("{0}".format(weapon.rb2),xpos+4.3*resolutionDPI, ypos)
              dc.DrawText("{0}".format(weapon.rb3),xpos+4.7*resolutionDPI, ypos)
              dc.DrawText("{0}".format(weapon.rb4),xpos+5.1*resolutionDPI, ypos)
              ypos += 0.20*resolutionDPI  #move the 'pointer' down ready for the next entry.
          if (squadron.heavyCruiser):
            for weapon in squadron.heavyCruiser.currentWeapons:
              if (weapon.category != 'Direct'):
                continue
              if (needHeader):  #print the Ship|Weapon Type|Weapon Arc|Range Increments header
                needHeader = False
                fillBrush = wx.Brush(gv.Report_DirectFireHeader_Color)
                dc.SetBrush(fillBrush)
                dc.DrawRectangle(xpos, ypos, 5.4*resolutionDPI, 0.20*resolutionDPI)
                dc.DrawText("Ship",xpos+0.05*resolutionDPI, ypos)
                dc.DrawText("Weapon Type",xpos+1.1*resolutionDPI, ypos)
                dc.DrawText("Weapon Arc",xpos+2.2*resolutionDPI, ypos)
                if (weaponType == 'Beam') and (weapon.category == 'Direct'):
                  dc.DrawText('<10"',xpos+3.9*resolutionDPI, ypos)
                  dc.DrawText('<20"',xpos+4.3*resolutionDPI, ypos)
                  dc.DrawText('<30"',xpos+4.7*resolutionDPI, ypos)
                  dc.DrawText('<40"',xpos+5.1*resolutionDPI, ypos)
                elif ((weaponType == 'Gun Rack') or (weaponType == 'Primary')) and (weapon.category == 'Direct'):
                  dc.DrawText('<8"',xpos+3.9*resolutionDPI, ypos)
                  dc.DrawText('<16"',xpos+4.3*resolutionDPI, ypos)
                  dc.DrawText('<24"',xpos+4.7*resolutionDPI, ypos)
                  dc.DrawText('<32"',xpos+5.1*resolutionDPI, ypos)
                elif (weaponType == 'Kinetic') and (weapon.category == 'Direct'):
                  dc.DrawText('<12"',xpos+3.9*resolutionDPI, ypos)
                  dc.DrawText('<24"',xpos+4.3*resolutionDPI, ypos)
                  dc.DrawText('<36"',xpos+4.7*resolutionDPI, ypos)
                  dc.DrawText('<48"',xpos+5.1*resolutionDPI, ypos)
                elif (weaponType == 'Scatter') and (weapon.category == 'Direct'):
                  dc.DrawText('<8"',xpos+3.9*resolutionDPI, ypos)
                  dc.DrawText('<16"',xpos+4.3*resolutionDPI, ypos)
                  dc.DrawText('<24"',xpos+4.7*resolutionDPI, ypos)
                  dc.DrawText('<32"',xpos+5.1*resolutionDPI, ypos)
                elif (weaponType == 'Nuclear') and (weapon.category == 'Direct'):
                  dc.DrawText('<8"',xpos+3.9*resolutionDPI, ypos)
                  dc.DrawText('<16"',xpos+4.3*resolutionDPI, ypos)
                  dc.DrawText('<24"',xpos+4.7*resolutionDPI, ypos)
                  dc.DrawText('<32"',xpos+5.1*resolutionDPI, ypos)
                ypos += 0.20*resolutionDPI  #move the 'pointer' down ready for the next entry.
              if (weaponType in weapon.weaponType) and (weapon.category == 'Direct'):
                fillBrush = wx.Brush(gv.Report_DirectFire_Color)
                dc.SetBrush(fillBrush)
                dc.DrawRectangle(xpos, ypos, 5.4*resolutionDPI, 0.20*resolutionDPI)
                dc.DrawText("{0}".format(squadron.heavyCruiser.className),xpos+0.05*resolutionDPI, ypos)
                dc.DrawText("{0} {1}".format(weapon.weaponType, weapon.weaponUpgrade),xpos+1.1*resolutionDPI, ypos)
                dc.DrawText("{0} {1}".format(weapon.arcDirection, weapon.arcRestriction),xpos+2.2*resolutionDPI, ypos)
                dc.DrawText("{0}".format(weapon.rb1),xpos+3.9*resolutionDPI, ypos)
                dc.DrawText("{0}".format(weapon.rb2),xpos+4.3*resolutionDPI, ypos)
                dc.DrawText("{0}".format(weapon.rb3),xpos+4.7*resolutionDPI, ypos)
                dc.DrawText("{0}".format(weapon.rb4),xpos+5.1*resolutionDPI, ypos)
                ypos += 0.20*resolutionDPI  #move the 'pointer' down ready for the next entry.
          if (squadron.ship.currentAccompaniment):
            for weapon in squadron.ship.currentAccompaniment.ship.currentWeapons:
              if (weapon.category != 'Direct'):
                continue
              if (needHeader):  #print the Ship|Weapon Type|Weapon Arc|Range Increments header
                needHeader = False
                fillBrush = wx.Brush(gv.Report_DirectFireHeader_Color)
                dc.SetBrush(fillBrush)
                dc.DrawRectangle(xpos, ypos, 5.4*resolutionDPI, 0.20*resolutionDPI)
                dc.DrawText("Ship",xpos+0.05*resolutionDPI, ypos)
                dc.DrawText("Weapon Type",xpos+1.1*resolutionDPI, ypos)
                dc.DrawText("Weapon Arc",xpos+2.2*resolutionDPI, ypos)
                if (weaponType == 'Beam') and (weapon.category == 'Direct'):
                  dc.DrawText('<10"',xpos+3.9*resolutionDPI, ypos)
                  dc.DrawText('<20"',xpos+4.3*resolutionDPI, ypos)
                  dc.DrawText('<30"',xpos+4.7*resolutionDPI, ypos)
                  dc.DrawText('<40"',xpos+5.1*resolutionDPI, ypos)
                elif ((weaponType == 'Gun Rack') or (weaponType == 'Primary')) and (weapon.category == 'Direct'):
                  dc.DrawText('<8"',xpos+3.9*resolutionDPI, ypos)
                  dc.DrawText('<16"',xpos+4.3*resolutionDPI, ypos)
                  dc.DrawText('<24"',xpos+4.7*resolutionDPI, ypos)
                  dc.DrawText('<32"',xpos+5.1*resolutionDPI, ypos)
                elif (weaponType == 'Kinetic') and (weapon.category == 'Direct'):
                  dc.DrawText('<12"',xpos+3.9*resolutionDPI, ypos)
                  dc.DrawText('<24"',xpos+4.3*resolutionDPI, ypos)
                  dc.DrawText('<36"',xpos+4.7*resolutionDPI, ypos)
                  dc.DrawText('<48"',xpos+5.1*resolutionDPI, ypos)
                elif (weaponType == 'Scatter') and (weapon.category == 'Direct'):
                  dc.DrawText('<8"',xpos+3.9*resolutionDPI, ypos)
                  dc.DrawText('<16"',xpos+4.3*resolutionDPI, ypos)
                  dc.DrawText('<24"',xpos+4.7*resolutionDPI, ypos)
                  dc.DrawText('<32"',xpos+5.1*resolutionDPI, ypos)
                elif (weaponType == 'Nuclear') and (weapon.category == 'Direct'):
                  dc.DrawText('<8"',xpos+3.9*resolutionDPI, ypos)
                  dc.DrawText('<16"',xpos+4.3*resolutionDPI, ypos)
                  dc.DrawText('<24"',xpos+4.7*resolutionDPI, ypos)
                  dc.DrawText('<32"',xpos+5.1*resolutionDPI, ypos)
                ypos += 0.20*resolutionDPI  #move the 'pointer' down ready for the next entry.
              if (weaponType in weapon.weaponType) and (weapon.category == 'Direct'):
                fillBrush = wx.Brush(gv.Report_DirectFire_Color)
                dc.SetBrush(fillBrush)
                dc.DrawRectangle(xpos, ypos, 5.4*resolutionDPI, 0.20*resolutionDPI)
                dc.DrawText("{0}".format(squadron.ship.currentAccompaniment.ship.className),xpos+0.05*resolutionDPI, ypos)
                dc.DrawText("{0} {1}".format(weapon.weaponType, weapon.weaponUpgrade),xpos+1.1*resolutionDPI, ypos)
                dc.DrawText("{0} {1}".format(weapon.arcDirection, weapon.arcRestriction),xpos+2.2*resolutionDPI, ypos)
                dc.DrawText("{0}".format(weapon.rb1),xpos+3.9*resolutionDPI, ypos)
                dc.DrawText("{0}".format(weapon.rb2),xpos+4.3*resolutionDPI, ypos)
                dc.DrawText("{0}".format(weapon.rb3),xpos+4.7*resolutionDPI, ypos)
                dc.DrawText("{0}".format(weapon.rb4),xpos+5.1*resolutionDPI, ypos)
                ypos += 0.20*resolutionDPI  #move the 'pointer' down ready for the next entry.  
      
      fillBrush = wx.Brush(gv.Report_IndirectFireHeader_Color)
      dc.SetBrush(fillBrush)
      xpos = 0
      ypos += 0.05*resolutionDPI
      dc.DrawRectangle(xpos, ypos ,5.5*resolutionDPI, 0.20*resolutionDPI)
      if (not indirectFire):
        dc.DrawText("No Indirect Fire Weapons Available", xpos, ypos)
        ypos += 0.20*resolutionDPI
      else:
        dc.DrawText( "Indirect Fire Weapons",xpos+0.05*resolutionDPI, ypos)
        dc.DrawText("RB1",xpos+4.0*resolutionDPI, ypos)
        dc.DrawText("RB2",xpos+4.4*resolutionDPI, ypos)
        dc.DrawText("RB3",xpos+4.8*resolutionDPI, ypos)
        dc.DrawText("RB4",xpos+5.2*resolutionDPI, ypos)
        xpos += 0.10*resolutionDPI  #offset entries under Indirect Fire Weapons
        ypos += 0.20*resolutionDPI
      
        for weaponType in dataStruct.indirectWeaponTypes:
          printBlock = False
          for weapon in squadron.ship.currentWeapons:
            if (weaponType in weapon.weaponType):
              printBlock = True
              break
          if (not printBlock):  #continue to the next weapon type if there are no matching weapons
            continue
          needHeader = True
          for weapon in squadron.ship.currentWeapons:
            if (weapon.category != 'Indirect'):
              continue
            if (needHeader):  #print the Ship|Weapon Type|Weapon Arc|Range Increments header
              needHeader = False
              fillBrush = wx.Brush(gv.Report_IndirectFireHeader_Color)
              dc.SetBrush(fillBrush)
              dc.DrawRectangle(xpos, ypos, 5.4*resolutionDPI, 0.20*resolutionDPI)
              dc.DrawText("Ship",xpos+0.05*resolutionDPI, ypos)
              dc.DrawText("Weapon Type",xpos+1.1*resolutionDPI, ypos)
              dc.DrawText("Weapon Arc",xpos+2.2*resolutionDPI, ypos)
              if (weaponType == 'Cyberwarfare'):
                dc.DrawText('<10"',xpos+3.9*resolutionDPI, ypos)
                dc.DrawText('<20"',xpos+4.3*resolutionDPI, ypos)
                dc.DrawText('<30"',xpos+4.7*resolutionDPI, ypos)
                dc.DrawText('<40"',xpos+5.1*resolutionDPI, ypos)
              elif (weaponType == 'Gravitational'):
                dc.DrawText('<8"',xpos+3.9*resolutionDPI, ypos)
                dc.DrawText('<16"',xpos+4.3*resolutionDPI, ypos)
                dc.DrawText('<24"',xpos+4.7*resolutionDPI, ypos)
                dc.DrawText('<32"',xpos+5.1*resolutionDPI, ypos)
              elif ((weaponType == 'Torpedo') or (weaponType == 'Nuclear Torpedo')):
                dc.DrawText('<12"',xpos+3.9*resolutionDPI, ypos)
                dc.DrawText('<24"',xpos+4.3*resolutionDPI, ypos)
                dc.DrawText('<36"',xpos+4.7*resolutionDPI, ypos)
                dc.DrawText('<48"',xpos+5.1*resolutionDPI, ypos)
              ypos += 0.20*resolutionDPI  #move the 'pointer' down ready for the next entry.
            if (weaponType in weapon.weaponType) and (weapon.category == 'Indirect'):
              fillBrush = wx.Brush(gv.Report_IndirectFire_Color)
              dc.SetBrush(fillBrush)
              dc.DrawRectangle(xpos, ypos, 5.4*resolutionDPI, 0.20*resolutionDPI)
              dc.DrawText("{0}".format(squadron.ship.className),xpos+0.05*resolutionDPI, ypos)
              dc.DrawText("{0} {1}".format(weapon.weaponType, weapon.weaponUpgrade),xpos+1.1*resolutionDPI, ypos)
              dc.DrawText("{0} {1}".format(weapon.arcDirection, weapon.arcRestriction),xpos+2.23*resolutionDPI, ypos)
              dc.DrawText("{0}".format(weapon.rb1),xpos+3.9*resolutionDPI, ypos)
              dc.DrawText("{0}".format(weapon.rb2),xpos+4.3*resolutionDPI, ypos)
              dc.DrawText("{0}".format(weapon.rb3),xpos+4.7*resolutionDPI, ypos)
              dc.DrawText("{0}".format(weapon.rb4),xpos+5.1*resolutionDPI, ypos)
              ypos += 0.20*resolutionDPI
          if (squadron.heavyCruiser):  #print out the indirect weapons on the Heavy Cruiser, if any
            for weapon in squadron.heavyCruiser.currentWeapons:
              if (weapon.category != 'Indirect'):
                continue
              if (needHeader):  #print the Ship|Weapon Type|Weapon Arc|Range Increments header
                needHeader = False
                fillBrush = wx.Brush(gv.Report_DirectFireHeader_Color)
                dc.SetBrush(fillBrush)
                dc.DrawRectangle(xpos, ypos, 5.4*resolutionDPI, 0.20*resolutionDPI)
                dc.DrawText("Ship",xpos+0.05*resolutionDPI, ypos)
                dc.DrawText("Weapon Type",xpos+1.1*resolutionDPI, ypos)
                dc.DrawText("Weapon Arc",xpos+2.2*resolutionDPI, ypos)
                if (weaponType == 'Cyberwarfare'):
                  dc.DrawText('<10"',xpos+3.9*resolutionDPI, ypos)
                  dc.DrawText('<20"',xpos+4.3*resolutionDPI, ypos)
                  dc.DrawText('<30"',xpos+4.7*resolutionDPI, ypos)
                  dc.DrawText('<40"',xpos+5.1*resolutionDPI, ypos)
                elif (weaponType == 'Gravitational'):
                  dc.DrawText('<8"',xpos+3.9*resolutionDPI, ypos)
                  dc.DrawText('<16"',xpos+4.3*resolutionDPI, ypos)
                  dc.DrawText('<24"',xpos+4.7*resolutionDPI, ypos)
                  dc.DrawText('<32"',xpos+5.1*resolutionDPI, ypos)
                elif ((weaponType == 'Torpedo') or (weaponType == 'Nuclear Torpedo')):
                  dc.DrawText('<12"',xpos+3.9*resolutionDPI, ypos)
                  dc.DrawText('<24"',xpos+4.3*resolutionDPI, ypos)
                  dc.DrawText('<36"',xpos+4.7*resolutionDPI, ypos)
                  dc.DrawText('<48"',xpos+5.1*resolutionDPI, ypos)
                  ypos += 0.20*resolutionDPI  #move the 'pointer' down ready for the next entry.
              if (weaponType in weapon.weaponType) and (weapon.category == 'Indirect'):
                fillBrush = wx.Brush(gv.Report_IndirectFire_Color)
                dc.SetBrush(fillBrush)
                dc.DrawRectangle(xpos, ypos, 5.4*resolutionDPI, 0.20*resolutionDPI)
                dc.DrawText("{0}".format(squadron.heavyCruiser.className),xpos+0.05*resolutionDPI, ypos)
                dc.DrawText("{0} {1}".format(weapon.weaponType, weapon.weaponUpgrade),xpos+1.1*resolutionDPI, ypos)
                dc.DrawText("{0} {1}".format(weapon.arcDirection, weapon.arcRestriction),xpos+2.2*resolutionDPI, ypos)
                dc.DrawText("{0}".format(weapon.rb1),xpos+3.9*resolutionDPI, ypos)
                dc.DrawText("{0}".format(weapon.rb2),xpos+4.3*resolutionDPI, ypos)
                dc.DrawText("{0}".format(weapon.rb3),xpos+4.7*resolutionDPI, ypos)
                dc.DrawText("{0}".format(weapon.rb4),xpos+5.1*resolutionDPI, ypos)
                ypos += 0.20*resolutionDPI  #move the 'pointer' down ready for the next entry.

          if (squadron.ship.currentAccompaniment):  #print out the indirect weapons on the Heavy Cruiser, if any
            for weapon in squadron.ship.currentAccompaniment.ship.currentWeapons:
              if (weapon.category != 'Indirect'):
                continue
              if (needHeader):  #print the Ship|Weapon Type|Weapon Arc|Range Increments header
                needHeader = False
                fillBrush = wx.Brush(gv.Report_DirectFireHeader_Color)
                dc.SetBrush(fillBrush)
                dc.DrawRectangle(xpos, ypos, 5.4*resolutionDPI, 0.20*resolutionDPI)
                dc.DrawText("Ship",xpos+0.05*resolutionDPI, ypos)
                dc.DrawText("Weapon Type",xpos+1.1*resolutionDPI, ypos)
                dc.DrawText("Weapon Arc",xpos+2.2*resolutionDPI, ypos)
                if (weaponType == 'Cyberwarfare'):
                  dc.DrawText('<10"',xpos+3.9*resolutionDPI, ypos)
                  dc.DrawText('<20"',xpos+4.3*resolutionDPI, ypos)
                  dc.DrawText('<30"',xpos+4.7*resolutionDPI, ypos)
                  dc.DrawText('<40"',xpos+5.1*resolutionDPI, ypos)
                elif (weaponType == 'Gravitational'):
                  dc.DrawText('<8"',xpos+3.9*resolutionDPI, ypos)
                  dc.DrawText('<16"',xpos+4.3*resolutionDPI, ypos)
                  dc.DrawText('<24"',xpos+4.7*resolutionDPI, ypos)
                  dc.DrawText('<32"',xpos+5.1*resolutionDPI, ypos)
                elif ((weaponType == 'Torpedo') or (weaponType == 'Nuclear Torpedo')):
                  dc.DrawText('<12"',xpos+3.9*resolutionDPI, ypos)
                  dc.DrawText('<24"',xpos+4.3*resolutionDPI, ypos)
                  dc.DrawText('<36"',xpos+4.7*resolutionDPI, ypos)
                  dc.DrawText('<48"',xpos+5.1*resolutionDPI, ypos)
                  ypos += 0.20*resolutionDPI  #move the 'pointer' down ready for the next entry.
              if (weaponType in weapon.weaponType) and (weapon.category == 'Indirect'):
                fillBrush = wx.Brush(gv.Report_IndirectFire_Color)
                dc.SetBrush(fillBrush)
                dc.DrawRectangle(xpos, ypos, 5.4*resolutionDPI, 0.20*resolutionDPI)
                dc.DrawText("{0}".format(squadron.ship.currentAccompaniment.ship.className),xpos+0.05*resolutionDPI, ypos)
                dc.DrawText("{0} {1}".format(weapon.weaponType, weapon.weaponUpgrade),xpos+1.1*resolutionDPI, ypos)
                dc.DrawText("{0} {1}".format(weapon.arcDirection, weapon.arcRestriction),xpos+2.2*resolutionDPI, ypos)
                dc.DrawText("{0}".format(weapon.rb1),xpos+3.9*resolutionDPI, ypos)
                dc.DrawText("{0}".format(weapon.rb2),xpos+4.3*resolutionDPI, ypos)
                dc.DrawText("{0}".format(weapon.rb3),xpos+4.7*resolutionDPI, ypos)
                dc.DrawText("{0}".format(weapon.rb4),xpos+5.1*resolutionDPI, ypos)
                ypos += 0.20*resolutionDPI  #move the 'pointer' down ready for the next entry.  

      lineHeight = 0
      #print out the MAR's, either full or partial text
      dc.SetFont(fontBoldSmall)
      (width, height) = dc.GetTextExtent("Model Assigned Rules (MAR)")
      xpos = 0
      dc.DrawText("Model Assigned Rules (MAR)", xpos, ypos)
      ypos += height
      dc.SetFont(fontSmall)
      if (squadron.ship.currentMAR == []):
        dc.SetFont(fontBoldSmall)  
        dc.DrawText("No MARs on {0}".format(squadron.ship.className), xpos, ypos)
        dc.SetFont(fontSmall)
        ypos += height
      for mar in squadron.ship.currentMAR:
        if (gv.PrintMAR != 'Simple'):
          if (gv.PrintMAR == 'Detailed'):
            text = mar.marText
          else:
            text = mar.marSummary
          if (ypos <= ypos_IMG):
            textBlock = ww.wordwrap(text, printAreaWidth-2.2*resolutionDPI, dc)
          else:
            textBlock = ww.wordwrap(text, printAreaWidth, dc)
          (width, ysize, lineHeight) = dc.GetMultiLineTextExtent(textBlock)
          xpos = 0.05*resolutionDPI
          dc.SetFont(fontBoldSmall)
          dc.DrawText((mar.marName+" ("+squadron.ship.className+")"), xpos, ypos)
          xpos = 0.1 * resolutionDPI
          ypos += lineHeight
          dc.SetFont(fontSmall)
          textList = textBlock.split('\n')
          for line in textList:
            dc.DrawText(line, xpos, ypos)
            ypos += lineHeight
        else:
          xpos = 0.05*resolutionDPI
          (width, height) = dc.GetTextExtent(mar.marName)
          dc.SetFont(fontBoldSmall)
          dc.DrawText((mar.marName+" ("+squadron.ship.className+")"), xpos, ypos)
          ypos += height
      dc.SetFont(fontSmall)
      
      #print out MARs from accompaniment, if any
      if (squadron.ship.currentAccompaniment):
        if (squadron.ship.currentAccompaniment.ship.currentMAR == []):
          dc.SetFont(fontBoldSmall)
          dc.DrawText("No MARs on {0}".format(squadron.ship.currentAccompaniment.ship.className), xpos+Xmargin, ypos)
          dc.SetFont(fontSmall)
          ypos += height
        for mar in squadron.ship.currentAccompaniment.ship.currentMAR:
          if (gv.PrintMAR != 'Simple'):
            if (gv.PrintMAR == 'Detailed'):
              text = mar.marText
            else:
              text = mar.marSummary
            if (ypos <= ypos_IMG):
              textBlock = ww.wordwrap(text, printAreaWidth-2.2*resolutionDPI, dc)
            else:
              textBlock = ww.wordwrap(text, printAreaWidth, dc)
            (width, ysize, lineHeight) = dc.GetMultiLineTextExtent(textBlock)
            xpos = 0.05*resolutionDPI
            dc.SetFont(fontBoldSmall)
            dc.DrawText((mar.marName+" ("+squadron.ship.currentAccompaniment.ship.className+")"), xpos, ypos)
            xpos = 0.1 * resolutionDPI
            ypos += lineHeight
            dc.SetFont(fontSmall)
            textList = textBlock.split('\n')
            for line in textList:
              dc.DrawText(line, xpos, ypos)
              ypos += lineHeight
          else:
            xpos = 0.05*resolutionDPI
            (width, height) = dc.GetTextExtent(mar.marName)
            dc.SetFont(fontBoldSmall)
            dc.DrawText((mar.marName+" ("+squadron.ship.currentAccompaniment.ship.className+")"), xpos, ypos)
            ypos += height
        dc.SetFont(fontSmall)
        
      #print out MARs from heavy cruiser, if any
      if (squadron.heavyCruiser):
        if (squadron.heavyCruiser.currentMAR == []):
          dc.SetFont(fontBoldSmall)
          dc.DrawText("No MARs on {0}".format(squadron.heavyCruiser.className), xpos+Xmargin, ypos)
          dc.SetFont(fontSmall)
          ypos += height
        for mar in squadron.heavyCruiser.currentMAR:
          if (gv.PrintMAR != 'Simple'):
            if (gv.PrintMAR == 'Detailed'):
              text = mar.marText
            else:
              text = mar.marSummary
            if (ypos <= ypos_IMG):
              textBlock = ww.wordwrap(text, printAreaWidth-2.2*resolutionDPI, dc)
            else:
              textBlock = ww.wordwrap(text, printAreaWidth, dc)
            (width, ysize, lineHeight) = dc.GetMultiLineTextExtent(textBlock)
            xpos = 0.05*resolutionDPI
            dc.SetFont(fontBoldSmall)
            dc.DrawText((mar.marName+" ("+squadron.heavyCruiser.className+")"), xpos, ypos)
            xpos = 0.1 * resolutionDPI
            ypos += lineHeight
            dc.SetFont(fontSmall)
            textList = textBlock.split('\n')
            for line in textList:
              dc.DrawText(line, xpos, ypos)
              ypos += lineHeight
          else:
            xpos = 0.05*resolutionDPI
            (width, height) = dc.GetTextExtent(mar.marName)
            dc.SetFont(fontBoldSmall)
            dc.DrawText((mar.marName+" ("+squadron.heavyCruiser.className+")"), xpos, ypos)
            ypos += height
        dc.SetFont(fontSmall)
        
      #Print out list of Upgrades for main ship
      if (squadron.ship.currentUpgrades != []):
        ypos += (lineHeight+0.05*resolutionDPI)  #spacing for the next entry
        xpos = 0
        dc.SetFont(fontBoldSmall)
        dc.DrawText("{0} Upgrades".format(squadron.ship.className), xpos, ypos)
        dc.SetFont(fontSmall)
        (tWidth, tHeight) = dc.GetTextExtent("Upgrades")
        ypos += tHeight
        xpos += Xmargin
        for upg in squadron.ship.currentUpgrades:
          if (upg.cost == 0):
            textLine = "{0} (free)".format(upg.text)
          else:
            textLine = "{0} ({1} points)".format(upg.text, upg.cost)
          if (ypos <= ypos_IMG):
            textBlock = ww.wordwrap(textLine, printAreaWidth-2.2*resolutionDPI, dc)
          else:
            textBlock = ww.wordwrap(textLine, printAreaWidth, dc)
          (width, ysize, lineHeight) = dc.GetMultiLineTextExtent(textBlock)
          dc.DrawText(textLine, xpos, ypos)
          ypos += lineHeight
      
      #Print out list of Upgrades for Accompaniment
      if (squadron.ship.currentAccompaniment) and (squadron.ship.currentAccompaniment.ship.currentUpgrades != []):
        xpos = 0
        ypos += (lineHeight+0.05*resolutionDPI)  #spacing for the next entry        
        dc.SetFont(fontBoldSmall)
        dc.DrawText("{0} Upgrades".format(squadron.ship.currentAccompaniment.ship.className), xpos, ypos)
        dc.SetFont(fontSmall)
        (tWidth, tHeight) = dc.GetTextExtent("Upgrades")
        ypos += tHeight
        xpos += Xmargin
        for upg in squadron.ship.currentAccompaniment.ship.currentUpgrades:
          if (upg.cost == 0):
            textLine = "{0} (free)".format(upg.text)
          else:
            textLine = "{0} ({1} points)".format(upg.text, upg.cost)
          if (ypos <= ypos_IMG):
            textBlock = ww.wordwrap(textLine, printAreaWidth-2.2*resolutionDPI, dc)
          else:
            textBlock = ww.wordwrap(textLine, printAreaWidth, dc)
          (width, ysize, lineHeight) = dc.GetMultiLineTextExtent(textBlock)
          dc.DrawText(textLine, xpos, ypos)
          ypos += lineHeight
        
      #Print out list of Upgrades for Heavy Cruiser
      if (squadron.heavyCruiser) and (squadron.heavyCruiser.currentUpgrades != []):
        xpos = 0
        ypos += (lineHeight+0.05*resolutionDPI)  #spacing for the next entry
        dc.SetFont(fontBoldSmall)
        dc.DrawText("{0} Upgrades".format(squadron.heavyCruiser.className), xpos, ypos)
        dc.SetFont(fontSmall)
        (tWidth, tHeight) = dc.GetTextExtent("Upgrades")
        ypos += tHeight
        xpos += Xmargin
        for upg in squadron.heavyCruiser.currentUpgrades:
          if (upg.cost == 0):
            textLine = "{0} (free)".format(upg.text)
          else:
            textLine = "{0} ({1} points)".format(upg.text, upg.cost)
          if (ypos <= ypos_IMG):
            textBlock = ww.wordwrap(textLine, printAreaWidth-2.2*resolutionDPI, dc)
          else:
            textBlock = ww.wordwrap(textLine, printAreaWidth, dc)
          (width, ysize, lineHeight) = dc.GetMultiLineTextExtent(textBlock)
          dc.DrawText(textLine, xpos, ypos)
          ypos += lineHeight
      
      #Print out list of Hardpoints added
      if (squadron.ship.currentHardpoints != []):
        xpos = 0
        ypos += (lineHeight+0.05*resolutionDPI)  #add padding
        dc.SetFont(fontBoldSmall)
        dc.DrawText("{0} Hardpoints".format(squadron.ship.className), xpos, ypos)
        (tWidth, tHeight) = dc.GetTextExtent("Hardpoints")
        ypos += tHeight
        dc.SetFont(fontSmall)
        xpos += Xmargin
        for hpt in squadron.ship.currentHardpoints:
          if (hpt.cost == 0):
            textLine = "{0} (free)".format(hpt.description)
          else:
            textLine = "{0} ({1} points)".format(hpt.description, hpt.cost)
          if (ypos <= ypos_IMG):
            textBlock = ww.wordwrap(textLine, printAreaWidth-2.2*resolutionDPI, dc)
          else:
            textBlock = ww.wordwrap(textLine, printAreaWidth, dc)
          (width, ysize, lineHeight) = dc.GetMultiLineTextExtent(textBlock)
          dc.DrawText(textLine, xpos, ypos)
          ypos += lineHeight
      
      #Print out list of Hardpoints added on the accompaniment
      if (squadron.ship.currentAccompaniment) and (squadron.ship.currentAccompaniment.ship.currentHardpoints != []):
        xpos = 0
        ypos += (lineHeight+0.05*resolutionDPI)  #add padding
        dc.SetFont(fontBoldSmall)
        dc.DrawText("{0} Hardpoints".format(squadron.ship.currentAccompaniment.ship.className), xpos, ypos)
        (tWidth, tHeight) = dc.GetTextExtent("Hardpoints")
        ypos += tHeight
        dc.SetFont(fontSmall)
        xpos += Xmargin
        for hpt in squadron.ship.currentAccompaniment.ship.currentHardpoints:
          if (hpt.cost == 0):
            textLine = "{0} (free)".format(hpt.description)
          else:
            textLine = "{0} ({1} points)".format(hpt.description, hpt.cost)
          if (ypos <= ypos_IMG):
            textBlock = ww.wordwrap(textLine, printAreaWidth-2.2*resolutionDPI, dc)
          else:
            textBlock = ww.wordwrap(textLine, printAreaWidth, dc)
          (width, ysize, lineHeight) = dc.GetMultiLineTextExtent(textBlock)
          dc.DrawText(textLine, xpos, ypos)
          ypos += lineHeight
      
      #Print out list of Hardpoints added on the heavy cruiser
      if (squadron.heavyCruiser) and (squadron.heavyCruiser.currentHardpoints != []):
        xpos = 0
        ypos += (lineHeight+0.05*resolutionDPI)  #add padding
        dc.SetFont(fontBoldSmall)
        dc.DrawText("{0} Hardpoints".format(squadron.heavyCruiser.className), xpos, ypos)
        (tWidth, tHeight) = dc.GetTextExtent("Hardpoints")
        ypos += tHeight
        dc.SetFont(fontSmall)
        xpos += Xmargin
        for hpt in squadron.heavyCruiser.currentHardpoints:
          if (hpt.cost == 0):
            textLine = "{0} (free)".format(hpt.description)
          else:
            textLine = "{0} ({1} points)".format(hpt.description, hpt.cost)
          if (ypos <= ypos_IMG):
            textBlock = ww.wordwrap(textLine, printAreaWidth-2.2*resolutionDPI, dc)
          else:
            textBlock = ww.wordwrap(textLine, printAreaWidth, dc)
          (width, ysize, lineHeight) = dc.GetMultiLineTextExtent(textBlock)
          dc.DrawText(textLine, xpos, ypos)
          ypos += lineHeight
      
      ypos += (lineHeight+0.25*resolutionDPI)  #add final padding between this and next squadron

      dc.SelectObject(wx.NullBitmap)
      squadron.printY = ypos
      currentStep += 1
      progressDlg.Update(currentStep)
    
    #print out the squadrons 
    dc = self.GetDC()    # this gets a dc for the printer
    dc.SetMapMode(wx.MM_TEXT)
    dc.SetFont(fontSmall)
    dc.SetTextBackground(wx.WHITE)
    dc.SetTextForeground(wx.BLACK)
    
    xpos = 0
    ypos = 0
    printHeader = True
    printList = sorted(self.fleet.squadronList, key=lambda fleet:fleet.printY, reverse=True)
    for squadron in printList:
      dc.SetDeviceOrigin(leftMargin, topMargin)
      if (printHeader): #print overall fleet information here
        Xmargin = 0
        dc.SetFont(fontBoldLarge)
        (tWidth, tHeight) = dc.GetTextExtent(self.fleet.fleetName)
        dc.DrawRectangle(Xmargin, ypos, 3.0*resolutionDPI, tHeight*1.5) #draw box for fleet name
        dc.DrawRectangle(Xmargin+3.0*resolutionDPI, ypos, 1.5*resolutionDPI, tHeight*1.5)  #draw box for race name
        dc.DrawRectangle(Xmargin+4.5*resolutionDPI, ypos, 3.0*resolutionDPI, tHeight*1.5)  #draw box for fleet size
        dc.DrawText(self.fleet.fleetName, Xmargin+0.25*resolutionDPI, ypos+tHeight/4)
        dc.DrawText(self.fleet.coreFaction, Xmargin+3.25*resolutionDPI, ypos+tHeight/4)
        ypos += 1.5 *tHeight  #move position down for next row of boxes
        if (self.fleet.MFV <= 800):
          text = "Patrol Fleet"
        elif (self.fleet.MFV <=1500):
          text = "Battle Fleet"
        else:
          text = "Grand Fleet"
        dc.DrawText("{0} ({1} points)".format(text, self.fleet.MFV), Xmargin+4.75*resolutionDPI, ypos+tHeight/4)
        dc.SetFont(fontBoldSmall)
        (tWidth, tHeight) = dc.GetTextExtent("Fleet Tactics")
        dc.DrawRectangle(Xmargin, ypos, 4.0*resolutionDPI, tHeight*1.5) #draw box for fleet tactics bonus
        dc.DrawRectangle(Xmargin+4.0*resolutionDPI, ypos, 3.5*resolutionDPI, tHeight*1.5)  #draw box for command distance
        dc.DrawText("Fleet Tactics Bonus: {0}".format(self.fleet.tacticsBonus), Xmargin+0.25*resolutionDPI, ypos+tHeight/4)
        dc.DrawText("Command Distance: {0}".format(self.fleet.commandDistance), Xmargin+4.25*resolutionDPI, ypos+tHeight/4)
        printHeader = False
        ypos += 1.5*tHeight
        
      Xmargin = 20
      Ymargin = 20
      if (0):
        squadron = dataStruct.squadronRecord()
        squadron.ship = dataStruct.ShipRecord()
        
      if ((ypos + squadron.printY) > printAreaHeight):  #skip to next page if current doesn't fit
        dc.EndPage()
        ypos = 0
      
      fillBrush = wx.WHITE_BRUSH
      dc.SetBrush(fillBrush)
      dc.SetPen(outlinePen)
      
      #faction image
      xpos_IMG = int(5.5*resolutionDPI)
      ypos_IMG = ypos
      
      imageX = int(1.5*resolutionDPI)
      imageY = int(1.5*resolutionDPI)
      
      dc.DrawRectangle(xpos_IMG, ypos_IMG,imageX, imageY)
      source = self.SetFactionImage(squadron.ship.faction)
      image = source.Scale(imageX, imageY, wx.IMAGE_QUALITY_HIGH)
      bitmap = wx.BitmapFromImage(image)
      dc.DrawBitmap(bitmap,xpos_IMG, ypos_IMG)
      
      ypos_IMG += imageY
      
      #ship image
      dc.DrawRectangle(xpos_IMG, ypos_IMG, imageX, imageY)
      if (squadron.ship.imageCustomFilename):
        source = wx.Image(os.path.join(gv.ship_custom_images_folder, squadron.ship.imageCustomFilename), wx.BITMAP_TYPE_ANY)
      elif (squadron.ship.imageFilename):
        source = wx.Image(os.path.join(gv.ship_images_folder, squadron.ship.imageFilename), wx.BITMAP_TYPE_ANY)
      else:
        source = wx.Image(os.path.join(gv.ship_images_folder, gv.defaultImage), wx.BITMAP_TYPE_PNG)
      image = source.Scale(imageX, imageY, wx.IMAGE_QUALITY_HIGH)
      bitmap = wx.BitmapFromImage(image)
      dc.DrawBitmap(bitmap,xpos_IMG, ypos_IMG)
      width, height = dc.GetTextExtent(squadron.ship.className)
      dc.DrawRotatedText(squadron.ship.className, xpos_IMG+imageX+height, ypos_IMG, -90)
      ypos_IMG += imageY  #move pointer to the next image position
      
      # heavy cruiser ship image, if applicable
      if (squadron.heavyCruiser):
        dc.DrawRectangle(xpos_IMG, ypos_IMG, imageX, imageY)
        if (squadron.heavyCruiser.imageFilename):
          source = wx.Image(os.path.join(gv.ship_images_folder, squadron.heavyCruiser.imageFilename), wx.BITMAP_TYPE_ANY)
        else:
          source = wx.Image(os.path.join(gv.ship_images_folder, gv.defaultImage), wx.BITMAP_TYPE_PNG)
        image = source.Scale(imageX, imageY, wx.IMAGE_QUALITY_HIGH)
        bitmap = wx.BitmapFromImage(image)
        dc.DrawBitmap(bitmap,xpos_IMG, ypos_IMG)
        width, height = dc.GetTextExtent(squadron.ship.className)
        dc.DrawRotatedText(squadron.heavyCruiser.className, xpos_IMG+imageX+height, ypos_IMG, -90)
        ypos_IMG += imageY  #move pointer to the next image position

      # accompaniment ship image, if applicable
      if (squadron.ship.currentAccompaniment):
        dc.DrawRectangle(xpos_IMG, ypos_IMG, imageX, imageY)
        if (squadron.ship.currentAccompaniment.ship.imageFilename):
          source = wx.Image(os.path.join(gv.ship_images_folder, squadron.ship.currentAccompaniment.ship.imageFilename), wx.BITMAP_TYPE_ANY)
        else:
          source = wx.Image(os.path.join(gv.ship_images_folder, gv.defaultImage), wx.BITMAP_TYPE_PNG)
        image = source.Scale(imageX, imageY, wx.IMAGE_QUALITY_HIGH)
        bitmap = wx.BitmapFromImage(image)
        dc.DrawBitmap(bitmap,xpos_IMG, ypos_IMG)
        width, height = dc.GetTextExtent(squadron.ship.className)
        dc.DrawRotatedText(squadron.ship.currentAccompaniment.ship.className, xpos_IMG+imageX+height, ypos_IMG, -90)
        ypos_IMG += imageY  #move pointer to the next image position

      #Print the SRS block (if any) below the faction and ship images
      if (squadron.ship.currentSRS != []):
        if (squadron.ship.currentSRS[0].srsType != 'None') or (squadron.ship.currentSRS[1].srsType != 'None'):
          textBlock = " {0} class {1}\n Short Range Spacecraft (SRS)\n".format(squadron.ship.className, squadron.ship.designation)
          for srs in squadron.ship.currentSRS:
            if (srs.srsType == 'None'):
              continue
            else:
              textBlock +=" {0} squadron, {1} Wings\n    {2}\n".format(srs.srsType, srs.numWings, dataStruct.srsTypes[srs.srsType])
          width,height,lineHeight = dc.GetMultiLineTextExtent(textBlock)
          width += 20 #add a little buffer because we are printing the first line in bold
          dc.SetBrush(wx.Brush(gv.Report_WC_Color))
          dc.DrawRectangle(xpos_IMG, ypos_IMG, width, height)
          textBlock = textBlock.rstrip('\n')
          textList = textBlock.split('\n')
          firstLine = True
          for line in textList:
            if (firstLine):
              dc.SetFont(fontBoldSmall)
              dc.DrawText(line, xpos_IMG, ypos_IMG)
              dc.SetFont(fontSmall) 
              firstLine = False
            else:
              dc.DrawText(line, xpos_IMG, ypos_IMG)
            ypos_IMG += lineHeight
          ypos_IMG += lineHeight #move down for next entry
        
      #Print the SRS block for Accompaniment (if any) below the faction and ship images
      if (squadron.ship.currentAccompaniment) and (squadron.ship.currentAccompaniment.ship.currentSRS != []):
        if (squadron.ship.currentAccompaniment.ship.currentSRS[0].srsType != 'None') or (squadron.ship.currentAccompaniment.ship.currentSRS[1].srsType != 'None'):
          textBlock = " {0} class {1}\n Short Range Spacecraft (SRS)\n".format(squadron.ship.currentAccompaniment.ship.className, squadron.ship.currentAccompaniment.ship.designation)
          for srs in squadron.ship.currentAccompaniment.ship.currentSRS:
            if (srs.srsType == 'None'):
              continue
            else:
              textBlock +=" {0} squadron, {1} Wings\n    {2}\n".format(srs.srsType, srs.numWings, dataStruct.srsTypes[srs.srsType])
          width,height,lineHeight = dc.GetMultiLineTextExtent(textBlock)
          width += 20 #add a little buffer because we are printing the first line in bold
          dc.SetBrush(wx.Brush(gv.Report_WC_Color))
          dc.DrawRectangle(xpos_IMG, ypos_IMG, width, height)
          textBlock = textBlock.rstrip('\n')
          textList = textBlock.split('\n')
          firstLine = True
          for line in textList:
            if (firstLine):
              dc.SetFont(fontBoldSmall)
              dc.DrawText(line, xpos_IMG, ypos_IMG)
              dc.SetFont(fontSmall) 
              firstLine = False
            else:
              dc.DrawText(line, xpos_IMG, ypos_IMG)
            ypos_IMG += lineHeight
          ypos_IMG += lineHeight #move down for next entry
        
      #Print the SRS block for the heavy cruiser (if any) below the faction and ship images
      if (squadron.heavyCruiser) and (squadron.heavyCruiser.currentSRS != []):
        if (squadron.heavyCruiser.currentSRS[0].srsType != 'None') or (squadron.heavyCruiser.currentSRS[1].srsType != 'None'):
          textBlock = " {0} class {1}\n Short Range Spacecraft (SRS)\n".format(squadron.heavyCruiser.className, squadron.heavyCruiser.designation)
          for srs in squadron.ship.heavyCruiser.currentSRS:
            if (srs.srsType == 'None'):
              continue
            else:
              textBlock +=" {0} squadron, {1} Wings\n    {2}\n".format(srs.srsType, srs.numWings, dataStruct.srsTypes[srs.srsType])
          width,height,lineHeight = dc.GetMultiLineTextExtent(textBlock)
          width += 20 #add a little buffer because we are printing the first line in bold
          dc.SetBrush(wx.Brush(gv.Report_WC_Color))
          dc.DrawRectangle(xpos_IMG, ypos_IMG, width, height)
          textBlock = textBlock.rstrip('\n')
          textList = textBlock.split('\n')
          firstLine = True
          for line in textList:
            if (firstLine):
              dc.SetFont(fontBoldSmall)
              dc.DrawText(line, xpos_IMG, ypos_IMG)
              dc.SetFont(fontSmall) 
              firstLine = False
            else:
              dc.DrawText(line, xpos_IMG, ypos_IMG)
            ypos_IMG += lineHeight
          ypos_IMG += lineHeight #move down for next entry
        
      #squadron name
      dc.SetBrush(wx.WHITE_BRUSH)
      dc.DrawRectangle(0, ypos, 5.5*resolutionDPI, 0.25*resolutionDPI)

      nameField = ""
      if (squadron.name != ""):
        nameField = squadron.name + ": "
      nameField += squadron.ship.className+" Class "+squadron.ship.designation + " squadron"
      if (squadron.ship.source != 'Official'):
        nameField += ' (Custom Ship)'
      dc.SetFont(fontBoldLarge)
      dc.DrawText(nameField, xpos, ypos)
      ypos += 0.25*resolutionDPI
      
      #number of points/tier/number of ships
      dc.DrawRectangle(0, ypos, 5.5*resolutionDPI, 0.25*resolutionDPI)
      if (squadron.ship.currentAccompaniment):
        text = "{0} points / {1} Capital ship / {2} {4} Escorts/ Tier {3} Squadron".format(squadron.ship.currentCost*squadron.size, squadron.size, squadron.ship.currentAccompaniment.numSelected,squadron.tier, squadron.ship.currentAccompaniment.ship.className)
      else:
        if (squadron.size == 1):
          text = "{0} points / {1} ship / Tier {2} Squadron".format(squadron.ship.currentCost*squadron.size, squadron.size, squadron.tier)
        elif (squadron.heavyCruiser):
          text = "{0} points / {1} ships (includes {3} Heavy Cruiser) / Tier {2} Squadron".format((squadron.heavyCruiser.currentCost+squadron.ship.currentCost*squadron.size), squadron.size+1, squadron.tier, squadron.heavyCruiser.className)
        else:
          text = "{0} points / {1} ships / Tier {2} Squadron".format(squadron.ship.currentCost*squadron.size, squadron.size, squadron.tier)
      xpos = Xmargin
      
      dc.SetFont(fontLarge)
      dc.DrawText(text, xpos, ypos)
      ypos += 0.25*resolutionDPI
      
      #main Stat block
      xpos = 0
      dc.SetFont(fontLarge)
      if (squadron.ship.currentAccompaniment) or (squadron.heavyCruiser):  #print the capital ship class name
        dc.SetFont(fontBoldSmall)
        dc.DrawText("{0}:".format(squadron.ship.className), xpos, ypos)
        ypos += 0.15*resolutionDPI
      
      blockWidth = 0.35*resolutionDPI
      if (True):  #print color stat block backgrounds for main ship
        fillBrush = wx.Brush(gv.Report_DR_Color)
        dc.SetBrush(fillBrush)
        dc.DrawRectangle(xpos,ypos,blockWidth, blockWidth)
        xpos += blockWidth
  
        fillBrush = wx.Brush(gv.Report_CR_Color)
        dc.SetBrush(fillBrush)
        dc.DrawRectangle(xpos,ypos,blockWidth, blockWidth)
        xpos += blockWidth
  
        fillBrush = wx.Brush(gv.Report_HP_Color)
        dc.SetBrush(fillBrush)
        dc.DrawRectangle(xpos,ypos,blockWidth, blockWidth)
        xpos += blockWidth
  
        fillBrush = wx.Brush(gv.Report_MV_Color)
        dc.SetBrush(fillBrush)
        dc.DrawRectangle(xpos,ypos,blockWidth, blockWidth)
        xpos += blockWidth
  
        fillBrush = wx.Brush(gv.Report_TL_Color)
        dc.SetBrush(fillBrush)
        dc.DrawRectangle(xpos,ypos,blockWidth, blockWidth)
        xpos += blockWidth
  
        fillBrush = wx.Brush(gv.Report_CP_Color)
        dc.SetBrush(fillBrush)
        dc.DrawRectangle(xpos,ypos,blockWidth, blockWidth)
        xpos += blockWidth
  
        fillBrush = wx.Brush(gv.Report_AP_Color)
        dc.SetBrush(fillBrush)
        dc.DrawRectangle(xpos,ypos,blockWidth, blockWidth)
        xpos += blockWidth
  
        fillBrush = wx.Brush(gv.Report_PD_Color)
        dc.SetBrush(fillBrush)
        dc.DrawRectangle(xpos,ypos,blockWidth, blockWidth)
        xpos += blockWidth
  
        fillBrush = wx.Brush(gv.Report_SR_Color)
        dc.SetBrush(fillBrush)
        dc.DrawRectangle(xpos,ypos,blockWidth, blockWidth)
        xpos += blockWidth
  
        fillBrush = wx.Brush(gv.Report_MN_Color)
        dc.SetBrush(fillBrush)
        dc.DrawRectangle(xpos,ypos,blockWidth, blockWidth)
        xpos += blockWidth
  
        fillBrush = wx.Brush(gv.Report_WC_Color)
        dc.SetBrush(fillBrush)
        dc.DrawRectangle(xpos,ypos,blockWidth, blockWidth)
      
      if (True):  #print stat header (DR, CR, HP, etc.) for main ship
        dc.SetFont(fontBoldSmall)
        ypos += 0.0*resolutionDPI
        xpos = 0.05*resolutionDPI
        dc.DrawText("DR", xpos, ypos)
        xpos += blockWidth
        dc.DrawText("CR", xpos, ypos)
        xpos += blockWidth
        dc.DrawText("HP", xpos, ypos)
        xpos += blockWidth
        dc.DrawText("MV", xpos, ypos)
        xpos += blockWidth
        dc.DrawText("TL", xpos, ypos)
        xpos += blockWidth
        dc.DrawText("CP", xpos, ypos)
        xpos += blockWidth
        dc.DrawText("AP", xpos, ypos)
        xpos += blockWidth
        dc.DrawText("PD", xpos, ypos)
        xpos += blockWidth
        if (squadron.ship.cloakingField):
          dc.DrawText("Cloak",xpos, ypos)
        else:
          dc.DrawText("SR", xpos, ypos)
        xpos += blockWidth
        dc.DrawText("MN", xpos, ypos)
        xpos += blockWidth
        dc.DrawText("WC", xpos, ypos)
      
      if (True):  #print stats for main ship class
        xpos = 0.05*resolutionDPI
        ypos += 0.15*resolutionDPI
        dc.SetFont(fontBoldSmall)
        dc.DrawText(str(squadron.ship.DR), xpos, ypos)
        xpos += blockWidth
        dc.DrawText(str(squadron.ship.CR), xpos, ypos)
        xpos += blockWidth
        dc.DrawText(str(squadron.ship.HP), xpos, ypos)
        xpos += blockWidth
        dc.DrawText(str(squadron.ship.MV), xpos, ypos)
        xpos += blockWidth
        dc.DrawText(str(squadron.ship.TL), xpos, ypos)
        xpos += blockWidth
        dc.DrawText(str(squadron.ship.CP), xpos, ypos)
        xpos += blockWidth
        dc.DrawText(str(squadron.ship.AP), xpos, ypos)
        xpos += blockWidth
        dc.DrawText(str(squadron.ship.PD), xpos, ypos)
        xpos += blockWidth
        if (squadron.ship.cloakingField):
          dc.DrawText("Field", xpos, ypos)
        else:
          dc.DrawText(str(squadron.ship.SR), xpos, ypos)
        xpos += blockWidth
        dc.DrawText(str(squadron.ship.MN), xpos, ypos)
        xpos += blockWidth
        dc.DrawText(str(squadron.ship.WC), xpos, ypos)
      
      if (squadron.heavyCruiser):  #stat block for heavy cruiser ships, if any
        xpos = 0
        ypos += 0.2*resolutionDPI
        dc.SetFont(fontBoldSmall)
        dc.DrawText("{0}:".format(squadron.heavyCruiser.className), xpos, ypos)
        ypos += 0.15*resolutionDPI
        
        fillBrush = wx.Brush(gv.Report_DR_Color)
        dc.SetBrush(fillBrush)
        dc.DrawRectangle(xpos,ypos,blockWidth, blockWidth)
        xpos += blockWidth
  
        fillBrush = wx.Brush(gv.Report_CR_Color)
        dc.SetBrush(fillBrush)
        dc.DrawRectangle(xpos,ypos,blockWidth, blockWidth)
        xpos += blockWidth
  
        fillBrush = wx.Brush(gv.Report_HP_Color)
        dc.SetBrush(fillBrush)
        dc.DrawRectangle(xpos,ypos,blockWidth, blockWidth)
        xpos += blockWidth
  
        fillBrush = wx.Brush(gv.Report_MV_Color)
        dc.SetBrush(fillBrush)
        dc.DrawRectangle(xpos,ypos,blockWidth, blockWidth)
        xpos += blockWidth
  
        fillBrush = wx.Brush(gv.Report_TL_Color)
        dc.SetBrush(fillBrush)
        dc.DrawRectangle(xpos,ypos,blockWidth, blockWidth)
        xpos += blockWidth
  
        fillBrush = wx.Brush(gv.Report_CP_Color)
        dc.SetBrush(fillBrush)
        dc.DrawRectangle(xpos,ypos,blockWidth, blockWidth)
        xpos += blockWidth
  
        fillBrush = wx.Brush(gv.Report_AP_Color)
        dc.SetBrush(fillBrush)
        dc.DrawRectangle(xpos,ypos,blockWidth, blockWidth)
        xpos += blockWidth
  
        fillBrush = wx.Brush(gv.Report_PD_Color)
        dc.SetBrush(fillBrush)
        dc.DrawRectangle(xpos,ypos,blockWidth, blockWidth)
        xpos += blockWidth
  
        fillBrush = wx.Brush(gv.Report_SR_Color)
        dc.SetBrush(fillBrush)
        dc.DrawRectangle(xpos,ypos,blockWidth, blockWidth)
        xpos += blockWidth
  
        fillBrush = wx.Brush(gv.Report_MN_Color)
        dc.SetBrush(fillBrush)
        dc.DrawRectangle(xpos,ypos,blockWidth, blockWidth)
        xpos += blockWidth
  
        fillBrush = wx.Brush(gv.Report_WC_Color)
        dc.SetBrush(fillBrush)
        dc.DrawRectangle(xpos,ypos,blockWidth, blockWidth)
        
        ypos += 0.0*resolutionDPI
        xpos = 0.05*resolutionDPI
        dc.DrawText("DR", xpos, ypos)
        xpos += blockWidth
        dc.DrawText("CR", xpos, ypos)
        xpos += blockWidth
        dc.DrawText("HP", xpos, ypos)
        xpos += blockWidth
        dc.DrawText("MV", xpos, ypos)
        xpos += blockWidth
        dc.DrawText("TL", xpos, ypos)
        xpos += blockWidth
        dc.DrawText("CP", xpos, ypos)
        xpos += blockWidth
        dc.DrawText("AP", xpos, ypos)
        xpos += blockWidth
        dc.DrawText("PD", xpos, ypos)
        xpos += blockWidth
        dc.DrawText("SR", xpos, ypos)
        xpos += blockWidth
        dc.DrawText("MN", xpos, ypos)
        xpos += blockWidth
        dc.DrawText("WC", xpos, ypos)
        
        xpos = 0.05*resolutionDPI
        ypos += 0.15*resolutionDPI
        dc.DrawText(str(squadron.heavyCruiser.DR), xpos, ypos)
        xpos += blockWidth
        dc.DrawText(str(squadron.heavyCruiser.CR), xpos, ypos)
        xpos += blockWidth
        dc.DrawText(str(squadron.heavyCruiser.HP), xpos, ypos)
        xpos += blockWidth
        dc.DrawText(str(squadron.heavyCruiser.MV), xpos, ypos)
        xpos += blockWidth
        dc.DrawText(str(squadron.heavyCruiser.TL), xpos, ypos)
        xpos += blockWidth
        dc.DrawText(str(squadron.heavyCruiser.CP), xpos, ypos)
        xpos += blockWidth
        dc.DrawText(str(squadron.heavyCruiser.AP), xpos, ypos)
        xpos += blockWidth
        dc.DrawText(str(squadron.heavyCruiser.PD), xpos, ypos)
        xpos += blockWidth
        dc.DrawText(str(squadron.heavyCruiser.SR), xpos, ypos)
        xpos += blockWidth
        dc.DrawText(str(squadron.heavyCruiser.MN), xpos, ypos)
        xpos += blockWidth
        dc.DrawText(str(squadron.heavyCruiser.WC), xpos, ypos)

      if (squadron.ship.currentAccompaniment):  #stat block for accompaniment ships, if any
        xpos = 0
        ypos += 0.2*resolutionDPI
        dc.SetFont(fontBoldSmall)
        dc.DrawText("{0}:".format(squadron.ship.currentAccompaniment.ship.className), xpos, ypos)
        ypos += 0.15*resolutionDPI
        
        fillBrush = wx.Brush(gv.Report_DR_Color)
        dc.SetBrush(fillBrush)
        dc.DrawRectangle(xpos,ypos,blockWidth, blockWidth)
        xpos += blockWidth
  
        fillBrush = wx.Brush(gv.Report_CR_Color)
        dc.SetBrush(fillBrush)
        dc.DrawRectangle(xpos,ypos,blockWidth, blockWidth)
        xpos += blockWidth
  
        fillBrush = wx.Brush(gv.Report_HP_Color)
        dc.SetBrush(fillBrush)
        dc.DrawRectangle(xpos,ypos,blockWidth, blockWidth)
        xpos += blockWidth
  
        fillBrush = wx.Brush(gv.Report_MV_Color)
        dc.SetBrush(fillBrush)
        dc.DrawRectangle(xpos,ypos,blockWidth, blockWidth)
        xpos += blockWidth
  
        fillBrush = wx.Brush(gv.Report_TL_Color)
        dc.SetBrush(fillBrush)
        dc.DrawRectangle(xpos,ypos,blockWidth, blockWidth)
        xpos += blockWidth
  
        fillBrush = wx.Brush(gv.Report_CP_Color)
        dc.SetBrush(fillBrush)
        dc.DrawRectangle(xpos,ypos,blockWidth, blockWidth)
        xpos += blockWidth
  
        fillBrush = wx.Brush(gv.Report_AP_Color)
        dc.SetBrush(fillBrush)
        dc.DrawRectangle(xpos,ypos,blockWidth, blockWidth)
        xpos += blockWidth
  
        fillBrush = wx.Brush(gv.Report_PD_Color)
        dc.SetBrush(fillBrush)
        dc.DrawRectangle(xpos,ypos,blockWidth, blockWidth)
        xpos += blockWidth
  
        fillBrush = wx.Brush(gv.Report_SR_Color)
        dc.SetBrush(fillBrush)
        dc.DrawRectangle(xpos,ypos,blockWidth, blockWidth)
        xpos += blockWidth
  
        fillBrush = wx.Brush(gv.Report_MN_Color)
        dc.SetBrush(fillBrush)
        dc.DrawRectangle(xpos,ypos,blockWidth, blockWidth)
        xpos += blockWidth
  
        fillBrush = wx.Brush(gv.Report_WC_Color)
        dc.SetBrush(fillBrush)
        dc.DrawRectangle(xpos,ypos,blockWidth, blockWidth)
        
        ypos += 0.0*resolutionDPI
        xpos = 0.05*resolutionDPI
        dc.DrawText("DR", xpos, ypos)
        xpos += blockWidth
        dc.DrawText("CR", xpos, ypos)
        xpos += blockWidth
        dc.DrawText("HP", xpos, ypos)
        xpos += blockWidth
        dc.DrawText("MV", xpos, ypos)
        xpos += blockWidth
        dc.DrawText("TL", xpos, ypos)
        xpos += blockWidth
        dc.DrawText("CP", xpos, ypos)
        xpos += blockWidth
        dc.DrawText("AP", xpos, ypos)
        xpos += blockWidth
        dc.DrawText("PD", xpos, ypos)
        xpos += blockWidth
        dc.DrawText("SR", xpos, ypos)
        xpos += blockWidth
        dc.DrawText("MN", xpos, ypos)
        xpos += blockWidth
        dc.DrawText("WC", xpos, ypos)
        
        xpos = 0.05*resolutionDPI
        ypos += 0.15*resolutionDPI
        dc.DrawText(str(squadron.ship.currentAccompaniment.ship.DR), xpos, ypos)
        xpos += blockWidth
        dc.DrawText(str(squadron.ship.currentAccompaniment.ship.CR), xpos, ypos)
        xpos += blockWidth
        dc.DrawText(str(squadron.ship.currentAccompaniment.ship.HP), xpos, ypos)
        xpos += blockWidth
        dc.DrawText(str(squadron.ship.currentAccompaniment.ship.MV), xpos, ypos)
        xpos += blockWidth
        dc.DrawText(str(squadron.ship.currentAccompaniment.ship.TL), xpos, ypos)
        xpos += blockWidth
        dc.DrawText(str(squadron.ship.currentAccompaniment.ship.CP), xpos, ypos)
        xpos += blockWidth
        dc.DrawText(str(squadron.ship.currentAccompaniment.ship.AP), xpos, ypos)
        xpos += blockWidth
        dc.DrawText(str(squadron.ship.currentAccompaniment.ship.PD), xpos, ypos)
        xpos += blockWidth
        dc.DrawText(str(squadron.ship.currentAccompaniment.ship.SR), xpos, ypos)
        xpos += blockWidth
        dc.DrawText(str(squadron.ship.currentAccompaniment.ship.MN), xpos, ypos)
        xpos += blockWidth
        dc.DrawText(str(squadron.ship.currentAccompaniment.ship.WC), xpos, ypos)

      dc.SetFont(fontSmall)
      #section for direct fire weapons, if any
      directFire = False
      indirectFire = False
      for weapon in squadron.ship.currentWeapons:  #which weapon types need to be printed?
        if (weapon.category == 'Direct'):
          directFire = True
        if (weapon.category == 'Indirect'):
          indirectFire = True

      if (squadron.ship.currentAccompaniment):  #which weapon types need to be printed?
        for weapon in squadron.ship.currentWeapons:
          if (weapon.category == 'Direct'):
            directFire = True
          if (weapon.category == 'Indirect'):
            indirectFire = True

      if (squadron.heavyCruiser):  #which weapon types need to be printed?
        for weapon in squadron.heavyCruiser.currentWeapons:
          if (weapon.category == 'Direct'):
            directFire = True
          if (weapon.category == 'Indirect'):
            indirectFire = True    
  
      fillBrush = wx.Brush(gv.Report_DirectFireHeader_Color)
      dc.SetBrush(fillBrush)
      xpos = 0
      ypos += 0.25*resolutionDPI
      dc.DrawRectangle(xpos, ypos ,5.5*resolutionDPI, 0.20*resolutionDPI)
      if (not directFire):
        dc.DrawText("No Direct Fire Weapons Available", xpos, ypos)
        ypos += 0.20*resolutionDPI
      else:
        dc.DrawText( "Direct Fire Weapons",xpos+0.05*resolutionDPI, ypos)
        dc.DrawText("RB1",xpos+4.0*resolutionDPI, ypos)
        dc.DrawText("RB2",xpos+4.4*resolutionDPI, ypos)
        dc.DrawText("RB3",xpos+4.8*resolutionDPI, ypos)
        dc.DrawText("RB4",xpos+5.2*resolutionDPI, ypos)
        xpos += 0.10*resolutionDPI  #offset entries under Direct Fire Weapons
        ypos += 0.20*resolutionDPI

        for weaponType in dataStruct.weaponTypes:
          printBlock = False
          for weapon in squadron.ship.currentWeapons:
            if (weaponType in weapon.weaponType):
              printBlock = True
              break
          if (not printBlock):  #continue to the next weapon type if there are no matching weapons
            continue
          needHeader = True
          for weapon in squadron.ship.currentWeapons:  #print stats for individual Direct fire weapons
            if (weapon.category != 'Direct'):
              continue
            if (needHeader):  #print the Ship|Weapon Type|Weapon Arc|Range Increments header
              needHeader = False
              fillBrush = wx.Brush(gv.Report_DirectFireHeader_Color)
              dc.SetBrush(fillBrush)
              dc.DrawRectangle(xpos, ypos, 5.4*resolutionDPI, 0.20*resolutionDPI)
              dc.DrawText("Ship",xpos+0.05*resolutionDPI, ypos)
              dc.DrawText("Weapon Type",xpos+1.1*resolutionDPI, ypos)
              dc.DrawText("Weapon Arc",xpos+2.2*resolutionDPI, ypos)
              if (weaponType == 'Beam') and (weapon.category == 'Direct'):
                dc.DrawText('<10"',xpos+3.9*resolutionDPI, ypos)
                dc.DrawText('<20"',xpos+4.3*resolutionDPI, ypos)
                dc.DrawText('<30"',xpos+4.7*resolutionDPI, ypos)
                dc.DrawText('<40"',xpos+5.1*resolutionDPI, ypos)
              elif ((weaponType == 'Gun Rack') or (weaponType == 'Primary')) and (weapon.category == 'Direct'):
                dc.DrawText('<8"',xpos+3.9*resolutionDPI, ypos)
                dc.DrawText('<16"',xpos+4.3*resolutionDPI, ypos)
                dc.DrawText('<24"',xpos+4.7*resolutionDPI, ypos)
                dc.DrawText('<32"',xpos+5.1*resolutionDPI, ypos)
              elif (weaponType == 'Kinetic') and (weapon.category == 'Direct'):
                dc.DrawText('<12"',xpos+3.9*resolutionDPI, ypos)
                dc.DrawText('<24"',xpos+4.3*resolutionDPI, ypos)
                dc.DrawText('<36"',xpos+4.7*resolutionDPI, ypos)
                dc.DrawText('<48"',xpos+5.1*resolutionDPI, ypos)
              elif (weaponType == 'Scatter') and (weapon.category == 'Direct'):
                dc.DrawText('<8"',xpos+3.9*resolutionDPI, ypos)
                dc.DrawText('<16"',xpos+4.3*resolutionDPI, ypos)
                dc.DrawText('<24"',xpos+4.7*resolutionDPI, ypos)
                dc.DrawText('<32"',xpos+5.1*resolutionDPI, ypos)
              elif (weaponType == 'Nuclear') and (weapon.category == 'Direct'):
                dc.DrawText('<8"',xpos+3.9*resolutionDPI, ypos)
                dc.DrawText('<16"',xpos+4.3*resolutionDPI, ypos)
                dc.DrawText('<24"',xpos+4.7*resolutionDPI, ypos)
                dc.DrawText('<32"',xpos+5.1*resolutionDPI, ypos)
              ypos += 0.20*resolutionDPI  #move the 'pointer' down ready for the next entry.
            if (weaponType in weapon.weaponType) and (weapon.category == 'Direct'):
              fillBrush = wx.Brush(gv.Report_DirectFire_Color)
              dc.SetBrush(fillBrush)
              dc.DrawRectangle(xpos, ypos, 5.4*resolutionDPI, 0.20*resolutionDPI)
              dc.DrawText("{0}".format(squadron.ship.className),xpos+0.05*resolutionDPI, ypos)
              dc.DrawText("{0} {1}".format(weapon.weaponType, weapon.weaponUpgrade),xpos+1.1*resolutionDPI, ypos)
              dc.DrawText("{0} {1}".format(weapon.arcDirection, weapon.arcRestriction),xpos+2.2*resolutionDPI, ypos)
              dc.DrawText("{0}".format(weapon.rb1),xpos+3.9*resolutionDPI, ypos)
              dc.DrawText("{0}".format(weapon.rb2),xpos+4.3*resolutionDPI, ypos)
              dc.DrawText("{0}".format(weapon.rb3),xpos+4.7*resolutionDPI, ypos)
              dc.DrawText("{0}".format(weapon.rb4),xpos+5.1*resolutionDPI, ypos)
              ypos += 0.20*resolutionDPI  #move the 'pointer' down ready for the next entry.
          if (squadron.heavyCruiser):
            for weapon in squadron.heavyCruiser.currentWeapons:
              if (weapon.category != 'Direct'):
                continue
              if (needHeader):  #print the Ship|Weapon Type|Weapon Arc|Range Increments header
                needHeader = False
                fillBrush = wx.Brush(gv.Report_DirectFireHeader_Color)
                dc.SetBrush(fillBrush)
                dc.DrawRectangle(xpos, ypos, 5.4*resolutionDPI, 0.20*resolutionDPI)
                dc.DrawText("Ship",xpos+0.05*resolutionDPI, ypos)
                dc.DrawText("Weapon Type",xpos+1.1*resolutionDPI, ypos)
                dc.DrawText("Weapon Arc",xpos+2.2*resolutionDPI, ypos)
                if (weaponType == 'Beam') and (weapon.category == 'Direct'):
                  dc.DrawText('<10"',xpos+3.9*resolutionDPI, ypos)
                  dc.DrawText('<20"',xpos+4.3*resolutionDPI, ypos)
                  dc.DrawText('<30"',xpos+4.7*resolutionDPI, ypos)
                  dc.DrawText('<40"',xpos+5.1*resolutionDPI, ypos)
                elif ((weaponType == 'Gun Rack') or (weaponType == 'Primary')) and (weapon.category == 'Direct'):
                  dc.DrawText('<8"',xpos+3.9*resolutionDPI, ypos)
                  dc.DrawText('<16"',xpos+4.3*resolutionDPI, ypos)
                  dc.DrawText('<24"',xpos+4.7*resolutionDPI, ypos)
                  dc.DrawText('<32"',xpos+5.1*resolutionDPI, ypos)
                elif (weaponType == 'Kinetic') and (weapon.category == 'Direct'):
                  dc.DrawText('<12"',xpos+3.9*resolutionDPI, ypos)
                  dc.DrawText('<24"',xpos+4.3*resolutionDPI, ypos)
                  dc.DrawText('<36"',xpos+4.7*resolutionDPI, ypos)
                  dc.DrawText('<48"',xpos+5.1*resolutionDPI, ypos)
                elif (weaponType == 'Scatter') and (weapon.category == 'Direct'):
                  dc.DrawText('<8"',xpos+3.9*resolutionDPI, ypos)
                  dc.DrawText('<16"',xpos+4.3*resolutionDPI, ypos)
                  dc.DrawText('<24"',xpos+4.7*resolutionDPI, ypos)
                  dc.DrawText('<32"',xpos+5.1*resolutionDPI, ypos)
                elif (weaponType == 'Nuclear') and (weapon.category == 'Direct'):
                  dc.DrawText('<8"',xpos+3.9*resolutionDPI, ypos)
                  dc.DrawText('<16"',xpos+4.3*resolutionDPI, ypos)
                  dc.DrawText('<24"',xpos+4.7*resolutionDPI, ypos)
                  dc.DrawText('<32"',xpos+5.1*resolutionDPI, ypos)
                ypos += 0.20*resolutionDPI  #move the 'pointer' down ready for the next entry.
              if (weaponType in weapon.weaponType) and (weapon.category == 'Direct'):
                fillBrush = wx.Brush(gv.Report_DirectFire_Color)
                dc.SetBrush(fillBrush)
                dc.DrawRectangle(xpos, ypos, 5.4*resolutionDPI, 0.20*resolutionDPI)
                dc.DrawText("{0}".format(squadron.heavyCruiser.className),xpos+0.05*resolutionDPI, ypos)
                dc.DrawText("{0} {1}".format(weapon.weaponType, weapon.weaponUpgrade),xpos+1.1*resolutionDPI, ypos)
                dc.DrawText("{0} {1}".format(weapon.arcDirection, weapon.arcRestriction),xpos+2.2*resolutionDPI, ypos)
                dc.DrawText("{0}".format(weapon.rb1),xpos+3.9*resolutionDPI, ypos)
                dc.DrawText("{0}".format(weapon.rb2),xpos+4.3*resolutionDPI, ypos)
                dc.DrawText("{0}".format(weapon.rb3),xpos+4.7*resolutionDPI, ypos)
                dc.DrawText("{0}".format(weapon.rb4),xpos+5.1*resolutionDPI, ypos)
                ypos += 0.20*resolutionDPI  #move the 'pointer' down ready for the next entry.
          if (squadron.ship.currentAccompaniment):
            for weapon in squadron.ship.currentAccompaniment.ship.currentWeapons:
              if (weapon.category != 'Direct'):
                continue
              if (needHeader):  #print the Ship|Weapon Type|Weapon Arc|Range Increments header
                needHeader = False
                fillBrush = wx.Brush(gv.Report_DirectFireHeader_Color)
                dc.SetBrush(fillBrush)
                dc.DrawRectangle(xpos, ypos, 5.4*resolutionDPI, 0.20*resolutionDPI)
                dc.DrawText("Ship",xpos+0.05*resolutionDPI, ypos)
                dc.DrawText("Weapon Type",xpos+1.1*resolutionDPI, ypos)
                dc.DrawText("Weapon Arc",xpos+2.2*resolutionDPI, ypos)
                if (weaponType == 'Beam') and (weapon.category == 'Direct'):
                  dc.DrawText('<10"',xpos+3.9*resolutionDPI, ypos)
                  dc.DrawText('<20"',xpos+4.3*resolutionDPI, ypos)
                  dc.DrawText('<30"',xpos+4.7*resolutionDPI, ypos)
                  dc.DrawText('<40"',xpos+5.1*resolutionDPI, ypos)
                elif ((weaponType == 'Gun Rack') or (weaponType == 'Primary')) and (weapon.category == 'Direct'):
                  dc.DrawText('<8"',xpos+3.9*resolutionDPI, ypos)
                  dc.DrawText('<16"',xpos+4.3*resolutionDPI, ypos)
                  dc.DrawText('<24"',xpos+4.7*resolutionDPI, ypos)
                  dc.DrawText('<32"',xpos+5.1*resolutionDPI, ypos)
                elif (weaponType == 'Kinetic') and (weapon.category == 'Direct'):
                  dc.DrawText('<12"',xpos+3.9*resolutionDPI, ypos)
                  dc.DrawText('<24"',xpos+4.3*resolutionDPI, ypos)
                  dc.DrawText('<36"',xpos+4.7*resolutionDPI, ypos)
                  dc.DrawText('<48"',xpos+5.1*resolutionDPI, ypos)
                elif (weaponType == 'Scatter') and (weapon.category == 'Direct'):
                  dc.DrawText('<8"',xpos+3.9*resolutionDPI, ypos)
                  dc.DrawText('<16"',xpos+4.3*resolutionDPI, ypos)
                  dc.DrawText('<24"',xpos+4.7*resolutionDPI, ypos)
                  dc.DrawText('<32"',xpos+5.1*resolutionDPI, ypos)
                elif (weaponType == 'Nuclear') and (weapon.category == 'Direct'):
                  dc.DrawText('<8"',xpos+3.9*resolutionDPI, ypos)
                  dc.DrawText('<16"',xpos+4.3*resolutionDPI, ypos)
                  dc.DrawText('<24"',xpos+4.7*resolutionDPI, ypos)
                  dc.DrawText('<32"',xpos+5.1*resolutionDPI, ypos)
                ypos += 0.20*resolutionDPI  #move the 'pointer' down ready for the next entry.
              if (weaponType in weapon.weaponType) and (weapon.category == 'Direct'):
                fillBrush = wx.Brush(gv.Report_DirectFire_Color)
                dc.SetBrush(fillBrush)
                dc.DrawRectangle(xpos, ypos, 5.4*resolutionDPI, 0.20*resolutionDPI)
                dc.DrawText("{0}".format(squadron.ship.currentAccompaniment.ship.className),xpos+0.05*resolutionDPI, ypos)
                dc.DrawText("{0} {1}".format(weapon.weaponType, weapon.weaponUpgrade),xpos+1.1*resolutionDPI, ypos)
                dc.DrawText("{0} {1}".format(weapon.arcDirection, weapon.arcRestriction),xpos+2.2*resolutionDPI, ypos)
                dc.DrawText("{0}".format(weapon.rb1),xpos+3.9*resolutionDPI, ypos)
                dc.DrawText("{0}".format(weapon.rb2),xpos+4.3*resolutionDPI, ypos)
                dc.DrawText("{0}".format(weapon.rb3),xpos+4.7*resolutionDPI, ypos)
                dc.DrawText("{0}".format(weapon.rb4),xpos+5.1*resolutionDPI, ypos)
                ypos += 0.20*resolutionDPI  #move the 'pointer' down ready for the next entry.  

      fillBrush = wx.Brush(gv.Report_IndirectFireHeader_Color)
      dc.SetBrush(fillBrush)
      xpos = 0
      ypos += 0.05*resolutionDPI
      dc.DrawRectangle(xpos, ypos ,5.5*resolutionDPI, 0.20*resolutionDPI)
      if (not indirectFire):
        dc.DrawText("No Indirect Fire Weapons Available", xpos, ypos)
        ypos += 0.20*resolutionDPI
      else:
        dc.DrawText( "Indirect Fire Weapons",xpos+0.05*resolutionDPI, ypos)
        dc.DrawText("RB1",xpos+4.0*resolutionDPI, ypos)
        dc.DrawText("RB2",xpos+4.4*resolutionDPI, ypos)
        dc.DrawText("RB3",xpos+4.8*resolutionDPI, ypos)
        dc.DrawText("RB4",xpos+5.2*resolutionDPI, ypos)
        xpos += 0.10*resolutionDPI  #offset entries under Indirect Fire Weapons
        ypos += 0.20*resolutionDPI
      
        for weaponType in dataStruct.indirectWeaponTypes:
          printBlock = False
          for weapon in squadron.ship.currentWeapons:
            if (weaponType in weapon.weaponType):
              printBlock = True
              break
          if (not printBlock):  #continue to the next weapon type if there are no matching weapons
            continue
          needHeader = True
          for weapon in squadron.ship.currentWeapons:
            if (weapon.category != 'Indirect'):
              continue
            if (needHeader):  #print the Ship|Weapon Type|Weapon Arc|Range Increments header
              needHeader = False
              fillBrush = wx.Brush(gv.Report_IndirectFireHeader_Color)
              dc.SetBrush(fillBrush)
              dc.DrawRectangle(xpos, ypos, 5.4*resolutionDPI, 0.20*resolutionDPI)
              dc.DrawText("Ship",xpos+0.05*resolutionDPI, ypos)
              dc.DrawText("Weapon Type",xpos+1.1*resolutionDPI, ypos)
              dc.DrawText("Weapon Arc",xpos+2.2*resolutionDPI, ypos)
              if (weaponType == 'Cyberwarfare'):
                dc.DrawText('<10"',xpos+3.9*resolutionDPI, ypos)
                dc.DrawText('<20"',xpos+4.3*resolutionDPI, ypos)
                dc.DrawText('<30"',xpos+4.7*resolutionDPI, ypos)
                dc.DrawText('<40"',xpos+5.1*resolutionDPI, ypos)
              elif (weaponType == 'Gravitational'):
                dc.DrawText('<8"',xpos+3.9*resolutionDPI, ypos)
                dc.DrawText('<16"',xpos+4.3*resolutionDPI, ypos)
                dc.DrawText('<24"',xpos+4.7*resolutionDPI, ypos)
                dc.DrawText('<32"',xpos+5.1*resolutionDPI, ypos)
              elif ((weaponType == 'Torpedo') or (weaponType == 'Nuclear Torpedo')):
                dc.DrawText('<12"',xpos+3.9*resolutionDPI, ypos)
                dc.DrawText('<24"',xpos+4.3*resolutionDPI, ypos)
                dc.DrawText('<36"',xpos+4.7*resolutionDPI, ypos)
                dc.DrawText('<48"',xpos+5.1*resolutionDPI, ypos)
              ypos += 0.20*resolutionDPI  #move the 'pointer' down ready for the next entry.
            if (weaponType in weapon.weaponType) and (weapon.category == 'Indirect'):
              fillBrush = wx.Brush(gv.Report_IndirectFire_Color)
              dc.SetBrush(fillBrush)
              dc.DrawRectangle(xpos, ypos, 5.4*resolutionDPI, 0.20*resolutionDPI)
              dc.DrawText("{0}".format(squadron.ship.className),xpos+0.05*resolutionDPI, ypos)
              dc.DrawText("{0} {1}".format(weapon.weaponType, weapon.weaponUpgrade),xpos+1.1*resolutionDPI, ypos)
              dc.DrawText("{0} {1}".format(weapon.arcDirection, weapon.arcRestriction),xpos+2.23*resolutionDPI, ypos)
              dc.DrawText("{0}".format(weapon.rb1),xpos+3.9*resolutionDPI, ypos)
              dc.DrawText("{0}".format(weapon.rb2),xpos+4.3*resolutionDPI, ypos)
              dc.DrawText("{0}".format(weapon.rb3),xpos+4.7*resolutionDPI, ypos)
              dc.DrawText("{0}".format(weapon.rb4),xpos+5.1*resolutionDPI, ypos)
              ypos += 0.20*resolutionDPI
          if (squadron.heavyCruiser):  #print out the indirect weapons on the Heavy Cruiser, if any
            for weapon in squadron.heavyCruiser.currentWeapons:
              if (weapon.category != 'Indirect'):
                continue
              if (needHeader):  #print the Ship|Weapon Type|Weapon Arc|Range Increments header
                needHeader = False
                fillBrush = wx.Brush(gv.Report_DirectFireHeader_Color)
                dc.SetBrush(fillBrush)
                dc.DrawRectangle(xpos, ypos, 5.4*resolutionDPI, 0.20*resolutionDPI)
                dc.DrawText("Ship",xpos+0.05*resolutionDPI, ypos)
                dc.DrawText("Weapon Type",xpos+1.1*resolutionDPI, ypos)
                dc.DrawText("Weapon Arc",xpos+2.2*resolutionDPI, ypos)
                if (weaponType == 'Cyberwarfare'):
                  dc.DrawText('<10"',xpos+3.9*resolutionDPI, ypos)
                  dc.DrawText('<20"',xpos+4.3*resolutionDPI, ypos)
                  dc.DrawText('<30"',xpos+4.7*resolutionDPI, ypos)
                  dc.DrawText('<40"',xpos+5.1*resolutionDPI, ypos)
                elif (weaponType == 'Gravitational'):
                  dc.DrawText('<8"',xpos+3.9*resolutionDPI, ypos)
                  dc.DrawText('<16"',xpos+4.3*resolutionDPI, ypos)
                  dc.DrawText('<24"',xpos+4.7*resolutionDPI, ypos)
                  dc.DrawText('<32"',xpos+5.1*resolutionDPI, ypos)
                elif ((weaponType == 'Torpedo') or (weaponType == 'Nuclear Torpedo')):
                  dc.DrawText('<12"',xpos+3.9*resolutionDPI, ypos)
                  dc.DrawText('<24"',xpos+4.3*resolutionDPI, ypos)
                  dc.DrawText('<36"',xpos+4.7*resolutionDPI, ypos)
                  dc.DrawText('<48"',xpos+5.1*resolutionDPI, ypos)
                  ypos += 0.20*resolutionDPI  #move the 'pointer' down ready for the next entry.
              if (weaponType in weapon.weaponType) and (weapon.category == 'Indirect'):
                fillBrush = wx.Brush(gv.Report_IndirectFire_Color)
                dc.SetBrush(fillBrush)
                dc.DrawRectangle(xpos, ypos, 5.4*resolutionDPI, 0.20*resolutionDPI)
                dc.DrawText("{0}".format(squadron.heavyCruiser.className),xpos+0.05*resolutionDPI, ypos)
                dc.DrawText("{0} {1}".format(weapon.weaponType, weapon.weaponUpgrade),xpos+1.1*resolutionDPI, ypos)
                dc.DrawText("{0} {1}".format(weapon.arcDirection, weapon.arcRestriction),xpos+2.2*resolutionDPI, ypos)
                dc.DrawText("{0}".format(weapon.rb1),xpos+3.9*resolutionDPI, ypos)
                dc.DrawText("{0}".format(weapon.rb2),xpos+4.3*resolutionDPI, ypos)
                dc.DrawText("{0}".format(weapon.rb3),xpos+4.7*resolutionDPI, ypos)
                dc.DrawText("{0}".format(weapon.rb4),xpos+5.1*resolutionDPI, ypos)
                ypos += 0.20*resolutionDPI  #move the 'pointer' down ready for the next entry.

          if (squadron.ship.currentAccompaniment):  #print out the indirect weapons on the Heavy Cruiser, if any
            for weapon in squadron.ship.currentAccompaniment.ship.currentWeapons:
              if (weapon.category != 'Indirect'):
                continue
              if (needHeader):  #print the Ship|Weapon Type|Weapon Arc|Range Increments header
                needHeader = False
                fillBrush = wx.Brush(gv.Report_DirectFireHeader_Color)
                dc.SetBrush(fillBrush)
                dc.DrawRectangle(xpos, ypos, 5.4*resolutionDPI, 0.20*resolutionDPI)
                dc.DrawText("Ship",xpos+0.05*resolutionDPI, ypos)
                dc.DrawText("Weapon Type",xpos+1.1*resolutionDPI, ypos)
                dc.DrawText("Weapon Arc",xpos+2.2*resolutionDPI, ypos)
                if (weaponType == 'Cyberwarfare'):
                  dc.DrawText('<10"',xpos+3.9*resolutionDPI, ypos)
                  dc.DrawText('<20"',xpos+4.3*resolutionDPI, ypos)
                  dc.DrawText('<30"',xpos+4.7*resolutionDPI, ypos)
                  dc.DrawText('<40"',xpos+5.1*resolutionDPI, ypos)
                elif (weaponType == 'Gravitational'):
                  dc.DrawText('<8"',xpos+3.9*resolutionDPI, ypos)
                  dc.DrawText('<16"',xpos+4.3*resolutionDPI, ypos)
                  dc.DrawText('<24"',xpos+4.7*resolutionDPI, ypos)
                  dc.DrawText('<32"',xpos+5.1*resolutionDPI, ypos)
                elif ((weaponType == 'Torpedo') or (weaponType == 'Nuclear Torpedo')):
                  dc.DrawText('<12"',xpos+3.9*resolutionDPI, ypos)
                  dc.DrawText('<24"',xpos+4.3*resolutionDPI, ypos)
                  dc.DrawText('<36"',xpos+4.7*resolutionDPI, ypos)
                  dc.DrawText('<48"',xpos+5.1*resolutionDPI, ypos)
                  ypos += 0.20*resolutionDPI  #move the 'pointer' down ready for the next entry.
              if (weaponType in weapon.weaponType) and (weapon.category == 'Indirect'):
                fillBrush = wx.Brush(gv.Report_IndirectFire_Color)
                dc.SetBrush(fillBrush)
                dc.DrawRectangle(xpos, ypos, 5.4*resolutionDPI, 0.20*resolutionDPI)
                dc.DrawText("{0}".format(squadron.ship.currentAccompaniment.ship.className),xpos+0.05*resolutionDPI, ypos)
                dc.DrawText("{0} {1}".format(weapon.weaponType, weapon.weaponUpgrade),xpos+1.1*resolutionDPI, ypos)
                dc.DrawText("{0} {1}".format(weapon.arcDirection, weapon.arcRestriction),xpos+2.2*resolutionDPI, ypos)
                dc.DrawText("{0}".format(weapon.rb1),xpos+3.9*resolutionDPI, ypos)
                dc.DrawText("{0}".format(weapon.rb2),xpos+4.3*resolutionDPI, ypos)
                dc.DrawText("{0}".format(weapon.rb3),xpos+4.7*resolutionDPI, ypos)
                dc.DrawText("{0}".format(weapon.rb4),xpos+5.1*resolutionDPI, ypos)
                ypos += 0.20*resolutionDPI  #move the 'pointer' down ready for the next entry.  
  
      lineHeight = 0
      #print out the MAR's, either full or partial text
      dc.SetFont(fontBoldSmall)
      (width, height) = dc.GetTextExtent("Model Assigned Rules (MAR)")
      xpos = 0
      dc.DrawText("Model Assigned Rules (MAR)", xpos, ypos)
      ypos += height
      dc.SetFont(fontSmall)
      if (squadron.ship.currentMAR == []):
        dc.SetFont(fontBoldSmall)  
        dc.DrawText("No MARs on {0}".format(squadron.ship.className), xpos, ypos)
        dc.SetFont(fontSmall)
        ypos += height
      for mar in squadron.ship.currentMAR:
        if (gv.PrintMAR != 'Simple'):
          if (gv.PrintMAR == 'Detailed'):
            text = mar.marText
          else:
            text = mar.marSummary
          if (ypos <= ypos_IMG):
            textBlock = ww.wordwrap(text, printAreaWidth-2.2*resolutionDPI, dc)
          else:
            textBlock = ww.wordwrap(text, printAreaWidth, dc)
          (width, ysize, lineHeight) = dc.GetMultiLineTextExtent(textBlock)
          xpos = 0.05*resolutionDPI
          dc.SetFont(fontBoldSmall)
          dc.DrawText((mar.marName+" ("+squadron.ship.className+")"), xpos, ypos)
          xpos = 0.1 * resolutionDPI
          ypos += lineHeight
          dc.SetFont(fontSmall)
          textList = textBlock.split('\n')
          for line in textList:
            dc.DrawText(line, xpos, ypos)
            ypos += lineHeight
          #ypos += (lineHeight+0.005*resolutionDPI)  #spacing for the next entry
        else:
          xpos = 0.05*resolutionDPI
          (width, height) = dc.GetTextExtent(mar.marName)
          dc.SetFont(fontBoldSmall)
          dc.DrawText((mar.marName+" ("+squadron.ship.className+")"), xpos, ypos)
          ypos += height
      dc.SetFont(fontSmall)
      
      #print out MARs from accompaniment, if any
      if (squadron.ship.currentAccompaniment):
        if (squadron.ship.currentAccompaniment.ship.currentMAR == []):
          dc.SetFont(fontBoldSmall)
          xpos = 0
          dc.DrawText("No MARs on {0}".format(squadron.ship.currentAccompaniment.ship.className), xpos, ypos)
          dc.SetFont(fontSmall)
          ypos += height
        for mar in squadron.ship.currentAccompaniment.ship.currentMAR:
          if (gv.PrintMAR != 'Simple'):
            if (gv.PrintMAR == 'Detailed'):
              text = mar.marText
            else:
              text = mar.marSummary
            if (ypos <= ypos_IMG):
              textBlock = ww.wordwrap(text, printAreaWidth-2.2*resolutionDPI, dc)
            else:
              textBlock = ww.wordwrap(text, printAreaWidth, dc)
            (width, ysize, lineHeight) = dc.GetMultiLineTextExtent(textBlock)
            xpos = 0.05*resolutionDPI
            dc.SetFont(fontBoldSmall)
            dc.DrawText((mar.marName+" ("+squadron.ship.currentAccompaniment.ship.className+")"), xpos, ypos)
            xpos = 0.1 * resolutionDPI
            ypos += lineHeight
            dc.SetFont(fontSmall)
            textList = textBlock.split('\n')
            for line in textList:
              dc.DrawText(line, xpos, ypos)
              ypos += lineHeight
            #ypos += (lineHeight+0.005*resolutionDPI)  #spacing for the next entry
          else:
            xpos = 0.05*resolutionDPI
            (width, height) = dc.GetTextExtent(mar.marName)
            dc.SetFont(fontBoldSmall)
            dc.DrawText((mar.marName+" ("+squadron.ship.currentAccompaniment.ship.className+")"), xpos, ypos)
            ypos += height
        dc.SetFont(fontSmall)
          
      #print out MARs from heavy cruiser, if any
      if (squadron.heavyCruiser):
        if (squadron.heavyCruiser.currentMAR == []):
          dc.SetFont(fontBoldSmall)
          dc.DrawText("No MARs on {0}".format(squadron.heavyCruiser.className), xpos, ypos)
          dc.SetFont(fontSmall)
          ypos += height
        for mar in squadron.heavyCruiser.currentMAR:
          if (gv.PrintMAR != 'Simple'):
            if (gv.PrintMAR == 'Detailed'):
              text = mar.marText
            else:
              text = mar.marSummary
            if (ypos <= ypos_IMG):
              textBlock = ww.wordwrap(text, printAreaWidth-2.2*resolutionDPI, dc)
            else:
              textBlock = ww.wordwrap(text, printAreaWidth, dc)
            (width, ysize, lineHeight) = dc.GetMultiLineTextExtent(textBlock)
            xpos = 0.05*resolutionDPI
            dc.SetFont(fontBoldSmall)
            dc.DrawText((mar.marName+" ("+squadron.heavyCruiser.className+")"), xpos, ypos)
            xpos = 0.1 * resolutionDPI
            ypos += lineHeight
            dc.SetFont(fontSmall)
            textList = textBlock.split('\n')
            for line in textList:
              dc.DrawText(line, xpos, ypos)
              ypos += lineHeight
          else:
            xpos = 0.05*resolutionDPI
            (width, height) = dc.GetTextExtent(mar.marName)
            dc.SetFont(fontBoldSmall)
            dc.DrawText((mar.marName+" ("+squadron.heavyCruiser.className+")"), xpos, ypos)
            ypos += height
        dc.SetFont(fontSmall)

      #Print out list of Upgrades for main ship
      if (squadron.ship.currentUpgrades != []):
        xpos = 0
        ypos += (lineHeight+0.05*resolutionDPI)  #add padding
        dc.SetFont(fontBoldSmall)
        dc.DrawText("{0} Upgrades".format(squadron.ship.className), xpos, ypos)
        dc.SetFont(fontSmall)
        (tWidth, tHeight) = dc.GetTextExtent("Upgrades")
        ypos += tHeight
        xpos += Xmargin
        for upg in squadron.ship.currentUpgrades:
          if (upg.cost == 0):
            textLine = "{0} (free)".format(upg.text)
          else:
            textLine = "{0} ({1} points)".format(upg.text, upg.cost)
          if (ypos <= ypos_IMG):
            textBlock = ww.wordwrap(textLine, printAreaWidth-2.2*resolutionDPI, dc)
          else:
            textBlock = ww.wordwrap(textLine, printAreaWidth, dc)
          (width, ysize, lineHeight) = dc.GetMultiLineTextExtent(textBlock)
          dc.DrawText(textLine, xpos, ypos)
          ypos += lineHeight
        
      #Print out list of Upgrades for Accompaniment
      if (squadron.ship.currentAccompaniment) and (squadron.ship.currentAccompaniment.ship.currentUpgrades != []):
        xpos = 0
        ypos += (lineHeight+0.05*resolutionDPI)  #add padding
        dc.SetFont(fontBoldSmall)
        dc.DrawText("{0} Upgrades".format(squadron.ship.currentAccompaniment.ship.className), xpos, ypos)
        dc.SetFont(fontSmall)
        (tWidth, tHeight) = dc.GetTextExtent("Upgrades")
        ypos += tHeight
        xpos += Xmargin
        for upg in squadron.ship.currentAccompaniment.ship.currentUpgrades:
          if (upg.cost == 0):
            textLine = "{0} (free)".format(upg.text)
          else:
            textLine = "{0} ({1} points)".format(upg.text, upg.cost)
          if (ypos <= ypos_IMG):
            textBlock = ww.wordwrap(textLine, printAreaWidth-2.2*resolutionDPI, dc)
          else:
            textBlock = ww.wordwrap(textLine, printAreaWidth, dc)
          (width, ysize, lineHeight) = dc.GetMultiLineTextExtent(textBlock)
          dc.DrawText(textLine, xpos, ypos)
          ypos += lineHeight
        
      #Print out list of Upgrades for Heavy Cruiser
      if (squadron.heavyCruiser) and (squadron.heavyCruiser.currentUpgrades != []):
        xpos = 0
        ypos += (lineHeight+0.05*resolutionDPI)  #add padding
        dc.SetFont(fontBoldSmall)
        dc.DrawText("{0} Upgrades".format(squadron.heavyCruiser.className), xpos, ypos)
        dc.SetFont(fontSmall)
        (tWidth, tHeight) = dc.GetTextExtent("Upgrades")
        ypos += tHeight
        xpos += Xmargin
        for upg in squadron.heavyCruiser.currentUpgrades:
          if (upg.cost == 0):
            textLine = "{0} (free)".format(upg.text)
          else:
            textLine = "{0} ({1} points)".format(upg.text, upg.cost)
          if (ypos <= ypos_IMG):
            textBlock = ww.wordwrap(textLine, printAreaWidth-2.2*resolutionDPI, dc)
          else:
            textBlock = ww.wordwrap(textLine, printAreaWidth, dc)
          (width, ysize, lineHeight) = dc.GetMultiLineTextExtent(textBlock)
          dc.DrawText(textLine, xpos, ypos)
          ypos += lineHeight
        
      #Print out list of Hardpoints added
      if (squadron.ship.currentHardpoints != []):
        xpos = 0
        ypos += (lineHeight+0.05*resolutionDPI)  #add padding
        dc.SetFont(fontBoldSmall)
        dc.DrawText("{0} Hardpoints".format(squadron.ship.className), xpos, ypos)
        (tWidth, tHeight) = dc.GetTextExtent("Hardpoints")
        ypos += tHeight
        dc.SetFont(fontSmall)
        xpos += Xmargin
        for hpt in squadron.ship.currentHardpoints:
          if (hpt.cost == 0):
            textLine = "{0} (free)".format(hpt.description)
          else:
            textLine = "{0} ({1} points)".format(hpt.description, hpt.cost)
          if (ypos <= ypos_IMG):
            textBlock = ww.wordwrap(textLine, printAreaWidth-2.2*resolutionDPI, dc)
          else:
            textBlock = ww.wordwrap(textLine, printAreaWidth, dc)
          (width, ysize, lineHeight) = dc.GetMultiLineTextExtent(textBlock)
          dc.DrawText(textLine, xpos, ypos)
          ypos += lineHeight
      
      #Print out list of Hardpoints added on the accompaniment
      if (squadron.ship.currentAccompaniment) and (squadron.ship.currentAccompaniment.ship.currentHardpoints != []):
        xpos = 0
        ypos += (lineHeight+0.05*resolutionDPI)  #add padding
        dc.SetFont(fontBoldSmall)
        dc.DrawText("{0} Hardpoints".format(squadron.ship.currentAccompaniment.ship.className), xpos, ypos)
        (tWidth, tHeight) = dc.GetTextExtent("Hardpoints")
        ypos += tHeight
        dc.SetFont(fontSmall)
        xpos += Xmargin
        for hpt in squadron.ship.currentAccompaniment.ship.currentHardpoints:
          if (hpt.cost == 0):
            textLine = "{0} (free)".format(hpt.description)
          else:
            textLine = "{0} ({1} points)".format(hpt.description, hpt.cost)
          if (ypos <= ypos_IMG):
            textBlock = ww.wordwrap(textLine, printAreaWidth-2.2*resolutionDPI, dc)
          else:
            textBlock = ww.wordwrap(textLine, printAreaWidth, dc)
          (width, ysize, lineHeight) = dc.GetMultiLineTextExtent(textBlock)
          dc.DrawText(textLine, xpos, ypos)
          ypos += lineHeight
      
      #Print out list of Hardpoints added on the heavy cruiser
      if (squadron.heavyCruiser) and (squadron.heavyCruiser.currentHardpoints != []):
        xpos = 0
        ypos += (lineHeight+0.05*resolutionDPI)  #add padding
        dc.SetFont(fontBoldSmall)
        dc.DrawText("{0} Hardpoints".format(squadron.heavyCruiser.className), xpos, ypos)
        (tWidth, tHeight) = dc.GetTextExtent("Hardpoints")
        ypos += tHeight
        dc.SetFont(fontSmall)
        xpos += Xmargin
        for hpt in squadron.heavyCruiser.currentHardpoints:
          if (hpt.cost == 0):
            textLine = "{0} (free)".format(hpt.description)
          else:
            textLine = "{0} ({1} points)".format(hpt.description, hpt.cost)
          if (ypos <= ypos_IMG):
            textBlock = ww.wordwrap(textLine, printAreaWidth-2.2*resolutionDPI, dc)
          else:
            textBlock = ww.wordwrap(textLine, printAreaWidth, dc)
          (width, ysize, lineHeight) = dc.GetMultiLineTextExtent(textBlock)
          dc.DrawText(textLine, xpos, ypos)
          ypos += lineHeight
      
      ypos += (lineHeight+0.25*resolutionDPI)  #add final padding between this and next squadron
      currentStep += 1
      progressDlg.Update(currentStep)


    dc.EndDoc()
  
    return True
  
  def SetFactionImage(self, faction):
    return wx.Image(os.path.join(gv.images_folder, dataStruct.factionBitmapPath[faction]), wx.BITMAP_TYPE_ANY)


#----------------------------------------------------------------------

class PrintPreviewBase(FirestormDialogs.PrintPreviewCore):
  def __init__(self, parent, fleet):
    FirestormDialogs.PrintPreviewCore.__init__(self, parent)
    self.printData = wx.PrintData()
    self.printData.SetPaperId(wx.PAPER_LETTER)
    self.printData.SetPrintMode(wx.PRINT_MODE_PRINTER)
    self.fleet = fleet
    return

  def OnPageSetup(self, evt):
    psdd = wx.PageSetupDialogData(self.printData)
    psdd.CalculatePaperSizeFromId()
    dlg = wx.PageSetupDialog(self, psdd)
    dlg.ShowModal()

    # this makes a copy of the wx.PrintData instead of just saving
    # a reference to the one inside the PrintDialogData that will
    # be destroyed when the dialog is destroyed
    self.printData = wx.PrintData( dlg.GetPageSetupData().GetPrintData() )

    dlg.Destroy()
    return
  
  def OnClose(self, event):
    self.EndModal(True)
    return

  def OnPrintPreview(self, event):
    data = wx.PrintDialogData(self.printData)
    printer = wx.Printer(data)
    printout = FleetPrintout(self.fleet)
    printout2 = FleetPrintout(self.fleet)
    self.preview = wx.PrintPreview(printout, printout2, data)

    if not self.preview.Ok():
        #("Houston, we have a problem...\n")
        return

    pfrm = wx.PreviewFrame(self.preview, gv.frame, "This is a print preview")
    pfrm.SetPosition(gv.frame.GetPosition())
    pfrm.SetSize(gv.frame.GetSize())
    pfrm.Initialize()
    pfrm.Show(True)
    return

  def OnPrint(self, event):
    pdd = wx.PrintDialogData(self.printData)
    #pdd.SetToPage(2)
    printer = wx.Printer(pdd)
    printout = FleetPrintout(self.fleet)

    if not printer.Print(self, printout, True):
      wx.MessageBox("There was a problem printing.\nPerhaps your current printer is not set correctly, or the print was cancelled?", "Printing", wx.OK)
    else:
      self.printData = wx.PrintData( printer.GetPrintDialogData().GetPrintData() )
    printout.Destroy()
    return
  
########################################################################
class SelectMarVariablesBase(FirestormDialogs.MarVariablesCore):
  #----------------------------------------------------------------------
  def __init__(self, parent, listType=None, intString1=None, intString2=None):
    """set listType to None to choose an integer value"""
    FirestormDialogs.MarVariablesCore.__init__(self, parent)
    if (listType == 'arcs'):
      self.cxStringChoice.AppendItems(dataStruct.arcs)
    elif (listType == 'designation'):
      self.cxStringChoice.AppendItems(dataStruct.designations)
    elif (listType == 'weapons'):
      self.cxStringChoice.AppendItems(dataStruct.weaponTypes)
    else:
      self.cxStringChoice.Hide()
      self.stChoiceValue.Hide()
      
    if (intString1):
      self.stIntValue1.SetLabel(intString1)
    else:
      self.stIntValue1.Hide()
      self.spIntValue1.Hide()
      
    if (intString2):
      self.stIntValue2.SetLabel(intString2)
    else:
      self.stIntValue2.Hide()
      self.spIntValue2.Hide()
    
    return
  
  def OnCancel(self, event):
    self.EndModal(False)
    return
  
  def OnOK(self,event):
    self.EndModal(True)
    return
    
  
    
class HelpDialog(FirestormDialogs.HelpDialogCore):
  """text is the help text string to be displayed"""
  def __init__(self, parent, text):
    olddir = os.getcwd()
    os.chdir(gv.root_folder)  #this is necessary so that bitmap paths are read by the dialog code correctly
    FirestormDialogs.HelpDialogCore.__init__(self, parent)
    os.chdir(olddir)
    self.txTextArea.ChangeValue(text)
    self.txTextArea.SetSelection(0,0)
    return

  def OnClose(self, evt):
    self.EndModal(True)
    return

#------------------------------------------------------------------------------
class ShipEditorBase(FirestormDialogs.ShipEditorCore):
  def __init__(self, parent, sourceShip=None):
    folder = os.getcwd()
    os.chdir(gv.root_folder)
    FirestormDialogs.ShipEditorCore.__init__(self, parent)
    os.chdir(folder)
    self.cxFaction.AppendItems(dataStruct.factions)
    self.cxShipDesignation.Clear()
    self.cxShipDesignation.AppendItems(dataStruct.designations)
    self.lcWeapons.InsertColumn(0,"Name")
    self.lcWeapons.InsertColumn(1, "Weapon Type")
    self.lcWeapons.InsertColumn(2, "Firing Arc")
    self.lcWeapons.InsertColumn(3, "Inc.")
    self.lcWeapons.InsertColumn(4, "RB 1")
    self.lcWeapons.InsertColumn(5, "RB 2")
    self.lcWeapons.InsertColumn(6, "RB 3")
    self.lcWeapons.InsertColumn(7, "RB 4")
    self.lcWeapons.SetColumnWidth(3,wx.LIST_AUTOSIZE_USEHEADER)
    self.lcWeapons.SetColumnWidth(4,wx.LIST_AUTOSIZE_USEHEADER)
    self.lcWeapons.SetColumnWidth(5,wx.LIST_AUTOSIZE_USEHEADER)
    self.lcWeapons.SetColumnWidth(6,wx.LIST_AUTOSIZE_USEHEADER)
    self.lcWeapons.SetColumnWidth(7,wx.LIST_AUTOSIZE_USEHEADER)
  
    self.lcHardpoints.InsertColumn(0,"Num Available")
    self.lcHardpoints.InsertColumn(1, "Cost")
    self.lcHardpoints.InsertColumn(2, "Description")
      
    self.lcUpgrades.InsertColumn(0,"Cost")
    self.lcUpgrades.InsertColumn(1,"Description")
    
    self.lcMarList.InsertColumn(0, '#')
    self.lcMarList.InsertColumn(1, 'MAR Name')
    self.lcMarList.InsertColumn(2, 'Description')
  
    self.lcAccompaniment.InsertColumn(0, "Number")
    self.lcAccompaniment.InsertColumn(1, "Cost Each")
    self.lcAccompaniment.InsertColumn(2, "Class Name")
    self.lcAccompaniment.InsertColumn(3, "Faction")
    self.lcAccompaniment.InsertColumn(4, "Designation")
    
    if (sourceShip):
      editShip = copy.deepcopy(sourceShip)
      #editShip = dataStruct.ShipRecord()
      self.cxFaction.SetStringSelection(editShip.faction)
      self.txClassName.SetValue(editShip.className)
      self.cxSizeClass.SetStringSelection(editShip.classSize)
      self.cxShipDesignation.SetStringSelection(editShip.designation)
      self.spBaseCost.SetValue(editShip.baseCost)
      self.spSquadronSizeMin.SetValue(editShip.squadronSizeMin)
      self.spSquadronSizeMax.SetValue(editShip.squadronSizeMax)
      self.spDR.SetValue(editShip.DR)
      self.spCR.SetValue(editShip.CR)
      self.spHP.SetValue(editShip.HP)
      self.spMV.SetValue(editShip.MV)
      self.spTL.SetValue(editShip.TL)
      self.spCP.SetValue(editShip.CP)
      self.spAP.SetValue(editShip.AP)
      self.spPD.SetValue(editShip.PD)
      self.spSR.SetValue(editShip.SR)
      self.spMN.SetValue(editShip.MN)
      self.spWC.SetValue(editShip.WC)
      
      self.cbCloakingField.SetValue(editShip.cloakingField)
      self.spSR.Enable(not editShip.cloakingField)
      if (editShip.secondaryCost != 0):
        self.cbSecondaryCost.SetValue(True)
        self.spSecondaryCost.Enable(True)
        self.spSecondaryCost.SetValue(editShip.secondaryCost)
      else:
        self.spSecondaryCost.Enable(False)
        self.cbSecondaryCost.SetValue(False)
      
      self.weaponsArray = editShip.weapons
      self.hardpointArray = editShip.availableHardPoints
      self.spHardpoints.SetValue(editShip.maxHardPoints)
      if (editShip.source == 'Official'):
        self.rbOfficialShip.SetValue(True)
      else:
        self.rbCustomShip.SetValue(True)
      if (editShip.maxHardPoints > 0):
        self.bAddHardpoint.Enable(True)
      self.upgradeArray = editShip.upgrades
      self.marArray = editShip.MAR
      self.accompanimentArray = editShip.accompanimentIncluded
      self.imageFilename = editShip.imageFilename
      self.txNotes.SetValue(editShip.notes)
      self.UpdateAccompArray()
      self.UpdateHardpoints()
      self.UpdateMarList()
      self.UpdateUpgradesList()
      self.UpdateWeaponsList()
    else:
      self.weaponsArray = []
      self.hardpointArray = []
      self.upgradeArray = []
      self.marArray = []
      self.accompanimentArray = []
      self.imageFilename = None
      self.cxFaction.SetStringSelection('Dindrenzi')  #default choice
    
    self.OnChooseFaction()
    self.UpdateImage()
    self.selectWeapon = None
    self.selectIndex = None
    self.selectMar = None
    self.selectAccomp = None
    self.selectHardpoint = None
    
    self.XOR_index = 0  #this index is used to differentiate between multiple XOR choices in upgrades
    self.XOR_blue = False  #has this color been used in XOR colors?
    self.XOR_red = False  #has this color been used in XOR colors?
    self.XOR_green = False  #has this color been used in XOR colors?
    self.XOR_orange = False  #has this color been used in XOR colors?
    self.XOR_purple = False  #has this color been used in XOR colors?
    
    return
  
  def OnClearXOR(self, event):
    for upg in self.upgradeArray:
      if (upg.selected):
        upg.xor = []
        if (upg.color == gv.XORblue):
          self.XOR_blue = False
        if (upg.color == gv.XORred):
          self.XOR_red = False
        if (upg.color == gv.XORgreen):
          self.XOR_green = False
        if (upg.color == gv.XORorange):
          self.XOR_orange = False
        if (upg.color == gv.XORpurple):
          self.XOR_purple = False
        upg.color = None
        upg.selected = None
    self.UpdateUpgradesList()
    return
  
  def OnMarkXOR(self, event):
    xorList = []
    #find an unused color
    if (not self.XOR_blue):
      XORcolor = gv.XORblue
      self.XOR_blue = True
    elif (not self.XOR_red):
      XORcolor = gv.XORred
      self.XOR_red = True
    elif (not self.XOR_green):
      XORcolor = gv.XORgreen
      self.XOR_green = True
    elif (not self.XOR_orange):
      XORcolor = gv.XORorange
      self.XOR_orange = True
    else:
      XORcolor = gv.XORpurple
      self.XOR_purple = True
      
    for upg in self.upgradeArray:
      if (upg.selected):
        xorList.append(upg.index)
    for upg in self.upgradeArray:
      if (upg.selected):
        upg.xor = xorList
        upg.color = XORcolor
      upg.selected = False  #reset the selected flag
    self.UpdateUpgradesList()
    self.bMarkXOR.Enable(False)
    self.bClearXOR.Enable(False)
    return
  
  def OnChooseFaction(self, event=None):
    #select the default image based on the faction chosen
    selection = self.cxFaction.GetStringSelection()
    self.factionImageFilename = dataStruct.factionBitmapPath[selection]
    self.UpdateImage()
    return
  
  def UpdateImage(self):
    if (self.imageFilename):
      try:
        filename = os.path.join(gv.ship_images_folder, self.imageFilename)
      except:
        logging.error("Could not find ship image {0}".format(self.imageFilename))
        filename = os.path.join(gv.ship_images_folder, gv.defaultImage)
        return
    else:
      try:
        filename = os.path.join(gv.ship_images_folder, gv.defaultImage)
      except:
        logging.critical("Could not find default image")
        return
      self.imageFilename = gv.defaultImage
      
    if (self.factionImageFilename):
      try:
        factionFilename = os.path.join(gv.faction_images_folder, self.factionImageFilename)
      except:
        logging.error("Could not find faction image {0}".format(self.factionImageFilename))
        factionFilename = os.path.join(gv.faction_images_folder, gv.defaultImage)
        return
    else:
      try:
        factionFilename = os.path.join(gv.faction_images_folder, gv.defaultImage)
      except:
        logging.critical("Could not find default image")
        return
    
    if (not os.access(filename, os.F_OK)):
      logging.warning("Could not find {0}".format(filename))
      self.imageFilename = gv.defaultImage
      filename = os.path.join(gv.images_folder, gv.defaultImage)
      
    image = wx.Image(filename)
    if (image.IsOk()):
      (width, height) = self.bmpShipImage.GetSize()
      scaledBitmap = wx.BitmapFromImage(image.Scale(width, height, wx.IMAGE_QUALITY_HIGH))
      self.bmpShipImage.SetBitmap(scaledBitmap)
    else:
      logging.error("Could not find ship image {0}".format(self.imageFilename))
    
    factionImage = wx.Image(factionFilename)
    if (factionImage.IsOk()):
      (width, height) = self.bmpFactionImage.GetSize()
      factionScaledBitmap = wx.BitmapFromImage(factionImage.Scale(width, height, wx.IMAGE_QUALITY_HIGH))
      self.bmpFactionImage.SetBitmap(factionScaledBitmap)
    else:
      logging.error("Could not find faction image {0}".format(self.factionImageFilename))
      
    return
  
  def OnChangeShipImage(self, event):
    dlg = wx.FileDialog(self, message='Choose ship image file', defaultDir=gv.ship_images_folder,
                        defaultFile="", wildcard=image_wildcard, style=wx.OPEN)
    if dlg.ShowModal() == wx.ID_OK:
      self.imageFilename = dlg.GetFilename()
    self.UpdateImage()
    return

  def OnChangeFactionImage(self, event):
    dlg = wx.FileDialog(self, message='Choose faction image file', defaultDir=gv.faction_images_folder,
                        defaultFile="", wildcard=image_wildcard, style=wx.OPEN)
    if dlg.ShowModal() == wx.ID_OK:
      self.factionImageFilename = dlg.GetFilename()
    self.UpdateImage()
    return
  
  def OnAddUpgrade(self, event):
    dlg = AddUpgradeBase(self)
    addUpgrade = dlg.ShowModal()
    if (addUpgrade):
      upgrade = dataStruct.UpgradeRecord()
      upgrade.cost = dlg.cost
      if (dlg.cbRemoveAblative.IsChecked()):
        upgrade = dataStruct.UpgradeRecord()
        upgrade.cost = dlg.cost
        upgrade.index = 1000
        upgrade.text = "Remove the Ablative Plating MAR"
        self.upgradeArray.append(upgrade)
      if (dlg.cbAmbushMAR.IsChecked()):
        upgrade = dataStruct.UpgradeRecord()
        upgrade.cost = dlg.cost
        upgrade.index = 1001
        upgrade.text = "Gain the Ambush (2) MAR"
        self.upgradeArray.append(upgrade)
      if (dlg.cbAssault2.IsChecked()):
        upgrade = dataStruct.UpgradeRecord()
        upgrade.cost = dlg.cost
        upgrade.index = 1002
        upgrade.text = "Gain +2 Assault Points"
        self.upgradeArray.append(upgrade)
      if (dlg.cbAssaultBlitz.IsChecked()):
        upgrade = dataStruct.UpgradeRecord()
        upgrade.cost = dlg.cost
        upgrade.index = 1003
        upgrade.text = "Gain the Assault Blitz MAR"
        self.upgradeArray.append(upgrade)
      if (dlg.cbBeamBiohazard.IsChecked()):
        upgrade = dataStruct.UpgradeRecord()
        upgrade.cost = dlg.cost
        upgrade.index = 1004
        upgrade.text = "Give all Beam Weapons the Biohazard MAR"
        self.upgradeArray.append(upgrade)
      if (dlg.cbBeamPrecisionStrike.IsChecked()):
        upgrade = dataStruct.UpgradeRecord()
        upgrade.cost = dlg.cost
        upgrade.index = 1006
        upgrade.text = "Give all Beam Weapons the Precision Strike MAR"
        self.upgradeArray.append(upgrade)
      if (dlg.cbBeamToHighEnergy.IsChecked()):
        upgrade = dataStruct.UpgradeRecord()
        upgrade.cost = dlg.cost
        upgrade.index = 1005
        upgrade.text = "Give all Beam Weapons the High Energy MAR"
        self.upgradeArray.append(upgrade)
      if (dlg.cbBeamToSplitFireMAR.IsChecked()):
        upgrade = dataStruct.UpgradeRecord()
        upgrade.cost = dlg.cost
        upgrade.index = 1007
        upgrade.text = "Give all Beam Weapons the Split Fire MAR"
        self.upgradeArray.append(upgrade)
      if (dlg.cbBiggerBatteriesMAR.IsChecked()):
        upgrade = dataStruct.UpgradeRecord()
        upgrade.cost = dlg.cost
        upgrade.index = 1008
        upgrade.text = "Gain the Bigger Batteries MAR"
        self.upgradeArray.append(upgrade)
      if (dlg.cbCountermeasures.IsChecked()):
        upgrade = dataStruct.UpgradeRecord()
        upgrade.cost = dlg.cost
        upgrade.index = 1009
        upgrade.text = "Gain the Countermeasures MAR"
        self.upgradeArray.append(upgrade)
      if (dlg.cbDeckCrews.IsChecked()):
        upgrade = dataStruct.UpgradeRecord()
        upgrade.cost = dlg.cost
        upgrade.index = 1010
        upgrade.text = "Gain the Deck Crews MAR"
        self.upgradeArray.append(upgrade)
      if (dlg.cbDoubleMines.IsChecked()):
        upgrade = dataStruct.UpgradeRecord()
        upgrade.cost = dlg.cost
        upgrade.index = 1011
        upgrade.text = "Gain the Double Mines MAR"
        self.upgradeArray.append(upgrade)
      if (dlg.cbDroneMines.IsChecked()):
        upgrade = dataStruct.UpgradeRecord()
        upgrade.cost = dlg.cost
        upgrade.index = 1012
        upgrade.text = "Gain the Drone Mines MAR"
        self.upgradeArray.append(upgrade)
      if (dlg.cbEnergyAll2.IsChecked()):
        upgrade = dataStruct.UpgradeRecord()
        upgrade.stringValue = 'All'
        upgrade.intValue1 = 2
        upgrade.cost = dlg.cost
        upgrade.index = 1016
        upgrade.text = "Gain the Energy Transfer (All, 2) MAR"
        self.upgradeArray.append(upgrade)
      if (dlg.cbEnergyTransfer1.IsChecked()):
        upgrade = dataStruct.UpgradeRecord()
        selectDlg = SelectMarVariablesBase(self, listType='weapons', 
                                          intString1='Transfer Value:', 
                                          intString2=None)
        ok = selectDlg.ShowModal()
        if (ok):
          upgrade.stringValue = selectDlg.cxStringChoice.GetStringSelection()
          upgrade.intValue1 = selectDlg.spIntValue1.GetValue()
        else:
          return
        upgrade.cost = dlg.cost
        upgrade.index = 1014
        upgrade.text = "Gain the Energy Transfer ({0},{1}) MAR".format(upgrade.stringValue, upgrade.intValue1)
        self.upgradeArray.append(upgrade)
      if (dlg.cbEnergyTransfer2.IsChecked()):
        upgrade = dataStruct.UpgradeRecord()
        selectDlg = SelectMarVariablesBase(self, listType='weapons', 
                                          intString1='Transfer Value:', 
                                          intString2=None)
        ok = selectDlg.ShowModal()
        if (ok):
          upgrade.stringValue = selectDlg.cxStringChoice.GetStringSelection()
          upgrade.intValue1 = selectDlg.spIntValue1.GetValue()
        else:
          return
        upgrade.cost = dlg.cost
        upgrade.index = 1015
        upgrade.text = "Gain the Energy Transfer ({0},{1}) MAR".format(upgrade.stringValue, upgrade.intValue1)
        self.upgradeArray.append(upgrade)
      if (dlg.cbExperiencedEngineers.IsChecked()):
        upgrade = dataStruct.UpgradeRecord()
        upgrade.cost = dlg.cost
        upgrade.index = 1013
        upgrade.text = "Gain the Experienced Engineers MAR"
        self.upgradeArray.append(upgrade)
      if (dlg.cbFFtoDecimatorMAR.IsChecked()):
        upgrade = dataStruct.UpgradeRecord()
        upgrade.cost = dlg.cost
        upgrade.index = 1017
        upgrade.text = "Give Fore Fixed Weapons the Decimator Warheads MAR"
        self.upgradeArray.append(upgrade)
      if (dlg.cbGunSplitFire.IsChecked()):
        upgrade = dataStruct.UpgradeRecord()
        upgrade.cost = dlg.cost
        upgrade.index = 1018
        upgrade.text = "Give Gun Rack Weapons the Split Fire MAR"
        self.upgradeArray.append(upgrade)
      if (dlg.cbKineticHighEnergy.IsChecked()):
        upgrade = dataStruct.UpgradeRecord()
        upgrade.cost = dlg.cost
        upgrade.index = 1019
        upgrade.text = "Give all Kinetic Weapons the High Energy MAR"
        self.upgradeArray.append(upgrade)
      if (dlg.cbKineticPrecision.IsChecked()):
        upgrade = dataStruct.UpgradeRecord()
        upgrade.cost = dlg.cost
        upgrade.index = 1020
        upgrade.text = "Give all Kinetic Weapons the Precision Strike MAR"
        self.upgradeArray.append(upgrade)
      if (dlg.cbManoeuverable.IsChecked()):
        upgrade = dataStruct.UpgradeRecord()
        upgrade.cost = dlg.cost
        upgrade.index = 1021
        upgrade.text = "Gain the Manoeuverable MAR"
        self.upgradeArray.append(upgrade)
      if (dlg.cbMineBiohazard.IsChecked()):
        upgrade = dataStruct.UpgradeRecord()
        upgrade.cost = dlg.cost
        upgrade.index = 1022
        upgrade.text = "Give Mines the Biohazard MAR"
        self.upgradeArray.append(upgrade)
      if (dlg.cbMines3.IsChecked()):
        upgrade = dataStruct.UpgradeRecord()
        upgrade.cost = dlg.cost
        upgrade.index = 1023
        upgrade.text = "Gain +3 Mines"
        self.upgradeArray.append(upgrade)
      if (dlg.cbPackHunters.IsChecked()):
        upgrade = dataStruct.UpgradeRecord()
        upgrade.cost = dlg.cost
        upgrade.index = 1024
        upgrade.text = "Gain the Pack Hunters MAR"
        self.upgradeArray.append(upgrade)
      if (dlg.cbPointDefenseBarrageMAR.IsChecked()):
        upgrade = dataStruct.UpgradeRecord()
        upgrade.cost = dlg.cost
        upgrade.index = 1025
        upgrade.text = "Gain the Point Defense Barrage MAR"
        self.upgradeArray.append(upgrade)
      if (dlg.cbPrimaryBiohazard.IsChecked()):
        upgrade = dataStruct.UpgradeRecord()
        upgrade.cost = dlg.cost
        upgrade.index = 1027
        upgrade.text = "Give all Primary Weapons the Biohazard MAR"
        self.upgradeArray.append(upgrade)
      if (dlg.cbPrimaryDecimator.IsChecked()):
        upgrade = dataStruct.UpgradeRecord()
        upgrade.cost = dlg.cost
        upgrade.index = 1028
        upgrade.text = "Give all Primary Weapons the Decimator Warheads MAR"
        self.upgradeArray.append(upgrade)
      if (dlg.cbPrimaryPrecisionStrike.IsChecked()):
        upgrade = dataStruct.UpgradeRecord()
        upgrade.cost = dlg.cost
        upgrade.index = 1030
        upgrade.text = "Give all Primary Weapons the Precision Strike MAR"
        self.upgradeArray.append(upgrade)
      if (dlg.cbPrimaryToHighEnergy.IsChecked()):
        upgrade = dataStruct.UpgradeRecord()
        upgrade.cost = dlg.cost
        upgrade.index = 1029
        upgrade.text = "Give all Primary Weapons the High Energy MAR"
        self.upgradeArray.append(upgrade)
      if (dlg.cbPrimaryToScatter.IsChecked()):
        upgrade = dataStruct.UpgradeRecord()
        upgrade.cost = dlg.cost
        upgrade.index = 1046
        upgrade.text = "Change Primary Weapons to Scatter Weapons MAR"
        self.upgradeArray.append(upgrade)
      if (dlg.cbProtectedSystems.IsChecked()):
        upgrade = dataStruct.UpgradeRecord()
        upgrade.cost = dlg.cost
        upgrade.index = 1026
        upgrade.text = "Gain the Protected Systems MAR"
        self.upgradeArray.append(upgrade)
      if (dlg.cbQuickLaunch.IsChecked()):
        upgrade = dataStruct.UpgradeRecord()
        upgrade.cost = dlg.cost
        upgrade.index = 1032
        upgrade.text = "Gain the Quick Launch MAR"
        self.upgradeArray.append(upgrade)
      if (dlg.cbReinforced.IsChecked()):
        upgrade = dataStruct.UpgradeRecord()
        upgrade.cost = dlg.cost
        upgrade.index = 1033
        upgrade.text = "Gain the Reinforced MAR"
        self.upgradeArray.append(upgrade)
      if (dlg.cbScatterSplitFire.IsChecked()):
        upgrade = dataStruct.UpgradeRecord()
        upgrade.cost = dlg.cost
        upgrade.index = 1034
        upgrade.text = "Give Scatter Weapons the Split Fire MAR"
        self.upgradeArray.append(upgrade)
      if (dlg.cbScatterStarPortSplitFire.IsChecked()):
        upgrade = dataStruct.UpgradeRecord()
        upgrade.cost = dlg.cost
        upgrade.index = 1035
        upgrade.text = "Give Starboard/Port Weapons the Split Fire MAR"
        self.upgradeArray.append(upgrade)
      if (dlg.cbSecondAssault.IsChecked()):
        upgrade = dataStruct.UpgradeRecord()
        upgrade.cost = dlg.cost
        upgrade.index = 1031
        upgrade.text = "Gain the Second Assault MAR"
        self.upgradeArray.append(upgrade)
      if (dlg.cbSectorShielding.IsChecked()):
        upgrade = dataStruct.UpgradeRecord()
        upgrade.cost = dlg.cost
        upgrade.index = 1036
        upgrade.text = "Gain the Sector Shielding MAR"
        self.upgradeArray.append(upgrade)
      if (dlg.cbSecuredBulkheads.IsChecked()):
        upgrade = dataStruct.UpgradeRecord()
        upgrade.cost = dlg.cost
        upgrade.index = 1037
        upgrade.text = "Gain the Secured Bulkheads MAR"
        self.upgradeArray.append(upgrade)
      if (dlg.cbSelfRepair.IsChecked()):
        upgrade = dataStruct.UpgradeRecord()
        upgrade.cost = dlg.cost
        upgrade.index = 1038
        upgrade.text = "Gain the Self Repair MAR"
        self.upgradeArray.append(upgrade)
      if (dlg.cbSpecialForcesMAR.IsChecked()):
        upgrade = dataStruct.UpgradeRecord()
        upgrade.cost = dlg.cost
        upgrade.index = 1039
        upgrade.text = "Gain the Special Forces MAR"
        self.upgradeArray.append(upgrade)
      if (dlg.cbSplitFireMAR.IsChecked()):
        upgrade = dataStruct.UpgradeRecord()
        upgrade.cost = dlg.cost
        upgrade.index = 1040
        upgrade.text = "Gain the Split Fire MAR"
        self.upgradeArray.append(upgrade)
      if (dlg.cbTorpedoBiohazard.IsChecked()):
        upgrade = dataStruct.UpgradeRecord()
        upgrade.cost = dlg.cost
        upgrade.index = 1041
        upgrade.text = "Give all Torpedo Weapons the Biohazard MAR"
        self.upgradeArray.append(upgrade)
      if (dlg.cbTorpedoToCorrosive.IsChecked()):
        upgrade = dataStruct.UpgradeRecord()
        upgrade.cost = dlg.cost
        upgrade.index = 1042
        upgrade.text = "Give all Torpedoes the Corrosive MAR"
        self.upgradeArray.append(upgrade)
      if (dlg.cbTorpedoesHighEnergyMAR.IsChecked()):
        upgrade = dataStruct.UpgradeRecord()
        upgrade.cost = dlg.cost
        upgrade.index = 1043
        upgrade.text = "Give all Torpedo Weapons the High Energy MAR"
        self.upgradeArray.append(upgrade)
      if (dlg.cbTorpedoSpook.IsChecked()):
        upgrade = dataStruct.UpgradeRecord()
        upgrade.cost = dlg.cost
        upgrade.index = 1044
        upgrade.text = "Gain the Torpedo Spook MAR"
        self.upgradeArray.append(upgrade)
      if (dlg.cbWeaponShieldingMAR.IsChecked()):
        upgrade = dataStruct.UpgradeRecord()
        upgrade.cost = dlg.cost
        upgrade.index = 1045
        upgrade.text = "Gain the Weapon Shielding MAR"
        self.upgradeArray.append(upgrade)
      if (dlg.cbUpgradeFFtoNuclear.IsChecked()):
        upgrade = dataStruct.UpgradeRecord()
        upgrade.cost = dlg.cost
        upgrade.index = 1047
        upgrade.text = "Upgrade Fore Fixed to Nuclear Weapons"
        self.upgradeArray.append(upgrade)
      if (dlg.cbUpgradeNuclearTorpedoes.IsChecked()):
        upgrade = dataStruct.UpgradeRecord()
        upgrade.cost = dlg.cost
        upgrade.index = 1048
        upgrade.text = "Upgrade all Torpedoes to Nuclear Torpedoes"
        self.upgradeArray.append(upgrade)
      if (dlg.cbUpgradePrimaryToBeams.IsChecked()):
        upgrade = dataStruct.UpgradeRecord()
        upgrade.cost = dlg.cost
        upgrade.index = 1049
        upgrade.text = "Upgrade Primary Weapons to Beam Weapons"
        self.upgradeArray.append(upgrade)
      if (dlg.cbUpgradeStarboardPortToBeam.IsChecked()):
        upgrade = dataStruct.UpgradeRecord()
        upgrade.cost = dlg.cost
        upgrade.index = 1050
        upgrade.text = "Upgrade Starboard/Port Weapons to Beam Weapons"
        self.upgradeArray.append(upgrade)
      if (dlg.cbUpgradeTurretsToNuclear.IsChecked()):
        upgrade = dataStruct.UpgradeRecord()
        upgrade.cost = dlg.cost
        upgrade.index = 1051
        upgrade.text = "Upgrade Turrets to Nuclear Turrets MAR"
        self.upgradeArray.append(upgrade)
      if (dlg.cbTorpedoToDecimator.IsChecked()):
        upgrade = dataStruct.UpgradeRecord()
        upgrade.cost = dlg.cost
        upgrade.index = 1052
        upgrade.text = "Give all Torpedoes the Decimator Warheads MAR"
        self.upgradeArray.append(upgrade)
      if (dlg.cbBeamToCorrosive.IsChecked()):
        upgrade = dataStruct.UpgradeRecord()
        upgrade.cost = dlg.cost
        upgrade.index = 1053
        upgrade.text = dataStruct.upgrades[upgrade.index]
        self.upgradeArray.append(upgrade)
      if (dlg.cbScoutMAR.IsChecked()):
        upgrade = dataStruct.UpgradeRecord()
        upgrade.cost = dlg.cost
        upgrade.index = 1054
        upgrade.text = dataStruct.upgrades[upgrade.index]
        self.upgradeArray.append(upgrade)
      if (dlg.cbPrimaryToCyberwarfare.IsChecked()):
        upgrade = dataStruct.UpgradeRecord()
        upgrade.cost = dlg.cost
        upgrade.index = 1055
        upgrade.text = dataStruct.upgrades[upgrade.index]
        self.upgradeArray.append(upgrade)
      if (dlg.cbBeamDecimator.IsChecked()):
        upgrade = dataStruct.UpgradeRecord()
        upgrade.cost = dlg.cost
        upgrade.index = 1056
        upgrade.text = "Give all Beam Weapons the Decimator Warheads MAR"
        self.upgradeArray.append(upgrade)
      if (dlg.cbPrimaryCorrosive.IsChecked()):
        upgrade = dataStruct.UpgradeRecord()
        upgrade.cost = dlg.cost
        upgrade.index = 1057
        upgrade.text = "Give all Primary Weapons the Corrosive MAR"
        self.upgradeArray.append(upgrade)
      if (dlg.cbStealthSystemsMAR.IsChecked()):
        upgrade = dataStruct.UpgradeRecord()
        upgrade.cost = dlg.cost
        upgrade.index = 1058
        upgrade.text = dataStruct.upgrades[upgrade.index]  #stealth systems MAR
        self.upgradeArray.append(upgrade)
      if (dlg.cbEnergy1Energy2.IsChecked()):
        upgrade = dataStruct.UpgradeRecord()
        upgrade.cost = dlg.cost
        upgrade.index = 1059
        upgrade.text = dataStruct.upgrades[upgrade.index]  #energy transfer 1 to 2 MAR
        self.upgradeArray.append(upgrade)
      if (dlg.cbCommand2.IsChecked()):
        upgrade = dataStruct.UpgradeRecord()
        upgrade.cost = dlg.cost
        upgrade.index = 1060
        upgrade.text = dataStruct.upgrades[upgrade.index]  #command +2
        self.upgradeArray.append(upgrade)
      if (dlg.cbMovement1.IsChecked()):
        upgrade = dataStruct.UpgradeRecord()
        upgrade.cost = dlg.cost
        upgrade.index = 1061
        upgrade.text = dataStruct.upgrades[upgrade.index]  #movement +1
        self.upgradeArray.append(upgrade)
      if (dlg.cbCommand3.IsChecked()):
        upgrade = dataStruct.UpgradeRecord()
        upgrade.cost = dlg.cost
        upgrade.index = 1062
        upgrade.text = dataStruct.upgrades[upgrade.index]  #command +3
        self.upgradeArray.append(upgrade)
      if (dlg.cbScatterToKinetic.IsChecked()):
        upgrade = dataStruct.UpgradeRecord()
        upgrade.cost = dlg.cost
        upgrade.index = 1063
        upgrade.text = dataStruct.upgrades[upgrade.index]  #scatter to kinetic
        self.upgradeArray.append(upgrade)
      if (dlg.cbPointDefense1.IsChecked()):
        upgrade = dataStruct.UpgradeRecord()
        upgrade.cost = dlg.cost
        upgrade.index = 1064
        upgrade.text = dataStruct.upgrades[upgrade.index]  # PD +1
        self.upgradeArray.append(upgrade)
      if (dlg.cbWeaponShieldingNoDurable.IsChecked()):
        upgrade = dataStruct.UpgradeRecord()
        upgrade.cost = dlg.cost
        upgrade.index = 1065
        upgrade.text = dataStruct.upgrades[upgrade.index]  # weapon shielding mar, no durable mar
        self.upgradeArray.append(upgrade)
      if (dlg.cbShield1.IsChecked()):
        upgrade = dataStruct.UpgradeRecord()
        upgrade.cost = dlg.cost
        upgrade.index = 1066
        upgrade.text = dataStruct.upgrades[upgrade.index]  # sr +1
        self.upgradeArray.append(upgrade)
      if (dlg.cbTorpedoCyberAssaultRobot.IsChecked()):
        upgrade = dataStruct.UpgradeRecord()
        upgrade.cost = dlg.cost
        upgrade.index = 1067
        upgrade.text = dataStruct.upgrades[upgrade.index]  # torpedo to cyberwarfare
        self.upgradeArray.append(upgrade)
      if (dlg.cbPrecisionStrikeMAR.IsChecked()):
        upgrade = dataStruct.UpgradeRecord()
        upgrade.cost = dlg.cost
        upgrade.index = 1068
        upgrade.text = dataStruct.upgrades[upgrade.index]  # gain precision strike mar
        self.upgradeArray.append(upgrade)
        
        
        #when adding in upgrades, also need to update ShipRecord::ApplyUpgrades() and dataStruct.upgrades() with the index number
        #update FleetEditorBase::AddUpgrade
      self.UpdateUpgradesList()
    return

  def UpdateUpgradesList(self):
    self.lcUpgrades.DeleteAllItems()
    upgrade = dataStruct.UpgradeRecord()
    self.upgradeArray.reverse()
    #determine how many different xor items to group together
    index = 0

    for upgrade in self.upgradeArray:
      listitem = wx.ListItem()
      listitem.SetText(upgrade.text)
      if (upgrade.color):  #display in separate color if marked XOR
        listitem.SetTextColour(upgrade.color)
      index = self.lcUpgrades.InsertItem(listitem)
      self.lcUpgrades.SetStringItem(index, 0, str(upgrade.cost))
      self.lcUpgrades.SetStringItem(index, 1, upgrade.text)
      
    self.lcUpgrades.SetColumnWidth(0, wx.LIST_AUTOSIZE_USEHEADER)
    self.lcUpgrades.SetColumnWidth(1, wx.LIST_AUTOSIZE)
    if (self.lcUpgrades.GetColumnWidth(1) < 100):
      self.lcUpgrades.SetColumnWidth(1,100)
    self.upgradeArray.reverse()
    return
  
  def OnDeselectAccomp(self, event):
    self.bEditAccomp.Enable(False)
    self.bDeleteAccompaniment.Enable(False)
    return
  
  def OnDeselectMAR(self, event):
    self.bDeleteMar.Enable(False)
    return
  

  def OnSelectCloakingField(self, event):
    self.spSR.Enable(not self.cbCloakingField.GetValue())
    return
  
  def OnDeselectWeapon(self, event):
    self.bEditWeapon.Enable(False)
    self.bDeleteWeapon.Enable(False)
    return
  
  def OnDeselectHardpoint(self, event):
    self.bDeleteHardpoint.Enable(False)
    return
  
  def OnSelectHardpoint(self, event):
    self.selectHardpoint = event.m_itemIndex
    self.bDeleteHardpoint.Enable(True)
    return
  
  def OnDeselectUpgrade(self, event):
    self.upgradeArray[event.m_itemIndex].selected = False
    num_selected = self.lcUpgrades.GetSelectedItemCount()
    if (num_selected == 0):
      self.bDeleteUpgrade.Enable(False)
      self.bMarkXOR.Enable(False)
      self.bClearXOR.Enable(False)
    elif (num_selected == 1):
      self.bDeleteUpgrade.Enable(True)
      self.bMarkXOR.Enable(False)
      self.bClearXOR.Enable(False)
    else:
      self.bDeleteUpgrade.Enable(False)
      self.bMarkXOR.Enable(True)
      self.bClearXOR.Enable(True)
    return
  
  def OnSelectUpgrade(self, event):
    self.selectUpgrade = event.m_itemIndex
    self.upgradeArray[event.m_itemIndex].selected = True
    num_selected = self.lcUpgrades.GetSelectedItemCount()
    if (num_selected == 0):
      self.bDeleteUpgrade.Enable(False)
      self.bMarkXOR.Enable(False)
      self.bClearXOR.Enable(False)
    elif (num_selected == 1):
      self.bDeleteUpgrade.Enable(True)
      self.bMarkXOR.Enable(False)
      self.bClearXOR.Enable(False)
    else:
      self.bDeleteUpgrade.Enable(False)
      self.bMarkXOR.Enable(True)
      self.bClearXOR.Enable(True)
    return
  
  def OnDeleteUpgrade(self, event):
    if (self.selectUpgrade != None):
      self.upgradeArray.pop(self.selectUpgrade)
      self.UpdateUpgradesList()
      self.selectUpgrade = None
      self.bDeleteUpgrade.Enable(False)
    return
  
  def OnAddShip(self, event):
    message = ''
    if (self.txClassName.GetValue() == ''):
      message = "Need to input a class name.\n"
    if (self.cxFaction.GetStringSelection() not in dataStruct.factions):
      message += "Need to select a faction.\n"
    
    if (message == ''):
      self.EndModal(True)
    else:
      wx.MessageBox(message)
    return
  
  def OnCancel(self, event):
    self.EndModal(False)
    return
  
  def OnAddWeapon(self, event):
    weaponsDlg = WeaponsEditorBase(self)
    add_weapon = weaponsDlg.ShowModal()
    if (add_weapon):
      weapon = dataStruct.WeaponRecord()
      weapon.weaponName = weaponsDlg.txWeaponName.GetValue()
      weapon.arcDirection = weaponsDlg.cxArcDirection.GetStringSelection()
      weapon.arcRestriction = weaponsDlg.cxArcType.GetStringSelection()
      if (weaponsDlg.rbDirectWeapon.GetValue()):  #direct weapon
        weapon.category = 'Direct'
        weapon.weaponType = weaponsDlg.cxDirectWeaponType.GetStringSelection()
        if (weapon.weaponType == 'Primary'):
          weapon.rangeIncrement = 8
        elif (weapon.weaponType == 'Gun Rack'):
          weapon.rangeIncrement = 8
        elif (weapon.weaponType == 'Scatter'):
          weapon.rangeIncrement = 8
        elif (weapon.weaponType == 'Kinetic'):
          weapon.rangeIncrement = 12
        elif (weapon.weaponType == 'Beam'):
          weapon.rangeIncrement = 10
        elif (weapon.weaponType == 'Nuclear'):
          weapon.rangeIncrement = 8
        else:
          weapon.weaponType = 'Primary'
          weapon.rangeIncrement = 8
      else:  #indirect weapon
        weapon.category = 'Indirect'
        weapon.weaponType = weaponsDlg.cxIndirectWeaponType.GetStringSelection()
        if (weapon.weaponType == 'Gravitational'):
          weapon.rangeIncrement = 8
        elif (weapon.weaponType == 'Torpedo'):
          weapon.rangeIncrement = 12
        elif (weapon.weaponType == 'Cyberwarfare'):
          weapon.rangeIncrement = 10
      weapon.rb1 = weaponsDlg.spRangeBand1.GetValue()
      weapon.rb2 = weaponsDlg.spRangeBand2.GetValue()
      weapon.rb3 = weaponsDlg.spRangeBand3.GetValue()
      weapon.rb4 = weaponsDlg.spRangeBand4.GetValue()
      weapon.hardpoint = weaponsDlg.cbHardpointWeapon.GetValue()
      self.weaponsArray.append(weapon)
      self.UpdateWeaponsList()
      weaponsDlg.Destroy()
    return
  
  def UpdateWeaponsList(self):
    self.lcWeapons.DeleteAllItems()
    weapon = dataStruct.WeaponRecord()
    self.weaponsArray.reverse()
    for weapon in self.weaponsArray:
      listitem = wx.ListItem()
      listitem.SetText(weapon.weaponName)
      index = self.lcWeapons.InsertItem(listitem)
      if (weapon.hardpoint):
        self.lcWeapons.SetItemTextColour(index, wx.RED)
      else:
        self.lcWeapons.SetItemTextColour(index, wx.BLACK)
      self.lcWeapons.SetStringItem(index, 0, weapon.weaponName)
      self.lcWeapons.SetStringItem(index, 1, weapon.weaponType)
      self.lcWeapons.SetStringItem(index, 2, (weapon.arcDirection +" "+ weapon.arcRestriction))
      self.lcWeapons.SetStringItem(index, 3, str(weapon.rangeIncrement))
      self.lcWeapons.SetStringItem(index, 4, str(weapon.rb1))
      self.lcWeapons.SetStringItem(index, 5, str(weapon.rb2))
      self.lcWeapons.SetStringItem(index, 6, str(weapon.rb3))
      self.lcWeapons.SetStringItem(index, 7, str(weapon.rb4))

    self.lcWeapons.SetColumnWidth(0, wx.LIST_AUTOSIZE)
    if (self.lcWeapons.GetColumnWidth(0) < 100):
      self.lcWeapons.SetColumnWidth(0,100)
    self.lcWeapons.SetColumnWidth(1, wx.LIST_AUTOSIZE_USEHEADER)
    if (self.lcWeapons.GetColumnWidth(1) < 100):
      self.lcWeapons.SetColumnWidth(1,100)
    self.lcWeapons.SetColumnWidth(2, wx.LIST_AUTOSIZE)
    if (self.lcWeapons.GetColumnWidth(2) < 100):
      self.lcWeapons.SetColumnWidth(2,100)
    self.weaponsArray.reverse()
    return
  
  def OnEditWeapon(self, event):
    #weapon = dataStruct.WeaponRecord()
    weapon = self.weaponsArray[self.selectWeapon]
    self.weaponsArray.reverse()
    dlg  = WeaponsEditorBase(self, weapon)
    dlg.bAddWeapon.SetLabel("Save Changes")
    update = dlg.ShowModal()
    if (update):
      weapon.weaponName = dlg.txWeaponName.GetValue()
      weapon.arcDirection = dlg.cxArcDirection.GetStringSelection()
      weapon.arcRestriction = dlg.cxArcType.GetStringSelection()
      weapon.hardpoint = dlg.cbHardpointWeapon.GetValue()
      if (dlg.rbDirectWeapon.GetValue()):  #direct weapon
        weapon.category = 'Direct'
        weapon.weaponType = dlg.cxDirectWeaponType.GetStringSelection()
        if (weapon.weaponType == 'Primary'):
          weapon.rangeIncrement = 8
        elif (weapon.weaponType == 'Gun Rack'):
          weapon.rangeIncrement = 8
        elif (weapon.weaponType == 'Scatter'):
          weapon.rangeIncrement = 8
        elif (weapon.weaponType == 'Kinetic'):
          weapon.rangeIncrement = 12
        elif (weapon.weaponType == 'Beam'):
          weapon.rangeIncrement = 10
        elif (weapon.weaponType == 'Nuclear'):
          weapon.rangeIncrement = 8
        else:
          weapon.weaponType = 'Primary'
          weapon.rangeIncrement = 8
      else:  #indirect weapon
        weapon.category = 'Indirect'
        weapon.weaponType = dlg.cxIndirectWeaponType.GetStringSelection()
        if (weapon.weaponType == 'Gravitational'):
          weapon.rangeIncrement = 8
        elif (weapon.weaponType == 'Torpedo'):
          weapon.rangeIncrement = 12
        elif (weapon.weaponType == 'Cyberwarfare'):
          weapon.rangeIncrement = 10
      weapon.rb1 = dlg.spRangeBand1.GetValue()
      weapon.rb2 = dlg.spRangeBand2.GetValue()
      weapon.rb3 = dlg.spRangeBand3.GetValue()
      weapon.rb4 = dlg.spRangeBand4.GetValue()
      self.weaponsArray.reverse()
      self.UpdateWeaponsList()
    return
  
  def OnSecondaryCost(self, event):
    if self.cbSecondaryCost.IsChecked():
      self.spSecondaryCost.Enable(True)
      self.spSecondaryCost.SetValue((self.spBaseCost.GetValue()+10))
    else:
      self.spSecondaryCost.Enable(False)
    return
  
  def OnSelectAccomp(self, event):
    self.selectAccomp = event.m_itemIndex
    self.bDeleteAccompaniment.Enable(True)
    self.bEditAccomp.Enable(True)
    return
  
  def OnEditAccomp(self, event):
    dlg = AddAccompanimentBase(self, self.accompanimentArray[self.selectAccomp])
    changeAccomp = dlg.ShowModal()
    if (changeAccomp):
      self.accompanimentArray[self.selectAccomp] = copy.deepcopy(dlg.accompClassList[0])
    dlg.Destroy()
    self.UpdateAccompArray()
    return
  
  def OnDeleteHardpoint(self, event):
    if (self.selectHardpoint != None):
      self.hardpointArray.reverse()
      self.hardpointArray.pop(self.selectHardpoint)
      self.hardpointArray.reverse()
      self.UpdateHardpoints()
      self.bDeleteHardpoint.Enable(False)
      self.selectHardpoint = None
      return
  
  def OnDeleteAccompaniment(self, event):
    if (self.selectAccomp != None):
      self.accompanimentArray.pop(self.selectAccomp)
      self.UpdateAccompArray()
      self.bDeleteAccompaniment.Enable(False)
      #self.bEditAccomp.Enable(False)
      self.selectAccomp = None
    return
  
  def OnSelectWeapon(self, event):
    self.selectWeapon = event.m_itemIndex
    self.bDeleteWeapon.Enable(True)
    self.bEditWeapon.Enable(True)
    return
  
  def OnSelectMAR(self, event):
    self.selectMar = event.m_itemIndex
    self.bDeleteMar.Enable(True)
    return
  
  def OnDeleteWeapon(self, event):
    if (self.selectWeapon != None):
      self.weaponsArray.pop(self.selectWeapon)
      self.UpdateWeaponsList()
      self.selectWeapon = None
      self.bDeleteWeapon.Enable(False)
    return
  
  def OnDeleteMar(self, event):
    if (self.selectMar != None):
      self.marArray.pop(self.selectMar)
      self.UpdateMarList()
      self.selectMar = None
      self.bDeleteMar.Enable(False)
    return
  
  def UpdateAccompArray(self):
    self.lcAccompaniment.DeleteAllItems()
    escort = dataStruct.AccompanimentRecord()
    ship = dataStruct.ShipRecord()
    self.accompanimentArray.reverse()
    for escort in self.accompanimentArray:
      for ship in gv.ShipClassList:
        if (ship.index == escort.shipIndex):
          break
      listitem = wx.ListItem()
      listitem.SetText(str(escort.shipIndex))
      index = self.lcAccompaniment.InsertItem(listitem)
      numText = "0-"+str(escort.numAllowed)
      self.lcAccompaniment.SetStringItem(index, 0, numText)
      self.lcAccompaniment.SetStringItem(index, 1, str(ship.baseCost))
      self.lcAccompaniment.SetStringItem(index, 2, ship.className)
      self.lcAccompaniment.SetStringItem(index, 3, ship.faction)
      self.lcAccompaniment.SetStringItem(index, 4, ship.designation)
    self.lcAccompaniment.SetColumnWidth(0,wx.LIST_AUTOSIZE_USEHEADER)
    self.lcAccompaniment.SetColumnWidth(1,wx.LIST_AUTOSIZE_USEHEADER)
    self.lcAccompaniment.SetColumnWidth(2,wx.LIST_AUTOSIZE)
    self.lcAccompaniment.SetColumnWidth(3,wx.LIST_AUTOSIZE)
    self.lcAccompaniment.SetColumnWidth(4,wx.LIST_AUTOSIZE_USEHEADER)
    self.accompanimentArray.reverse()
    return
  
  def UpdateMarList(self):
    self.lcMarList.DeleteAllItems()
    mar = dataStruct.MarRecord()
    self.marArray.reverse()
    for mar in self.marArray:
      listitem = wx.ListItem()
      listitem.SetText(mar.marName)
      index = self.lcMarList.InsertItem(listitem)
      self.lcMarList.SetStringItem(index, 0, str(mar.marNumber))
      self.lcMarList.SetStringItem(index, 1, mar.marName)
      self.lcMarList.SetStringItem(index, 2, mar.marText)
    self.lcMarList.SetColumnWidth(0, wx.LIST_AUTOSIZE)
    self.lcMarList.SetColumnWidth(1, wx.LIST_AUTOSIZE)
    self.lcMarList.SetColumnWidth(2, wx.LIST_AUTOSIZE)
    self.marArray.reverse()
    return
  
  def UpdateHardpoints(self):
    self.lcHardpoints.DeleteAllItems()
    hardpoint = dataStruct.HardpointRecord()
    self.hardpointArray.reverse()
    for hardpoint in self.hardpointArray:
      listitem = wx.ListItem()
      listitem.SetText(hardpoint.description)
      index = self.lcHardpoints.InsertItem(listitem)
      availText = "0-"+str(hardpoint.numAvailable)
      self.lcHardpoints.SetStringItem(index, 0, availText)
      self.lcHardpoints.SetStringItem(index, 1, str(hardpoint.cost))
      self.lcHardpoints.SetStringItem(index, 2, hardpoint.description)
    self.lcHardpoints.SetColumnWidth(0, wx.LIST_AUTOSIZE_USEHEADER)
    self.lcHardpoints.SetColumnWidth(1, wx.LIST_AUTOSIZE_USEHEADER)
    self.lcHardpoints.SetColumnWidth(2, wx.LIST_AUTOSIZE)
    self.weaponsArray.reverse()
    return
  
  def OnChangeHardpoints(self, event):
    if (self.spHardpoints.GetValue() > 0):
      self.bAddHardpoint.Enable(True)
    else:
      self.bAddHardpoint.Enable(False)
      self.bDeleteHardpoint.Enable(False)
    return
  
  def OnAddAccompaniment(self, event):
    dlg = AddAccompanimentBase(self, faction=self.cxFaction.GetStringSelection())
    addAccomp = dlg.ShowModal()
    if (addAccomp != False):  #the return code may come back as a 0; ie, shipIndex = 0
      for item in dlg.accompClassList:
        self.accompanimentArray.append(item)
      self.UpdateAccompArray()
    dlg.Destroy()
    return
  
  def OnAddMar(self, event):
    dlg = SelectMarBase(self)
    addMAR = dlg.ShowModal()
    if (addMAR and dlg.selectedItem != None):
      self.marArray.append(dlg.marRecord)
    self.UpdateMarList()
    dlg.Destroy()
    return
  
  def OnAddHardpoint(self, event):
    """add to the dialog, and to ShipRecord::ApplyHardpoints"""
    dlg = AddHardpointBase(self)
    add_hardpoint = dlg.ShowModal()
    if (add_hardpoint):
      #get the selected information
      if (dlg.cbDecreaseTL.IsChecked()):
        hardpoint = dataStruct.HardpointRecord()
        hardpoint.intValue1 = dlg.spTL.GetValue()
        hardpoint.cost = dlg.cost
        hardpoint.index = 1
        hardpoint.description = "Decrease Turn Limit by "+str(hardpoint.intValue1)
        hardpoint.numAvailable = dlg.spNumAvailable.GetValue()
        self.hardpointArray.append(hardpoint)
      if (dlg.cbIncreaseWC.IsChecked()):
        hardpoint = dataStruct.HardpointRecord()
        hardpoint.intValue1 = dlg.spWC.GetValue()
        hardpoint.cost = dlg.cost
        hardpoint.index = 2
        hardpoint.description = "Increase Wing Capacity (WC) by "+str(hardpoint.intValue1)
        hardpoint.numAvailable = dlg.spNumAvailable.GetValue()
        self.hardpointArray.append(hardpoint)
      if (dlg.cbChangeWeaponArc.IsChecked()):
        hardpoint = dataStruct.HardpointRecord()
        hardpoint.cost = dlg.cost
        hardpoint.index = 3
        hardpoint.description = "Change torpedoes to forward arc"
        hardpoint.numAvailable = dlg.spNumAvailable.GetValue()
        self.hardpointArray.append(hardpoint)
      if (dlg.cbIncreaseMV.IsChecked()):
        hardpoint = dataStruct.HardpointRecord()
        hardpoint.cost = dlg.cost
        hardpoint.intValue1 = dlg.spMV.GetValue()
        hardpoint.index = 4
        hardpoint.description = "Increase Movement (MV) by "+str(hardpoint.intValue1)
        hardpoint.numAvailable = dlg.spNumAvailable.GetValue()
        self.hardpointArray.append(hardpoint)
      if (dlg.cbIncreaseSR.IsChecked()):
        hardpoint = dataStruct.HardpointRecord()
        hardpoint.cost = dlg.cost
        hardpoint.intValue1 = dlg.spSR.GetValue()
        hardpoint.index = 5
        hardpoint.description = "Increase Shields (SR) by "+str(hardpoint.intValue1)
        hardpoint.numAvailable = dlg.spNumAvailable.GetValue()
        self.hardpointArray.append(hardpoint)
      if (dlg.cbUpgradeTorpedo.IsChecked()):
        hardpoint = dataStruct.HardpointRecord()
        hardpoint.cost = dlg.cost
        hardpoint.index = 6
        hardpoint.description = "Upgrade to nuclear torpedoes"
        hardpoint.numAvailable = dlg.spNumAvailable.GetValue()
        self.hardpointArray.append(hardpoint)
      if (dlg.cbIncreaseHP.IsChecked()):
        hardpoint = dataStruct.HardpointRecord()
        hardpoint.cost = dlg.cost
        hardpoint.intValue1 = dlg.spHP.GetValue()
        hardpoint.index = 7
        hardpoint.description = "Increase Hit Points (HP) by "+str(hardpoint.intValue1)
        hardpoint.numAvailable = dlg.spNumAvailable.GetValue()
        self.hardpointArray.append(hardpoint)
      if (dlg.cbIncreaseMN.IsChecked()):
        hardpoint = dataStruct.HardpointRecord()
        hardpoint.cost = dlg.cost
        hardpoint.intValue1 = dlg.spMN.GetValue()
        hardpoint.index = 8
        hardpoint.description = "Increase Mines (MN) by "+str(hardpoint.intValue1)
        hardpoint.numAvailable = dlg.spNumAvailable.GetValue()
        self.hardpointArray.append(hardpoint)
      if (dlg.cbRemoveAblativeMar.IsChecked()):
        hardpoint = dataStruct.HardpointRecord()
        hardpoint.cost = dlg.cost
        hardpoint.index = 9
        hardpoint.description = "Remove the Ablative Plating MAR"
        hardpoint.numAvailable = dlg.spNumAvailable.GetValue()
        self.hardpointArray.append(hardpoint)
      if (dlg.cbGainDifficultTargetMAR.IsChecked()):
        hardpoint = dataStruct.HardpointRecord()
        hardpoint.cost = dlg.cost
        hardpoint.index = 10
        hardpoint.description = "Gain the Difficult Target MAR"
        hardpoint.numAvailable = dlg.spNumAvailable.GetValue()
        self.hardpointArray.append(hardpoint)
      if (dlg.cbIncreaseCommandDistance.IsChecked()):
        hardpoint = dataStruct.HardpointRecord()
        hardpoint.cost = dlg.cost
        hardpoint.intValue1 = dlg.spCMD.GetValue()
        hardpoint.index = 11
        hardpoint.description = "Increase Command Distance by "+str(hardpoint.intValue1)
        hardpoint.numAvailable = dlg.spNumAvailable.GetValue()
        self.hardpointArray.append(hardpoint)
      if (dlg.cbUpgradePrimaryWeapons.IsChecked()):
        hardpoint = dataStruct.HardpointRecord()
        hardpoint.cost = dlg.cost
        hardpoint.index = 12
        hardpoint.description = "Upgrade Primary Weapons to Beam Weapons"
        hardpoint.numAvailable = dlg.spNumAvailable.GetValue()
        self.hardpointArray.append(hardpoint)
      if (dlg.cbChangeFixedForeToScatter.IsChecked()):
        hardpoint = dataStruct.HardpointRecord()
        hardpoint.cost = dlg.cost
        hardpoint.index = 13
        hardpoint.description = "Change Fixed (Fore) weapons to Scatter Weapons"
        hardpoint.numAvailable = dlg.spNumAvailable.GetValue()
        self.hardpointArray.append(hardpoint)
      if (dlg.cbIncreasePD.IsChecked()):
        hardpoint = dataStruct.HardpointRecord()
        hardpoint.intValue1 = dlg.spPD.GetValue()
        hardpoint.cost = dlg.cost
        hardpoint.index = 14
        hardpoint.description = "Increase Point Defense (PD) by "+str(hardpoint.intValue1)
        hardpoint.numAvailable = dlg.spNumAvailable.GetValue()
        if (dlg.cbBiggerBatteriesPD.IsChecked()):
          hardpoint.marValue = 6
          hardpoint.description += ", Gain the Bigger Batteries MAR"
        self.hardpointArray.append(hardpoint)
      if (dlg.cbIncreaseAP.IsChecked()):
        hardpoint = dataStruct.HardpointRecord()
        hardpoint.intValue1 = dlg.spAP.GetValue()
        hardpoint.cost = dlg.cost
        hardpoint.index = 15
        hardpoint.description = "Increase Assault Points (AP) by "+str(hardpoint.intValue1)
        hardpoint.numAvailable = dlg.spNumAvailable.GetValue()
        if (dlg.cbAP_SecondAssaultMAR.IsChecked()):  #add the second assault mar
          hardpoint.marValue = 39
          hardpoint.description += "; Gain the Second Assault MAR"
        if (dlg.cbAP_SpecialForcesMAR.IsChecked()):  #add the special forces mar
          hardpoint.marValue = 45
          hardpoint.description += "; Gain the Special Forces MAR"
        self.hardpointArray.append(hardpoint)
      if (dlg.cbQuickLaunch.IsChecked()):
        hardpoint = dataStruct.HardpointRecord()
        hardpoint.cost = dlg.cost
        hardpoint.index = 16
        hardpoint.description = "Gain the Quick Launch MAR"
        hardpoint.numAvailable = dlg.spNumAvailable.GetValue()
        self.hardpointArray.append(hardpoint)
      if (dlg.cbChangeEnergyTransferBeam.IsChecked()):
        hardpoint = dataStruct.HardpointRecord()
        hardpoint.cost = dlg.cost
        hardpoint.index = 17
        hardpoint.description = "Replace the Energy Transfer (Beam,1) MAR with the Energy Transfer (Beam,2) MAR"
        hardpoint.numAvailable = dlg.spNumAvailable.GetValue()
        self.hardpointArray.append(hardpoint)
      if (dlg.cbGainMinefieldsMAR.IsChecked()):
        hardpoint = dataStruct.HardpointRecord()
        hardpoint.cost = dlg.cost
        hardpoint.index = 18
        hardpoint.description = "Gain the Minefields (4, 6 AD) MAR"
        hardpoint.numAvailable = dlg.spNumAvailable.GetValue()
        self.hardpointArray.append(hardpoint)
      if (dlg.cbIncreaseCP.IsChecked()):
        hardpoint = dataStruct.HardpointRecord()
        hardpoint.cost = dlg.cost
        hardpoint.intValue1 = dlg.spCP.GetValue()
        hardpoint.index = 19
        hardpoint.description = "Increase Crew Points (CP) by "+str(hardpoint.intValue1)
        hardpoint.numAvailable = dlg.spNumAvailable.GetValue()
        self.hardpointArray.append(hardpoint)
      if (dlg.cbGainShuntMatrix4.IsChecked()):
        hardpoint = dataStruct.HardpointRecord()
        hardpoint.cost = dlg.cost
        hardpoint.index = 20
        hardpoint.description = "Gain the Shunt Matrix (4) MAR"
        hardpoint.numAvailable = dlg.spNumAvailable.GetValue()
        self.hardpointArray.append(hardpoint)
      if (dlg.cbGainOpsCenterMar.IsChecked()):
        hardpoint = dataStruct.HardpointRecord()
        hardpoint.cost = dlg.cost
        hardpoint.index = 21
        hardpoint.description = "Gain the Ops Center MAR"
        hardpoint.numAvailable = dlg.spNumAvailable.GetValue()
        self.hardpointArray.append(hardpoint)
      if (dlg.cbGainDeckCrewsMar.IsChecked()):
        hardpoint = dataStruct.HardpointRecord()
        hardpoint.cost = dlg.cost
        hardpoint.index = 22
        hardpoint.description = "Gain the Deck Crews MAR"
        hardpoint.numAvailable = dlg.spNumAvailable.GetValue()
        self.hardpointArray.append(hardpoint)
      if (dlg.cbGainTorpedoSpookMar.IsChecked()):
        hardpoint = dataStruct.HardpointRecord()
        hardpoint.cost = dlg.cost
        hardpoint.index = 23
        hardpoint.description = "Gain the Torpedo Spook MAR"
        hardpoint.numAvailable = dlg.spNumAvailable.GetValue()
        self.hardpointArray.append(hardpoint)
      if (dlg.cbGainSpecialForcesMar.IsChecked()):
        hardpoint = dataStruct.HardpointRecord()
        hardpoint.cost = dlg.cost
        hardpoint.index = 24
        hardpoint.description = "Gain the Special Forces MAR"
        hardpoint.numAvailable = dlg.spNumAvailable.GetValue()
        self.hardpointArray.append(hardpoint)
      if (dlg.cbSectorShieldingMar.IsChecked()):
        hardpoint = dataStruct.HardpointRecord()
        hardpoint.cost = dlg.cost
        hardpoint.index = 25
        hardpoint.description = "Gain the Sector Shielding MAR"
        hardpoint.numAvailable = dlg.spNumAvailable.GetValue()
        self.hardpointArray.append(hardpoint)
      if (dlg.cbShieldProjectorMAR.IsChecked()):
        hardpoint = dataStruct.HardpointRecord()
        hardpoint.cost = dlg.cost
        hardpoint.intValue1 = 0
        hardpoint.index = 26
        hardpoint.description = "Gain the Shield Projector (Self) MAR"
        hardpoint.numAvailable = dlg.spNumAvailable.GetValue()
        self.hardpointArray.append(hardpoint)
      if (dlg.cbShieldProjector8MAR.IsChecked()):
        hardpoint = dataStruct.HardpointRecord()
        hardpoint.cost = dlg.cost
        hardpoint.intValue1 = 8
        hardpoint.index = 27
        hardpoint.description = "Gain the Shield Projector (8) MAR"
        hardpoint.numAvailable = dlg.spNumAvailable.GetValue()
        self.hardpointArray.append(hardpoint)
      if (dlg.cbCountermeasuresMar.IsChecked()):
        hardpoint = dataStruct.HardpointRecord()
        hardpoint.cost = dlg.cost
        hardpoint.index = 28
        hardpoint.description = "Gain the Countermeasures MAR"
        hardpoint.numAvailable = dlg.spNumAvailable.GetValue()
        self.hardpointArray.append(hardpoint)
      if (dlg.cbDecimatorWarheads.IsChecked()):
        hardpoint = dataStruct.HardpointRecord()
        hardpoint.cost = dlg.cost
        hardpoint.index = 29
        hardpoint.description = "All torpedos gain the Decimator Warheads MAR"
        hardpoint.numAvailable = dlg.spNumAvailable.GetValue()
        self.hardpointArray.append(hardpoint)
      if (dlg.cbIncludeCyberwarfareWeapons.IsChecked()):
        hardpoint = dataStruct.HardpointRecord()
        hardpoint.cost = dlg.cost
        hardpoint.index = 30
        hardpoint.description = "Include the Cyberwarfare Weapons"
        hardpoint.numAvailable = dlg.spNumAvailable.GetValue()
        hardpoint.strValue1 = 'Cyberwarfare'
        self.hardpointArray.append(hardpoint)
      if (dlg.cbGainSelfRepairMAR.IsChecked()):
        hardpoint = dataStruct.HardpointRecord()
        hardpoint.cost = dlg.cost
        hardpoint.index = 31
        hardpoint.description = "Gain the Self Repair MAR"
        hardpoint.numAvailable = dlg.spNumAvailable.GetValue()
        self.hardpointArray.append(hardpoint)
      if (dlg.cbIncreaseCR.IsChecked()):
        hardpoint = dataStruct.HardpointRecord()
        hardpoint.intValue1 = dlg.spCR.GetValue()
        hardpoint.cost = dlg.cost
        hardpoint.index = 32
        hardpoint.description = "Increase Critical Rating (CR) by "+str(hardpoint.intValue1)
        hardpoint.numAvailable = dlg.spNumAvailable.GetValue()
        self.hardpointArray.append(hardpoint)
      if (dlg.cbGainSecondAssaultMAR.IsChecked()):
        hardpoint = dataStruct.HardpointRecord()
        hardpoint.cost = dlg.cost
        hardpoint.index = 33
        hardpoint.description = "Gain the Second Assault MAR"
        hardpoint.numAvailable = dlg.spNumAvailable.GetValue()
        self.hardpointArray.append(hardpoint)
      if (dlg.cbLaunchTubesMar.IsChecked()):
        hardpoint = dataStruct.HardpointRecord()
        hardpoint.cost = dlg.cost
        hardpoint.index = 34
        hardpoint.description = "Gain the Launch Tubes MAR"
        hardpoint.numAvailable = dlg.spNumAvailable.GetValue()
        self.hardpointArray.append(hardpoint)
      if (dlg.cbRemoveVulnerable.IsChecked()):
        hardpoint = dataStruct.HardpointRecord()
        hardpoint.cost = dlg.cost
        hardpoint.index = 35
        hardpoint.description = "Remove the Vulnerable MAR"
        hardpoint.numAvailable = dlg.spNumAvailable.GetValue()
        self.hardpointArray.append(hardpoint)
      if (dlg.cbAmbushMAR.IsChecked()):
        hardpoint = dataStruct.HardpointRecord()
        hardpoint.cost = dlg.cost
        hardpoint.index = 36
        hardpoint.description = "Gain the Ambush (1) MAR"
        hardpoint.numAvailable = dlg.spNumAvailable.GetValue()
        self.hardpointArray.append(hardpoint)
      if (dlg.cbTurretsSplitFire.IsChecked()):
        hardpoint = dataStruct.HardpointRecord()
        hardpoint.cost = dlg.cost
        hardpoint.index = 37
        hardpoint.description = "Give the Turrets the Split Fire MAR"
        hardpoint.numAvailable = dlg.spNumAvailable.GetValue()
        self.hardpointArray.append(hardpoint)
      if (dlg.cbGainStealthSystemsAndReduceSR.IsChecked()):
        hardpoint = dataStruct.HardpointRecord()
        hardpoint.cost = dlg.cost
        hardpoint.index = 38
        hardpoint.description = "Gain the Stealth Systems MAR and Reduce SR to 1"
        hardpoint.numAvailable = dlg.spNumAvailable.GetValue()
        self.hardpointArray.append(hardpoint)
      if (dlg.cbGainDirtySecrets.IsChecked()):
        hardpoint = dataStruct.HardpointRecord()
        hardpoint.cost = dlg.cost
        hardpoint.index = 39
        hardpoint.description = "Gain the Dirty Secrets MAR"
        hardpoint.numAvailable = dlg.spNumAvailable.GetValue()
        self.hardpointArray.append(hardpoint)
      if (dlg.cbBeamForeToCyberwarfare.IsChecked()):
        hardpoint = dataStruct.HardpointRecord()
        hardpoint.cost = dlg.cost
        hardpoint.index = 40
        hardpoint.description = "Change Beam (Fore) to a Cyberwarfare weapon"
        hardpoint.numAvailable = dlg.spNumAvailable.GetValue()
        self.hardpointArray.append(hardpoint)
      if (dlg.cbStealthSystemsToHiddenKillerMAR.IsChecked()):
        hardpoint = dataStruct.HardpointRecord()
        hardpoint.cost = dlg.cost
        hardpoint.index = 41
        hardpoint.description = "Replace the Stealth Systems MAR with the Hidden Killer MAR"
        hardpoint.numAvailable = dlg.spNumAvailable.GetValue()
        self.hardpointArray.append(hardpoint)
      if (dlg.cbBeamToCyberwarfare.IsChecked()):
        hardpoint = dataStruct.HardpointRecord()
        hardpoint.cost = dlg.cost
        hardpoint.index = 42
        hardpoint.description = "Change Beam weapons to Cyberwarfare weapons"
        hardpoint.numAvailable = dlg.spNumAvailable.GetValue()
        self.hardpointArray.append(hardpoint)
      if (dlg.cbTorpedoesGainCorrosiveMAR.IsChecked()):
        hardpoint = dataStruct.HardpointRecord()
        hardpoint.cost = dlg.cost
        hardpoint.index = 43
        hardpoint.description = "Torpedoes gain the Corrosive Ammo MAR"
        hardpoint.numAvailable = dlg.spNumAvailable.GetValue()
        self.hardpointArray.append(hardpoint)
      if (dlg.cbTorpedoesGainBiohazard.IsChecked()):
        hardpoint = dataStruct.HardpointRecord()
        hardpoint.cost = dlg.cost
        hardpoint.index = 44
        hardpoint.description = "Torpedoes gain the Biohazard Ammo MAR"
        hardpoint.numAvailable = dlg.spNumAvailable.GetValue()
        self.hardpointArray.append(hardpoint)
      if (dlg.cbTakeBeams.IsChecked()):
        hardpoint = dataStruct.HardpointRecord()
        hardpoint.cost = dlg.cost
        hardpoint.index = 45
        hardpoint.description = "Take the Beam Weapons"
        hardpoint.strValue1 = 'Beam'
        hardpoint.numAvailable = dlg.spNumAvailable.GetValue()
        self.hardpointArray.append(hardpoint)
      if (dlg.cbTakeCyberwarfare.IsChecked()):
        hardpoint = dataStruct.HardpointRecord()
        hardpoint.cost = dlg.cost
        hardpoint.index = 46
        hardpoint.description = "Take the Cyberwarfare Weapons"
        hardpoint.strValue1 = 'Cyberwarfare'
        hardpoint.numAvailable = dlg.spNumAvailable.GetValue()
        self.hardpointArray.append(hardpoint)
      if (dlg.cbTakeGravity.IsChecked()):
        hardpoint = dataStruct.HardpointRecord()
        hardpoint.cost = dlg.cost
        hardpoint.index = 47
        hardpoint.description = "Take the Gravity Weapons and Compressor Beam MAR"
        hardpoint.strValue1 = "Gravitational"
        hardpoint.numAvailable = dlg.spNumAvailable.GetValue()
        self.hardpointArray.append(hardpoint)
      if (dlg.cbCyberwarfareAssaultRobot.IsChecked()):
        hardpoint = dataStruct.HardpointRecord()
        hardpoint.cost = dlg.cost
        hardpoint.index = 48
        hardpoint.description = "Take the Cyberwarfare Weapons with the Assault Robot Torpedoes MAR"
        hardpoint.strValue1 = "Cyberwarfare"
        hardpoint.numAvailable = dlg.spNumAvailable.GetValue()
        self.hardpointArray.append(hardpoint)
      if (dlg.cbIncludeTorpedoWeapons.IsChecked()):
        hardpoint = dataStruct.HardpointRecord()
        hardpoint.cost = dlg.cost
        hardpoint.index = 49
        hardpoint.description = "Take the Torpedo Weapons"
        hardpoint.strValue1 = 'Torpedo'
        hardpoint.numAvailable = dlg.spNumAvailable.GetValue()
        self.hardpointArray.append(hardpoint)
      if (dlg.cbGainWeaponShielding.IsChecked()):
        hardpoint = dataStruct.HardpointRecord()
        hardpoint.cost = dlg.cost
        hardpoint.index = 50
        hardpoint.description = "Gain the Weapon Shielding MAR"
        hardpoint.numAvailable = dlg.spNumAvailable.GetValue()
        self.hardpointArray.append(hardpoint)
      if (dlg.cbGainPackHuntersMAR.IsChecked()):
        hardpoint = dataStruct.HardpointRecord()
        hardpoint.cost = dlg.cost
        hardpoint.index = 51
        hardpoint.description = "Gain the Pack Hunters MAR"
        hardpoint.numAvailable = dlg.spNumAvailable.GetValue()
        self.hardpointArray.append(hardpoint)
      if (dlg.cbBeamsHighEnergyMAR.IsChecked()):
        hardpoint = dataStruct.HardpointRecord()
        hardpoint.cost = dlg.cost
        hardpoint.index = 52
        hardpoint.description = "Give the Beam Weapons the High Energy MAR"
        hardpoint.numAvailable = dlg.spNumAvailable.GetValue()
        self.hardpointArray.append(hardpoint)
      if (dlg.cbPointDefenseMAR.IsChecked()):
        hardpoint = dataStruct.HardpointRecord()
        hardpoint.cost = dlg.cost
        hardpoint.index = 53
        hardpoint.description = "Gain the Point Defense Barrage MAR"
        hardpoint.numAvailable = dlg.spNumAvailable.GetValue()
        self.hardpointArray.append(hardpoint)
      if (dlg.cbGainShuntMatrix2.IsChecked()):
        hardpoint = dataStruct.HardpointRecord()
        hardpoint.cost = dlg.cost
        hardpoint.index = 54
        hardpoint.description = "Gain the Shunt Matrix (2) MAR"
        hardpoint.numAvailable = dlg.spNumAvailable.GetValue()
        self.hardpointArray.append(hardpoint)
      if (dlg.cbAmbush23.IsChecked()):
        hardpoint = dataStruct.HardpointRecord()
        hardpoint.cost = dlg.cost
        hardpoint.index = 55
        hardpoint.description = "Replace the Ambush(2) MAR with the Ambush (3) MAR"
        hardpoint.numAvailable = dlg.spNumAvailable.GetValue()
        self.hardpointArray.append(hardpoint)
      if (dlg.cbRemoveNoFSD.IsChecked()):
        hardpoint = dataStruct.HardpointRecord()
        hardpoint.cost = dlg.cost
        hardpoint.index = 56
        hardpoint.description = "Remove the No FSD MAR"
        hardpoint.numAvailable = dlg.spNumAvailable.GetValue()
        self.hardpointArray.append(hardpoint)
      if (dlg.cbGainStealthSystems.IsChecked()):
        hardpoint = dataStruct.HardpointRecord()
        hardpoint.cost = dlg.cost
        hardpoint.index = 57
        hardpoint.description = "Gain the Stealth Systems MAR"
        hardpoint.numAvailable = dlg.spNumAvailable.GetValue()
        self.hardpointArray.append(hardpoint)
      if (dlg.cbShield1Move1.IsChecked()):
        hardpoint = dataStruct.HardpointRecord()
        hardpoint.cost = dlg.cost
        hardpoint.index = 58
        hardpoint.description = "Gain +1 Shield, -1 Move"
        hardpoint.numAvailable = dlg.spNumAvailable.GetValue()
        self.hardpointArray.append(hardpoint)
      if (dlg.cbGainDifficult1HP.IsChecked()):
        hardpoint = dataStruct.HardpointRecord()
        hardpoint.cost = dlg.cost
        hardpoint.index = 59
        hardpoint.description = "Gain the Difficult Target MAR, -1 HP"
        hardpoint.numAvailable = dlg.spNumAvailable.GetValue()
        self.hardpointArray.append(hardpoint)
      if (dlg.cbWing3NoPlanetfall.IsChecked()):
        hardpoint = dataStruct.HardpointRecord()
        hardpoint.cost = dlg.cost
        hardpoint.index = 60
        hardpoint.description = "Gain +3 Wings, Remove the Planetfall MAR"
        hardpoint.numAvailable = dlg.spNumAvailable.GetValue()
        self.hardpointArray.append(hardpoint)
      if (dlg.cbGainPlanetfall1LoseDurable.IsChecked()):
        hardpoint = dataStruct.HardpointRecord()
        hardpoint.cost = dlg.cost
        hardpoint.index = 61
        hardpoint.description = "Gain +1 Planetfall value, Remove the Durable MAR"
        hardpoint.numAvailable = dlg.spNumAvailable.GetValue()
        self.hardpointArray.append(hardpoint)
      if (dlg.cbGainDifficultRemoveDurable.IsChecked()):
        hardpoint = dataStruct.HardpointRecord()
        hardpoint.cost = dlg.cost
        hardpoint.index = 62
        hardpoint.description = "Gain the Difficult Target MAR, Remove the Durable MAR"
        hardpoint.numAvailable = dlg.spNumAvailable.GetValue()
        self.hardpointArray.append(hardpoint)
      if (dlg.cbMine2RemovePlanetfall.IsChecked()):
        hardpoint = dataStruct.HardpointRecord()
        hardpoint.cost = dlg.cost
        hardpoint.index = 63
        hardpoint.description = "Gain +2 Mines, Remove the Planetfall MAR"
        hardpoint.numAvailable = dlg.spNumAvailable.GetValue()
        self.hardpointArray.append(hardpoint)
      if (dlg.cbMineDrone2RemovePlanetfall.IsChecked()):
        hardpoint = dataStruct.HardpointRecord()
        hardpoint.cost = dlg.cost
        hardpoint.index = 64
        hardpoint.description = "Gain +2 Mines and the Drone Mines MAR, Remove the Planetfall MAR"
        hardpoint.numAvailable = dlg.spNumAvailable.GetValue()
        self.hardpointArray.append(hardpoint)
      if (dlg.cbReinforcedFore.IsChecked()):
        hardpoint = dataStruct.HardpointRecord()
        hardpoint.cost = dlg.cost
        hardpoint.index = 65
        hardpoint.description = "Gain the Reinforced (Fore) MAR"
        hardpoint.numAvailable = dlg.spNumAvailable.GetValue()
        self.hardpointArray.append(hardpoint)
      if (dlg.cbShield1HP1.IsChecked()):
        hardpoint = dataStruct.HardpointRecord()
        hardpoint.cost = dlg.cost
        hardpoint.index = 66
        hardpoint.description = "Gain +1 SR, +1 HP"
        hardpoint.numAvailable = dlg.spNumAvailable.GetValue()
        self.hardpointArray.append(hardpoint)
      if (dlg.cbEnergyTransfer1.IsChecked()):
        hardpoint = dataStruct.HardpointRecord()
        hardpoint.cost = dlg.cost
        hardpoint.index = 67
        hardpoint.description = "Gain the Energy Transfer (1) MAR"
        hardpoint.numAvailable = dlg.spNumAvailable.GetValue()
        self.hardpointArray.append(hardpoint)
      if (dlg.cbManeoverable.IsChecked()):
        hardpoint = dataStruct.HardpointRecord()
        hardpoint.cost = dlg.cost
        hardpoint.index = 68
        hardpoint.description = "Gain the Maneoverable MAR"
        hardpoint.numAvailable = dlg.spNumAvailable.GetValue()
        self.hardpointArray.append(hardpoint)
      if (dlg.cbForeScatterForeKinetic.IsChecked()):
        hardpoint = dataStruct.HardpointRecord()
        hardpoint.cost = dlg.cost
        hardpoint.index = 69
        hardpoint.description = "Change the Fore Scatter Weapons to Fore Fixed Kinetics"
        hardpoint.numAvailable = dlg.spNumAvailable.GetValue()
        self.hardpointArray.append(hardpoint)
      if (dlg.cbReinforcedPortStarboard.IsChecked()):
        hardpoint = dataStruct.HardpointRecord()
        hardpoint.cost = dlg.cost
        hardpoint.index = 70
        hardpoint.description = "Gain the Reinforced (Port/Starboard) MAR"
        hardpoint.numAvailable = dlg.spNumAvailable.GetValue()
        self.hardpointArray.append(hardpoint)
      if (dlg.cbSecondAssaultNoPlanetfall.IsChecked()):
        hardpoint = dataStruct.HardpointRecord()
        hardpoint.cost = dlg.cost
        hardpoint.index = 71
        hardpoint.description = "Gain the Second Assault MAR, Remove the Planetfall MAR"
        hardpoint.numAvailable = dlg.spNumAvailable.GetValue()
        self.hardpointArray.append(hardpoint)
      if (dlg.cbPointBarrageNoPlanetfall.IsChecked()):
        hardpoint = dataStruct.HardpointRecord()
        hardpoint.cost = dlg.cost
        hardpoint.index = 72
        hardpoint.description = "Gain the Point Defense Barrage MAR, Remove the Planetfall MAR"
        hardpoint.numAvailable = dlg.spNumAvailable.GetValue()
        self.hardpointArray.append(hardpoint)
      if (dlg.cbHP1NoDurable.IsChecked()):
        hardpoint = dataStruct.HardpointRecord()
        hardpoint.cost = dlg.cost
        hardpoint.index = 73
        hardpoint.description = "Gain +1 HP, Remove the Durable MAR"
        hardpoint.numAvailable = dlg.spNumAvailable.GetValue()
        self.hardpointArray.append(hardpoint)
      if (dlg.cbNuclearTurretsNoPlanetfall.IsChecked()):
        hardpoint = dataStruct.HardpointRecord()
        hardpoint.cost = dlg.cost
        hardpoint.index = 74
        hardpoint.description = "Upgrade turrets to Nuclear weapons, remove the Planetfall MAR"
        hardpoint.numAvailable = dlg.spNumAvailable.GetValue()
        self.hardpointArray.append(hardpoint)
      if (dlg.cbPlanetfall1WC2.IsChecked()):
        hardpoint = dataStruct.HardpointRecord()
        hardpoint.cost = dlg.cost
        hardpoint.index = 75
        hardpoint.description = "Gain +1 Planetfall value, -2 Wings"
        hardpoint.numAvailable = dlg.spNumAvailable.GetValue()
        self.hardpointArray.append(hardpoint)
      if (dlg.cbAP3NoPlanetfall.IsChecked()):
        hardpoint = dataStruct.HardpointRecord()
        hardpoint.cost = dlg.cost
        hardpoint.index = 76
        hardpoint.description = "Gain +3 AP, Remove the Planetfall MAR"
        hardpoint.numAvailable = dlg.spNumAvailable.GetValue()
        self.hardpointArray.append(hardpoint)
      if (dlg.cbGainTurrets.IsChecked()):
        hardpoint = dataStruct.HardpointRecord()
        hardpoint.cost = dlg.cost
        hardpoint.index = 77
        hardpoint.description = "Gain the Turret weapons"
        hardpoint.numAvailable = dlg.spNumAvailable.GetValue()
        self.hardpointArray.append(hardpoint)
      if (dlg.cbShield1ShieldProjector.IsChecked()):
        hardpoint = dataStruct.HardpointRecord()
        hardpoint.cost = dlg.cost
        hardpoint.index = 78
        hardpoint.description = "Gain +1 Shield with the Shield Projector (0) MAR"
        hardpoint.numAvailable = dlg.spNumAvailable.GetValue()
        self.hardpointArray.append(hardpoint)
      if (dlg.cbEscortNoPlanetfall.IsChecked()):
        hardpoint = dataStruct.HardpointRecord()
        hardpoint.cost = dlg.cost
        hardpoint.index = 79
        hardpoint.description = "Change the designation to Escort and remove the Planetfall MAR"
        hardpoint.numAvailable = dlg.spNumAvailable.GetValue()
        self.hardpointArray.append(hardpoint)
      if (dlg.cbPrimaryScatter.IsChecked()):
        hardpoint = dataStruct.HardpointRecord()
        hardpoint.cost = dlg.cost
        hardpoint.index = 80
        hardpoint.description = "Change Primary Weapons to Scatter Weapons"
        hardpoint.numAvailable = dlg.spNumAvailable.GetValue()
        self.hardpointArray.append(hardpoint)
      if (dlg.cbRemoveAblativeDurable.IsChecked()):
        hardpoint = dataStruct.HardpointRecord()
        hardpoint.cost = dlg.cost
        hardpoint.index = 81
        hardpoint.description = "Remove the Ablative Plating and Durable MARs"
        hardpoint.numAvailable = dlg.spNumAvailable.GetValue()
        self.hardpointArray.append(hardpoint)
      if (dlg.cbPlanetfall1WC3.IsChecked()):
        hardpoint = dataStruct.HardpointRecord()
        hardpoint.cost = dlg.cost
        hardpoint.index = 82
        hardpoint.description = "Gain +1 Planetfall, -3 Wings"
        hardpoint.numAvailable = dlg.spNumAvailable.GetValue()
        self.hardpointArray.append(hardpoint)
      if (dlg.cbPlanetfall1AP3.IsChecked()):
        hardpoint = dataStruct.HardpointRecord()
        hardpoint.cost = dlg.cost
        hardpoint.index = 83
        hardpoint.description = "Gain +1 Planetfall, -3 AP"
        hardpoint.numAvailable = dlg.spNumAvailable.GetValue()
        self.hardpointArray.append(hardpoint)
      if (dlg.cbPlanetfall1Shield1.IsChecked()):
        hardpoint = dataStruct.HardpointRecord()
        hardpoint.cost = dlg.cost
        hardpoint.index = 84
        hardpoint.description = "Gain +1 Planetfall, -1 SR"
        hardpoint.numAvailable = dlg.spNumAvailable.GetValue()
        self.hardpointArray.append(hardpoint)
      if (dlg.cbAP2NoPlanetfall.IsChecked()):
        hardpoint = dataStruct.HardpointRecord()
        hardpoint.cost = dlg.cost
        hardpoint.index = 85
        hardpoint.description = "Gain +2 AP, -1 Planetfall"
        hardpoint.numAvailable = dlg.spNumAvailable.GetValue()
        self.hardpointArray.append(hardpoint)
      if (dlg.cbAP6NoPlanetfall.IsChecked()):
        hardpoint = dataStruct.HardpointRecord()
        hardpoint.cost = dlg.cost
        hardpoint.index = 86
        hardpoint.description = "Gain +6 AP, -1 Planetfall"
        hardpoint.numAvailable = dlg.spNumAvailable.GetValue()
        self.hardpointArray.append(hardpoint)
      if (dlg.cbBiggerBatteriesNoPlanetfall.IsChecked()):
        hardpoint = dataStruct.HardpointRecord()
        hardpoint.cost = dlg.cost
        hardpoint.index = 87
        hardpoint.description = "Gain the Bigger Batteries MAR, remove the Planetfall MAR"
        hardpoint.numAvailable = dlg.spNumAvailable.GetValue()
        self.hardpointArray.append(hardpoint)
      if (dlg.cbImprovedShunt3to4.IsChecked()):
        hardpoint = dataStruct.HardpointRecord()
        hardpoint.cost = dlg.cost
        hardpoint.index = 88
        hardpoint.description = "Upgrade the Improved Shunt Matrix from (3) to (4)"
        hardpoint.numAvailable = dlg.spNumAvailable.GetValue()
        self.hardpointArray.append(hardpoint)
      if (dlg.cbWC4QuickLaunchNoPlanetfall.IsChecked()):
        hardpoint = dataStruct.HardpointRecord()
        hardpoint.cost = dlg.cost
        hardpoint.index = 89
        hardpoint.description = "Gain +4 WC and the Quick Launch MAR, remove the Planetfall MAR"
        hardpoint.numAvailable = dlg.spNumAvailable.GetValue()
        self.hardpointArray.append(hardpoint)
      if (dlg.cbCloakNoPlanetfall.IsChecked()):
        hardpoint = dataStruct.HardpointRecord()
        hardpoint.cost = dlg.cost
        hardpoint.index = 91
        hardpoint.description = "Gain the Cloaking Field MAR, and remove the Planetfall MAR"
        hardpoint.numAvailable = dlg.spNumAvailable.GetValue()
        self.hardpointArray.append(hardpoint)
      if (dlg.cbMine3Move2.IsChecked()):
        hardpoint = dataStruct.HardpointRecord()
        hardpoint.cost = dlg.cost
        hardpoint.index = 92
        hardpoint.description = "Gain +3 Mines, -2 Move"
        hardpoint.numAvailable = dlg.spNumAvailable.GetValue()
        self.hardpointArray.append(hardpoint)
      if (dlg.cbAP1CP1.IsChecked()):
        hardpoint = dataStruct.HardpointRecord()
        hardpoint.cost = dlg.cost
        hardpoint.index = 93
        hardpoint.description = "Gain +1 AP, -1 CP"
        hardpoint.numAvailable = dlg.spNumAvailable.GetValue()
        self.hardpointArray.append(hardpoint)
      if (dlg.cbMove2LoseDurable1.IsChecked()):
        hardpoint = dataStruct.HardpointRecord()
        hardpoint.cost = dlg.cost
        hardpoint.index = 94
        hardpoint.description = "Gain +2 Movement, remove the Durable MAR"
        hardpoint.numAvailable = dlg.spNumAvailable.GetValue()
        self.hardpointArray.append(hardpoint)
      if (dlg.cbTorpedoNoPlanetfall.IsChecked()):
        hardpoint = dataStruct.HardpointRecord()
        hardpoint.cost = dlg.cost
        hardpoint.index = 95
        hardpoint.description = "Gain the Torpedo weapons, remove the Planetfall MAR"
        hardpoint.numAvailable = dlg.spNumAvailable.GetValue()
        self.hardpointArray.append(hardpoint)
      if (dlg.cbPlanetfall1PD2.IsChecked()):
        hardpoint = dataStruct.HardpointRecord()
        hardpoint.cost = dlg.cost
        hardpoint.index = 96
        hardpoint.description = "Gain +1 Planetfall, -2 PD"
        hardpoint.numAvailable = dlg.spNumAvailable.GetValue()
        self.hardpointArray.append(hardpoint)
      if (dlg.cbShield1NoDurable.IsChecked()):
        hardpoint = dataStruct.HardpointRecord()
        hardpoint.cost = dlg.cost
        hardpoint.index = 97
        hardpoint.description = "Gain +1 Shield, Remove the Durable MAR"
        hardpoint.numAvailable = dlg.spNumAvailable.GetValue()
        self.hardpointArray.append(hardpoint)
      if (dlg.cbTakeTorpedoesWithBiohazard.IsChecked()):
        hardpoint = dataStruct.HardpointRecord()
        hardpoint.cost = dlg.cost
        hardpoint.index = 98
        hardpoint.description = "Take the Torpedo weapons, with the Biohazard Ammo MAR"
        hardpoint.numAvailable = dlg.spNumAvailable.GetValue()
        self.hardpointArray.append(hardpoint)
      if (dlg.cbScout.IsChecked()):
        hardpoint = dataStruct.HardpointRecord()
        hardpoint.cost = dlg.cost
        hardpoint.index = 99
        hardpoint.description = "Gain the Scout MAR"
        hardpoint.numAvailable = dlg.spNumAvailable.GetValue()
        self.hardpointArray.append(hardpoint)
        
        
#also edit AddHardpointBase::__init__, ShipRecord::ApplyHardpoints(), FleetEditorBase::RemoveHardpoint, FleetEditorBase::AddHardpoint
    self.UpdateHardpoints()
    dlg.Destroy()
    return
#------------------------------------------------------------------------------
########################################################################
class AddHardpointBase(FirestormDialogs.AddNewHardpointCore):
  """Add hardpoints to the ship class editor
     Also add new hardpoints to ShipRecord::ApplyHardpoints
  """

  #----------------------------------------------------------------------
  def __init__(self, parent, hpoint=None):
    """Constructor"""
    hpoint = dataStruct.HardpointRecord()
    folder = os.getcwd()
    os.chdir(gv.root_folder)
    FirestormDialogs.AddNewHardpointCore.__init__(self,parent)
    os.chdir(folder)
    self.cost = 5
    
    if (hpoint):
      if (hpoint.index == 1):  #decrease TL by value1
        self.cbDecreaseTL.SetValue(True)
        self.spTL.Enable(True)
        self.spTL.SetValue(hpoint.intValue1)
      elif (hpoint.index == 2):  #increase WC by value1
        self.cbIncreaseWC.SetValue(True)
        self.spWC.Enable(True)
        self.spWC.SetValue(hpoint.intValue1)
      elif (hpoint.index == 3):  #change torpedoes to forward arc
        self.cbChangeWeaponArc.SetValue(True)
      elif (hpoint.index == 4):  #increase MV by value1
        self.cbIncreaseMV.SetValue(True)
        self.spMV.Enable(True)
        self.spMV.SetValue(hpoint.intValue1)
      elif (hpoint.index == 5):  #increase shields
        self.cbIncreaseSR.SetValue(True)
        self.spSR.Enable(True)
        self.spSR.SetValue(hpoint.intValue1)
      elif (hpoint.index == 6):    #upgrade to nuclear torpedoes
        self.cbUpgradeTorpedo.SetValue(True)
      elif (hpoint.index == 7):    #increase HP
        self.cbIncreaseHP.SetValue(True)
        self.spHP.Enable(True)
        self.spHP.SetValue(hpoint.intValue1)
      elif (hpoint.index == 8):    #increase mines
        self.cbIncreaseMN.SetValue(True)
        self.spMN.Enable(True)
        self.spMN.SetValue(hpoint.intValue1)
      elif (hpoint.index == 9):    #Remove Ablative Plating MAR
        self.cbRemoveAblativeMar.SetValue(True)
      elif (hpoint.index == 10):   #Gain Difficult Target MAR
        self.cbGainDifficultTargetMAR.SetValue(True)
      elif (hpoint.index == 11):   #increase command distance
        self.cbIncreaseCommandDistance.SetValue(True)
        self.spCMD.Enable(True)
        self.spCMD.SetValue(hpoint.intValue1)
      elif (hpoint.index == 12):   #upgrade primaries to beam
        self.cbUpgradePrimaryWeapons.SetValue(True)
      elif (hpoint.index == 13):   #fore fixed to scatter
        self.cbChangeFixedForeToScatter.SetValue(True)
      elif (hpoint.index == 14):   #increase PD
        self.cbIncreasePD.SetValue(True)
        self.spPD.Enable(True)
        self.spPD.SetValue(hpoint.intValue1)
        self.cbBiggerBatteriesPD.Enable(True)
        if (hpoint.marValue == 6): #add the bigger batteries MAR
          self.cbBiggerBatteriesPD.SetValue(True)
      elif (hpoint.index == 15):   #increase AP
        self.cbIncreaseAP.SetValue(True)
        self.spAP.Enable(True)
        self.cbAP_SecondAssaultMAR.Show(True)
        self.cbAP_SpecialForcesMAR.Show(True)
        if (hpoint.marValue == 39):
          self.cbAP_SecondAssaultMAR.SetValue(True)
        if (hpoint.marValue == 45):
          self.cbAP_SpecialForcesMAR.SetValue(True)
        self.spAP.SetValue(hpoint.intValue1)
      elif (hpoint.index == 16):  #quick launch MAR
        self.cbQuickLaunch.SetValue(True)
      elif (hpoint.index == 17):  #change Energy Transfer Beam MAR
        self.cbChangeEnergyTransferBeam.SetValue(True)
      elif (hpoint.index == 18):  #gain minefields mar
        self.cbGainMinefieldsMAR.SetValue(True)
      elif (hpoint.index == 19):  #increase CP
        self.cbIncreaseCP.SetValue(True)
        self.spCP.Enable(True)
        self.spCP.SetValue(hpoint.intValue1)
      elif (hpoint.index == 20):  #Gain Shunt Matrix (4) MAR
        self.cbGainShuntMatrix4.SetValue(True)
      elif (hpoint.index == 21):  #gain ops center mar
        self.cbGainOpsCenterMar.SetValue(True)
      elif (hpoint.index == 22):  #gain deck crews mar
        self.cbGainDeckCrewsMar.SetValue(True)
      elif (hpoint.index == 23):  #gain torpedo spook mar
        self.cbGainTorpedoSpookMar.SetValue(True)
      elif (hpoint.index == 24):  #gain special forces mar
        self.cbGainSpecialForcesMar.SetValue(True)
      elif (hpoint.index == 25):  #gain sector shielding mar
        self.cbSectorShieldingMar.SetValue(True)
      elif (hpoint.index == 26):  #gain shield projector self mar
        self.cbShieldProjectorMAR.SetValue(True)
      elif (hpoint.index == 27):  #gain shield projector8 mar
        self.cbShieldProjector8MAR.SetValue(True)
      elif (hpoint.index == 28):  #gain countermeasures mar
        self.cbCountermeasuresMar.SetValue(True)
      elif (hpoint.index == 29):  #gain decimator warheads mar
        self.cbDecimatorWarheads.SetValue(True)
      elif (hpoint.index == 30):  #include cyberwarfare weapons
        self.cbIncludeCyberwarfareWeapons.SetValue(True)
      elif (hpoint.index == 31):  #gain self repair mar
        self.cbGainSelfRepairMAR.SetValue(True)
      elif (hpoint.index == 32):  #increase CR
        self.cbIncreaseCR.SetValue(True)
        self.spCR.Enable(True)
        self.spCR.SetValue(hpoint.intValue1)
      elif (hpoint.index == 33):  #gain second assault mar
        self.cbGainSecondAssaultMAR.SetValue(True)
      elif (hpoint.index == 34):  #gain launch tubes mar
        self.cbLaunchTubesMar.SetValue(True)
      elif (hpoint.index == 35):  #gain launch tubes mar
        self.cbRemoveVulnerable.SetValue(True)
      elif (hpoint.index == 36):  #gain ambush (1) mar
        self.cbAmbushMAR.SetValue(True)
      elif (hpoint.index == 37):  #gain turrets split fire mar
        self.cbTurretsSplitFire.SetValue(True)
      elif (hpoint.index == 38):  #gain stealth systems mar and SR=1
        self.cbGainStealthSystemsAndReduceSR.SetValue(True)
      elif (hpoint.index == 39):  #gain dirty secrets mar
        self.cbGainDirtySecrets.SetValue(True)
      elif (hpoint.index == 40):  #gain dirty secrets mar
        self.cbBeamForeToCyberwarfare.SetValue(True)
      elif (hpoint.index == 41):  #replace stealth with hidden killer
        self.cbStealthSystemsToHiddenKillerMAR.SetValue(True)
      elif (hpoint.index == 42):  #change beam to cyberwarfare
        self.cbBeamToCyberwarfare.SetValue(True)
      elif (hpoint.index == 43):  #torpedo gain corrosive
        self.cbTorpedoesGainCorrosiveMAR.SetValue(True)
      elif (hpoint.index == 44):  #torpedo gain biohazard
        self.cbTorpedoesGainBiohazard.SetValue(True)
      elif (hpoint.index == 45):  #take beams
        self.cbTakeBeams.SetValue(True)
      elif (hpoint.index == 46):  #take cyberwarfare
        self.cbTakeCyberwarfare.SetValue(True)
      elif (hpoint.index == 47):  #take gravity weapons
        self.cbTakeGravity.SetValue(True)
      elif (hpoint.index == 48):  #take cyberwarfare with assault robots
        self.cbCyberwarfareAssaultRobot.SetValue(True)
      elif (hpoint.index == 49):  #take torpedo weapons
        self.cbIncludeTorpedoWeapons.SetValue(True)
      elif (hpoint.index == 50):  #gain weapon shielding mar
        self.cbGainWeaponShielding.SetValue(True)
      elif (hpoint.index == 51):  #gain the pack hunter mar
        self.cbGainPackHuntersMAR.SetValue(True)
      elif (hpoint.index == 52):  #beams to high energy mar
        self.cbBeamsHighEnergyMAR.SetValue(True)
      elif (hpoint.index == 53):  #gain point defense barrage mar
        self.cbPointDefenseMAR.SetValue(True)
      elif (hpoint.index == 54):  #gain shunt matrix2
        self.cbGainShuntMatrix2.SetValue(True)
      elif (hpoint.index == 55):  #replace ambush2 with ambush3 mar
        self.cbAmbush23.SetValue(True)
      elif (hpoint.index == 56):  #remove no fsd mar 
        self.cbRemoveNoFSD.SetValue(True)
      elif (hpoint.index == 57):  #gain stealth systems
        self.cbGainStealthSystems.SetValue(True)
      elif (hpoint.index == 58):  #gain +1 SR, -1 MV
        self.cbShield1Move1.SetValue(True)
      elif (hpoint.index == 59):  #gain -1 HP, Difficult Target mar
        self.cbGainDifficult1HP.SetValue(True)
      elif (hpoint.index == 60):  #gain +3 WC, lose planetfall mar
        self.cbWing3NoPlanetfall.SetValue(True)
      elif (hpoint.index == 61):  #gain planetfall+1, remove durable
        self.cbGainPlanetfall1LoseDurable.SetValue(True)
      elif (hpoint.index == 62):  #gain difficult, remove durable
        self.cbGainDifficultRemoveDurable.SetValue(True)
      elif (hpoint.index == 63):  #gain +2 MN, remove planetfall
        self.cbMine2RemovePlanetfall.SetValue(True)
      elif (hpoint.index == 64):  #gain +2 MN and Drone Mines, remove planetfall
        self.cbMineDrone2RemovePlanetfall.SetValue(True)
      elif (hpoint.index == 65):  #gain reinforced fore mar
        self.cbReinforcedFore.SetValue(True)
      elif (hpoint.index == 66):  #+1SR, +1HP
        self.cbShield1HP1.SetValue(True)
      elif (hpoint.index == 67):  #gain energy transfer 1
        self.cbEnergyTransfer1.SetValue(True)
      elif (hpoint.index == 68):  #gain maneoverable mar
        self.cbEnergyTransfer1.SetValue(True)
      elif (hpoint.index == 69):  #change fore scatter to fore fixed kinetic
        self.cbForeScatterForeKinetic.SetValue(True)
      elif (hpoint.index == 70):  #gain reinforced fore mar
        self.cbReinforcedPortStarboard.SetValue(True)
      elif (hpoint.index == 71):  #gain  second assault, remove planetfall mar
        self.cbSecondAssaultNoPlanetfall.SetValue(True)
      elif (hpoint.index == 72):  #gain point barrage, remove planetfall mar
        self.cbPointBarrageNoPlanetfall.SetValue(True)
      elif (hpoint.index == 73):  #gain +1 HP, remove durable
        self.cbHP1NoDurable.SetValue(True)
      elif (hpoint.index == 74):  #upgrade to nuclear turrets, remove planetfall
        self.cbNuclearTurretsNoPlanetfall.SetValue(True)
      elif (hpoint.index == 75):  # +1 planetfall, -2 WC
        self.cbPlanetfall1WC2.SetValue(True)
      elif (hpoint.index == 76):  #+3 AP, remove planetfall
        self.cbAP3NoPlanetfall.SetValue(True)
      elif (hpoint.index == 77):  #Gain the Turret Weapons
        self.cbGainTurrets.SetValue(True)
      elif (hpoint.index == 78):  #Gain +1 SR and Shield Projector 0 MAR
        self.cbShield1ShieldProjector.SetValue(True)
      elif (hpoint.index == 79):  #change to Escort, remove planetfall
        self.cbEscortNoPlanetfall.SetValue(True)
      elif (hpoint.index == 80):  #primary to scatter
        self.cbPrimaryScatter.SetValue(True)
      elif (hpoint.index == 81):  #remove ablative and durable mars
        self.cbRemoveAblativeDurable.SetValue(True)
      elif (hpoint.index == 82):  # +1 planetfall, -3 WC
        self.cbPlanetfall1WC3.SetValue(True)
      elif (hpoint.index == 83):  # +1 planetfall, -3 AP
        self.cbPlanetfall1AP3.SetValue(True)
      elif (hpoint.index == 84):  # +1 planetfall, -1 SR
        self.cbPlanetfall1Shield1.SetValue(True)
      elif (hpoint.index == 85):  # +2 AP , -1 planetfall
        self.cbAP2NoPlanetfall.SetValue(True)
      elif (hpoint.index == 86):  # +6 AP , -1 planetfall
        self.cbAP6NoPlanetfall.SetValue(True)
      elif (hpoint.index == 87):  # gain bigger batteries mar, remove planetfall
        self.cbBiggerBatteriesNoPlanetfall.SetValue(True)
      elif (hpoint.index == 88):  # upgrade improved shunt matrix from 3 to 4
        self.cbImprovedShunt3to4.SetValue(True)
      elif (hpoint.index == 89):  # gain +4 WC and Quick launch, remove planetfall
        self.cbWC4QuickLaunchNoPlanetfall.SetValue(True)
      elif (hpoint.index == 91):  # gain cloaking field, remove planetfall
        self.cbCloakNoPlanetfall.SetValue(True)
      elif (hpoint.index == 92):  # +3 MN, -2 MV
        self.cbMine3Move2.SetValue(True)
      elif (hpoint.index == 93):  # +1 AP, -1 CP
        self.cbAP1CP1.SetValue(True)
      elif (hpoint.index == 94):  # +1 AP, -1 CP
        self.cbMove2LoseDurable1.SetValue(True)
      elif (hpoint.index == 95):  # gain torpedo, remove planetfall
        self.cbTorpedoNoPlanetfall.SetValue(True)
      elif (hpoint.index == 96):  # +1 planetfall, -2 PD
        self.cbPlanetfall1WC2.SetValue(True)
      elif (hpoint.index == 97):  # +1 SR, remove durable
        self.cbShield1NoDurable.SetValue(True)
      elif (hpoint.index == 98):  #  Take torpedoes as hardpoint, with biohazard
        self.cbTakeTorpedoesWithBiohazard.SetValue(True)
      elif (hpoint.index == 99):  #  Gain the Scout MAR
        self.cbTakeTorpedoesWithBiohazard.SetValue(True)



#also edit ShipEditorBase::OnAddHardpoint, ShipRecord::ApplyHardpoints() if weapons or stats are affected.
      self.cost = hpoint.cost

      if (self.cost == 0):
        self.stPrefixCost.SetLabel("for free.")
        self.stCost.SetLabel("")
      else:
        self.stPrefixCost.SetLabel("for a cost of ")
        self.stCost.SetLabel(str(self.cost))
    return
  
  def EnableSpinAP(self, event):
    self.spAP.Enable(self.cbIncreaseAP.GetValue())
    self.cbAP_SecondAssaultMAR.Enable(self.cbIncreaseAP.GetValue())
    self.cbAP_SpecialForcesMAR.Enable(self.cbIncreaseAP.GetValue())
    self.spAP.SetFocus()
    return
  
  def EnableSpinCP(self, event):
    self.spCP.Enable(self.cbIncreaseCP.GetValue())
    self.spCP.SetFocus()
    return
  
  def EnableSpinCR(self, event):
    self.spCR.Enable(self.cbIncreaseCR.GetValue())
    self.spCR.SetFocus()
    return
  def EnableSpinCMD(self, event):
    self.spCMD.Enable(self.cbIncreaseCommandDistance.GetValue())
    self.spCMD.SetFocus() 
    return
  def EnableSpinHP(self, event):
    self.spHP.Enable(self.cbIncreaseHP.GetValue())
    self.spHP.SetFocus()
    return
  def EnableSpinMN(self, event):
    self.spMN.Enable(self.cbIncreaseMN.GetValue())
    self.spMN.SetFocus()
    return
  def EnableSpinMV(self, event):
    self.spMV.Enable(self.cbIncreaseMV.GetValue())
    self.spMV.SetFocus()
    return
  def EnableSpinPD(self, event):
    self.spPD.Enable(self.cbIncreasePD.GetValue())
    self.cbBiggerBatteriesPD.Enable(self.cbIncreasePD.GetValue())
    self.spPD.SetFocus()
    return
  def EnableSpinSR(self, event):
    self.spSR.Enable(self.cbIncreaseSR.GetValue())
    self.spSR.SetFocus()
    return
  def EnableSpinTL(self, event):
    self.spTL.Enable(self.cbDecreaseTL.GetValue())
    self.spTL.SetFocus()
    return
  def EnableSpinWC(self, event):
    self.spWC.Enable(self.cbIncreaseWC.GetValue())
    self.spWC.SetFocus()
    return
  
  def OnAddHardpoint(self, event):
    self.EndModal(True)
    return
  
  def OnCancel(self, event):
    self.EndModal(False)
    
  def OnHelp(self, event):
    text = "Select the hardpoints to add.  Multiple hardpoints can be added from this dialog, they will all be added as separate hardpoints in the ship list."
    helper = HelpDialog(self, text)
    helper.ShowModal()
    return
  
  def OnIncreaseCost(self, event):
    self.cost += 5
    self.stPrefixCost.SetLabel("for a cost of ")
    self.stCost.SetLabel(str(self.cost))
    return
  def OnDecreaseCost(self, event):
    if (self.cost > 0):
      self.cost -= 5
    if (self.cost == 0):
      self.stPrefixCost.SetLabel("for free.")
      self.stCost.SetLabel("")
    else:
      self.stPrefixCost.SetLabel("for a cost of ")
      self.stCost.SetLabel(str(self.cost))
    return
    
  
#------------------------------------------------------------------------------
########################################################################
class SelectSquadronSizeBase(FirestormDialogs.SquadronSizeCore):
  """"""
  #----------------------------------------------------------------------
  def __init__(self, parent, squadronMin, squadronMax, heavyCruiser=None, heavyList=None):
    """Constructor"""
    FirestormDialogs.SquadronSizeCore.__init__(self, parent)
    self.spSquadronSize.SetRange(squadronMin, squadronMax)
    self.stPrompt.SetLabel("This squadron has a minimum size of {0} and a maximum size of {1}".format(squadronMin, squadronMax))
    if (heavyCruiser):
      for ship in heavyList:
        self.cxHeavyCruiser.Append(ship.className +" (" + ship.designation +")")
    else:
      self.cxHeavyCruiser.Hide()
      self.stHeavyCruiser.Hide()
    return
  
  def OnCancel(self, event):
    self.EndModal(False)
    return
  
  def OnOK(self, event):
    self.EndModal(True)
    return
  
########################################################################
class AddUpgradeBase(FirestormDialogs.AddNewUpgradeCore):
  """"""
  #----------------------------------------------------------------------
  def __init__(self, parent):
    """Constructor"""
    FirestormDialogs.AddNewUpgradeCore.__init__(self, parent)
    self.cost = 5
    return
  
  def OnAddUpgrade(self, event):
    self.EndModal(True)
    return
  
  def OnCancel(self, event):
    self.EndModal(False)
    return
  
  def OnHelp(self, event):
    text = "Check the upgrades to add.  Each selected upgrade will cost the same amount as shown in the top left value."
    dlg = HelpDialog(self, text)
    dlg.ShowModal()
    return
    
  def OnIncreaseCost(self, event):
    self.cost += 5
    self.stCost.SetLabel(str(self.cost))
    return
  
  def OnDecreaseCost(self, event):
    if (self.cost > 0):
      self.cost -= 5
    if (self.cost == 0):
      self.stCost.SetLabel("Free")
    else:
      self.stCost.SetLabel(str(self.cost))
    return
  

#------------------------------------------------------------------------------
########################################################################
class SelectMarBase(FirestormDialogs.SelectMarCore):
  """"""
  #----------------------------------------------------------------------
  def __init__(self, parent):
    """Constructor"""
    FirestormDialogs.SelectMarCore.__init__(self, parent)
    self.lcMarList.InsertColumn(0, "#")
    self.lcMarList.InsertColumn(1, "MAR Name")
    self.lcMarList.InsertColumn(2, "Description")
    self.selectedItem = None
    self.marRecord = dataStruct.MarRecord()
    gv.MarList.reverse()
    for mar in gv.MarList:
      listitem = wx.ListItem()
      listitem.SetText(mar.marName)
      index = self.lcMarList.InsertItem(listitem)
      self.lcMarList.SetStringItem(index, 0, str(mar.marIndex))
      self.lcMarList.SetStringItem(index, 1, mar.marName)
      self.lcMarList.SetStringItem(index, 2, mar.marText)
    self.lcMarList.SetColumnWidth(0, wx.LIST_AUTOSIZE)
    self.lcMarList.SetColumnWidth(1, wx.LIST_AUTOSIZE)
    self.lcMarList.SetColumnWidth(2, wx.LIST_AUTOSIZE)
    gv.MarList.reverse()
    self.value = None
    return
  
  def OnSelectItem(self, event):
    self.selectedItem = event.m_itemIndex
    return
  
  def OnDeselectItem(self, event):
    self.selectedItem = None
    return
  
  def OnCancel(self, event):
    self.EndModal(False)
    return
  
  def OnAddMAR(self, event):
    if (gv.MarList[self.selectedItem].valueRequired):
      choice = None
      string1 = None
      string2 = None
      #the selectedItem is 0 based, it is 1 off of the standard MAR list
      #add mar descriptions to export.py::ReadShipClassData() to modify text.
      if (gv.MarList[self.selectedItem].marNumber == 3):
        choice = None
        string1 = 'Ambush Value:'
        string2 = None
      elif (gv.MarList[self.selectedItem].marNumber == 20):
        choice = 'weapons'
        string1 = 'Select System:'
        string2 = None
      elif (gv.MarList[self.selectedItem].marNumber == 22):
        choice = 'weapons'
        string1 = None
        string2 = None
      elif (gv.MarList[self.selectedItem].marNumber == 28):
        choice = None
        string1 = 'Number of Mines:'
        string2 = 'Attack Dice (AD):'
      elif (gv.MarList[self.selectedItem].marNumber == 36):
        choice = 'arcs'
        string1 = None
        string2 = None
      elif (gv.MarList[self.selectedItem].marNumber == 37):
        choice = 'weapons'
        string1 = None
        string2 = None
      elif (gv.MarList[self.selectedItem].marNumber == 43):
        choice = None
        string1 = 'Shield Range'
        string2 = None
      elif (gv.MarList[self.selectedItem].marNumber == 44):
        choice = None
        string1 = 'Shunt Value:'
        string2 = None
      elif (gv.MarList[self.selectedItem].marNumber == 46):
        choice = 'arcs'
        string1 = None
        string2 = None
      elif (gv.MarList[self.selectedItem].marNumber == 58):
        choice = None
        string1 = 'Planetfall Value:'
        string2 = None
      elif (gv.MarList[self.selectedItem].marNumber == 59):
        choice = None
        string1 = 'Matrix Value:'
        string2 = None
      elif (gv.MarList[self.selectedItem].marNumber == 60):
        choice = None
        string1 = 'Locus Value:'
        string2 = None
      
      valueDlg = SelectMarVariablesBase(self, listType=choice, intString1=string1, intString2=string2)
      result = valueDlg.ShowModal()
      if (result):
        self.marRecord.strValue1 = valueDlg.cxStringChoice.GetStringSelection()
        self.marRecord.intValue1 = valueDlg.spIntValue1.GetValue()
        self.marRecord.intValue2 = valueDlg.spIntValue2.GetValue()
      else:
        return
    self.marRecord.marIndex = self.selectedItem
    self.marRecord.marNumber = gv.MarList[self.selectedItem].marNumber
    self.marRecord.valueRequired = gv.MarList[self.selectedItem].valueRequired
    if (self.marRecord.valueRequired):
      self.SetMarName(self.marRecord.marNumber)
    else:
      self.marRecord.marName = gv.MarList[self.selectedItem].marName
    self.marRecord.marText = gv.MarList[self.selectedItem].marText
    
    self.EndModal(True)
    return
  
  def SetMarName(self, marNumber):
    if (marNumber == 3):
      self.marRecord.marName = 'Ambush {0}'.format(self.marRecord.intValue1)
    elif (marNumber == 20):
      self.marRecord.marName = 'Energy Transfer ({0}, {1})'.format(self.marRecord.strValue1, self.marRecord.intValue1)
    elif (marNumber == 22):
      self.marRecord.marName = 'High Energy ({0})'.format(self.marRecord.strValue1)
    elif (marNumber == 28):
      self.marRecord.marName = 'Minefields ({0},{1} AD)'.format(self.marRecord.intValue1, self.marRecord.intValue2)
    elif (marNumber == 36):
      self.marRecord.marName = 'Reinforced ({0})'.format(self.marRecord.strValue1)
    elif (marNumber == 37):
      self.marRecord.marName = 'Retractable Plating ({0})'.format(self.marRecord.strValue1)
    elif (marNumber == 43):
      self.marRecord.marName = 'Shield Projector ({0})'.format(self.marRecord.intValue1)
    elif (marNumber == 44):
      self.marRecord.marName = 'Shunt Matrix ({0})'.format(self.marRecord.intValue1)
    elif (marNumber == 46):
      self.marRecord.marName = 'Split Fire ({0})'.format(self.marRecord.strValue1)
    elif (marNumber == 58):
      self.marRecord.marName = 'Planetfall ({0})'.format(self.marRecord.intValue1)
    elif (marNumber == 59):
      self.marRecord.marName = 'Improved Shunt Matrix ({0})'.format(self.marRecord.intValue1)
    elif (marNumber == 60):
      self.marRecord.marName = 'Energy Locus ({0})'.format(self.marRecord.intValue1)
    return
  
class WeaponsEditorBase(FirestormDialogs.AddNewWeaponCore):
  def __init__(self, parent, weapon=None):
    folder = os.getcwd()
    os.chdir(gv.root_folder)
    FirestormDialogs.AddNewWeaponCore.__init__(self,parent)
    os.chdir(folder)

    #weapon = dataStruct.WeaponRecord()
    if (weapon):
      self.txWeaponName.SetValue(weapon.weaponName)
      self.cxArcDirection.SetStringSelection(weapon.arcDirection)
      self.cxArcType.SetStringSelection(weapon.arcRestriction)
      if (weapon.hardpoint):
        self.cbHardpointWeapon.SetValue(True)
      if (weapon.category == 'Direct'):
        self.rbDirectWeapon.SetValue(True)
        self.cxDirectWeaponType.Clear()
        self.cxDirectWeaponType.AppendItems(dataStruct.weaponTypes)
        self.cxDirectWeaponType.SetStringSelection(weapon.weaponType)
        self.OnDirectWeaponType()
      else:  #indirect weapons
        self.rbIndirectWeapon.SetValue(True)
        self.cxIndirectWeaponType.Clear()
        self.cxIndirectWeaponType.AppendItems(dataStruct.indirectWeaponTypes)
        self.cxIndirectWeaponType.SetStringSelection(weapon.weaponType)
        self.OnIndirectWeaponType()
      self.spRangeBand1.SetValue(weapon.rb1)
      self.spRangeBand2.SetValue(weapon.rb2)
      self.spRangeBand3.SetValue(weapon.rb3)
      self.spRangeBand4.SetValue(weapon.rb4)
    else:
      self.OnDirectWeaponType()  #set default range band values
    return

  def OnAddWeapon(self, event):
    self.EndModal(True)
    return

  def OnCancel(self, event):
    self.EndModal(False)
    return
  
  def OnHelp(self, event):
    text = 'Input weapon details.  The range band values will change depending on the weapon category chosen.'
    dlg = HelpDialog(gv.frame,text)
    dlg.ShowModal()
    return
  
  def OnSelectDirect(self, event):
    self.cxDirectWeaponType.Show()
    self.cxIndirectWeaponType.Hide()
    self.OnDirectWeaponType()
    self.Layout()
    return
  
  def OnSelectIndirect(self, event=None):
    self.cxDirectWeaponType.Hide()
    self.cxIndirectWeaponType.Show()
    self.OnIndirectWeaponType()
    self.Layout()
    return
  
  def OnIndirectWeaponType(self, event=None):
    """Set range bands by weapon type"""
    weaponType = self.cxIndirectWeaponType.GetStringSelection()
    if (weaponType == 'Torpedo'):
      #set range bands to 
      self.stRange1.SetLabel('0"-12"')
      self.stRange2.SetLabel('12"-24"')
      self.stRange3.SetLabel('24"-36"')
      self.stRange4.SetLabel('36"-48"')
    elif (weaponType == 'Gravitational'):
      self.stRange1.SetLabel('0"-8"')
      self.stRange2.SetLabel('8"-16"')
      self.stRange3.SetLabel('16"-24"')
      self.stRange4.SetLabel('24"-32"')
    elif (weaponType == 'Cyberwarfare'):
      self.stRange1.SetLabel('0"-10"')
      self.stRange2.SetLabel('10"-20"')
      self.stRange3.SetLabel('20"-30"')
      self.stRange4.SetLabel('30"-40"')
    else:
      self.stRange1.SetLabel('0"-8"')
      self.stRange2.SetLabel('8"-16"')
      self.stRange3.SetLabel('16"-24"')
      self.stRange4.SetLabel('24"-32"')
    return

  
  def OnDirectWeaponType(self, event=None):
    """Set range bands by weapon type"""
    weaponType = self.cxDirectWeaponType.GetStringSelection()
    if (weaponType == 'Primary'):
      #set range bands
      self.stRange1.SetLabel('0"-8"')
      self.stRange2.SetLabel('8"-16"')
      self.stRange3.SetLabel('16"-24"')
      self.stRange4.SetLabel('24"-32"')
    elif (weaponType == 'Gun Rack'):
      self.stRange1.SetLabel('0"-8"')
      self.stRange2.SetLabel('8"-16"')
      self.stRange3.SetLabel('16"-24"')
      self.stRange4.SetLabel('24"-32"')
    elif (weaponType == 'Scatter'):
      self.stRange1.SetLabel('0"-8"')
      self.stRange2.SetLabel('8"-16"')
      self.stRange3.SetLabel('16"-24"')
      self.stRange4.SetLabel('24"-32"')
    elif (weaponType == 'Kinetic'):
      self.stRange1.SetLabel('0"-12"')
      self.stRange2.SetLabel('12"-24"')
      self.stRange3.SetLabel('24"-36"')
      self.stRange4.SetLabel('36"-48"')
    elif (weaponType == 'Beam'):
      self.stRange1.SetLabel('0"-10"')
      self.stRange2.SetLabel('10"-20"')
      self.stRange3.SetLabel('20"-30"')
      self.stRange4.SetLabel('30"-40"')
    elif (weaponType == 'Nuclear'):
      self.stRange1.SetLabel('0"-8"')
      self.stRange2.SetLabel('8"-16"')
      self.stRange3.SetLabel('16"-24"')
      self.stRange4.SetLabel('24"-32"')
    else:
      self.stRange1.SetLabel('0"-8"')
      self.stRange2.SetLabel('8"-16"')
      self.stRange3.SetLabel('16"-24"')
      self.stRange4.SetLabel('24"-32"')
    return  
      

#------------------------------------------------------------------------------

class FleetManagerFrame(FirestormDialogs.Mainframe):
  def __init__(self, parent=None):
    FirestormDialogs.Mainframe.__init__(self, parent)
    
    gv.root_folder = os.getcwd()
    
    gv.data_folder = os.path.join(gv.root_folder, 'data')
    if (not os.access(gv.data_folder, os.F_OK)):
      logging.error('Could not find data folder, default to root folder')
      gv.data_folder = gv.root_folder
    else:
      logging.info('data folder:%s',gv.data_folder)

    gv.backup_folder = os.path.join(gv.root_folder, 'backup_folder')
    if (not os.access(gv.backup_folder, os.F_OK)):
      logging.error('Could not find data backup folder, create in root folder')
      os.mkdir(gv.backup_folder)
    else:
      logging.info('backup folder:%s', gv.backup_folder)
  
    gv.fleets_folder = os.path.join(gv.root_folder, 'fleets')
    if (not os.access(gv.fleets_folder, os.F_OK)):
      logging.error('Could not find fleets folder, default to root folder')
      gv.fleets_folder = gv.root_folder
    else:
      logging.info('fleets folder:%s', gv.fleets_folder)
    
    gv.images_folder = os.path.join(gv.root_folder, 'images')
    if (not os.access(gv.images_folder, os.F_OK)):
      logging.error('Could not find data folder, default to root folder')
      gv.images_folder = gv.root_folder
    else:
      logging.info('images folder:%s', gv.images_folder)
      
    gv.ship_images_folder = os.path.join(gv.images_folder, 'ships')
    if (not os.access(gv.ship_images_folder, os.F_OK)):
      logging.error('Could not find images/ships folder, default to root folder')
      gv.ship_images_folder = gv.root_folder
    else:
      logging.info('ship images folder:%s', gv.ship_images_folder)
      
    gv.ship_custom_images_folder = os.path.join(gv.images_folder, 'custom')
    if (not os.access(gv.ship_custom_images_folder, os.F_OK)):
      logging.error('Could not find images/custom folder, default to root folder')
      gv.ship_custom_images_folder = gv.root_folder
    else:
      logging.info('ship custom images folder:%s', gv.ship_custom_images_folder)
  
    gv.faction_images_folder = os.path.join(gv.images_folder, 'factions')
    if (not os.access(gv.faction_images_folder, os.F_OK)):
      logging.error('Could not find images/factions folder, default to root folder')
      gv.faction_images_folder = gv.root_folder
    else:
      logging.info('faction images folder:%s', gv.faction_images_folder)
    
    imported_MAR = False
    gv.MarDataFile = os.path.join(gv.data_folder, 'mar.xml')
    if (os.access(gv.MarDataFile, os.F_OK)):
      imported_MAR = export.ReadMarData(gv.MarDataFile)
    else:
      logging.critical('Could not find MAR datafile from %s', gv.MarDataFile)

    
    if (imported_MAR != True):
      logging.error("Problem importing MAR data")
      logging.error(repr(imported_MAR))
      #try to restore from backup
      backupFile = os.path.join(gv.backup_folder, 'mar.xml')
      if (os.access(backupFile, os.F_OK)):
        shutil.copyfile(backupFile, gv.MarDataFile)  #copy from backup
        logging.info("Restored mar.xml from backup copy")
        imported_MAR = export.ReadMarData(gv.MarDataFile)
      else:
        logging.critical('Could not recreate mar.xml from backup')

    
    if (imported_MAR != True):  #backup failed
      logging.critical('Could not recover mar.xml from backup')
      return
    else:
      #make a backup copy of the data file
      backupFile = os.path.join(gv.backup_folder, 'mar.xml')
      shutil.copyfile(gv.MarDataFile, backupFile)
      logging.info('Backed up MAR data')
      logging.info('Imported MAR data from %s',gv.MarDataFile)



    importedFleetComposition = False
    gv.FleetCompositionFile = os.path.join(gv.data_folder, 'fleet_composition.xml')
    
    if (os.access(gv.FleetCompositionFile, os.F_OK)):
      importedFleetComposition = export.ReadFleetCompositionData(gv.FleetCompositionFile)
    else:
      logging.critical('Could not find fleet composition data file:%s',gv.FleetCompositionFile)

    
    if (importedFleetComposition != True):
      logging.error("Problem importing FleetComposition data")
      logging.error(repr(importedFleetComposition))
      #try to restore from backup
      backupFile = os.path.join(gv.backup_folder, 'fleet_composition.xml')
      if (os.access(backupFile, os.F_OK)):
        shutil.copyfile(backupFile, gv.FleetCompositionFile)  #copy from backup
        gv.FleetComposition = export.ReadFleetCompositionData(gv.FleetCompositionFile)
        importedFleetComposition = True
      else:
        logging.critical('Could not recreate fleet_composition.xml from backup')

    
    if (importedFleetComposition != True):  #backup failed
      logging.critical('Could not recover fleet_composition.xml from backup')
      return
    else:
      #make a backup copy of the data file
      backupFile = os.path.join(gv.backup_folder, 'fleet_composition.xml')
      shutil.copyfile(gv.FleetCompositionFile, backupFile)
      logging.info('Backed up FleetComposition data')
      logging.info('Imported fleet composition data from %s',gv.FleetCompositionFile)


    importedShipCatalog = False
    gv.ShipClassFile = os.path.join(gv.data_folder, 'catalog.xml')
    if (os.access(gv.ShipClassFile, os.F_OK)):
      importedShipCatalog = export.ReadShipClassData(gv.ShipClassFile, gv.ShipClassList)
    else:
      logging.error('Could not load ship catalog data')
    
    if (importedShipCatalog != True):
      logging.error("Problem importing ShipCatalog data")
      logging.error(repr(importedShipCatalog))
      #try to restore from backup
      backupFile = os.path.join(gv.backup_folder, 'catalog.xml')
      if (os.access(backupFile, os.F_OK)):
        shutil.copyfile(backupFile, gv.ShipClassFile)  #copy from backup
        importedShipCatalog = export.ReadShipClassData(gv.ShipClassFile, gv.ShipClassList)
      else:
        logging.critical('Could not recreate catalog.xml from backup')

    
    if (importedShipCatalog != True):  #backup failed
      logging.critical('Could not recover catalog.xml from backup')
      return
    else:
      #make a backup copy of the data file
      backupFile = os.path.join(gv.backup_folder, 'catalog.xml')
      shutil.copyfile(gv.ShipClassFile, backupFile)
      logging.info('Backed up Catalog data')
      logging.info('Imported ship catalog data from %s',gv.ShipClassFile)
    
    
    importedShipInventory = False
    gv.ShipInventoryFile = os.path.join(gv.data_folder, 'inventory.xml')
    #check existence of inventory file
    if (os.access(gv.ShipInventoryFile, os.F_OK)):
      importedShipInventory = export.ReadInventoryData(gv.ShipInventoryFile)
    else:
      logging.error('Could not load ship inventory data')

    if (importedShipInventory != True):
      logging.error("Problem importing ShipInventory data")
      logging.error(repr(importedShipInventory))
      #try to restore from backup
      backupFile = os.path.join(gv.backup_folder, 'inventory.xml')
      if (os.access(backupFile, os.F_OK)):
        shutil.copyfile(backupFile, gv.ShipInventoryFile)  #copy from backup
        importedShipInventory = export.ReadInventoryData(gv.ShipInventoryFile)
      else:
        logging.critical('Could not recreate inventory.xml from backup')
    
    if (not importedShipInventory):  #backup failed
      logging.error('Could not recover inventory.xml from backup')
    else:
      #make a backup copy of the data file
      backupFile = os.path.join(gv.backup_folder, 'inventory.xml')
      shutil.copyfile(gv.ShipInventoryFile, backupFile)
      logging.info('Backed up Inventory data')
      logging.info('Imported ship inventory data from %s',gv.ShipInventoryFile)
    
    
    gv.FleetRosterFile = os.path.join(gv.fleets_folder, 'fleets.xml')
    
    self.fleetsRoster = None  #holds a list of fleets;  should be a list if imported correctly
    if (os.access(gv.FleetRosterFile, os.F_OK)):
      self.fleetsRoster = export.ReadFleetData(gv.FleetRosterFile)
    else:
      logging.error('Could not access fleets.xml data file')
    
    if (not isinstance(self.fleetsRoster, list)):  #missing fleet data, try to restore from backup
      logging.error("Problem importing FleetRoster data")
      logging.error(repr(self.fleetsRoster))
      backupFile = os.path.join(gv.backup_folder, 'fleets.xml')
      
      if (os.access(backupFile, os.F_OK)):
        logging.info('Restored FleetRoster datafile from backup')
        shutil.copyfile(backupFile, gv.FleetRosterFile)  #copy from backup
        self.fleetsRoster = export.ReadFleetData(gv.FleetRosterFile)
        
    if (isinstance(self.fleetsRoster, list)):
      #make a backup copy of the data file
      backupFile = os.path.join(gv.backup_folder, 'fleets.xml')
      shutil.copyfile(gv.FleetRosterFile, backupFile)
      logging.info('Backed up Inventory data')
      logging.info('Imported FleetRoster data')
    else:
      logging.error('Could not recover fleets.xml from backup')
      self.fleetsRoster = []

  
    #Read the .ini file
    init_filename = os.path.join(gv.data_folder, 'settings.xml')
    if (os.access(init_filename, os.F_OK)):
      export.ReadIniFile(init_filename)
    else:
      logging.error('Could not load settings.xml data file')
    
    #update the ship index values
    for ship in gv.ShipClassList:
      if (ship.index < 1000000):
        gv.nextMasterIndex = max(gv.nextMasterIndex, ship.index)
      else:
        gv.nextUserIndex = max(gv.nextUserIndex, ship.index)
    gv.nextMasterIndex += 1
    gv.nextUserIndex += 1
    
    self.selectedFleetIndex = None
    self.lcFleetList.InsertColumn(0, "Core Faction")
    self.lcFleetList.InsertColumn(1, "Cost/MFV")
    self.lcFleetList.InsertColumn(2, "Fleet Name")
    
    self.UpdateRosterList()
    
    self.PrintData = None
    
    #set the program icon
    iconFilename = os.path.join(gv.root_folder, "FleetManager.ico")
    icon = wx.Icon(iconFilename, wx.BITMAP_TYPE_ICO)
    self.SetIcon(icon)
    
    return
  
  def OnProgramOptions(self, event):
    return
  
  def OnProgramExit(self, event):
    logging.info("Begin shutdown")
    
    if (gv.SaveMarData):
      logging.info("Save MAR data")
      export.SaveMarData(gv.MarDataFile)
    if (gv.SaveFleetData) and (self.fleetsRoster != []):
      logging.info("Begin SaveFleetData")
      export.SaveFleetData(gv.FleetRosterFile, self.fleetsRoster)
    
    logging.info("Destroy() frame")
    self.Destroy()
    logging.info("Firestorm Fleet Manager Exit")
    logging.shutdown()
    return  
  
  def OnFleetNew(self, event):
    dlg = FleetEditorBase(self)
    addFleet = dlg.ShowModal()
    
    
    if (addFleet):
      self.fleetsRoster.append(dlg.fleet)
      self.UpdateRosterList()
    return
  
  def UpdateRosterList(self):
    fleet = dataStruct.FleetRecord()
    self.lcFleetList.DeleteAllItems()
    self.fleetsRoster.reverse()
    for fleet in self.fleetsRoster:
      listitem = wx.ListItem()
      listitem.SetText(fleet.fleetName)
      index = self.lcFleetList.InsertItem(listitem)
      self.lcFleetList.SetStringItem(index, 0, fleet.coreFaction)
      valueText = str(fleet.CFV) + "/" + str(fleet.MFV)
      self.lcFleetList.SetStringItem(index, 1, valueText)
      self.lcFleetList.SetStringItem(index, 2, fleet.fleetName)
    self.fleetsRoster.reverse()
    self.lcFleetList.SetColumnWidth(0, wx.LIST_AUTOSIZE_USEHEADER)
    self.lcFleetList.SetColumnWidth(1, wx.LIST_AUTOSIZE_USEHEADER)
    self.lcFleetList.SetColumnWidth(2, wx.LIST_AUTOSIZE)
    if (self.lcFleetList.GetColumnWidth(2) < 200):
      self.lcFleetList.SetColumnWidth(2,200)
    return
  
  def OnSelectFleet(self, event):
    dlg = FleetEditorBase(self, fleet=self.fleetsRoster[event.m_itemIndex])
    if (dlg.ShowModal()):
      #make edit to selected fleet
      self.fleetsRoster[event.m_itemIndex] = dlg.fleet
    dlg.Destroy()
    self.UpdateRosterList()
    self.txFleetDetails.Clear()
    return
    
  
  def OnDisplayFleetDetails(self, event):
    self.UpdateFleetDetails(self.fleetsRoster[event.m_itemIndex])
    self.selectedFleetIndex = event.m_itemIndex
    self.MenuBar.Enable(FirestormDialogs.idPrintFleetRoster, True)
    self.bDeleteFleet.Enable(True)
    return
  
  def UpdateFleetDetails(self, fleet):
    #fleet = dataStruct.FleetRecord()
    self.txFleetDetails.Clear()

    text = fleet.fleetName+"\n"
    if (fleet.MFV <= 800):
      fleetSize = "Patrol Fleet"
    elif (fleet.MFV <= 1200):
      fleetSize = 'Battle Fleet'
    else:
      fleetSize = 'Grand Fleet'
      
    text += "{0} {1}, {2}/{3} points\n".format(fleet.coreFaction, fleetSize, fleet.CFV, fleet.MFV)
    
    tier1Text = 'Tier 1 Squadrons:\n'
    tier2Text = 'Tier 2 Squadrons:\n'
    tier3Text = 'Tier 3 Squadrons:\n'
    tier1Ship = ''
    tier2Ship = ''
    tier3Ship = ''
    for squadron in fleet.squadronList:
      if (squadron.tier == 1):
        tier1Ship += '   {0} {1} ({3}x), {2} Class'.format(squadron.ship.faction, squadron.ship.designation, squadron.ship.className, squadron.size)
        if (squadron.heavyCruiser):
          tier1Ship += ' with {0}, {1} Class\n'.format(squadron.heavyCruiser.designation, squadron.heavyCruiser.className)
        else:
          tier1Ship += '\n'
      elif (squadron.tier == 2):
        tier2Ship += '   {0} {1} ({3}x), {2} Class'.format(squadron.ship.faction, squadron.ship.designation, squadron.ship.className, squadron.size)
        if (squadron.heavyCruiser):
          tier2Ship += ' with {0}, {1} Class\n'.format(squadron.heavyCruiser.designation, squadron.heavyCruiser.className)
        else:
          tier2Ship += '\n'
      elif (squadron.tier == 3):
        tier3Ship += '   {0} {1} ({3}x), {2} Class'.format(squadron.ship.faction, squadron.ship.designation, squadron.ship.className, squadron.size)
        if (squadron.heavyCruiser):
          tier3Ship += ' with {0}, {1} Class\n'.format(squadron.heavyCruiser.designation, squadron.heavyCruiser.className)
        else:
          tier3Ship += '\n'
    if (tier1Ship == ''):
      tier1Ship = '   None\n'
    if (tier2Ship == ''):
      tier2Ship = '   None\n'
    if (tier3Ship == ''):
      tier3Ship = '   None\n'
      
    text += tier1Text + tier1Ship + tier2Text + tier2Ship + tier3Text + tier3Ship
    self.txFleetDetails.AppendText(text)
    self.txFleetDetails.ShowPosition(0)
    return
  
  def OnDeselectFleet(self, event):
    self.txFleetDetails.Clear()
    self.bDeleteFleet.Enable(False)
    return
  
  def OnDeleteFleet(self, event):
    dlg = wx.MessageDialog(self, "Delete Fleet?", "Confirm Deletion")
    result = dlg.ShowModal()
    if (result == wx.ID_OK):
      self.fleetsRoster.pop(self.selectedFleetIndex)
      self.UpdateRosterList()
      self.txFleetDetails.Clear()
    dlg.Destroy()
    return
  
  def OnFleetOpen(self, event):
    return
  
  def OnFleetSave(self, event):
    return
  
  def OnFleetSaveAs(self, event):
    return
  
  def OnInventoryManage(self, event):
    dlg = ShipCatalogBase(self, inventory=True)
    dlg.ShowModal()
    return
  
  def OnPrintFleetRoster(self, event):
    #add provision for selecting a fleet
    if (self.selectedFleetIndex == None):
      return
    else:
      dlg = PrintPreviewBase(self, self.fleetsRoster[self.selectedFleetIndex])
      dlg.ShowModal()
    return
  
  def OnPrintOptions(self, event):
    dlg = PrintOptionsBase(self)
    result = dlg.ShowModal()
    if (result):
      #change colors to those selected in the dialog.
      gv.Report_DR_Color = dlg.cpkDRcolor.GetColour()
      gv.Report_CR_Color = dlg.cpkCRcolor.GetColour()
      gv.Report_HP_Color = dlg.cpkHPcolor.GetColour()
      gv.Report_MV_Color = dlg.cpkMVcolor.GetColour()
      gv.Report_TL_Color = dlg.cpkTLcolor.GetColour()
      gv.Report_CP_Color = dlg.cpkCPcolor.GetColour()
      gv.Report_AP_Color = dlg.cpkAPcolor.GetColour()
      gv.Report_PD_Color = dlg.cpkPDcolor.GetColour()
      gv.Report_SR_Color = dlg.cpkSRcolor.GetColour()
      gv.Report_MN_Color = dlg.cpkMNcolor.GetColour()
      gv.Report_WC_Color = dlg.cpkWCcolor.GetColour()
    
      gv.Report_DirectFireHeader_Color = dlg.cpkDirectFireHeader.GetColour()
      gv.Report_DirectFire_Color = dlg.cpkDirectFireEntry.GetColour()
      gv.Report_IndirectFireHeader_Color = dlg.cpkIndirectFireHeader.GetColour()
      gv.Report_IndirectFire_Color = dlg.cpkIndirectFireEntry.GetColour()
      gv.Report_SRS_Color = dlg.cpkSrsEntry.GetColour()
      gv.Report_Hardpoint_Color = dlg.cpkHardpointEntry.GetColour()
      gv.Report_Upgrade_Color = dlg.cpkUpgradeEntry.GetColour()
      gv.Report_MAR_Color = dlg.cpkMarEntry.GetColour()
    
      gv.PrintBW = dlg.cbPrintBW.GetValue()
      if (dlg.rbDetailedMAR.GetValue()):
        gv.PrintMAR = 'Detailed'
      elif (dlg.rbSummaryMAR.GetValue()):
        gv.PrintMAR = 'Summary'
      else:
        gv.PrintMAR = 'Simple'
    dlg.Destroy()
    return
  
  def OnPrinterSetup(self, event):
    
    return
  
  def OnDataImport(self, event):
    return
  
  def OnDataFactionInsignia(self, event):
    dlg = FactionInsigniaBase(self)
    result = dlg.ShowModal()
    return
  
  def OnFleetComposition(self, event):
    dlg = FleetCompositionBase(self)
    saveData = dlg.ShowModal()
    export.SaveFleetCompositionData(gv.FleetCompositionFile, gv.FleetComposition)
    return
  
  def OnDataAddShip(self, event):
    """Add ships to the general catalog"""
    dlg = ShipCatalogBase(self)
    add_ship = dlg.ShowModal()
    return

  def OnDataViewMars(self, event):
    
    return
  
  def OnDataAddMar(self, event):
    dlg = AddNewMarBase(self)
    add_mar = dlg.ShowModal()
    if (add_mar):
      newMar = MarRecord(dlg.txMarNumber.GetValue(), dlg.txMarName.GetValue(),
                         dlg.txMarDescription.GetValue())
      if (dlg.txMarValue1.GetValue() != ''):
        newMar.value1 = dlg.txMarValue1.GetValue()
      if (dlg.txMarValue2.GetValue() != ''):
        newMar.value2 = dlg.txMarValue2.GetValue()
      if (dlg.txMarValue3.GetValue() != ''):
        newMar.value3 = dlg.txMarValue3.GetValue()
      gv.MarList.append(newMar)
    return
  
  def OnDataAddHardpoint(self, event):
    return
  
  def OnHelpAbout(self, event):
    info = wx.AboutDialogInfo()
    info.Name = "Firestorm Fleet Manager"
    info.Copyright = "This software is copyright (C) 2017 Michael Seely\n\nThe Firestorm Armada game rules and statistics are copyright of Spartan Games."
    info.Developers = ["Michael Seely"]
    info.Description = "This software provides fleet building assistance for the 2nd edition of the Firestorm Armada tabletop miniatures game."
    info.Licence = "Gnu General Public License, version 3.\nSee the installation folder for the complete license."
    info.WebSite = "http://www.spartangames.co.uk/products/firestorm-galaxy/firestorm-armada"
    info.Version = str(VERSION)
    wx.AboutBox(info)
    return
#--------------------------------------------------------------------------------------------------

########################################################################
class FactionInsigniaBase(FirestormDialogs.FactionImagesCore):
  """Select the default images to use for faction insignia"""

  #----------------------------------------------------------------------
  def __init__(self, parent):
    """Constructor"""
    FirestormDialogs.FactionImagesCore.__init__(self, parent)
    
    self.bbAquan.SetBitmapLabel(wx.Bitmap(dataStruct.factionBitmapPath['Aquan']))
    self.bbBaKash.SetBitmapLabel(wx.Bitmap(dataStruct.factionBitmapPath["Ba'Kash"]))
    self.bbCorsairs.SetBitmapLabel(wx.Bitmap(dataStruct.factionBitmapPath['OSO Corsairs']))
    self.bbDindrenzi.SetBitmapLabel(wx.Bitmap(dataStruct.factionBitmapPath['Dindrenzi']))
    self.bbDirectorate.SetBitmapLabel(wx.Bitmap(dataStruct.factionBitmapPath['Directorate']))
    self.bbHawker.SetBitmapLabel(wx.Bitmap(dataStruct.factionBitmapPath['Hawker Industries']))
    self.bbKedorian.SetBitmapLabel(wx.Bitmap(dataStruct.factionBitmapPath['Kedorian']))
    self.bbOmnidyne.SetBitmapLabel(wx.Bitmap(dataStruct.factionBitmapPath['OSO Omnidyne']))
    self.bbOroshan.SetBitmapLabel(wx.Bitmap(dataStruct.factionBitmapPath['Oroshan']))
    self.bbPathogen.SetBitmapLabel(wx.Bitmap(dataStruct.factionBitmapPath['Pathogen']))
    self.bbRelthoza.SetBitmapLabel(wx.Bitmap(dataStruct.factionBitmapPath['Relthoza']))
    self.bbRenseSystemNavy.SetBitmapLabel(wx.Bitmap(dataStruct.factionBitmapPath['Rense System Navy']))
    self.bbRyushi.SetBitmapLabel(wx.Bitmap(dataStruct.factionBitmapPath['Ryushi']))
    self.bbSorylian.SetBitmapLabel(wx.Bitmap(dataStruct.factionBitmapPath['Sorylian']))
    self.bbSyndicate.SetBitmapLabel(wx.Bitmap(dataStruct.factionBitmapPath['STL Syndicate']))
    self.bbTarakian.SetBitmapLabel(wx.Bitmap(dataStruct.factionBitmapPath['Tarakian']))
    self.bbTerquai.SetBitmapLabel(wx.Bitmap(dataStruct.factionBitmapPath['Terquai']))
    self.bbTerran.SetBitmapLabel(wx.Bitmap(dataStruct.factionBitmapPath['Terran']))
    self.bbTraders.SetBitmapLabel(wx.Bitmap(dataStruct.factionBitmapPath['STL Traders']))
    self.bbVeydreth.SetBitmapLabel(wx.Bitmap(dataStruct.factionBitmapPath['Veydreth']))
    self.bbWorksRaptor.SetBitmapLabel(wx.Bitmap(dataStruct.factionBitmapPath['Works Raptor']))
    self.bbXelocian.SetBitmapLabel(wx.Bitmap(dataStruct.factionBitmapPath['Xelocian']))
    return
  
  def OnOK(self, event):
    self.EndModal(True)
    return
  
  def OnCancel(self, event):
    self.EndModal(False)
    return
  
  def ChangeImage(self, event):
    wildcard = "PNG files (*.png) |*.png |" \
               "JPG Files (*.jpg) | *.jpg"
    filedlg = wx.FileDialog(self, "Choose image file", gv.images_folder, "", wildcard, wx.OPEN|wx.CHANGE_DIR)
    result = filedlg.ShowModal()
    if (result == wx.ID_CANCEL):  #no file selected
      filedlg.Destroy()
      return
    if (event.GetId() == FirestormDialogs.idFactionAquan):
      
      pass
    
    filedlg.Destroy()
    return
    
  

########################################################################
class FleetCompositionBase(FirestormDialogs.FleetCompositionCore):
  """Defines the compositions for use in fleet building"""
  #----------------------------------------------------------------------
  def __init__(self, parent):
    """Constructor"""
    FirestormDialogs.FleetCompositionCore.__init__(self, parent)
    
    self.lcPatrol1.InsertColumn(0, "Designation            ")
    self.lcPatrol1.InsertColumn(1, "# Allowed")
    self.lcPatrol2.InsertColumn(0, "Designation            ")
    self.lcPatrol2.InsertColumn(1, "# Allowed")
    self.lcPatrol3.InsertColumn(0, "Designation            ")
    self.lcPatrol3.InsertColumn(1, "# Allowed")
    
    self.lcBattle1.InsertColumn(0, "Designation            ")
    self.lcBattle1.InsertColumn(1, "# Allowed")
    self.lcBattle2.InsertColumn(0, "Designation            ")
    self.lcBattle2.InsertColumn(1, "# Allowed")
    self.lcBattle3.InsertColumn(0, "Designation            ")
    self.lcBattle3.InsertColumn(1, "# Allowed")
   
    self.lcGrand1.InsertColumn(0, "Designation            ")
    self.lcGrand1.InsertColumn(1, "# Allowed")
    self.lcGrand2.InsertColumn(0, "Designation            ")
    self.lcGrand2.InsertColumn(1, "# Allowed")
    self.lcGrand3.InsertColumn(0, "Designation            ")
    self.lcGrand3.InsertColumn(1, "# Allowed")
    
    self.lcPatrol1.SetColumnWidth(0, wx.LIST_AUTOSIZE_USEHEADER)
    self.lcPatrol1.SetColumnWidth(1, wx.LIST_AUTOSIZE_USEHEADER)
    self.lcPatrol2.SetColumnWidth(0, wx.LIST_AUTOSIZE_USEHEADER)
    self.lcPatrol2.SetColumnWidth(1, wx.LIST_AUTOSIZE_USEHEADER)
    self.lcPatrol3.SetColumnWidth(0, wx.LIST_AUTOSIZE_USEHEADER)
    self.lcPatrol3.SetColumnWidth(1, wx.LIST_AUTOSIZE_USEHEADER)
    
    self.lcBattle1.SetColumnWidth(0, wx.LIST_AUTOSIZE_USEHEADER)
    self.lcBattle1.SetColumnWidth(1, wx.LIST_AUTOSIZE_USEHEADER)
    self.lcBattle2.SetColumnWidth(0, wx.LIST_AUTOSIZE_USEHEADER)
    self.lcBattle2.SetColumnWidth(1, wx.LIST_AUTOSIZE_USEHEADER)
    self.lcBattle3.SetColumnWidth(0, wx.LIST_AUTOSIZE_USEHEADER)
    self.lcBattle3.SetColumnWidth(1, wx.LIST_AUTOSIZE_USEHEADER)
   
    self.lcGrand1.SetColumnWidth(0, wx.LIST_AUTOSIZE_USEHEADER)
    self.lcGrand1.SetColumnWidth(1, wx.LIST_AUTOSIZE_USEHEADER)
    self.lcGrand2.SetColumnWidth(0, wx.LIST_AUTOSIZE_USEHEADER)
    self.lcGrand2.SetColumnWidth(1, wx.LIST_AUTOSIZE_USEHEADER)
    self.lcGrand3.SetColumnWidth(0, wx.LIST_AUTOSIZE_USEHEADER)
    self.lcGrand3.SetColumnWidth(1, wx.LIST_AUTOSIZE_USEHEADER)

    self.cxFaction.AppendItems(dataStruct.factions)
    self.cxFaction.SetSelection(0)
    
    self.UpdateFleetCompositions()
    self.UpdateSquadronLimits()
    return
  
  def OnHelp(self, event):
    helpText = "This screen sets the allowable ship classes in a patrol, battle, or grand fleet.\n\nThese values are found in the various fleet guides by race or faction.\n\n   To modify the values, first select the faction from the drop-down list, and then select the fleet size (patrol/battle/grand) to modify.\n   To add a squadron to the tier, simply double-click the ship class.  To remove a squadron, right-click on the ship class."  
    dlg = HelpDialog(self, helpText)
    dlg.ShowModal()
    return
  
  def UpdateSquadronLimits(self):
    self.spMinPatrol1.SetValue(self.fleetComposition['minPatrolTier1'])
    self.spMaxPatrol1.SetValue(self.fleetComposition['maxPatrolTier1'])
    self.spMinPatrol2.SetValue(self.fleetComposition['minPatrolTier2'])
    self.spMaxPatrol2.SetValue(self.fleetComposition['maxPatrolTier2'])
    self.spMinPatrol3.SetValue(self.fleetComposition['minPatrolTier3'])
    self.spMaxPatrol3.SetValue(self.fleetComposition['maxPatrolTier3'])
    
    self.spMinBattle1.SetValue(self.fleetComposition['minBattleTier1'])
    self.spMaxBattle1.SetValue(self.fleetComposition['maxBattleTier1'])
    self.spMinBattle2.SetValue(self.fleetComposition['minBattleTier2'])
    self.spMaxBattle2.SetValue(self.fleetComposition['maxBattleTier2'])
    self.spMinBattle3.SetValue(self.fleetComposition['minBattleTier3'])
    self.spMaxBattle3.SetValue(self.fleetComposition['maxBattleTier3'])
    
    self.spMinGrand1.SetValue(self.fleetComposition['minGrandTier1'])
    self.spMaxGrand1.SetValue(self.fleetComposition['maxGrandTier1'])
    self.spMinGrand2.SetValue(self.fleetComposition['minGrandTier2'])
    self.spMaxGrand2.SetValue(self.fleetComposition['maxGrandTier2'])
    self.spMinGrand3.SetValue(self.fleetComposition['minGrandTier3'])
    self.spMaxGrand3.SetValue(self.fleetComposition['maxGrandTier3'])
    return
  
  def OnChangeSquadronLimits(self, event):
    self.fleetComposition['minPatrolTier1'] = self.spMinPatrol1.GetValue()
    self.fleetComposition['maxPatrolTier1'] = self.spMaxPatrol1.GetValue()
    self.fleetComposition['minPatrolTier2'] = self.spMinPatrol2.GetValue()
    self.fleetComposition['maxPatrolTier2'] = self.spMaxPatrol2.GetValue()
    self.fleetComposition['minPatrolTier3'] = self.spMinPatrol3.GetValue()
    self.fleetComposition['maxPatrolTier3'] = self.spMaxPatrol3.GetValue()
    
    self.fleetComposition['minBattleTier1'] = self.spMinBattle1.GetValue()
    self.fleetComposition['maxBattleTier1'] = self.spMaxBattle1.GetValue()
    self.fleetComposition['minBattleTier2'] = self.spMinBattle2.GetValue()
    self.fleetComposition['maxBattleTier2'] = self.spMaxBattle2.GetValue()
    self.fleetComposition['minBattleTier3'] = self.spMinBattle3.GetValue()
    self.fleetComposition['maxBattleTier3'] = self.spMaxBattle3.GetValue()
    
    self.fleetComposition['minGrandTier1'] = self.spMinGrand1.GetValue()
    self.fleetComposition['maxGrandTier1'] = self.spMaxGrand1.GetValue()
    self.fleetComposition['minGrandTier2'] = self.spMinGrand2.GetValue()
    self.fleetComposition['maxGrandTier2'] = self.spMaxGrand2.GetValue()
    self.fleetComposition['minGrandTier3'] = self.spMinGrand3.GetValue()
    self.fleetComposition['maxGrandTier3'] = self.spMaxGrand3.GetValue()
    return
  
  def OnChooseFaction(self, event):
    selection = self.cxFaction.GetStringSelection()
    for factionID, fleetComp in gv.FleetComposition.iteritems():
      if (factionID == selection):
        self.fleetComposition = gv.FleetComposition[factionID]
        self.UpdateFleetCompositions()
        self.UpdateSquadronLimits()
        break
    return
  
  def OnActivateItem(self, event):
    if (event.GetId() == FirestormDialogs.idPatrol1):
      key = self.lcPatrol1.GetItemText(event.m_itemIndex)
      self.fleetComposition['patrolTier1'][key] += 1
    elif (event.GetId() == FirestormDialogs.idPatrol2):
      key = self.lcPatrol2.GetItemText(event.m_itemIndex)
      self.fleetComposition['patrolTier2'][key] += 1
    elif (event.GetId() == FirestormDialogs.idPatrol3):
      key = self.lcPatrol3.GetItemText(event.m_itemIndex)
      self.fleetComposition['patrolTier3'][key] += 1
    elif (event.GetId() == FirestormDialogs.idBattle1):
      key = self.lcBattle1.GetItemText(event.m_itemIndex)
      self.fleetComposition['battleTier1'][key] += 1
    elif (event.GetId() == FirestormDialogs.idBattle2):
      key = self.lcBattle2.GetItemText(event.m_itemIndex)
      self.fleetComposition['battleTier2'][key] += 1
    elif (event.GetId() == FirestormDialogs.idBattle3):
      key = self.lcBattle3.GetItemText(event.m_itemIndex)
      self.fleetComposition['battleTier3'][key] += 1
    elif (event.GetId() == FirestormDialogs.idGrand1):
      key = self.lcGrand1.GetItemText(event.m_itemIndex)
      self.fleetComposition['grandTier1'][key] += 1
    elif (event.GetId() == FirestormDialogs.idGrand2):
      key = self.lcGrand2.GetItemText(event.m_itemIndex)
      self.fleetComposition['grandTier2'][key] += 1
    elif (event.GetId() == FirestormDialogs.idGrand3):
      key = self.lcGrand3.GetItemText(event.m_itemIndex)
      self.fleetComposition['grandTier3'][key] += 1
      
    self.UpdateFleetCompositions()
    return
  
  def OnRightClick(self, event):
    if (event.GetId() == FirestormDialogs.idPatrol1):
      key = self.lcPatrol1.GetItemText(event.m_itemIndex)
      self.fleetComposition['patrolTier1'][key] -= 1
      self.fleetComposition['patrolTier1'][key] = max(self.fleetComposition['patrolTier1'][key], 0)
    elif (event.GetId() == FirestormDialogs.idPatrol2):
      key = self.lcPatrol2.GetItemText(event.m_itemIndex)
      self.fleetComposition['patrolTier2'][key] -= 1
      self.fleetComposition['patrolTier2'][key] = max(self.fleetComposition['patrolTier2'][key], 0)
    elif (event.GetId() == FirestormDialogs.idPatrol3):
      key = self.lcPatrol3.GetItemText(event.m_itemIndex)
      self.fleetComposition['patrolTier3'][key] -= 1
      self.fleetComposition['patrolTier3'][key] = max(self.fleetComposition['patrolTier3'][key], 0)
    elif (event.GetId() == FirestormDialogs.idBattle1):
      key = self.lcBattle1.GetItemText(event.m_itemIndex)
      self.fleetComposition['battleTier1'][key] -= 1
      self.fleetComposition['battleTier1'][key] = max(self.fleetComposition['battleTier1'][key], 0)
    elif (event.GetId() == FirestormDialogs.idBattle2):
      key = self.lcBattle2.GetItemText(event.m_itemIndex)
      self.fleetComposition['battleTier2'][key] -= 1
      self.fleetComposition['battleTier2'][key] = max(self.fleetComposition['battleTier2'][key], 0)
    elif (event.GetId() == FirestormDialogs.idBattle3):
      key = self.lcBattle3.GetItemText(event.m_itemIndex)
      self.fleetComposition['battleTier3'][key] -= 1
      self.fleetComposition['battleTier3'][key] = max(self.fleetComposition['battleTier3'][key], 0)
    elif (event.GetId() == FirestormDialogs.idGrand1):
      key = self.lcGrand1.GetItemText(event.m_itemIndex)
      self.fleetComposition['grandTier1'][key] -= 1
      self.fleetComposition['grandTier1'][key] = max(self.fleetComposition['grandTier1'][key], 0)
    elif (event.GetId() == FirestormDialogs.idGrand2):
      key = self.lcGrand2.GetItemText(event.m_itemIndex)
      self.fleetComposition['grandTier2'][key] -= 1
      self.fleetComposition['grandTier2'][key] = max(self.fleetComposition['grandTier2'][key], 0)
    elif (event.GetId() == FirestormDialogs.idGrand3):
      key = self.lcGrand3.GetItemText(event.m_itemIndex)
      self.fleetComposition['grandTier3'][key] -= 1
      self.fleetComposition['grandTier3'][key] = max(self.fleetComposition['grandTier3'][key], 0)
    self.UpdateFleetCompositions()
    return
  
  def HideUnavailableClasses(self, event):
    self.UpdateFleetCompositions()
    return
  
  def UpdateFleetCompositions(self):
    
    self.lcPatrol1.DeleteAllItems()
    self.lcPatrol2.DeleteAllItems()
    self.lcPatrol3.DeleteAllItems()
    self.lcBattle1.DeleteAllItems()
    self.lcBattle2.DeleteAllItems()
    self.lcBattle3.DeleteAllItems()
    self.lcGrand1.DeleteAllItems()
    self.lcGrand2.DeleteAllItems()
    self.lcGrand3.DeleteAllItems()
    
    #identify the current selected race
    for faction, fleetComp in gv.FleetComposition.iteritems():
      if (faction == self.cxFaction.GetStringSelection()):
        self.fleetComposition = fleetComp
        break    
        
    for key,value in self.fleetComposition.iteritems():
      if ('max' in key) or ('min' in key) or (key == 'faction'):
        continue
      if (key == 'patrolTier1'):
        for key2,value2 in sorted(self.fleetComposition[key].iteritems(), reverse=True):
          if (value2 == 0 and self.cbHideUnavailableClasses.IsChecked()):
            continue
          listitem = wx.ListItem()
          listitem.SetText(key)
          index = self.lcPatrol1.InsertItem(listitem)
          self.lcPatrol1.SetStringItem(index, 0, key2)
          self.lcPatrol1.SetStringItem(index, 1, str(value2))
      elif (key == 'patrolTier2'):
        for key2,value2 in sorted(self.fleetComposition[key].iteritems(), reverse=True):
          if (value2 == 0 and self.cbHideUnavailableClasses.IsChecked()):
            continue
          listitem = wx.ListItem()
          listitem.SetText(key)
          index = self.lcPatrol2.InsertItem(listitem)
          self.lcPatrol2.SetStringItem(index, 0, key2)
          self.lcPatrol2.SetStringItem(index, 1, str(value2))
      elif (key == 'patrolTier3'):
        for key2,value2 in sorted(self.fleetComposition[key].iteritems(), reverse=True):
          if (value2 == 0 and self.cbHideUnavailableClasses.IsChecked()):
            continue
          listitem = wx.ListItem()
          listitem.SetText(key)
          index = self.lcPatrol3.InsertItem(listitem)
          self.lcPatrol3.SetStringItem(index, 0, key2)
          self.lcPatrol3.SetStringItem(index, 1, str(value2))
      elif (key == 'battleTier1'):
        for key2,value2 in sorted(self.fleetComposition[key].iteritems(), reverse=True):
          if (value2 == 0 and self.cbHideUnavailableClasses.IsChecked()):
            continue
          listitem = wx.ListItem()
          listitem.SetText(key)
          index = self.lcBattle1.InsertItem(listitem)
          self.lcBattle1.SetStringItem(index, 0, key2)
          self.lcBattle1.SetStringItem(index, 1, str(value2))
      elif (key == 'battleTier2'):
        for key2,value2 in sorted(self.fleetComposition[key].iteritems(), reverse=True):
          if (value2 == 0 and self.cbHideUnavailableClasses.IsChecked()):
            continue
          listitem = wx.ListItem()
          listitem.SetText(key)
          index = self.lcBattle2.InsertItem(listitem)
          self.lcBattle2.SetStringItem(index, 0, key2)
          self.lcBattle2.SetStringItem(index, 1, str(value2))
      elif (key == 'battleTier3'):
        for key2,value2 in sorted(self.fleetComposition[key].iteritems(), reverse=True):
          if (value2 == 0 and self.cbHideUnavailableClasses.IsChecked()):
            continue
          listitem = wx.ListItem()
          listitem.SetText(key)
          index = self.lcBattle3.InsertItem(listitem)
          self.lcBattle3.SetStringItem(index, 0, key2)
          self.lcBattle3.SetStringItem(index, 1, str(value2))
      elif (key == 'grandTier1'):
        for key2,value2 in sorted(self.fleetComposition[key].iteritems(), reverse=True):
          if (value2 == 0 and self.cbHideUnavailableClasses.IsChecked()):
            continue
          listitem = wx.ListItem()
          listitem.SetText(key)
          index = self.lcGrand1.InsertItem(listitem)
          self.lcGrand1.SetStringItem(index, 0, key2)
          self.lcGrand1.SetStringItem(index, 1, str(value2))
      elif (key == 'grandTier2'):
        for key2,value2 in sorted(self.fleetComposition[key].iteritems(), reverse=True):
          if (value2 == 0 and self.cbHideUnavailableClasses.IsChecked()):
            continue
          listitem = wx.ListItem()
          listitem.SetText(key)
          index = self.lcGrand2.InsertItem(listitem)
          self.lcGrand2.SetStringItem(index, 0, key2)
          self.lcGrand2.SetStringItem(index, 1, str(value2))
      elif (key == 'grandTier3'):
        for key2,value2 in sorted(self.fleetComposition[key].iteritems(), reverse=True):
          if (value2 == 0 and self.cbHideUnavailableClasses.IsChecked()):
            continue
          listitem = wx.ListItem()
          listitem.SetText(key)
          index = self.lcGrand3.InsertItem(listitem)
          self.lcGrand3.SetStringItem(index, 0, key2)
          self.lcGrand3.SetStringItem(index, 1, str(value2))
    return
  
  def OnCancel(self, event):
    self.EndModal(False)
    return
  
  def OnOK(self, event):
    self.EndModal(True)
    return
########################################################################
class FleetEditorBase(FirestormDialogs.FleetEditorCore):
  """"""

  #----------------------------------------------------------------------
  def __init__(self, parent, fleet=None):
    """Constructor"""
    FirestormDialogs.FleetEditorCore.__init__(self, parent)
    self.inventoryRoot = self.trInventoryDisplay.AddRoot("Core Faction")
    self.inventoryTier1 = self.trInventoryDisplay.AppendItem(self.inventoryRoot, "Tier 1")
    self.inventoryTier2 = self.trInventoryDisplay.AppendItem(self.inventoryRoot, "Tier 2")
    self.inventoryTier3 = self.trInventoryDisplay.AppendItem(self.inventoryRoot, "Tier 3")
    
    self.fleetRoot = self.trFleetDisplay.AddRoot("Fleet")
    self.fleetTier1 = self.trFleetDisplay.AppendItem(self.fleetRoot, "Tier 1")
    self.fleetTier2 = self.trFleetDisplay.AppendItem(self.fleetRoot, "Tier 2")
    self.fleetTier3 = self.trFleetDisplay.AppendItem(self.fleetRoot, "Tier 3")
    
    self.trFleetDisplay.ExpandAll()
    self.trInventoryDisplay.ExpandAll()
    
    self.selectedFleetShipID = None      #treeID of the item selected
    self.selectedInventoryShipID = None  #treeID of the item selected
    self.shipList = []   # shipRecord      #this list contains the ships showing in the selection list
    self.cxCoreFaction.AppendItems(dataStruct.factions)
    self.cxTokenType1.AppendItems(dataStruct.srsTokens)
    self.cxTokenType2.AppendItems(dataStruct.srsTokens)
    self.cxTokenType1.SetSelection(0)  #default to 'None'
    self.cxTokenType2.SetSelection(0)  #default to 'None'

    self.bChangeSquadronImage.Enable(False)
    self.bRevertToDefaultImage.Enable(False)
    
    if (fleet):
      self.fleet = fleet.__deepcopy__(fleet)
      #if we send in a fleet, it should update with all appropriate values.
      self.cbRestrictInventory.SetValue(self.fleet.restrictToInventory)
      self.cbIncludeMercenaries.SetValue(self.fleet.includeMercenaries)
      self.cbRestrictToCore.SetValue(self.fleet.restrictToCore)
      self.spMFV.SetValue(self.fleet.MFV)
      self.txFleetNotes.SetValue(self.fleet.notes)
      
      if (self.fleet.coreFaction != ''):
        self.cxCoreFaction.SetStringSelection(self.fleet.coreFaction)
        self.OnSelectFaction()
      self.stCurrentFleetValue.SetLabel(str(self.fleet.CFV))
      self.txFleetName.SetValue(self.fleet.fleetName)
      self.UpdateFleetList()
      self.UpdateFleetCost()
    else:
      self.fleet = dataStruct.FleetRecord()
      self.cxCoreFaction.SetStringSelection(gv.defaultCoreFaction)
      self.OnSelectFaction()
      self.fleet.SetFleetMFV(self.spMFV.GetValue())

    self.pnAccompaniment.SetBackgroundColour(gv.FleetBackgroundColor)
    self.pnHardpoints.SetBackgroundColour(gv.FleetBackgroundColor)
    self.pnUpgrades.SetBackgroundColour(gv.FleetBackgroundColor)
    self.pnSRS.SetBackgroundColour(gv.FleetBackgroundColor)
    self.trFleetDisplay.SetBackgroundColour(gv.FleetBackgroundColor)
    self.trInventoryDisplay.SetBackgroundColour(gv.InventoryBackgroundColor)
    
    nullShip = dataStruct.ShipRecord()
    self.SetHardpoints(nullShip)
    self.SetUpgrades(nullShip)
    self.SetAccompaniment(nullShip)
    self.SetSRS(nullShip)
    
    self.CheckFleet()
    self.MarDisplay = None
    return
  
  def OnSaveFleet(self, event):
    #once back from the FleetEditor, remove the following references, or it will crash when doing a deepcopy
    self.fleet.notes = self.txFleetNotes.GetValue()
    for squadron in self.fleet.squadronList:
      squadron.treeItemShip = None
      squadron.treeItemSquadron = None
      squadron.treeItemShipAccomp = None
      squadron.treeItemShipHeavyCruiser = None
    
    self.EndModal(True)
    return
  
  def OnClose(self, event):
    self.EndModal(False)
    return
  
  def OnSelectFaction(self, event=None):
    alliance = []
    self.shipList = []
    self.trFleetDisplay.Enable(True)
    self.trInventoryDisplay.Enable(True)
    
    self.fleet.coreFaction = self.cxCoreFaction.GetStringSelection()
    self.fleet.restrictToCore = self.cbRestrictToCore.IsChecked()
    self.fleet.restrictToInventory = self.cbRestrictInventory.IsChecked()
    self.fleet.includeMercenaries = self.cbIncludeMercenaries.IsChecked()
    
    if (self.fleet.coreFaction not in dataStruct.coreFactions):
      self.fleet.includeNaturalAllies = False
      self.cbIncludeNaturalAllies.Enable(False)
      self.cbIncludeNaturalAllies.SetLabel("No Natural Allies")
    else:
      self.fleet.includeNaturalAllies = self.cbIncludeNaturalAllies.IsChecked()
      self.cbIncludeNaturalAllies.Enable(True)
      self.cbIncludeNaturalAllies.SetLabel("Include Natural Allies ({0})".format(dataStruct.naturalAllies[self.fleet.coreFaction]))
    
    self.fleet.SetFleetMFV(self.spMFV.GetValue())
    self.fleet.SetFleetTacticsAndCommand()
    
    self.FilterShipList()
    self.UpdateInventoryList()
    self.UpdateFleetList()
    
    self.CheckFleet()
    return
  
  def OnSelectFleetShip(self, event):
    ship = dataStruct.ShipRecord()
    if (not isinstance(event, wx.TreeEvent)):
      event.Skip()
      if (not self.selectedFleetShipID):
        self.bChangeSquadronImage.Enable(False)
        self.bRevertToDefaultImage.Enable(False)
        return
      treeData = self.trFleetDisplay.GetItemData(self.selectedFleetShipID)
    else:
      self.selectedFleetShipID = event.GetItem()
      treeData = self.trFleetDisplay.GetItemData(self.selectedFleetShipID)
    ship = treeData.GetData()
    if (not ship):
      self.bChangeSquadronImage.Enable(False)
      self.bRevertToDefaultImage.Enable(False)
      nullShip = dataStruct.ShipRecord()
      self.SetHardpoints(nullShip)
      self.SetUpgrades(nullShip)
      self.SetAccompaniment(nullShip)
      self.SetSRS(nullShip)
      return
    
    self.bChangeSquadronImage.Enable(True)
    if (isinstance(ship, dataStruct.ShipRecord) and (ship.imageCustomFilename)):
      self.bRevertToDefaultImage.Enable(True)
    else:
      self.bRevertToDefaultImage.Enable(False)
    self.SetAccompaniment(ship)
    self.SetUpgrades(ship)
    self.SetHardpoints(ship)
    self.SetSRS(ship)
    self.UpdateShipDetail(ship, 'fleet')
    
    self.bAddSquadronShip.Enable(False)
    self.bRemoveSquadronShip.Enable(False)

    for squadron in self.fleet.squadronList:
      if (squadron.ship == ship):
        if (squadron.size > 1):
          self.bRemoveSquadronShip.Enable(True)
        else:
          self.bRemoveSquadronShip.Enable(False)
          
        if (squadron.heavyCruiser) and (squadron.size+1 >= ship.squadronSizeMax):
          self.bAddSquadronShip.Enable(False)
        else:
          self.bAddSquadronShip.Enable(True)
          
        if (squadron.size >= ship.squadronSizeMax):
          self.bAddSquadronShip.Enable(False)
        else:
          self.bAddSquadronShip.Enable(True)
        break
    event.Skip()
    return
  
  
  def OnRemoveSquadronShip(self, event):
    ship = dataStruct.ShipRecord()
    treeData = self.trFleetDisplay.GetItemData(self.selectedFleetShipID)
    ship = treeData.GetData()
    for squadron in self.fleet.squadronList:
      if (squadron.ship == ship):
        squadron.size -= 1
        if (squadron.size == 1):
          self.bRemoveSquadronShip.Enable(False)
        self.UpdateFleetCost()
        self.trFleetDisplay.SetItemText(squadron.treeItemShip,"{0}x {1} ({2} {3})".format(squadron.size,squadron.ship.className,squadron.ship.faction,squadron.ship.designation))
        self.CheckFleet()
        break
    return
  
  def OnAddSquadronShip(self, event):
    ship = dataStruct.ShipRecord()
    treeData = self.trFleetDisplay.GetItemData(self.selectedFleetShipID)
    ship = treeData.GetData()
    for squadron in self.fleet.squadronList:
      if (squadron.ship == ship):
        squadron.size += 1
        if (squadron.size >= squadron.ship.squadronSizeMax):
          self.bAddSquadronShip.Enable(False)
        if (squadron.heavyCruiser) and (squadron.size+1 >= ship.squadronSizeMax):
          self.bAddSquadronShip.Enable(False)
        self.bRemoveSquadronShip.Enable(True)
        self.UpdateFleetCost()
        self.trFleetDisplay.SetItemText(squadron.treeItemShip,"{0}x {1} ({2} {3})".format(squadron.size,squadron.ship.className,squadron.ship.faction,squadron.ship.designation))
        self.CheckFleet()
        break
    return
  
  def OnAddShipFromInventory(self, event):
    ship = dataStruct.ShipRecord()
    squadron = dataStruct.squadronRecord()
    itemID = event.GetItem()
    treeData = self.trInventoryDisplay.GetItemData(itemID)
    ship = copy.deepcopy(treeData.GetData())
    if (not ship):
      return
    else:
      #add new squadron of ships
      if (ship.squadronSizeMax > 1):
        if (ship.designation == 'Cruiser') or (ship.designation == 'Light_Cruiser'):
          #Cruiser squadron may have 1 Heavy_Cruiser or 1 Escort_Carrier
          #Light_Cruiser squadron may have 1 Cruiser, 1 Gunship, or 1 Escort_Carrier
          mixedCruiserFlag = True
        else:
          mixedCruiserFlag = False
        mixedList = []
        if (ship.designation == 'Cruiser'):
          for heavyCruiserShip in gv.ShipClassList:
            if (ship.faction == heavyCruiserShip.faction) and (heavyCruiserShip.designation == 'Heavy_Cruiser'):
              mixedList.append(heavyCruiserShip)
            if (ship.faction == heavyCruiserShip.faction) and (heavyCruiserShip.designation == 'Escort_Carrier'):
              mixedList.append(heavyCruiserShip)
        elif (ship.designation == 'Light_Cruiser'):
          for heavyCruiserShip in gv.ShipClassList:
            if (ship.faction == heavyCruiserShip.faction) and (heavyCruiserShip.designation == 'Cruiser'):
              mixedList.append(heavyCruiserShip)
            if (ship.faction == heavyCruiserShip.faction) and (heavyCruiserShip.designation == 'Gunship'):
              mixedList.append(heavyCruiserShip)
            if (ship.faction == heavyCruiserShip.faction) and (heavyCruiserShip.designation == 'Escort_Carrier'):
              mixedList.append(heavyCruiserShip)
        dlg = SelectSquadronSizeBase(self, ship.squadronSizeMin, ship.squadronSizeMax, mixedCruiserFlag, mixedList)
        if (dlg.ShowModal()):
          squadron.size = dlg.spSquadronSize.GetValue()
          if (mixedCruiserFlag):  #find if a heavy was selected
            selectHeavy = dlg.cxHeavyCruiser.GetStringSelection()
            if (selectHeavy != 'None'):
              for heavyCruiserShip in mixedList:
                if (heavyCruiserShip.className in selectHeavy):
                  squadron.heavyCruiser = heavyCruiserShip
                  squadron.size -= 1  #squadron.size is the size not including the heavy
                  break
          squadron.ship = ship
          self.fleet.AddSquadron(squadron)
        else:
          return
      else:
        squadron.size = 1
        squadron.ship = ship
        self.fleet.AddSquadron(squadron)
        
    
    self.UpdateFleetList()
    self.UpdateFleetCost()
    self.CheckFleet()
    return
  
  def UpdateFleetCost(self):
    """Updates the cost of each squadron in the fleet, and updates the fleet display with costs"""
    squadron = dataStruct.squadronRecord()
    squadron.ship = dataStruct.ShipRecord()
    
    fleetCost = 0
    shipBaseCost = 0
    accompCost = 0
    
    for squadron in self.fleet.squadronList:
      squadron.cost = 0
      shipBaseCost = squadron.ship.baseCost
      accompCost = 0
      #check upgrade costs
      for upgrade in squadron.ship.currentUpgrades:
        shipBaseCost += upgrade.cost
      #check hardpoint costs
      for hardpoint in squadron.ship.currentHardpoints:
        shipBaseCost += (hardpoint.cost * hardpoint.numSelected)
      #check accompaniment costs
      if (squadron.ship.currentAccompaniment):
        accompBaseCost = squadron.ship.currentAccompaniment.ship.baseCost

        #check upgrade costs on Accompaniment
        for upgrade in squadron.ship.currentAccompaniment.ship.currentUpgrades:
          accompBaseCost += upgrade.cost

        #check hardpoint costs on Accompaniment
        for hardpoint in squadron.ship.currentAccompaniment.ship.currentHardpoints:
          accompBaseCost += (hardpoint.cost * hardpoint.numSelected)

        #check srs costs on Accompaniment
        for srs in squadron.ship.currentAccompaniment.ship.currentSRS:
          accompBaseCost += srs.cost
        accompCost = (accompBaseCost * squadron.ship.currentAccompaniment.numSelected)

      #check SRS costs on main squadron
      srsCost = 0
      for srs in squadron.ship.currentSRS:
        srs.cost = srs.numWings * 5
        srsCost += srs.cost
        
      shipBaseCost += srsCost
      squadron.ship.currentCost = shipBaseCost
      squadron.cost += accompCost
      squadron.cost += (squadron.size * shipBaseCost)
      if (squadron.heavyCruiser):
        heavyBaseCost = squadron.heavyCruiser.baseCost
        #check upgrade costs
        for upgrade in squadron.heavyCruiser.currentUpgrades:
          heavyBaseCost += upgrade.cost
        #check hardpoint costs
        for hardpoint in squadron.heavyCruiser.currentHardpoints:
          heavyBaseCost += (hardpoint.cost * hardpoint.numSelected)
        #check accompaniment costs
        if (squadron.heavyCruiser.currentAccompaniment):
          heavyBaseCost += (squadron.heavyCruiser.currentAccompaniment.baseCost* squadron.heavyCruiser.currentAccompaniment.numSelected)
        #check SRS costs
        for srs in squadron.heavyCruiser.currentSRS:
          heavyBaseCost += srs.cost
        squadron.heavyCruiser.currentCost = heavyBaseCost
        squadron.cost += heavyBaseCost
      self.trFleetDisplay.SetItemText(squadron.treeItemSquadron, "{0} squadron ({1} points)".format(squadron.ship.designation,squadron.cost))
      fleetCost += squadron.cost
    self.fleet.CFV = fleetCost
    self.stCurrentFleetValue.SetLabel(" "+str(fleetCost)+" ")
    return
  
  def SetFleetName(self, event):
    self.fleet.fleetName = self.txFleetName.GetValue()
    self.CheckFleet()
    return

  def CheckFleet(self):
    """Validate the current fleet build.  Display errors where needed.  This should
    be called whenever the fleet has changed."""
    ship = dataStruct.ShipRecord()
    squadron = dataStruct.squadronRecord()
    factionPointTotals = copy.deepcopy(dataStruct.fleetTactics)
    for race,value in factionPointTotals.iteritems():
      factionPointTotals[race] = 0
      
    
    self.txErrors.Clear()
    errorDisplay = ''
    coreFaction = self.cxCoreFaction.GetStringSelection()
    
    if (self.fleet.fleetName == ''):
      fleetName = self.txFleetName.GetValue()
      if (fleetName != ''):
        self.fleet.fleetName = fleetName
      else:
        errorDisplay += 'Please enter a name for this fleet.\n'
        
    if (self.fleet.coreFaction == ''):
      errorDisplay += 'Please select a core faction for the fleet.\n'
    
    #check if minimum number of squadrons have been met
    numTier1 = 0
    numTier2 = 0
    numTier3 = 0
    
    #check tier 1 requirements
    for shipClass,value in self.fleet.allowedTier1.iteritems():
      if (self.fleet.allowedTier1[shipClass]):
        numTier1 += self.fleet.squadronCount[shipClass]
        if (self.fleet.squadronCount[shipClass] > value):  #too many squadrons
          errorDisplay += "Fleet has too many {0} squadrons selected for Tier 1.\n".format(shipClass)
    
    #check tier 2 requirements
    for shipClass,value in self.fleet.allowedTier2.iteritems():
      if (self.fleet.allowedTier2[shipClass]):
        numTier2 += self.fleet.squadronCount[shipClass]
        if (self.fleet.squadronCount[shipClass] > value):  #too many squadrons
          errorDisplay += "Fleet has too many {0} squadrons selected for Tier 2.\n".format(shipClass)
        
    #check tier 3 requirements
    for shipClass,value in self.fleet.allowedTier3.iteritems():
      if (self.fleet.allowedTier3[shipClass]):
        numTier3 += self.fleet.squadronCount[shipClass]
        if (self.fleet.squadronCount[shipClass] > value):  #too many squadrons
          errorDisplay += "Fleet has too many {0} squadrons selected for Tier 3.\n".format(shipClass)
    
    if (numTier1 > self.fleet.maxTier1):
      errorDisplay += "There are too many squadrons selected in Tier 1 (maximum {0} allowed).\n".format(self.fleet.maxTier1)
    if (numTier2 > self.fleet.maxTier2):
      errorDisplay += "There are too many squadrons selected in Tier 2 (maximum {0} allowed).\n".format(self.fleet.maxTier2)
    if (numTier3 > self.fleet.maxTier3):
      errorDisplay += "There are too many squadrons selected in Tier 3 (maximum {0} allowed).\n".format(self.fleet.maxTier3)
      
    if (numTier1 < self.fleet.minTier1):
      errorDisplay += "Need to select additional squadrons for Tier 1 (minimum {0} required).\n".format(self.fleet.minTier1)
    if (numTier2 < self.fleet.minTier2):
      errorDisplay += "Need to select additional squadrons for Tier 2 (minimum {0} required).\n".format(self.fleet.minTier2)
    if (numTier3 < self.fleet.minTier3):
      errorDisplay += "Need to select additional squadrons for Tier 3 (minimum {0} required).\n".format(self.fleet.minTier3)
      
    
    #check point cost of the fleet
    if (self.fleet.CFV> self.fleet.MFV):
      errorDisplay += 'The cost of the current fleet is greater than the Maximum Fleet Value by {0} points.\n'.format((self.fleet.CFV - self.fleet.MFV))
      self.stCurrentFleetValue.SetBackgroundColour(wx.RED)
      self.stCurrentFleetValue.SetForegroundColour(wx.WHITE)
    else:
      self.stCurrentFleetValue.SetBackgroundColour(wx.GREEN)
      self.stCurrentFleetValue.SetForegroundColour(wx.BLACK)
  
    coreTier1 = 0  #minimum tier requirements must be filled by the core faction
    coreTier2 = 0
    coreTier3 = 0

    for squadron in self.fleet.squadronList:
      #count coreFaction squadrons in each tier
      if (squadron.ship.faction == self.fleet.coreFaction):
        if (squadron.ship.tier == 1):
          coreTier1 += 1
        elif (squadron.ship.tier ==2):
          coreTier2 += 1
        elif (squadron.ship.tier == 3):
          coreTier3 += 1
      #check if either/or upgrades have illegal selections
      blueCount = []
      redCount = []
      greenCount = []
      orangeCount = []
      purpleCount = []
      for upgrade in squadron.ship.currentUpgrades:
        if (upgrade.color == gv.XORblue):
          blueCount.append(upgrade.text)
        elif (upgrade.color == gv.XORred):
          redCount.append(upgrade.text)
        elif (upgrade.color == gv.XORgreen):
          greenCount.append(upgrade.text)
        elif (upgrade.color == gv.XORorange):
          orangeCount.append(upgrade.text)
        elif (upgrade.color == gv.XORpurple):
          purpleCount.append(upgrade.text)
      if (len(blueCount) > 1):
        errorDisplay += '{2} has too many upgrades: Select EITHER {0} OR {1}\n'.format(blueCount[0], blueCount[1], squadron.ship.className)
      if (len(redCount) > 1):
        errorDisplay += '{2} has too many upgrades:: Select EITHER {0} OR {1}\n'.format(redCount[0], redCount[1], squadron.ship.className)
      if (len(greenCount) > 1):
        errorDisplay += '{2} has too many upgrades: Select EITHER {0} OR {1}\n'.format(greenCount[0], greenCount[1], squadron.ship.className)
      if (len(orangeCount) > 1):
        errorDisplay += '{2} has too many upgrades: Select EITHER {0} OR {1}\n'.format(orangeCount[0], orangeCount[1], squadron.ship.className)
      if (len(purpleCount) > 1):
        errorDisplay += '{2} has too many upgrades: Select EITHER {0} OR {1}\n'.format(purpleCount[0], purpleCount[1], squadron.ship.className)
        
      if (squadron.heavyCruiser):
        blueCountHC = []
        redCountHC = []
        greenCountHC = []
        orangeCountHC = []
        purpleCountHC = []
        for upgrade in squadron.heavyCruiser.currentUpgrades:
          if (upgrade.color == gv.XORblue):
            blueCountHC.append(upgrade.text)
          elif (upgrade.color == gv.XORred):
            redCountHC.append(upgrade.text)
          elif (upgrade.color == gv.XORgreen):
            greenCountHC.append(upgrade.text)
          elif (upgrade.color == gv.XORorange):
            orangeCountHC.append(upgrade.text)
          elif (upgrade.color == gv.XORpurple):
            purpleCountHC.append(upgrade.text)
        if (len(blueCountHC) > 1):
          errorDisplay += '{2} has too many upgrades: Select EITHER {0} OR {1}\n'.format(blueCountHC[0], blueCountHC[1], squadron.heavyCruiser.className)
        if (len(redCountHC) > 1):
          errorDisplay += '{2} has too many upgrades:: Select EITHER {0} OR {1}\n'.format(redCountHC[0], redCountHC[1], squadron.heavyCruiser.className)
        if (len(greenCountHC) > 1):
          errorDisplay += '{2} has too many upgrades: Select EITHER {0} OR {1}\n'.format(greenCountHC[0], greenCountHC[1], squadron.heavyCruiser.className)
        if (len(orangeCountHC) > 1):
          errorDisplay += '{2} has too many upgrades: Select EITHER {0} OR {1}\n'.format(orangeCountHC[0], orangeCountHC[1], squadron.heavyCruiser.className)
        if (len(purpleCountHC) > 1):
          errorDisplay += '{2} has too many upgrades: Select EITHER {0} OR {1}\n'.format(purpleCountHC[0], purpleCountHC[1], squadron.heavyCruiser.className)
        #check hardpoint selections
        if (len(squadron.ship.currentHardpoints) > squadron.ship.maxHardPoints):
          errorDisplay += '{0} has too many hardpoints selected.\n'.format(squadron.ship.className)
        if (squadron.heavyCruiser):
          if (len(squadron.heavyCruiser.currentHardpoints) > squadron.heavyCruiser.maxHardPoints):
            errorDisplay += '{0} has too many hardpoints selected.\n'.format(squadron.heavyCruiser.className)
        if (squadron.ship.currentAccompaniment):
          if (len(squadron.ship.currentAccompaniment.ship.currentHardpoints) > squadron.ship.currentAccompaniment.ship.maxHardPoints):
            errorDisplay += '{0} has too many hardpoints selected.\n'.format(squadron.ship.currentAccompaniment.ship.className)
      
      #check wing count
      wingCount = 0
      HCwingCount = 0  #heavy cruiser (or other mixed cruiser)
      AccompWingCount = 0
      for srs in squadron.ship.currentSRS:
        wingCount += srs.numWings
      if (wingCount > squadron.ship.WC):
        errorDisplay += "{0} has too many wings ({1}) assigned.\n".format(squadron.ship.className, wingCount)
      if (squadron.heavyCruiser):
        for srs in squadron.heavyCruiser.currentSRS:
          HCwingCount += srs.numWings
        if (HCwingCount > squadron.heavyCruiser.WC):
          errorDisplay += "{0} has too many wings ({1}) assigned.\n".format(squadron.heavyCruiser.className, HCwingCount)
      if (squadron.ship.currentAccompaniment):
        for srs in squadron.ship.currentAccompaniment.ship.currentSRS:
          AccompWingCount += srs.numWings
        if (AccompWingCount > squadron.ship.currentAccompaniment.ship.WC):
          errorDisplay += "{0} has too many wings ({1}) assigned.\n".format(squadron.ship.currentAccompaniment.ship.className, AccompWingCount)
      
      #check points used by non-core races
      factionPointTotals[squadron.ship.faction] += squadron.cost
    
    #check for minimum tier amounts filled by core faction squadrons
    if (coreTier1 < self.fleet.minTier1):
      errorDisplay += "Minimum tier requirements {1} for Tier 1 must be occupied by {0} squadrons.\n".format(coreFaction, self.fleet.minTier1)
    if (coreTier2 < self.fleet.minTier2):
      errorDisplay += "Minimum tier requirements {1} for Tier 2 must be occupied by {0} squadrons.\n".format(coreFaction, self.fleet.minTier2)
    if (coreTier3 < self.fleet.minTier3):
      errorDisplay += "Minimum tier requirements {1} for Tier 3 must be occupied by {0} squadrons.\n".format(coreFaction, self.fleet.minTier3)
    #check natural allies in a core fleet
    if (coreFaction in dataStruct.naturalAllies):
      if (factionPointTotals[dataStruct.naturalAllies[coreFaction]] > (0.5*self.spMFV.GetValue())):
        errorDisplay += "{0} has too many points in the current fleet (natural allies can have up to {1} points in this fleet)\n".format(dataStruct.naturalAllies[coreFaction], int(0.5*self.spMFV.GetValue()))
      #check for max allowable points from other allies
      for faction, points in factionPointTotals.iteritems():
        #check special case of Terran Fleet with 50% STL
        if (coreFaction == 'Terran') and ((faction == 'STL Traders') or (faction == 'STL Syndicate')):
          STLpoints = factionPointTotals['STL Traders'] + factionPointTotals['STL Syndicate']
          if STLpoints > (self.spMFV.GetValue()/2):
            errorDisplay += "Fleet has too many points ({0}) included from STL Traders and/or Syndicate ships. This fleet may have no more than half value ({1}) in STL ships\n".format(STLpoints, self.spMFV.GetValue()/2)
        #check special case of Directorate Fleet with 50% Omnidyne
        elif (coreFaction == 'Directorate') and (faction == 'OSO Omnidyne'):
          if factionPointTotals['OSO Omnidyne'] > (self.spMFV.GetValue()/2):
            errorDisplay += "Fleet has too many points ({0}) included from OSO Omnidyne ships. This fleet may have no more than half value ({1}) in OSO Omnidyne ships\n".format(factionPointTotals['OSO Omnidyne'], self.spMFV.GetValue()/2)
        elif (faction != coreFaction) and (faction != dataStruct.naturalAllies[coreFaction]) and (points > int(0.25*self.spMFV.GetValue())):
          errorDisplay += "{0} has too many points in the current fleet (allies can have up to {1} points in this fleet)\n".format(faction, int(0.25*self.spMFV.GetValue()))
    
    #check special case of no Works Raptor in same fleet as Omnidyne
    if (factionPointTotals['Works Raptor'] > 0) and (factionPointTotals['OSO Omnidyne'] > 0):
      errorDisplay += 'Fleet cannot contain both Works Raptor and OSO Omnidyne ships\n'

    if (errorDisplay == ''):
      self.pnErrorLight.SetBackgroundColour(wx.GREEN)
      errorDisplay = "No errors found."
    else:
      self.pnErrorLight.SetBackgroundColour(wx.RED)
    self.pnErrorLight.Refresh()
    self.txErrors.AppendText(errorDisplay)
    return
  
  def OnDeleteFleetShip(self, event):
    treeData = self.trFleetDisplay.GetItemData(self.selectedFleetShipID)
    deleteShip = treeData.GetData()
    if (not deleteShip):
      return
    else:
      for squadron in self.fleet.squadronList:
        if (squadron.ship == deleteShip) or (squadron.heavyCruiser == deleteShip):
          self.fleet.RemoveSquadron(squadron)
    self.UpdateFleetCost()
    self.UpdateFleetList()
    self.CheckFleet()
    return
  
  def OnChangeMaxFleetValue(self, event):
    self.fleet.SetFleetMFV(self.spMFV.GetValue())
    self.FilterShipList()
    self.CheckFleet()
    return
  
  def FilterShipList(self):
    self.shipList = []
    alliance = []
    if (not self.fleet.restrictToCore):
      if (self.fleet.coreFaction in dataStruct.kurak):
        alliance += dataStruct.kurak
      else:
        alliance += dataStruct.zenian
    else:
      alliance.append(self.fleet.coreFaction)
      
    if (self.fleet.includeMercenaries):
      alliance += dataStruct.mercenary
    if (self.fleet.includeNaturalAllies):
      alliance.append(dataStruct.naturalAllies[self.fleet.coreFaction])
    
    for ship in gv.ShipClassList:
      if (ship.deleted_ship):
        continue
      if (ship.faction in alliance):  #add ship to list based on faction
        ship.tier = None  #this filters out the ship, then turn it back on if its part of the allowed tier
        #check the tier classification of the ship, based on the core faction and MFV
        #some ships change tiers based on the MFV
        if (self.fleet.allowedTier1[ship.designation] > 0):
            ship.tier = 1
        elif (self.fleet.allowedTier2[ship.designation] > 0):
          ship.tier = 2
        elif (self.fleet.allowedTier3[ship.designation] > 0):
          ship.tier = 3
          
        if (self.fleet.restrictToInventory) and (ship.numOwned > 0):
          self.shipList.append(ship)
        elif (not self.fleet.restrictToInventory):
          self.shipList.append(ship)
          
        
          
    self.UpdateInventoryList()
    return
  
  def UpdateInventoryList(self):
    self.trInventoryDisplay.DeleteChildren(self.inventoryTier1)
    self.trInventoryDisplay.DeleteChildren(self.inventoryTier2)
    self.trInventoryDisplay.DeleteChildren(self.inventoryTier3)
    for ship in self.shipList:
      if (ship.tier == 1):
        treeData = wx.TreeItemData()
        treeData.SetData(ship)
        self.trInventoryDisplay.AppendItem(self.inventoryTier1, (ship.className+" ("+ship.faction+" "+ship.designation+")"), data=treeData)
      elif (ship.tier == 2):
        treeData = wx.TreeItemData()
        treeData.SetData(ship)
        self.trInventoryDisplay.AppendItem(self.inventoryTier2, (ship.className+" ("+ship.faction+" "+ship.designation+")"), data=treeData)
      elif (ship.tier == 3):
        treeData = wx.TreeItemData()
        treeData.SetData(ship)
        self.trInventoryDisplay.AppendItem(self.inventoryTier3, (ship.className+" ("+ship.faction+" "+ship.designation+")"), data=treeData)
      self.trInventoryDisplay.Expand(self.inventoryRoot)
      self.trInventoryDisplay.Expand(self.inventoryTier1)
      self.trInventoryDisplay.Expand(self.inventoryTier2)
      self.trInventoryDisplay.Expand(self.inventoryTier3)
    return

  def UpdateFleetList(self):
    """Update each ship tier based on the fleet composition and MFV"""
    squadron = dataStruct.squadronRecord()
    ship = dataStruct.ShipRecord()
    
    self.trFleetDisplay.DeleteChildren(self.fleetTier1)
    self.trFleetDisplay.DeleteChildren(self.fleetTier2)
    self.trFleetDisplay.DeleteChildren(self.fleetTier3)
    for squadron in self.fleet.squadronList:
      if (self.fleet.allowedTier1[squadron.ship.designation] > 0):
        treeData = wx.TreeItemData()
        treeData.SetData(squadron.ship)
        treeDataSquadron = wx.TreeItemData()
        treeDataSquadron.SetData(squadron)
        squadron.tier = 1
        squadron.treeItemSquadron = self.trFleetDisplay.AppendItem(self.fleetTier1, (squadron.ship.designation+ " squadron ({0} points)".format(squadron.cost)), data=treeDataSquadron)
        squadron.treeItemShip = self.trFleetDisplay.AppendItem(squadron.treeItemSquadron, (("{0}x ".format(squadron.size))+squadron.ship.className+" ("+squadron.ship.faction+ " "+squadron.ship.designation+")"), data=treeData)
        if (squadron.heavyCruiser):
          treeDataHeavyCruiser = wx.TreeItemData()
          treeDataHeavyCruiser.SetData(squadron.heavyCruiser)
          squadron.treeItemShipHeavyCruiser = self.trFleetDisplay.AppendItem(squadron.treeItemSquadron, ("1x "+squadron.heavyCruiser.className+" ("+squadron.ship.faction+ " "+squadron.heavyCruiser.designation+")"), data=treeDataHeavyCruiser)
        if (squadron.ship.currentAccompaniment):
          treeDataAccomp = wx.TreeItemData()
          treeDataAccomp.SetData(squadron.ship.currentAccompaniment.ship)
          squadron.treeItemShipAccomp = self.trFleetDisplay.AppendItem(squadron.treeItemSquadron, (("{0}x ".format(squadron.ship.currentAccompaniment.numSelected))+squadron.ship.currentAccompaniment.ship.className+" ("+squadron.ship.currentAccompaniment.ship.faction+ " "+squadron.ship.currentAccompaniment.ship.designation+")"), data=treeDataAccomp)
      elif (self.fleet.allowedTier2[squadron.ship.designation] > 0):
        treeData = wx.TreeItemData()
        treeData.SetData(squadron.ship)
        treeDataSquadron = wx.TreeItemData()
        treeDataSquadron.SetData(squadron)
        squadron.tier = 2
        squadron.treeItemSquadron = self.trFleetDisplay.AppendItem(self.fleetTier2, (squadron.ship.designation+ " squadron ({0} points)".format(squadron.cost)), data=treeDataSquadron)
        squadron.treeItemShip = self.trFleetDisplay.AppendItem(squadron.treeItemSquadron, (("{0}x ".format(squadron.size))+squadron.ship.className+" ("+squadron.ship.faction+ " "+squadron.ship.designation+")"), data=treeData)
        if (squadron.heavyCruiser):
          treeDataHeavyCruiser = wx.TreeItemData()
          treeDataHeavyCruiser.SetData(squadron.heavyCruiser)
          squadron.treeItemShipHeavyCruiser = self.trFleetDisplay.AppendItem(squadron.treeItemSquadron, ("1x "+squadron.heavyCruiser.className+" ("+squadron.ship.faction+ " "+squadron.heavyCruiser.designation+")"), data=treeDataHeavyCruiser)        
        if (squadron.ship.currentAccompaniment):
          treeDataAccomp = wx.TreeItemData()
          treeDataAccomp.SetData(squadron.ship.currentAccompaniment)
          squadron.treeItemShipAccomp = self.trFleetDisplay.AppendItem(squadron.treeItemSquadron, (("{0}x ".format(squadron.ship.currentAccompaniment.numSelected))+squadron.ship.currentAccompaniment.ship.className+" ("+squadron.ship.faction+ " "+squadron.ship.currentAccompaniment.ship.designation+")"), data=treeDataAccomp)
      elif (self.fleet.allowedTier3[squadron.ship.designation] > 0):
        treeData = wx.TreeItemData()
        treeData.SetData(squadron.ship)
        treeDataSquadron = wx.TreeItemData()
        treeDataSquadron.SetData(squadron)
        squadron.tier = 3
        squadron.treeItemSquadron = self.trFleetDisplay.AppendItem(self.fleetTier3, (squadron.ship.designation+ " squadron ({0} points)".format(squadron.cost)), data=treeDataSquadron)
        squadron.treeItemShip = self.trFleetDisplay.AppendItem(squadron.treeItemSquadron, (("{0}x ".format(squadron.size))+squadron.ship.className+" ("+squadron.ship.faction+ " "+squadron.ship.designation+")"), data=treeData)
        if (squadron.heavyCruiser):
          treeDataHeavyCruiser = wx.TreeItemData()
          treeDataHeavyCruiser.SetData(squadron.heavyCruiser)
          squadron.treeItemShipHeavyCruiser = self.trFleetDisplay.AppendItem(squadron.treeItemSquadron, ("1x "+squadron.heavyCruiser.className+" ("+squadron.ship.faction+ " "+squadron.heavyCruiser.designation+")"), data=treeDataHeavyCruiser)        
        if (squadron.ship.currentAccompaniment):
          treeDataAccomp = wx.TreeItemData()
          treeDataAccomp.SetData(squadron.ship.currentAccompaniment)
          squadron.treeItemShipAccomp = self.trFleetDisplay.AppendItem(squadron.treeItemSquadron, (("{0}x ".format(squadron.ship.currentAccompaniment.numSelected))+squadron.ship.currentAccompaniment.ship.className+" ("+squadron.ship.faction+ " "+squadron.ship.currentAccompaniment.ship.designation+")"), data=treeDataAccomp)    
    self.trFleetDisplay.ExpandAllChildren(self.fleetRoot)
    return
  
  def OnSelectInventoryShip(self, event):
    """Update the ship information to the Inventory Detail """
    ship = dataStruct.ShipRecord()
    
    self.bChangeSquadronImage.Enable(False)
    self.bRevertToDefaultImage.Enable(False)
    
    if (isinstance(event, wx.MouseEvent)):
      event.Skip()
      if (not self.selectedInventoryShipID):   #skip if there is no ship selected
        return
      treeData = self.trInventoryDisplay.GetItemData(self.selectedInventoryShipID)
    else:
      self.selectedInventoryShipID = event.GetItem()
      treeData = self.trInventoryDisplay.GetItemData(self.selectedInventoryShipID)
    ship = treeData.GetData()
    if (not ship):
      nullShip = dataStruct.ShipRecord()
      self.SetHardpoints(nullShip)
      self.SetUpgrades(nullShip)
      self.SetAccompaniment(nullShip)
      self.SetSRS(nullShip)
      return
    self.UpdateShipDetail(ship, 'inventory')
    return
  
  def OnRevertShipImage(self, event):
    if (not self.selectedFleetShipID):
      return
    treeData = self.trFleetDisplay.GetItemData(self.selectedFleetShipID)
    ship = treeData.GetData()
    if (not isinstance(ship, dataStruct.ShipRecord)):
      return
    ship.imageCustomFilename = None
    self.UpdateShipDetail(ship,'fleet')
    self.bRevertToDefaultImage.Enable(False)
    return
    
  
  def OnChangeSqImage(self, event):
    if (not self.selectedFleetShipID):
      return
    dlg = wx.FileDialog(self, message='Choose ship image file', defaultDir=gv.ship_custom_images_folder,
                        defaultFile="", wildcard=image_wildcard, style=wx.OPEN)
    if dlg.ShowModal() == wx.ID_OK:
      treeData = self.trFleetDisplay.GetItemData(self.selectedFleetShipID)
      ship = treeData.GetData()
      ship.imageCustomFilename = dlg.GetFilename()
      self.bRevertToDefaultImage.Enable(True)
      self.UpdateShipDetail(ship,'fleet')
    return
    
  def UpdateShipDetail(self, ship, source):
    """source is either fleet, escort, or inventory"""
    if (not isinstance(ship, dataStruct.ShipRecord)):
      self.bViewMars.Enable(False)
      return
    else:
      self.bViewMars.Enable(True)
    self.txShipDetail.Clear()
    if (source == 'inventory'):
      self.txShipDetail.SetBackgroundColour(gv.InventoryBackgroundColor)
    elif (source == 'escort'):
      self.txShipDetail.SetBackgroundColour(gv.EscortBackgroundColor)
    else:
      self.txShipDetail.SetBackgroundColour(gv.FleetBackgroundColor)

    imageCustomPath = None
    if (ship.imageCustomFilename):
      imageCustomPath = os.path.join(gv.ship_custom_images_folder, ship.imageCustomFilename)
    if (ship.imageFilename):
      imagePath = os.path.join(gv.ship_images_folder, ship.imageFilename)
    else:
      imagePath = os.path.join(gv.ship_images_folder, gv.defaultImage)
    
    image = wx.Image(imagePath)
    if (source == 'inventory') and (image.IsOk()):
      (width, height) = self.bmpInventoryImage.GetSize()
      scaledBitmap = wx.BitmapFromImage(image.Scale(width, height, wx.IMAGE_QUALITY_HIGH))
      self.bmpInventoryImage.SetBitmap(scaledBitmap)
    else:
      logging.error("Could not find ship image {0}".format(imagePath))
    
    if (imageCustomPath):  #use the custom image, otherwise, the standard ship image
      customImage = wx.Image(imageCustomPath)
      if (not customImage.IsOk()):
        logging.error("Could not find custom ship image {0}".format(imageCustomPath))
        customImage = wx.Image(imagePath)
    else:
      customImage = wx.Image(imagePath)
      
    if (source == 'fleet') or (source == 'escort'):
      (width, height) = self.bmpSquadronShipImage.GetSize()
      scaledBitmap = wx.BitmapFromImage(customImage.Scale(width, height, wx.IMAGE_QUALITY_HIGH))
      self.bmpSquadronShipImage.SetBitmap(scaledBitmap)

    
    imagePath = os.path.join(gv.ship_images_folder, dataStruct.factionBitmapPath[ship.faction])

    image = wx.Image(imagePath)
    if (image.IsOk()):
      (width, height) = self.bmpFactionImage.GetSize()
      scaledBitmap = wx.BitmapFromImage(image.Scale(width, height, wx.IMAGE_QUALITY_HIGH))
      self.bmpFactionImage.SetBitmap(scaledBitmap)
    else:
      logging.error("Could not find faction image {0}".format(imagePath))
    
    self.stDR.SetLabel(str(ship.DR))
    self.stCR.SetLabel(str(ship.CR))
    self.stHP.SetLabel(str(ship.HP))
    self.stMV.SetLabel(str(ship.MV))
    self.stTL.SetLabel(str(ship.TL))
    self.stCP.SetLabel(str(ship.CP))
    self.stAP.SetLabel(str(ship.AP))
    self.stPD.SetLabel(str(ship.PD))
    if (ship.cloakingField):
      self.stSR.SetLabel("Cloak")
    else:
      self.stSR.SetLabel(str(ship.SR))
    self.stMN.SetLabel(str(ship.MN))
    self.stWC.SetLabel(str(ship.WC))
    if (source == 'inventory') or (source == 'escort'):
      text = '{0} {1}, {2} Class; Tier {4}, Base Cost:{3} points\n'.format(ship.faction, ship.designation, ship.className, ship.baseCost, ship.tier)
    else:
      text = '{0} {1}, {2} Class; Tier {4}, Base Ship Cost:{3} points, Current Ship Cost:{5}\n'.format(ship.faction, ship.designation, ship.className, ship.baseCost, ship.tier, ship.currentCost)
    text += 'Weapons Array:\n'
    weapon = dataStruct.WeaponRecord()
    if (ship.weapons == []):
      text += '   None\n'
    else:
      if (source == 'inventory') or (source == 'escort'):
        weaponsList = ship.weapons
      else:  #source == 'fleet'
        weaponsList = ship.currentWeapons
        
      for weapon in weaponsList:
        if (source == 'fleet' and weapon.hardpoint and not weapon.includeAsHardpoint):
          continue  #skip if the weapon is a hardpoint attachment, but the hardpoint has not been selected
        elif (source == 'inventory' and weapon.hardpoint):
          text += "Hardpoint Option: "
        if (gv.IncludeFlavorText):
          text += '   {0},'.format(weapon.weaponName)
        text += '  {8} {0} weapon, {1} {2}, Increment:{3}, AD:{4} | {5} | {6} | {7}\n'.format(weapon.weaponType, weapon.arcDirection, weapon.arcRestriction, weapon.rangeIncrement, weapon.rb1, weapon.rb2, weapon.rb3, weapon.rb4, weapon.category)

    
    if (ship.WC > 0) and (source != 'inventory'):
      text += "\n"
      text += "Short Range Spacecraft (SRS):\n"
      if (ship.currentSRS[0].srsType == 'None' and ship.currentSRS[1].srsType == 'None'):  #update SRS
        text += "   None\n"
      else:
        for srs in ship.currentSRS:
          text += "   SRS: {0}, {1} wings  ({2} points)\n".format(srs.srsType, srs.numWings, (srs.cost))
        
    if (source == 'inventory') or (source == 'escort'):
      marList = ship.MAR
    else:
      marList = ship.currentMAR
      
    text += '\n'
    if (marList == []):
      text += 'MARs:  None\n'
    else:
      text += 'MARs:\n'
      mar = dataStruct.MarRecord()
      listLength = len(marList)
      listIndex = 0
      self.MarDisplay = ""
      for mar in marList:
        text += '   {0}\n'.format(mar.marName)
        self.MarDisplay += '{0}:  {1}\n\n'.format(mar.marName, mar.marText)


    text += '\n'
    if (source == 'inventory') or (source == 'escort'):
      hardpointList = ship.availableHardPoints
    else:
      hardpointList = ship.currentHardpoints
      
    if (hardpointList == []):
      text += 'Hardpoints:  None\n'
    else:
      if (source == 'inventory') or (source == 'escort'):
        text += 'Hardpoints (select up to {0} hardpoints):\n'.format(ship.maxHardPoints)
      else:
        text += 'Hardpoints attached:\n'
      hpoint = dataStruct.HardpointRecord()
      for hpoint in hardpointList:
        if (source == 'inventory'):
          text += '   {0} points, 0-{1}, {2}\n'.format(hpoint.cost, hpoint.numAvailable, hpoint.description)
        else:
          text += '   {0} points, {2} (x{1})\n'.format((hpoint.cost*hpoint.numSelected), hpoint.numSelected, hpoint.description)

    if (source == 'inventory') or (source == 'escort'):
      upgradeList = ship.upgrades
    else:
      upgradeList = ship.currentUpgrades
      
    text += '\n'
    if (upgradeList == []):
      text += 'Upgrades:  None\n'
    else:
      text += 'Upgrades:\n'
      upgrade = dataStruct.UpgradeRecord()
      for upgrade in upgradeList:
        text += '   {0} points, {1}\n'.format(upgrade.cost, upgrade.text)
    
    text += '\n'
    
    if (source == 'inventory') or (source == 'escort'):
      accompList = ship.accompanimentIncluded
      if (accompList == []):
        text += 'Accompaniment:  None\n'
      else:
        text += 'Accompaniment:\n'
        for accomp in accompList:
          text += '   0-{0}, {1} points, {2} {3} {4}\n'.format(accomp.numAllowed, gv.ShipClassList[accomp.shipIndex].baseCost, gv.ShipClassList[accomp.shipIndex].faction,gv.ShipClassList[accomp.shipIndex].className,gv.ShipClassList[accomp.shipIndex].designation)
    else:
      if (ship.currentAccompaniment):
        text += 'Accompaniment:\n'
        text += '   {0}x {2} {3} {4} ({1} points each)\n'.format(ship.currentAccompaniment.numSelected,
                                                                 ship.currentAccompaniment.ship.baseCost,
                                                                 ship.currentAccompaniment.ship.faction,
                                                                 ship.currentAccompaniment.ship.className,
                                                                 ship.currentAccompaniment.ship.designation)
      else:
        text += 'Accompaniment:  None\n'
    self.txShipDetail.SetLabel(text)
    return
  
  def OnChangeHardpoint(self, event):
    if (event.GetEventObject().IsChecked()):
      self.AddHardpoint(event)
    else:
      self.RemoveHardpoint(event)
    return
  
  def OnChangeUpgrade(self, event):
    if (event.GetEventObject().IsChecked()):
      self.AddUpgrade(event)
    else:
      self.RemoveUpgrade(event)
    return
    
  def RemoveUpgrade(self, event):
    mar = dataStruct.MarRecord()
    ship = dataStruct.ShipRecord()
    upgrade = dataStruct.UpgradeRecord()
    weapon = dataStruct.WeaponRecord()
    treeData = self.trFleetDisplay.GetItemData(self.selectedFleetShipID)
    ship = treeData.GetData()
    
    if (ship.currentUpgrades == []):
      return  #nothing to remove...how did we even get here?
    
    if (event.GetEventObject().GetName() == 'idUpgrade0'):
      listedUpgrade = ship.upgrades[0]
    elif (event.GetEventObject().GetName() == 'idUpgrade1'):
      listedUpgrade = ship.upgrades[1]
    elif (event.GetEventObject().GetName() == 'idUpgrade2'):
      listedUpgrade = ship.upgrades[2]
    elif (event.GetEventObject().GetName() == 'idUpgrade3'):
      listedUpgrade = ship.upgrades[3]
    elif (event.GetEventObject().GetName() == 'idUpgrade4'):
      listedUpgrade = ship.upgrades[4]
    elif (event.GetEventObject().GetName() == 'idUpgrade5'):
      listedUpgrade = ship.upgrades[5]
    elif (event.GetEventObject().GetName() == 'idUpgrade6'):
      listedUpgrade = ship.upgrades[6]
    else:
      return
    for upgrade in ship.currentUpgrades:
      if (upgrade.index == listedUpgrade.index):
        break
      
    #modify ship stats based on upgrade de-selected

    if (upgrade.index == 1000):  #remove ablative plating MAR
      mar = copy.deepcopy(gv.MarList[1])
      ship.currentMAR.append(mar)
    elif (upgrade.index == 1001): #gain ambush 2 MAR
      for mar in ship.currentMAR:
        if (mar.marIndex == 3):
          ship.currentMAR.remove(mar)
          break
    elif (upgrade.index == 1002): #gain +2 AP
      ship.AP -= 2
    elif (upgrade.index == 1003):  #gain assault blitz mar
      for mar in ship.currentMAR:
        if (mar.marIndex == 4):
          ship.currentMAR.remove(mar)
          break
    elif (upgrade.index == 1004): #beam gains biohazard ammo mar
      for mar in ship.currentMAR:
        if (mar.marIndex == 7):
          ship.currentMAR.remove(mar)
          break
    elif (upgrade.index == 1005): #beam gains High Energy mar
      for mar in ship.currentMAR:
        if (mar.marIndex == 22):
          ship.currentMAR.remove(mar)
          break
    elif (upgrade.index == 1006): # beam gains precision strike mar
      for mar in ship.currentMAR:
        if (mar.marIndex == 33):
          ship.currentMAR.remove(mar)
          break
    elif (upgrade.index == 1007): #beam gains split fire mar
      for mar in ship.currentMAR:
        if (mar.marIndex == 46):
          ship.currentMAR.remove(mar)
          break
    elif (upgrade.index == 1008): # bigger batteries mar
      for mar in ship.currentMAR:
        if (mar.marIndex == 6):
          ship.currentMAR.remove(mar)
          break
    elif (upgrade.index == 1009): # countermeasures mar
      for mar in ship.currentMAR:
        if (mar.marIndex == 10):
          ship.currentMAR.remove(mar)
          break
    elif (upgrade.index == 1010): # deck crews mar
      for mar in ship.currentMAR:
        if (mar.marIndex == 22):
          ship.currentMAR.remove(mar)
          break
    elif (upgrade.index == 1011): # double mines mar
      for mar in ship.currentMAR:
        if (mar.marIndex == 17):
          ship.currentMAR.remove(mar)
          break
    elif (upgrade.index == 1012): # drone mines mar
      for mar in ship.currentMAR:
        if (mar.marIndex == 16):
          ship.currentMAR.remove(mar)
          break
    elif (upgrade.index == 1013): # experienced engineers
      for mar in ship.currentMAR:
        if (mar.marIndex == 21):
          ship.currentMAR.remove(mar)
          break
    elif (upgrade.index == 1014): # beam energy transfer 1 mar
      for mar in ship.currentMAR:
        if (mar.marIndex == 20):
          ship.currentMAR.remove(mar)
          break
    elif (upgrade.index == 1015): # beam energy transfer 2 mar
      for mar in ship.currentMAR:
        if (mar.marIndex == 20):
          ship.currentMAR.remove(mar)
          break
    elif (upgrade.index == 1016): # energy transfer 2,all
      for mar in ship.currentMAR:
        if (mar.marIndex == 20):
          ship.currentMAR.remove(mar)
          break
    elif (upgrade.index == 1017): # fore fixed gain decimator
      for mar in ship.currentMAR:
        if (mar.marIndex == 11):
          ship.currentMAR.remove(mar)
          break
    elif (upgrade.index == 1018): # gun racks gain split fire mar
      for mar in ship.currentMAR:
        if (mar.marIndex == 46):
          ship.currentMAR.remove(mar)
          break
    elif (upgrade.index == 1019): # kinetics gain high energy mar
      for mar in ship.currentMAR:
        if (mar.marIndex == 22):
          ship.currentMAR.remove(mar)
          break
    elif (upgrade.index == 1020): # kinetics gain precision strike
      for mar in ship.currentMAR:
        if (mar.marIndex == 33):
          ship.currentMAR.remove(mar)
          break
    elif (upgrade.index == 1021): # manoeuverable
      for mar in ship.currentMAR:
        if (mar.marIndex == 27):
          ship.currentMAR.remove(mar)
          break
    elif (upgrade.index == 1022): # mines gain biohazard ammo
      for mar in ship.currentMAR:
        if (mar.marIndex == 7):
          ship.currentMAR.remove(mar)
          break
    elif (upgrade.index == 1023): # gain +3 mines
      ship.MN -= 3
    elif (upgrade.index == 1024): # pack hunter mar
      for mar in ship.currentMAR:
        if (mar.marIndex == 31):
          ship.currentMAR.remove(mar)
          break
    elif (upgrade.index == 1025): # point defense barrage
      for mar in ship.currentMAR:
        if (mar.marIndex == 32):
          ship.currentMAR.remove(mar)
          break
    elif (upgrade.index == 1026): # protected systems
      for mar in ship.currentMAR:
        if (mar.marIndex == 34):
          ship.currentMAR.remove(mar)
          break
    elif (upgrade.index == 1027): # primaries gain biohazard
      for mar in ship.currentMAR:
        if (mar.marIndex == 7):
          ship.currentMAR.remove(mar)
          break
    elif (upgrade.index == 1028): # primary decimator warheads
      for mar in ship.currentMAR:
        if (mar.marIndex == 11):
          ship.currentMAR.remove(mar)
          break
    elif (upgrade.index == 1029): # primary high energy
      for mar in ship.currentMAR:
        if (mar.marIndex == 22):
          ship.currentMAR.remove(mar)
          break
    elif (upgrade.index == 1030): # primary precision strike
      for mar in ship.currentMAR:
        if (mar.marIndex == 33):
          ship.currentMAR.remove(mar)
          break
    elif (upgrade.index == 1031): # second assault mar
      for mar in ship.currentMAR:
        if (mar.marIndex == 39):
          ship.currentMAR.remove(mar)
          break
    elif (upgrade.index == 1032): # quick launch
      for mar in ship.currentMAR:
        if (mar.marIndex == 35):
          ship.currentMAR.remove(mar)
          break
    elif (upgrade.index == 1033): # reinforced
      for mar in ship.currentMAR:
        if (mar.marIndex == 36):
          ship.currentMAR.remove(mar)
          break
    elif (upgrade.index == 1034): # scatter gains split fire mar
      for mar in ship.currentMAR:
        if (mar.marIndex == 46):
          ship.currentMAR.remove(mar)
          break
    elif (upgrade.index == 1035): # 
      for mar in ship.currentMAR:
        if (mar.marIndex == 46):
          ship.currentMAR.remove(mar)
          break
    elif (upgrade.index == 1036): # sector shielding
      for mar in ship.currentMAR:
        if (mar.marIndex == 41):
          ship.currentMAR.remove(mar)
          break
    elif (upgrade.index == 1037): # secured bulkhead
      for mar in ship.currentMAR:
        if (mar.marIndex == 40):
          ship.currentMAR.remove(mar)
          break
    elif (upgrade.index == 1038): # self repair
      for mar in ship.currentMAR:
        if (mar.marIndex == 42):
          ship.currentMAR.remove(mar)
          break
    elif (upgrade.index == 1039): # special forces mar
      for mar in ship.currentMAR:
        if (mar.marIndex == 45):
          ship.currentMAR.remove(mar)
          break
    elif (upgrade.index == 1040): # 
      for mar in ship.currentMAR:
        if (mar.marIndex == 46):
          ship.currentMAR.remove(mar)
          break
    elif (upgrade.index == 1041): # torpedo gains biohazard
      for mar in ship.currentMAR:
        if (mar.marIndex == 7):
          ship.currentMAR.remove(mar)
          break
    elif (upgrade.index == 1042): # torp corrosive
      for mar in ship.currentMAR:
        if (mar.marIndex == 9):
          ship.currentMAR.remove(mar)
          break
    elif (upgrade.index == 1043): # torp high energy
      for mar in ship.currentMAR:
        if (mar.marIndex == 22):
          ship.currentMAR.remove(mar)
          break
    elif (upgrade.index == 1044): # torpedo spook
      for mar in ship.currentMAR:
        if (mar.marIndex == 51):
          ship.currentMAR.remove(mar)
          break
    elif (upgrade.index == 1045): # weapon shielding
      for mar in ship.currentMAR:
        if (mar.marIndex == 54):
          ship.currentMAR.remove(mar)
          break
    elif (upgrade.index == 1046): # primary to scatter
      for weapon in ship.currentWeapons:
        if (weapon.weaponType == 'Scatter'):
          weapon.weaponType = 'Primary'
    elif (upgrade.index == 1047): # fore fixed weapons gain nuclear type
      for weapon in ship.currentWeapons:
        if (weapon.arcDirection == 'Forward') and (weapon.arcRestriction == 'Fixed'):
          if ('Beam' in weapon.weaponType):
            weapon.weaponType = 'Beam'
          elif ('Primary' in weapon.weaponType):
            weapon.weaponType = 'Primary'
          elif ('Kinetic' in weapon.weaponType):
            weapon.weaponType = 'Kinetic'
          elif ('Scatter' in weapon.weaponType):
            weapon.weaponType = 'Scatter'
    elif (upgrade.index == 1048): # torpedoes gain nuclear type
      for weapon in ship.currentWeapons:
        if (weapon.weaponType == 'Nuclear Torpedo'):
          weapon.weaponType = "Torpedo"
    elif (upgrade.index == 1049): # primary to beams
      for weapon in ship.currentWeapons:
        if (weapon.weaponType == 'Beam'):
          weapon.weaponType = "Primary"
    elif (upgrade.index == 1050): # starboard/port to beams
      for weapon in ship.currentWeapons:
        if (weapon.arcDirection == 'Starboard') or (weapon.arcDirection == 'Port'):
          weapon.weaponType = "Primary"
    elif (upgrade.index == 1051): #turrets gain nuclear
      for weapon in ship.currentWeapons:
        if (weapon.arcDirection == 'Any'):
          weapon.weaponType = "Primary"
    elif (upgrade.index == 1053): #beam gains Corrosive mar
      for mar in ship.currentMAR:
        if (mar.marIndex == 9):
          ship.currentMAR.remove(mar)
          break
    elif (upgrade.index == 1054): # scout mar 
      for mar in ship.currentMAR:
        if (mar.marIndex == 38):
          ship.currentMAR.remove(mar)
          break
    elif (upgrade.index == 1055): # primary to beams
      for weapon in ship.currentWeapons:
        if (weapon.weaponType == 'Cyberwarfare'):
          weapon.weaponType = "Primary"
    elif (upgrade.index == 1056): # beam decimator warheads
      for mar in ship.currentMAR:
        if (mar.marIndex == 11):
          ship.currentMAR.remove(mar)
          break
    elif (upgrade.index == 1057): # primary gain corrosive
      for mar in ship.currentMAR:
        if (mar.marIndex == 9):
          ship.currentMAR.remove(mar)
          break
    elif (upgrade.index == 1058): # remove stealth systems mar
      for mar in ship.currentMAR:
        if (mar.marIndex == 47):
          ship.currentMAR.remove(mar)
          break
    elif (upgrade.index == 1059): # remove energy transfer 2 with energy transfer 1 
      for mar in ship.currentMAR:
        if (mar.marIndex == 20):
          mar.intValue1 = 1
          break
    elif (upgrade.index == 1060):  #command -2
      ship.commandDistance -= 2
    elif (upgrade.index == 1061):  # MV -1
      ship.MV -= 1
    elif (upgrade.index == 1062):  #command -3
      ship.commandDistance -= 3
    elif (upgrade.index == 1063):  #scatter to kinetic
      for weapon in ship.currentWeapons:
        if (weapon.weaponType == 'Kinetic'):
          weapon.weaponType = 'Scatter'
    elif (upgrade.index == 1064):  # PD -1
      ship.PD -= 1
    elif (upgrade.index == 1065): # remove weapons shielding mar, gain durable mar
      for mar in ship.currentMAR:
        if (mar.marIndex == 54):
          ship.currentMAR.remove(mar)
          break
      mar = copy.deepcopy(gv.MarList[57])
      ship.currentMAR.append(mar)
    elif (upgrade.index == 1066):  # sr -1
      ship.SR -= 1
    elif (upgrade.index == 1067): # remove assault robots mar
      for mar in ship.currentMAR:
        if (mar.marIndex == 5):
          ship.currentMAR.remove(mar)
          break
    elif (upgrade.index == 1068): # remove precision strike mar
      for mar in ship.currentMAR:
        if (mar.marIndex == 33):
          ship.currentMAR.remove(mar)
          break
    
  
    ship.currentUpgrades.remove(upgrade)
    self.UpdateFleetCost()
    self.UpdateShipDetail(ship, 'fleet')
    self.CheckFleet()
    return
  
  def RemoveHardpoint(self, event):
    ship = dataStruct.ShipRecord()
    treeData = self.trFleetDisplay.GetItemData(self.selectedFleetShipID)
    ship = treeData.GetData()
    
    if (ship.currentHardpoints == []):
      return
    
    if (event.GetEventObject().GetName() == 'idHardpoint0'):
      listedHardpoint = ship.availableHardPoints[0]
      numHardpoint = self.spHardpoint0.GetValue()
    elif (event.GetEventObject().GetName() == 'idHardpoint1'):
      listedHardpoint = ship.availableHardPoints[1]
      numHardpoint = self.spHardpoint1.GetValue()
    elif (event.GetEventObject().GetName() == 'idHardpoint2'):
      listedHardpoint = ship.availableHardPoints[2]
      numHardpoint = self.spHardpoint2.GetValue()
    elif (event.GetEventObject().GetName() == 'idHardpoint3'):
      listedHardpoint = ship.availableHardPoints[3]
      numHardpoint = self.spHardpoint3.GetValue()
    elif (event.GetEventObject().GetName() == 'idHardpoint4'):
      listedHardpoint = ship.availableHardPoints[4]
      numHardpoint = self.spHardpoint4.GetValue()
    elif (event.GetEventObject().GetName() == 'idHardpoint5'):
      listedHardpoint = ship.availableHardPoints[5]
      numHardpoint = self.spHardpoint5.GetValue()
    elif (event.GetEventObject().GetName() == 'idHardpoint6'):
      listedHardpoint = ship.availableHardPoints[6]
      numHardpoint = self.spHardpoint6.GetValue()
    else:
      return

    for hardpoint in ship.currentHardpoints:
      if (hardpoint.index == listedHardpoint.index):
        break
    #reverse the changes made by the hardpoint
    if (hardpoint.index == 1):  #increase TL by amount
      ship.TL += max(hardpoint.intValue1, 0)  #cannot go below 0
    elif (hardpoint.index == 2):  #decrease WC
      ship.WC -= (numHardpoint * hardpoint.intValue1)
      self.SetSRS(ship)
    elif (hardpoint.index == 3):  #change torpedo arc
      for weapon in ship.currentWeapons:
        if (weapon.weaponType == 'Torpedo'):
          weapon.arcRestriction = 'Fixed'
    elif (hardpoint.index == 4):  #decrease MV
      ship.MV -= (numHardpoint * hardpoint.intValue1)
    elif (hardpoint.index == 5):  #decrease SR
      ship.SR -= (numHardpoint * hardpoint.intValue1)
    elif (hardpoint.index == 6):  #upgrade to Nuclear Torp
      for weapon in ship.currentWeapons:
        if (weapon.weaponType == 'Nuclear Torpedo'):
          weapon.weaponType = 'Torpedo'
    elif (hardpoint.index == 7):  #decrease HP
      ship.HP -= (numHardpoint * hardpoint.intValue1)
    elif (hardpoint.index == 8):  #decrease MN
      ship.MN -= (numHardpoint * hardpoint.intValue1)
    elif (hardpoint.index == 9):  #Remove Ablative Plating MAR
      ship.currentMAR.append(gv.MarList[1])
    elif (hardpoint.index == 11):  #decrease Command Distance
      pass
    elif (hardpoint.index == 12):  #upgrade Primary to Beam
      for weapon in ship.currentWeapons:
        if (weapon.weaponType == 'Beam'):
          weapon.weaponType = 'Primary'
    elif (hardpoint.index == 13):  #change fixed fore to Scatter
      for weapon in ship.currentWeapons:
        if (weapon.weaponType == 'Scatter'):
          weapon.weaponType = 'Primary'
          weapon.arcRestriction = 'Fixed'
    elif (hardpoint.index == 14):  #decrease PD
      ship.PD -= (numHardpoint * hardpoint.intValue1)
      if (hardpoint.marValue == 6):
        for mar in ship.currentMAR:
          if (mar.marIndex == 6):
            ship.currentMAR.remove(mar)
            break
    elif (hardpoint.index == 15):  #decrease AP
      ship.AP -= (numHardpoint * hardpoint.intValue1)
      if (hardpoint.marValue == 39):
        for mar in ship.currentMAR:
          if (mar.marIndex == 39):
            ship.currentMAR.remove(mar)
            break
      if (hardpoint.marValue == 45):
        for mar in ship.currentMAR:
          if (mar.marIndex == 45):
            ship.currentMAR.remove(mar)
            break
    elif (hardpoint.index == 16):  # gain quick launch mar
      for mar in ship.currentMAR:
        if (mar.marIndex == 35):
          ship.currentMAR.remove(mar)
          break
    elif (hardpoint.index == 17):  #replace energy transfer (beam,1) with (beam,2) mar
      for mar in ship.currentMAR:
        if (mar.marIndex == 20):
          mar.intValue1 = 1
          mar.marName = mar.marName.replace('2', '1')
          break
    elif (hardpoint.index == 18):  #gain minefields mar
      for mar in ship.currentMAR:
        if (mar.marIndex == 28):
          ship.currentMAR.remove(28)
    elif (hardpoint.index == 19):  #decrease CP
      ship.CP -= (numHardpoint * hardpoint.intValue1)
    elif (hardpoint.index == 20):  # remove shunt matrix 4 mar
      for mar in ship.currentMAR:
        if (mar.marIndex == 44):
          ship.currentMAR.remove(mar)
          break
    elif (hardpoint.index == 21):  # remove ops center mar
      for mar in ship.currentMAR:
        if (mar.marIndex == 30):
          ship.currentMAR.remove(mar)
          break
    elif (hardpoint.index == 22):  # remove deck crews mar
      for mar in ship.currentMAR:
        if (mar.marIndex == 44):
          ship.currentMAR.remove(mar)
          break
    elif (hardpoint.index == 23):  # remove torpedo spook mar
      for mar in ship.currentMAR:
        if (mar.marIndex == 51):
          ship.currentMAR.remove(mar)
          break
    elif (hardpoint.index == 24):  # remove special forces mar
      for mar in ship.currentMAR:
        if (mar.marIndex == 45):
          ship.currentMAR.remove(mar)
          break
    elif (hardpoint.index == 25):  # remove special forces mar
      for mar in ship.currentMAR:
        if (mar.marIndex == 41):
          ship.currentMAR.remove(mar)
          break
    elif (hardpoint.index == 26):  # remove shield projector self mar
      for mar in ship.currentMAR:
        if (mar.marIndex == 43):
          ship.currentMAR.remove(mar)
          break
    elif (hardpoint.index == 27):  # remove shield projector 8 mar
      for mar in ship.currentMAR:
        if (mar.marIndex == 43):
          ship.currentMAR.remove(mar)
          break
    elif (hardpoint.index == 28):  # remove countermeasures mar
      for mar in ship.currentMAR:
        if (mar.marIndex == 10):
          ship.currentMAR.remove(mar)
          break
    elif (hardpoint.index == 29):  # remove decimator warheads mar
      for mar in ship.currentMAR:
        if (mar.marIndex == 11):
          ship.currentMAR.remove(mar)
          break
    elif (hardpoint.index == 30):  # remove cyberwarfare weapons
      for weapon in ship.currentWeapons:
        if (weapon.weaponType == hardpoint.strValue1):
          weapon.includeAsHardpoint = False
          break
    elif (hardpoint.index == 31):  # remove self repair mar
      for mar in ship.currentMAR:
        if (mar.marIndex == 42):
          ship.currentMAR.remove(mar)
          break
    elif (hardpoint.index == 32):  #decrease CR
      ship.CR -= (numHardpoint * hardpoint.intValue1)
    elif (hardpoint.index == 33):  # remove second assault mar
      for mar in ship.currentMAR:
        if (mar.marIndex == 39):
          ship.currentMAR.remove(mar)
          break
    elif (hardpoint.index == 34):  # remove launch tubes mar
      for mar in ship.currentMAR:
        if (mar.marIndex == 25):
          ship.currentMAR.remove(mar)
          break
    elif (hardpoint.index == 35):  #Remove Vulnerable MAR
      ship.currentMAR.append(gv.MarList[53])
    elif (hardpoint.index == 36):  # remove ambush1 mar
      for mar in ship.currentMAR:
        if (mar.marIndex == 3):
          ship.currentMAR.remove(mar)
          break
    elif (hardpoint.index == 37):  #Remove Split Fire MAR
      ship.currentMAR.append(gv.MarList[46])
    elif (hardpoint.index == 38):  #remove stealth systems mar and increase SR to 1
      for mar in ship.currentMAR:
        if (mar.marIndex == 47):
          ship.SR = mar.intValue1
          ship.currentMAR.remove(mar)
          break
    elif (hardpoint.index == 39):  #Remove dirty secrets MAR
      ship.currentMAR.append(gv.MarList[14])
    elif (hardpoint.index == 40):  #change beam fore to cyberwarfare
      for weapon in ship.currentWeapons:
        if (weapon.weaponType == 'Cyberwarfare'):
          weapon.weaponType = 'Beam'
          break
    elif (hardpoint.index == 41):  # replace stealth with hidden killer
      for mar in ship.currentMAR:
        if (mar.marIndex == 23):
          ship.currentMAR.remove(mar)
          break
      mar = copy.deepcopy(gv.MarList[47])
      ship.currentMAR.append(mar)
    elif (hardpoint.index == 42):  #change beam to cyberwarfare
      for weapon in ship.currentWeapons:
        if (weapon.weaponType == 'Cyberwarfare'):
          weapon.weaponType = 'Beam'
          break
    elif (hardpoint.index == 43):  # remove corrosive mar
      for mar in ship.currentMAR:
        if (mar.marIndex == 9):
          ship.currentMAR.remove(mar)
          break
    elif (hardpoint.index == 44):  # remove biohazard mar
      for mar in ship.currentMAR:
        if (mar.marIndex == 7):
          ship.currentMAR.remove(mar)
          break
    elif (hardpoint.index == 45):  #include the beam weapons
      for weapon in ship.currentWeapons:
        if (weapon.weaponType == 'Beam'):
          weapon.includeAsHardpoint = False
    elif (hardpoint.index == 46):  #include cyberwarfare weapons
      for weapon in ship.currentWeapons:
        if (weapon.weaponType == 'Cyberwarfare'):
          weapon.includeAsHardpoint = False
    elif (hardpoint.index == 47):  #include the gravitational weapons and compressor MAR
      for weapon in ship.currentWeapons:
        if (weapon.weaponType == 'Gravitational'):
          weapon.includeAsHardpoint = False
      for mar in ship.currentMAR:
        if (mar.marIndex == 55):
          ship.currentMAR.remove(mar)
          break
    elif (hardpoint.index == 48):  #include the cyberwarfare weapons and assault robots MAR
      for weapon in ship.currentWeapons:
        if (weapon.weaponType == 'Cyberwarfare'):
          weapon.includeAsHardpoint = False
      for mar in ship.currentMAR:
        if (mar.marIndex == 5):
          ship.currentMAR.remove(mar)
          break
    elif (hardpoint.index == 49):  #include cyberwarfare weapons
      for weapon in ship.currentWeapons:
        if (weapon.weaponType == 'Torpedo'):
          weapon.includeAsHardpoint = False
    elif (hardpoint.index == 50):  #remove the weapon shielding MAR
      for mar in ship.currentMAR:
        if (mar.marIndex == 54):
          ship.currentMAR.remove(mar)
          break
    elif (hardpoint.index == 51):  #remove pack hunters MAR
      for mar in ship.currentMAR:
        if (mar.marIndex == 31):
          ship.currentMAR.remove(mar)
          break
    elif (hardpoint.index == 52):  #remove beams to high energy MAR
      for mar in ship.currentMAR:
        if (mar.marIndex == 22):
          ship.currentMAR.remove(mar)
          break
    elif (hardpoint.index == 53):  # remove point defense barrage mar
      for mar in ship.currentMAR:
        if (mar.marIndex == 32):
          ship.currentMAR.remove(mar)
          break
    elif (hardpoint.index == 54):  # remove shunt matrix 2
      for mar in ship.currentMAR:
        if (mar.marIndex == 44):
          ship.currentMAR.remove(mar)
          break
    elif (hardpoint.index == 55):  # replace ambush3 with ambush2
      for mar in ship.currentMAR:
        if (mar.marIndex == 3):
          mar.intValue1 = 2
          mar.marName = mar.marName.replace('3', '2')
          break
    elif (hardpoint.index == 56):  #add No FSD mar
      mar = copy.deepcopy(gv.MarList[29])
      ship.currentMAR.append(mar)
    elif (hardpoint.index == 57):  # remove stealth systems mar
      for mar in ship.currentMAR:
        if (mar.marIndex == 47):
          ship.currentMAR.remove(mar)
          break
    elif (hardpoint.index == 58):  #restore +1 MV, -1 SR
      ship.SR -= 1
      ship.MV += 1
    elif (hardpoint.index == 59):  # remove difficult target mar, +1 hp 
      for mar in ship.currentMAR:
        if (mar.marIndex == 13):
          ship.currentMAR.remove(mar)
          break
        ship.HP += 1
    elif (hardpoint.index == 60):  # remove +3 WC, gain planetfall 
      for mar in ship.MAR:
        if (mar.marIndex == 58):
          copyMar = copy.deepcopy(mar)
          ship.currentMAR.append(copyMar)
          break
        ship.WC -= 3
    elif (hardpoint.index == 61):  #-1 planetfall, restore durable
      for mar in ship.currentMAR:
        if (mar.marIndex == 58):
          mar.intValue1 -= 1
          break
      for mar in ship.MAR:
        if (mar.marIndex == 57):
          copyMar = copy.deepcopy(mar)
          ship.currentMAR.append(copyMar)
          break
    elif (hardpoint.index == 62):  #remove difficult target, restore durable
      for mar in ship.currentMAR:
        if (mar.marIndex == 13):
          ship.currentMAR.remove(mar)
          break
      for mar in ship.MAR:
        if (mar.marIndex == 57):
          copyMar = copy.deepcopy(mar)
          ship.currentMAR.append(copyMar)
          break
    elif (hardpoint.index == 63):  #lose 2 MN, restore planetfall
      ship.MN -= 2
      for mar in ship.MAR:
        if (mar.marIndex == 58):
          copyMar = copy.deepcopy(mar)
          ship.currentMAR.append(copyMar)
          break
    elif (hardpoint.index == 64):  #lose 2 MN and Drone Mines, restore planetfall
      ship.MN -= 2
      for mar in ship.currentMAR:
        if (mar.marIndex == 16):
          ship.currentMAR.remove(mar)
          break
      for mar in ship.MAR:
        if (mar.marIndex == 58):
          copyMar = copy.deepcopy(mar)
          ship.currentMAR.append(copyMar)
          break
    elif (hardpoint.index == 65):  #remove reinforced fore mar
      for mar in ship.currentMAR:
        if (mar.marIndex == 36):
          ship.currentMAR.remove(mar)
          break
    elif (hardpoint.index == 66):  #reduce sr by 1, hp by 1
      ship.SR -= 1
      ship.HP -= 1
    elif (hardpoint.index == 67):  #remove energy transfer 1 MAR
      for mar in ship.currentMAR:
        if (mar.marIndex == 20):
          ship.currentMAR.remove(mar)
          break
    elif (hardpoint.index == 68):  #remove maneouverable MAR
      for mar in ship.currentMAR:
        if (mar.marIndex == 20):
          ship.currentMAR.remove(mar)
          break
    elif (hardpoint.index == 69):  #change fore scatter to fore fixed kinetic
      for weapon in ship.currentWeapons:
        if (weapon.weaponType == 'Kinetic' and weapon.arcDirection == 'Fixed'):
          weapon.weaponType = 'Scatter'
          weapon.arcRestriction = 'Arc'
    elif (hardpoint.index == 70):  #remove maneouverable MAR
      for mar in ship.currentMAR:
        if (mar.marIndex == 13):
          ship.currentMAR.remove(mar)
          break
    elif (hardpoint.index == 71):  #remove second assault, gain planetfall mar
      for newMar in ship.MAR:
        if (newMar.marIndex == 58):
          newMar = copy.deepcopy(gv.MarList[58])
          ship.currentMAR.append(mar)
          break
      for mar in ship.currentMAR:
        if (mar.marIndex == 39):
          ship.currentMAR.remove(mar)
          break
    elif (hardpoint.index == 72):  #remove PD barrage, gain planetfall mar
      for newMar in ship.MAR:
        if (newMar.marIndex == 58):
          newMar = copy.deepcopy(gv.MarList[58])
          ship.currentMAR.append(mar)
          break
      for mar in ship.currentMAR:
        if (mar.marIndex == 32):
          ship.currentMAR.remove(mar)
          break
    elif (hardpoint.index == 73):  #lose 1HP, gain durable mar
      ship.HP -= 1
      for newMar in ship.MAR:
        if (newMar.marIndex == 57):
          newMar = copy.deepcopy(gv.MarList[57])
          ship.currentMAR.append(mar)
          break
    elif (hardpoint.index == 74):  #turrets to nuclear, remove planetfall
      for newMar in ship.MAR:
        if (newMar.marIndex == 58):
          newMar = copy.deepcopy(gv.MarList[58])
          ship.currentMAR.append(mar)
          break
      for weapon in ship.currentWeapons:
        if (weapon.weaponType == 'Nuclear' and weapon.category == 'Direct'):
          weapon.weaponType = 'Primary'
          break
    elif (hardpoint.index == 75):  # -1 planetfall, +2 WC
      for mar in ship.currentMAR:
        if (mar.marIndex == 58):
          mar.intValue1 -= 1
          break
      ship.WC += 2
    elif (hardpoint.index == 76):  # -3 AP, add planetfall mar
      ship.AP -= 3
      for mar in ship.MAR:
        if (mar.marIndex == 58):
          newMar = copy.deepcopy(mar)
          ship.currentMAR.append(newMar)
          break
    elif (hardpoint.index == 77):  #remove optional turret weapons
      for weapon in self.currentWeapons:
        if (weapon.arcDirection == 'Any' and weapon.hardpoint == True):
          weapon.includeAsHardpoint = False
    elif (hardpoint.index == 78):  # +3 AP, remove planetfall mar
      ship.SR -= 1
      for mar in ship.currentMAR:
        if (mar.marIndex == 43):
          ship.currentMAR.remove(mar)
          break
    elif (hardpoint.index == 79):  # designation to Frigate, remove planetfall mar
      ship.designation = 'Frigate'
      for mar in ship.MAR:
        if (mar.marIndex == 58):
          newMar = copy.deepcopy(mar)
          ship.currentMAR.append(newMar)
          break
    elif (hardpoint.index == 80):
      for weapon in ship.currentWeapons:
        if (weapon.weaponType == 'Scatter'):
          weapon.weaponType = 'Primary'
    elif (hardpoint.index == 81):
      mar = copy.deepcopy(gv.MarList[1])
      ship.currentMAR.append(mar)
      mar = copy.deepcopy(gv.MarList[57])
      ship.currentMAR.append(mar)
    elif (hardpoint.index == 82):
      for mar in ship.currentMAR:
        if (mar.marIndex == 58):
          mar.intValue1 -= 1
      ship.WC += 3
    elif (hardpoint.index == 83):
      for mar in ship.currentMAR:
        if (mar.marIndex == 58):
          mar.intValue1 -= 1
      ship.AP += 3
    elif (hardpoint.index == 84):  # -1 planetfall, SR +1
      for mar in ship.currentMAR:
        if (mar.marIndex == 58):
          mar.intValue1 -= 1
      ship.SR += 1
    elif (hardpoint.index == 85):  # -2 AP, add planetfall mar
      ship.AP -= 2
      for mar in ship.MAR:
        if (mar.marIndex == 58):
          newMar = copy.deepcopy(mar)
          ship.currentMAR.append(newMar)
          break
    elif (hardpoint.index == 86):  # -6 AP, add planetfall mar
      ship.AP -= 6
      for mar in ship.MAR:
        if (mar.marIndex == 58):
          newMar = copy.deepcopy(mar)
          ship.currentMAR.append(newMar)
          break
    elif (hardpoint.index == 87):  #remove bigger batteries, gain planetfall mar
      for newMar in ship.MAR:
        if (newMar.marIndex == 58):
          newMar = copy.deepcopy(gv.MarList[58])
          ship.currentMAR.append(mar)
          break
      for mar in ship.currentMAR:
        if (mar.marIndex == 6):
          ship.currentMAR.remove(mar)
          break
    elif (hardpoint.index == 88): #upgrade improved shunt matrix from 3 to 4
      for mar in ship.currentMAR:
        if (mar.marIndex == 59):
          mar.intValue1 = 3
          break
    elif (hardpoint.index == 89):  #remove quick launch and -4 WC, gain planetfall mar
      ship.WC -= 4
      for newMar in ship.MAR:
        if (newMar.marIndex == 58):
          newMar = copy.deepcopy(gv.MarList[58])
          ship.currentMAR.append(mar)
          break
      for mar in ship.currentMAR:
        if (mar.marIndex == 35):
          ship.currentMAR.remove(mar)
          break
    elif (hardpoint.index == 90):  # -3 WC, gain planetfall mar
      ship.WC -= 3
      for newMar in ship.MAR:
        if (newMar.marIndex == 58):
          newMar = copy.deepcopy(gv.MarList[58])
          ship.currentMAR.append(mar)
          break
    elif (hardpoint.index == 91):  #gain cloaking field, remove planetfall mar
      ship.cloakingField = False
    elif (hardpoint.index == 92):  # -3 MN, +2 MV
      ship.MV += 2
      ship.MN -= 3
    elif (hardpoint.index == 93):  # -1 AP, +1 CP
      ship.AP -= 1
      ship.MN += 1
    elif (hardpoint.index == 94):  # -2 MV, add durable mar
      ship.MV -= 2
      newMar = copy.deepcopy(gv.MarList[57])
      ship.currentMAR.append(mar)
    elif (hardpoint.index == 95):  # lose torp, gain planetfall 
      for weapon in ship.currentWeapons:
        if (weapon.weaponType == 'Torpedo') and (weapon.hardpoint == True):
          weapon.includeAsHardpoint = False
      for newMar in ship.MAR:
        if (newMar.marIndex == 58):
          newMar = copy.deepcopy(gv.MarList[58])
          ship.currentMAR.append(mar)
          break
    elif (hardpoint.index == 96): # -1 planetfall, +2 PD
      for mar in ship.currentMAR:
        if (mar.marIndex == 58):
          mar.intValue1 -= 1
          break
      ship.PD += 2
    elif (hardpoint.index == 97):  # -1 SR, add durable mar
      ship.SR -= 1
      newMar = copy.deepcopy(gv.MarList[57])
      ship.currentMAR.append(mar)
    elif (hardpoint.index == 98):  #remove torp, remove biohzard mar
      for mar in ship.currentMAR:
        if (mar.marIndex == 7):
          ship.currentMAR.remove(mar)
          break
      for weapon in ship.currentWeapons:
        if (weapon.weaponType == 'Torpedo') and (weapon.hardpoint == True):
          weapon.includeAsHardpoint = False
    elif (hardpoint.index == 99):  #remove the Scout MAR
      for mar in ship.currentMAR:
        if (mar.marIndex == 38):
          ship.currentMAR.remove(mar)
    
        
      
      
      
        
#ShipEditorBase::OnAddHardpoint, ShipRecord::ApplyHardpoints(), FleetEditorBase::AddHardpoint

    ship.currentHardpoints.remove(hardpoint)
    self.UpdateFleetCost()
    self.UpdateShipDetail(ship, 'fleet')
    self.CheckFleet()
    return
  
  def OnChangeHardpointQty(self, event):
    ship = dataStruct.ShipRecord()
    hardpoint = dataStruct.HardpointRecord()
    treeData = self.trFleetDisplay.GetItemData(self.selectedFleetShipID)
    ship = treeData.GetData()
    if (ship == None or ship.currentHardpoints == []):
      return
    if (event.GetId()== FirestormDialogs.spHardpointQty0) and (self.cbHardpoint0.IsChecked()):
      for selectHardpoint in ship.currentHardpoints:
        if (selectHardpoint.index == ship.availableHardPoints[0].index):
          selectHardpoint.numSelected = self.spHardpoint0.GetValue()
          break
    elif (event.GetId()== FirestormDialogs.spHardpointQty1) and (self.cbHardpoint1.IsChecked()):
      for selectHardpoint in ship.currentHardpoints:
        if (selectHardpoint.index == ship.availableHardPoints[1].index):
          selectHardpoint.numSelected = self.spHardpoint1.GetValue()
          break      
    elif (event.GetId()== FirestormDialogs.spHardpointQty2) and (self.cbHardpoint2.IsChecked()):
      for selectHardpoint in ship.currentHardpoints:
        if (selectHardpoint.index == ship.availableHardPoints[2].index):
          selectHardpoint.numSelected = self.spHardpoint2.GetValue()
          break
    elif (event.GetId()== FirestormDialogs.spHardpointQty3) and (self.cbHardpoint3.IsChecked()):
      for selectHardpoint in ship.currentHardpoints:
        if (selectHardpoint.index == ship.availableHardPoints[3].index):
          selectHardpoint.numSelected = self.spHardpoint3.GetValue()
          break
    elif (event.GetId()== FirestormDialogs.spHardpointQty4) and (self.cbHardpoint4.IsChecked()):
      for selectHardpoint in ship.currentHardpoints:
        if (selectHardpoint.index == ship.availableHardPoints[4].index):
          selectHardpoint.numSelected = self.spHardpoint4.GetValue()
          break
    elif (event.GetId()== FirestormDialogs.spHardpointQty5) and (self.cbHardpoint5.IsChecked()):
      for selectHardpoint in ship.currentHardpoints:
        if (selectHardpoint.index == ship.availableHardPoints[5].index):
          selectHardpoint.numSelected = self.spHardpoint5.GetValue()
          break
    elif (event.GetId()== FirestormDialogs.spHardpointQty6) and (self.cbHardpoint6.IsChecked()):
      for selectHardpoint in ship.currentHardpoints:
        if (selectHardpoint.index == ship.availableHardPoints[6].index):
          selectHardpoint.numSelected = self.spHardpoint5.GetValue()
          break
    
    self.UpdateFleetCost()
    self.UpdateShipDetail(ship, 'fleet')
    self.CheckFleet()
    return
  
  def AddHardpoint(self, event):
    ship = dataStruct.ShipRecord()
    hardpoint = dataStruct.HardpointRecord()
    weapon = dataStruct.WeaponRecord()
    mar = dataStruct.MarRecord()
    treeData = self.trFleetDisplay.GetItemData(self.selectedFleetShipID)
    ship = treeData.GetData()
    if (event.GetEventObject().GetName() == 'idHardpoint0'):
      hardpoint = copy.deepcopy(ship.availableHardPoints[0])
      numHardpoint = self.spHardpoint0.GetValue()
    elif (event.GetEventObject().GetName() == 'idHardpoint1'):
      hardpoint = copy.deepcopy(ship.availableHardPoints[1])
      numHardpoint = self.spHardpoint1.GetValue()
    elif (event.GetEventObject().GetName() == 'idHardpoint2'):
      hardpoint = copy.deepcopy(ship.availableHardPoints[2])
      numHardpoint = self.spHardpoint2.GetValue()
    elif (event.GetEventObject().GetName() == 'idHardpoint3'):
      hardpoint = copy.deepcopy(ship.availableHardPoints[3])
      numHardpoint = self.spHardpoint3.GetValue()
    elif (event.GetEventObject().GetName() == 'idHardpoint4'):
      hardpoint = copy.deepcopy(ship.availableHardPoints[4])
      numHardpoint = self.spHardpoint4.GetValue()
    elif (event.GetEventObject().GetName() == 'idHardpoint5'):
      hardpoint = copy.deepcopy(ship.availableHardPoints[5])
      numHardpoint = self.spHardpoint5.GetValue()
    elif (event.GetEventObject().GetName() == 'idHardpoint6'):
      hardpoint = copy.deepcopy(ship.availableHardPoints[6])
      numHardpoint = self.spHardpoint6.GetValue()
    else:
      return

    if (hardpoint.index == 1):  #decrease TL by amount
      ship.TL -= max(hardpoint.intValue1, 0)  #cannot go below 0
    elif (hardpoint.index == 2):  #increase WC
      ship.WC += (numHardpoint * hardpoint.intValue1)
      self.SetSRS(ship)
    elif (hardpoint.index == 3):  #change torpedo arc
      for weapon in ship.currentWeapons:
        if (weapon.weaponType == 'Torpedo'):
          weapon.arcRestriction = 'Arc'
    elif (hardpoint.index == 4):  #increase MV, may be selected multiple on some ships
      ship.MV += (numHardpoint * hardpoint.intValue1)
    elif (hardpoint.index == 5):  #increase SR
      ship.SR += (numHardpoint * hardpoint.intValue1)
    elif (hardpoint.index == 6):  #upgrade to Nuclear Torp
      for weapon in ship.currentWeapons:
        if (weapon.weaponType == 'Torpedo'):
          weapon.weaponType = 'Nuclear Torpedo'
    elif (hardpoint.index == 7):  #increase HP
      ship.HP += (numHardpoint * hardpoint.intValue1)
    elif (hardpoint.index == 8):  #increase MN
      ship.MN += (numHardpoint * hardpoint.intValue1)
    elif (hardpoint.index == 9):  #Remove Ablative Plating MAR
      for mar in ship.currentMAR:
        if (mar.marIndex == 1):
          ship.currentMAR.remove(mar)
          break
    elif (hardpoint.index == 11):  #increase Command Distance
      pass
    elif (hardpoint.index == 12):  #upgrade Primary to Beam
      for weapon in ship.currentWeapons:
        if (weapon.weaponType == 'Primary'):
          weapon.weaponType = 'Beam'
    elif (hardpoint.index == 13):  #change fixed fore to Scatter
      for weapon in ship.currentWeapons:
        if ((weapon.arcDirection == 'Forward') and (weapon.arcRestriction == 'Fixed')):
          weapon.weaponType = 'Scatter'
          weapon.arcRestriction = 'Arc'
    elif (hardpoint.index == 14):  #increase PD
      ship.PD += (numHardpoint * hardpoint.intValue1)
      if (hardpoint.marValue == 6):  #add the bigger batteries MAR
        mar = copy.deepcopy(gv.MarList[6])
        ship.currentMAR.append(mar)
    elif (hardpoint.index == 15):  #increase AP
      ship.AP += (numHardpoint * hardpoint.intValue1)
      if (hardpoint.marValue == 39):  #add the second assault mar
        mar = copy.deepcopy(gv.MarList[39])
        ship.currentMAR.append(mar) 
      if (hardpoint.marValue == 45):  #add the special forces mar
        mar = copy.deepcopy(gv.MarList[45])
        ship.currentMAR.append(mar) 
    elif (hardpoint.index == 16):  # replace energy transfer
      mar = copy.deepcopy(gv.MarList[35])
      ship.currentMAR.append(mar)
    elif (hardpoint.index == 17):  #replace energy transfer (beam,1) with (beam,2) mar
      for mar in ship.currentMAR:
        if (mar.marIndex == 20):
          mar.intValue1 = 2
          mar.marName = mar.marName.replace('1', '2') 
          break
    elif (hardpoint.index == 18):  #gain minefields mar
      mar = copy.deepcopy(gv.MarList[28])
      mar.intValue1 = 4
      mar.intValue2 = 6
      ship.currentMAR.append(mar)
    elif (hardpoint.index == 19):  #increase CP
      ship.CP += (numHardpoint * hardpoint.intValue1)
    elif (hardpoint.index == 20):  #gain shunt matrix (4) mar
      mar = copy.deepcopy(gv.MarList[44])
      mar.intValue1 = 4
      mar.marName = mar.marName.replace('value', '4')
      ship.currentMAR.append(mar)
    elif (hardpoint.index == 21):  #gain ops center mar
      mar = copy.deepcopy(gv.MarList[30])
      ship.currentMAR.append(mar)
    elif (hardpoint.index == 22):  #gain deck crews mar
      mar = copy.deepcopy(gv.MarList[12])
      ship.currentMAR.append(mar)
    elif (hardpoint.index == 23):  #gain torpedo spook mar
      mar = copy.deepcopy(gv.MarList[51])
      ship.currentMAR.append(mar)
    elif (hardpoint.index == 24):  #gain special forces mar
      mar = copy.deepcopy(gv.MarList[45])
      ship.currentMAR.append(mar)
    elif (hardpoint.index == 25):  #gain sector shielding mar
      mar = copy.deepcopy(gv.MarList[41])
      ship.currentMAR.append(mar)
    elif (hardpoint.index == 26):  #gain shield projector self mar
      mar = copy.deepcopy(gv.MarList[43])
      mar.intValue1 = 0
      mar.marName = mar.marName.replace('Range', 'Self')
      ship.currentMAR.append(mar)
    elif (hardpoint.index == 27):  #gain shield projector8 mar
      mar = copy.deepcopy(gv.MarList[43])
      mar.intValue1 = 8
      mar.marName = mar.marName.replace('Range', '8')
      ship.currentMAR.append(mar)
    elif (hardpoint.index == 28):  #gain countermeasures mar
      mar = copy.deepcopy(gv.MarList[10])
      ship.currentMAR.append(mar)
    elif (hardpoint.index == 29):  #torpedos gain decimator warheads mar
      mar = copy.deepcopy(gv.MarList[11])
      mar.strValue1 = 'Torpedoes'
      ship.currentMAR.append(mar)
    elif (hardpoint.index == 30):  #include the cyberwarfare weapons
      for weapon in ship.currentWeapons:
        if (weapon.weaponType == hardpoint.strValue1):
          weapon.includeAsHardpoint = True
    elif (hardpoint.index == 31):  #gain self repair mar
      mar = copy.deepcopy(gv.MarList[42])
      ship.currentMAR.append(mar)
    elif (hardpoint.index == 32):  #increase CR
      ship.CR += (numHardpoint * hardpoint.intValue1)
    elif (hardpoint.index == 33):  #gain second assault mar
      mar = copy.deepcopy(gv.MarList[39])
      ship.currentMAR.append(mar)
    elif (hardpoint.index == 34):  #gain launch tubes mar
      mar = copy.deepcopy(gv.MarList[25])
      ship.currentMAR.append(mar)
    elif (hardpoint.index == 35):  #Remove Vulnerable MAR
      for mar in ship.currentMAR:
        if (mar.marIndex == 53):
          ship.currentMAR.remove(mar)
          break
    elif (hardpoint.index == 36):  #gain ambush (1) mar
      mar = copy.deepcopy(gv.MarList[3])
      mar.intValue1 = 1
      mar.marName = mar.marName.replace('value', '1')
      ship.currentMAR.append(mar)
    elif (hardpoint.index == 37):  #turrets gain split fire mar
      mar = copy.deepcopy(gv.MarList[46])
      mar.strValue1 = 'Turrets'
      ship.currentMAR.append(mar)
    elif (hardpoint.index == 38):  #gain stealth systems mar and reduce SR to 1
      mar = copy.deepcopy(gv.MarList[47])
      mar.intValue1 = ship.SR
      ship.sr = 1
      ship.currentMAR.append(mar)
    elif (hardpoint.index == 39):  #gain dirty secrets mar
      mar = copy.deepcopy(gv.MarList[14])
      ship.currentMAR.append(mar)
    elif (hardpoint.index == 40):  #change beam fore to cyberwarfare
      for weapon in ship.currentWeapons:
        if ((weapon.arcDirection == 'Forward') and (weapon.weaponType == 'Beam')):
          weapon.weaponType = 'Cyberwarfare'
          break
    elif (hardpoint.index == 41):  #replace stealth systems with hidden killer MAR
      for mar in ship.currentMAR:
        if (mar.marIndex == 47):
          ship.currentMAR.remove(mar)
          break
      mar = copy.deepcopy(gv.MarList[23])
      ship.currentMAR.append(mar)
    elif (hardpoint.index == 42):  #change beam to cyberwarfare
      for weapon in ship.currentWeapons:
        if (weapon.weaponType == 'Beam'):
          weapon.weaponType = 'Cyberwarfare'
          break
    elif (hardpoint.index == 43):  #torpedos gain corrosive warheads mar
      mar = copy.deepcopy(gv.MarList[9])
      mar.strValue1 = 'Torpedoes'
      ship.currentMAR.append(mar)
    elif (hardpoint.index == 44):  #torpedos gain biohazard warheads mar
      mar = copy.deepcopy(gv.MarList[7])
      mar.strValue1 = 'Torpedoes'
      ship.currentMAR.append(mar)
    elif (hardpoint.index == 45):  #include the beam weapons
      for weapon in ship.currentWeapons:
        if (weapon.weaponType == hardpoint.strValue1):
          weapon.includeAsHardpoint = True
    elif (hardpoint.index == 46):  #include the cyberwarfare weapons
      for weapon in ship.currentWeapons:
        if (weapon.weaponType == hardpoint.strValue1):
          weapon.includeAsHardpoint = True
    elif (hardpoint.index == 47):  #include the gravitational weapons and compressor MAR
      for weapon in ship.currentWeapons:
        if (weapon.weaponType == hardpoint.strValue1):
          weapon.includeAsHardpoint = True
      mar = copy.deepcopy(gv.MarList[55])
      ship.currentMAR.append(mar)
    elif (hardpoint.index == 48):  #include the cyberwarfare with Assault Robots MAR
      for weapon in ship.currentWeapons:
        if (weapon.weaponType == hardpoint.strValue1):
          weapon.includeAsHardpoint = True
      mar = copy.deepcopy(gv.MarList[5])
      ship.currentMAR.append(mar)
    elif (hardpoint.index == 49):  #include the torpedo weapons
      for weapon in ship.currentWeapons:
        if (weapon.weaponType == hardpoint.strValue1):
          weapon.includeAsHardpoint = True
    elif (hardpoint.index == 50):  #gain weapon shielding mar
      mar = copy.deepcopy(gv.MarList[54])
      ship.currentMAR.append(mar)
    elif (hardpoint.index == 51):  #gain pack hunters mar
      mar = copy.deepcopy(gv.MarList[31])
      ship.currentMAR.append(mar)
    elif (hardpoint.index == 52):  #gain high energy mar
      mar = copy.deepcopy(gv.MarList[22])
      mar.strValue1 = 'Beam'
      ship.currentMAR.append(mar)
    elif (hardpoint.index == 53):  #gain point defense barrage mar
      mar = copy.deepcopy(gv.MarList[32])
      ship.currentMAR.append(mar)
    elif (hardpoint.index == 54):  #gain shunt matrix (2) mar
      mar = copy.deepcopy(gv.MarList[44])
      mar.intValue1 = 2
      mar.marName = mar.marName.replace('value', '2')
      ship.currentMAR.append(mar)
    elif (hardpoint.index == 55):  #replace ambush (2) with ambush (3)
      for mar in ship.currentMAR:
        if (mar.marIndex == 3):
          mar.intValue1 = 3
          mar.marName = mar.marName.replace('2', '3')
          break
    elif (hardpoint.index == 56):  #Remove No FSD MAR
      for mar in ship.currentMAR:
        if (mar.marIndex == 29):
          ship.currentMAR.remove(mar)
          break
    elif (hardpoint.index == 57):  #gain stealth systems mar
      mar = copy.deepcopy(gv.MarList[47])
      ship.currentMAR.append(mar)
    elif (hardpoint.index == 58):  #gain +1 SR, -1 MV
      ship.SR += 1
      ship.MV -= 1
    elif (hardpoint.index == 59):  #gain difficult target, -1 HP
      mar = copy.deepcopy(gv.MarList[13])
      ship.currentMAR.append(mar)
      ship.HP -= 1
    elif (hardpoint.index == 60):  #gain +3 WC, lose planetfall
      ship.WC += 3
      for mar in ship.currentMAR:
        if (mar.marIndex == 58):
          ship.currentMAR.remove(mar)
          break
    elif (hardpoint.index == 61):  #gain +1 planetfall, lose durable mar
      for mar in ship.currentMAR:
        if (mar.marIndex == 58):
          mar.intValue1 += 1
      for mar in ship.currentMAR:
        if (mar.marIndex == 57):
          ship.currentMAR.remove(mar)
    elif (hardpoint.index == 62):  #gain difficult target, lose durable mar
      mar = copy.deepcopy(gv.MarList[13])
      ship.currentMAR.append(mar)
      for mar in ship.currentMAR:
        if (mar.marIndex == 57):
          ship.currentMAR.remove(mar)
    elif (hardpoint.index == 63):  #gain +2 MN, lose planetfall
      ship.MN += 2
      for mar in ship.currentMAR:
        if (mar.marIndex == 58):
          ship.currentMAR.remove(mar)
          break
    elif (hardpoint.index == 64):  #gain +2 MN and Drone Mines, lose planetfall
      ship.MN += 2
      mar = copy.deepcopy(gv.MarList[16])
      ship.currentMAR.append(mar)
      for mar in ship.currentMAR:
        if (mar.marIndex == 58):
          ship.currentMAR.remove(mar)
          break
    elif (hardpoint.index == 65):  #gain reinforced fore mar
      mar = copy.deepcopy(gv.MarList[13])
      mar.strValue1 = "Forward"
      ship.currentMAR.append(mar)
    elif (hardpoint.index == 66):  #gain 1SR, 1HP
      ship.SR += 1
      ship.HP += 1
    elif (hardpoint.index == 67): #gain energy transfer 1 MAR
      mar = copy.deepcopy(gv.MarList[20])
      mar.strValue1 = "Beam"
      mar.intValue1 = 1
      ship.currentMAR.append(mar)
    elif (hardpoint.index == 68): #gain maneouverable MAR
      mar = copy.deepcopy(gv.MarList[27])
      ship.currentMAR.append(mar)
    elif (hardpoint.index == 69): #change fore scatter to fore fixed kinetic
      for weapon in ship.currentWeapons:
        if (weapon.weaponType == 'Scatter' and weapon.arcDirection == 'Forward'):
          weapon.weaponType = 'Kinetic'
          weapon.arcRestriction = 'Fixed'
    elif (hardpoint.index == 70):  #gain reinforced port/starboard mar
      mar = copy.deepcopy(gv.MarList[13])
      mar.strValue1 = "Port/Starboard"
      ship.currentMAR.append(mar)
    elif (hardpoint.index == 71):  #gain second assault, remove planetfall mar
      mar = copy.deepcopy(gv.MarList[39])
      ship.currentMAR.append(mar)
      for mar in ship.currentMAR:
        if (mar.marIndex == 58):
          ship.currentMAR.remove(mar)
          break
    elif (hardpoint.index == 72):  #gain PD barrage, remove planetfall mar
      mar = copy.deepcopy(gv.MarList[32])
      ship.currentMAR.append(mar)
      for mar in ship.currentMAR:
        if (mar.marIndex == 58):
          ship.currentMAR.remove(mar)
          break
    elif (hardpoint.index == 73):  #gain +1 HP, remove durable mar
      ship.HP += 1
      for mar in ship.currentMAR:
        if (mar.marIndex == 57):
          ship.currentMAR.remove(mar)
          break
    elif (hardpoint.index == 74):  #upgrade to nuclear turrets, remove planetfall
      for weapon in ship.currentWeapons:
        if (weapon.category == 'Direct' and weapon.weaponType == 'Primary' and weapon.arcDirection == 'Any'):
          weapon.weaponType = 'Nuclear'
      for mar in ship.currentMAR:
        if (mar.marIndex == 58):
          ship.currentMAR.remove(mar)
          break
    elif (hardpoint.index == 75): #+1 planetfall, -2 WC
      for mar in ship.currentMAR:
        if (mar.marIndex == 58):
          mar.intValue1 += 1
          break
      ship.WC -= 2
    elif (hardpoint.index == 76):  # +3 AP, remove planetfall mar
      ship.AP += 3
      for mar in ship.currentMAR:
        if (mar.marIndex == 58):
          ship.currentMAR.remove(mar)
          break
    elif (hardpoint.index == 77):  #gain turret weapons
      for weapon in self.currentWeapons:
        if (weapon.arcDirection == 'Any' and weapon.hardpoint == True):
          weapon.includeAsHardpoint = True
    elif (hardpoint.index == 78):  #gain +1 SR and shield projector 0 mar
      ship.SR += 1
      mar = copy.deepcopy(gv.MarList[43])
      mar.intValue1 = 0
      ship.currentMAR.append(mar)
    elif (hardpoint.index == 79):  # change to Escort, remove planetfall mar
      ship.designation = 'Escort'
      for mar in ship.currentMAR:
        if (mar.marIndex == 58):
          ship.currentMAR.remove(mar)
          break
    elif (hardpoint.index == 80): #change primaries to scatter
      for weapon in ship.currentWeapons:
        if (weapon.weaponType == 'Primary'):
          weapon.weaponType = 'Scatter'
    elif (hardpoint.index == 81): #remove ablative and durable mars
      for mar in ship.currentMAR:
        if (mar.marIndex == 1):
          ship.currentMAR.remove(mar)
          break
      for mar in ship.currentMAR:
        if (mar.marIndex == 57):
          ship.currentMAR.remove(mar)
          break
    elif (hardpoint.index == 82): # gain +1 planetfall, -3 WC
      for mar in ship.currentMAR:
        if (mar.marIndex == 58):
          mar.intValue1 += 1
      ship.WC -= 3
    elif (hardpoint.index == 83): # gain +1 planetfall, -3 AP
      for mar in ship.currentMAR:
        if (mar.marIndex == 58):
          mar.intValue1 += 1
      ship.AP -= 3
    elif (hardpoint.index == 84): # gain +1 planetfall, -1 SR
      for mar in ship.currentMAR:
        if (mar.marIndex == 58):
          mar.intValue1 += 1
      ship.SR -= 1
    elif (hardpoint.index == 85):  # +2 AP, remove planetfall mar
      ship.AP += 2
      for mar in ship.currentMAR:
        if (mar.marIndex == 58):
          ship.currentMAR.remove(mar)
          break
    elif (hardpoint.index == 86):  # +6 AP, remove planetfall mar
      ship.AP += 6
      for mar in ship.currentMAR:
        if (mar.marIndex == 58):
          ship.currentMAR.remove(mar)
          break
    elif (hardpoint.index == 87):  #gain bigger batteries, remove planetfall mar
      mar = copy.deepcopy(gv.MarList[6])
      ship.currentMAR.append(mar)
      for mar in ship.currentMAR:
        if (mar.marIndex == 58):
          ship.currentMAR.remove(mar)
          break
    elif (hardpoint.index == 88): #upgrade improved shunt matrix from 3 to 4
      for mar in ship.currentMAR:
        if (mar.marIndex == 59):
          mar.intValue1 = 4
          break
    elif (hardpoint.index == 89):  #gain +4 WC and quick launch, remove planetfall mar
      mar = copy.deepcopy(gv.MarList[35])
      ship.currentMAR.append(mar)
      ship.WC += 4
      for mar in ship.currentMAR:
        if (mar.marIndex == 58):
          ship.currentMAR.remove(mar)
          break
    elif (hardpoint.index == 90):  #gain +3 WC, remove planetfall mar
      ship.WC += 3
      for mar in ship.currentMAR:
        if (mar.marIndex == 58):
          ship.currentMAR.remove(mar)
          break
    elif (hardpoint.index == 91):  #gain cloaking field, remove planetfall mar
      ship.cloakingField = True
      for mar in ship.currentMAR:
        if (mar.marIndex == 58):
          ship.currentMAR.remove(mar)
          break
    elif (hardpoint.index == 92):  # +3 MN, -2 MV
      ship.MV -= 2
      ship.MN += 3
    elif (hardpoint.index == 93):  # +1 AP, -1 CP
      ship.AP += 1
      ship.MN -= 1
    elif (hardpoint.index == 94):  # +2 MV, remove durable mar
      ship.MV += 2
      for mar in ship.currentMAR:
        if (mar.marIndex == 57):
          ship.currentMAR.remove(mar)
          break      
    elif (hardpoint.index == 95):  # gain torp , remove planetfall mar
      for weapon in ship.currentWeapons:
        if (weapon.weaponType == 'Torpedo') and (weapon.hardpoint == True):
          weapon.includeAsHardpoint = True
      for mar in ship.currentMAR:
        if (mar.marIndex == 58):
          ship.currentMAR.remove(mar)
          break
    elif (hardpoint.index == 96): #+1 planetfall, -2 PD
      for mar in ship.currentMAR:
        if (mar.marIndex == 58):
          mar.intValue1 += 1
          break
      ship.PD -= 2
    elif (hardpoint.index == 97):  # +1 SR, remove durable mar
      ship.SR += 1
      for mar in ship.currentMAR:
        if (mar.marIndex == 57):
          ship.currentMAR.remove(mar)
          break
    elif (hardpoint.index == 98):# gain torp and biohazard mar
      for weapon in ship.currentWeapons:
        if (weapon.weaponType == 'Torpedo') and (weapon.hardpoint == True):
          weapon.includeAsHardpoint = True
        mar = copy.deepcopy(gv.MarList[7])
        mar.strValue1 = 'Torpedo'
        ship.currentMAR.append(mar)
    elif (hardpoint.index == 99): #gain Scout MAR
      mar = copy.deepcopy(gv.MarList[38])
      ship.currentMAR.append(mar)
      
    
      
#ShipEditorBase::OnAddHardpoint, ShipRecord::ApplyHardpoints(), FleetEditorBase::RemoveHardpoint
      
    hardpoint.numSelected = numHardpoint
    ship.currentHardpoints.append(hardpoint)
    self.UpdateFleetCost()
    self.UpdateShipDetail(ship, 'fleet')
    self.CheckFleet()
    return
  
  
  
  def AddUpgrade(self, event):
    mar = dataStruct.MarRecord()
    ship = dataStruct.ShipRecord()
    upgrade = dataStruct.UpgradeRecord()
    weapon = dataStruct.WeaponRecord()
    treeData = self.trFleetDisplay.GetItemData(self.selectedFleetShipID)
    ship = treeData.GetData()
    if (event.GetEventObject().GetName() == 'idUpgrade0'):
      upgrade = copy.deepcopy(ship.upgrades[0])
    elif (event.GetEventObject().GetName() == 'idUpgrade1'):
      upgrade = copy.deepcopy(ship.upgrades[1])
    elif (event.GetEventObject().GetName() == 'idUpgrade2'):
      upgrade = copy.deepcopy(ship.upgrades[2])
    elif (event.GetEventObject().GetName() == 'idUpgrade3'):
      upgrade = copy.deepcopy(ship.upgrades[3])
    elif (event.GetEventObject().GetName() == 'idUpgrade4'):
      upgrade = copy.deepcopy(ship.upgrades[4])
    elif (event.GetEventObject().GetName() == 'idUpgrade5'):
      upgrade = copy.deepcopy(ship.upgrades[5])
    elif (event.GetEventObject().GetName() == 'idUpgrade6'):
      upgrade = copy.deepcopy(ship.upgrades[6])
    else:
      return
    
    #modify ship stats based on upgrade chosen
    if (upgrade.index == 1000):  #remove ablative plating MAR
      for mar in ship.currentMAR:
        if (mar.marIndex == 1):
          ship.currentMAR.remove(mar)
          break
    elif (upgrade.index == 1001): #gain ambush 2 MAR
      mar = copy.deepcopy(gv.MarList[3])
      mar.intValue1 = 2
      mar.marText = "Gain the Ambush (2) MAR"
      ship.currentMAR.append(gv.MarList)
    elif (upgrade.index == 1002): #gain +2 AP
      ship.AP += 2
    elif (upgrade.index == 1003):  #gain assault blitz mar
      mar = copy.deepcopy(gv.MarList[4])
      ship.currentMAR.append(mar)
    elif (upgrade.index == 1004): #beam gains biohazard ammo mar
      mar = copy.deepcopy(gv.MarList[7])
      mar.strValue1 = 'Beam'
      mar.marName = mar.marName.replace('Weapon', 'Beam')
      ship.currentMAR.append(mar)
    elif (upgrade.index == 1005): #beam gains High Energy mar
      mar = copy.deepcopy(gv.MarList[22])
      mar.strValue1 = 'Beam'
      mar.marName = mar.marName.replace('Weapon', 'Beam')
      ship.currentMAR.append(mar)
    elif (upgrade.index == 1006): # beam gains precision strike mar
      mar = copy.deepcopy(gv.MarList[33])
      mar.strValue1 = 'Beam'
      mar.marName = mar.marName.replace('Weapon', 'Beam')
      ship.currentMAR.append(mar)
    elif (upgrade.index == 1007): #beam gains split fire mar
      mar = copy.deepcopy(gv.MarList[46])
      mar.strValue1 = 'Beam'
      mar.marName = mar.marName.replace('Weapon', 'Beam')
      ship.currentMAR.append(mar)
    elif (upgrade.index == 1008): # bigger batteries mar
      mar = copy.deepcopy(gv.MarList[6])
      ship.currentMAR.append(mar)
    elif (upgrade.index == 1009): # countermeasures mar
      mar = copy.deepcopy(gv.MarList[10])
      ship.currentMAR.append(mar)
    elif (upgrade.index == 1010): # deck crews mar
      mar = copy.deepcopy(gv.MarList[12])
      ship.currentMAR.append(mar)
    elif (upgrade.index == 1011): # double mines mar
      mar = copy.deepcopy(gv.MarList[17])
      ship.currentMAR.append(mar)
    elif (upgrade.index == 1012): # drone mines mar
      mar = copy.deepcopy(gv.MarList[16])
      ship.currentMAR.append(mar)
    elif (upgrade.index == 1013): # experienced engineers
      mar = copy.deepcopy(gv.MarList[21])
      ship.currentMAR.append(mar)
    elif (upgrade.index == 1014): # beam energy transfer 1 mar
      mar = copy.deepcopy(gv.MarList[20])
      mar.intValue1 = 1
      mar.strValue1 = 'Beam'
      mar.marName = mar.marName.replace('type', mar.strValue1)
      mar.marName = mar.marName.replace('value', str(mar.intValue1))
      ship.currentMAR.append(mar)
    elif (upgrade.index == 1015): # beam energy transfer 2 mar
      mar = copy.deepcopy(gv.MarList[20])
      mar.intValue1 = 2
      mar.strValue1 = 'Beam'
      mar.marName = mar.marName.replace('type', mar.strValue1)
      mar.marName = mar.marName.replace('value', str(mar.intValue1))
      ship.currentMAR.append(mar)
    elif (upgrade.index == 1016): # energy transfer 2,all
      mar = copy.deepcopy(gv.MarList[20])
      mar.intValue1 = 2
      mar.strValue1 = 'All'
      mar.marName = mar.marName.replace('type', mar.strValue1)
      mar.marName = mar.marName.replace('value', str(mar.intValue1))
      ship.currentMAR.append(mar)
    elif (upgrade.index == 1017): # fore fixed gain decimator
      mar = copy.deepcopy(gv.MarList[11])
      ship.currentMAR.append(mar)
    elif (upgrade.index == 1018): # gun racks gain split fire mar
      mar = copy.deepcopy(gv.MarList[46])
      mar.strValue1 = 'Gun Rack'
      mar.marName = mar.marName.replace('Weapon', mar.strValue1)
      ship.currentMAR.append(mar)
    elif (upgrade.index == 1019): # kinetics gain high energy mar
      mar = copy.deepcopy(gv.MarList[22])
      mar.strValue1 = 'Kinetic'
      mar.marName = mar.marName.replace('Weapon', mar.strValue1)
      ship.currentMAR.append(mar)
    elif (upgrade.index == 1020): # kinetics gain precision strike
      mar = copy.deepcopy(gv.MarList[33])
      mar.strValue1 = 'Kinetic'
      mar.marName = mar.marName.replace('Weapon', mar.strValue1)
      ship.currentMAR.append(mar)
    elif (upgrade.index == 1021): # manoeuverable
      mar = copy.deepcopy(gv.MarList[27])
      ship.currentMAR.append(mar)
    elif (upgrade.index == 1022): # mines gain biohazard ammo
      mar = copy.deepcopy(gv.MarList[7])
      mar.strValue1 = 'Mines'
      mar.marName = mar.marName.replace('Weapon', mar.strValue1)
      ship.currentMAR.append(mar)
    elif (upgrade.index == 1023): # gain +3 mines
      ship.MN += 3
    elif (upgrade.index == 1024): # pack hunter mar
      mar = copy.deepcopy(gv.MarList[31])
      ship.currentMAR.append(mar)
    elif (upgrade.index == 1025): # point defense barrage
      mar = copy.deepcopy(gv.MarList[32])
      ship.currentMAR.append(mar)
    elif (upgrade.index == 1026): # protected systems
      mar = copy.deepcopy(gv.MarList[34])
      ship.currentMAR.append(mar)
    elif (upgrade.index == 1027): # primaries gain biohazard
      mar = copy.deepcopy(gv.MarList[7])
      mar.strValue1 = 'Primary'
      mar.marName = mar.marName.replace('Weapon', mar.strValue1)
      ship.currentMAR.append(mar)
    elif (upgrade.index == 1028): # primary decimator warheads
      mar = copy.deepcopy(gv.MarList[11])
      mar.strValue1 = 'Primary'
      mar.marName = mar.marName.replace('Weapon', mar.strValue1)
      ship.currentMAR.append(mar)
    elif (upgrade.index == 1029): # primary high energy
      mar = copy.deepcopy(gv.MarList[22])
      mar.strValue1 = 'Primary'
      mar.marName = mar.marName.replace('Weapon', mar.strValue1)
      ship.currentMAR.append(mar)
    elif (upgrade.index == 1030): # primary precision strike
      mar = copy.deepcopy(gv.MarList[33])
      mar.strValue1 = 'Primary'
      mar.marName = mar.marName.replace('Weapon', mar.strValue1)
      ship.currentMAR.append(mar)
    elif (upgrade.index == 1031): # second assault mar
      mar = copy.deepcopy(gv.MarList[39])
      ship.currentMAR.append(mar)
    elif (upgrade.index == 1032): # quick launch
      mar = copy.deepcopy(gv.MarList[35])
      ship.currentMAR.append(mar)
    elif (upgrade.index == 1033): # reinforced
      mar = copy.deepcopy(gv.MarList[36])
      mar.strValue1 = 'Starboard/Port'
      mar.marName = mar.marName.replace('Arc', mar.strValue1)
      ship.currentMAR.append(mar)
    elif (upgrade.index == 1034): # scatter gains split fire mar
      mar = copy.deepcopy(gv.MarList[46])
      mar.strValue1 = 'Scatter'
      mar.marName = mar.marName.replace('Weapon', mar.strValue1)
      ship.currentMAR.append(mar)
    elif (upgrade.index == 1035): # 
      mar = copy.deepcopy(gv.MarList[46])
      mar.strValue1 = 'Scatter'
      mar.strValue2 = 'Starboard/Port'
      mar.marName = mar.marName.replace('Weapon', mar.strValue1)
      ship.currentMAR.append(mar)
    elif (upgrade.index == 1036): # sector shielding
      mar = copy.deepcopy(gv.MarList[41])
      ship.currentMAR.append(mar)
    elif (upgrade.index == 1037): # secured bulkhead
      mar = copy.deepcopy(gv.MarList[40])
      ship.currentMAR.append(mar)
    elif (upgrade.index == 1038): # self repair
      mar = copy.deepcopy(gv.MarList[42])
      ship.currentMAR.append(mar)
    elif (upgrade.index == 1039): # special forces mar
      mar = copy.deepcopy(gv.MarList[45])
      ship.currentMAR.append(mar)
    elif (upgrade.index == 1040): # 
      mar = copy.deepcopy(gv.MarList[46])
      mar.strValue1 = 'All'
      mar.strValue2 = 'Starboard/Port'
      mar.marName = mar.marName.replace('Weapon', mar.strValue1)
      ship.currentMAR.append(mar)
    elif (upgrade.index == 1041): # torpedo gains biohazard
      mar = copy.deepcopy(gv.MarList[7])
      mar.strValue1 = 'Torpedo'
      mar.marName = mar.marName.replace('Weapon', mar.strValue1)
      ship.currentMAR.append(mar)
    elif (upgrade.index == 1042): # torp corrosive
      mar = copy.deepcopy(gv.MarList[9])
      mar.strValue1 = 'Torpedo'
      mar.marName = mar.marName.replace('Weapon', mar.strValue1)
      ship.currentMAR.append(mar)
    elif (upgrade.index == 1043): # torp high energy
      mar = copy.deepcopy(gv.MarList[22])
      mar.strValue1 = 'Torpedo'
      mar.marName = mar.marName.replace('Weapon', mar.strValue1)
      ship.currentMAR.append(mar)
    elif (upgrade.index == 1044): # torpedo spook
      mar = copy.deepcopy(gv.MarList[51])
      ship.currentMAR.append(mar)
    elif (upgrade.index == 1045): # weapon shielding
      mar = copy.deepcopy(gv.MarList[54])
      ship.currentMAR.append(mar)
    elif (upgrade.index == 1046): # primary to scatter
      for weapon in ship.currentWeapons:
        if (weapon.weaponType == 'Primary'):
          weapon.weaponType = 'Scatter'
    elif (upgrade.index == 1047): # fore fixed weapons gain nuclear type
      for weapon in ship.currentWeapons:
        if (weapon.arcDirection == 'Forward') and (weapon.arcRestriction == 'Fixed'):
          weapon.weaponType = "Nuclear "+weapon.weaponType
    elif (upgrade.index == 1048): # torpedoes gain nuclear type
      for weapon in ship.currentWeapons:
        if (weapon.weaponType == 'Torpedo'):
          weapon.weaponType = "Nuclear Torpedo"
    elif (upgrade.index == 1049): # primary to beams
      for weapon in ship.currentWeapons:
        if (weapon.weaponType == 'Primary'):
          weapon.weaponType = "Beam"
    elif (upgrade.index == 1050): # starboard/port to beams
      for weapon in ship.currentWeapons:
        if (weapon.arcDirection == 'Starboard') or (weapon.arcDirection == 'Port'):
          weapon.weaponType = "Beam"
    elif (upgrade.index == 1051): # gun rack or turrets gain nuclear
      for weapon in ship.currentWeapons:
        if (weapon.arcDirection == 'Any'):
          weapon.weaponType = "Nuclear " + weapon.weaponType
    elif (upgrade.index == 1053): #beam gains corrosive mar
      mar = copy.deepcopy(gv.MarList[9])
      mar.strValue1 = 'Beam'
      mar.marName = mar.marName.replace('Weapon', 'Beam')
      ship.currentMAR.append(mar)
    elif (upgrade.index == 1054): # gain scout MAR
      mar = copy.deepcopy(gv.MarList[38])
      ship.currentMAR.append(mar)
    elif (upgrade.index == 1055):
      for weapon in ship.currentWeapons:
        if (weapon.weaponType == 'Primary'):
          weapon.weaponType = 'Cyberwarfare'
    elif (upgrade.index == 1056): # primary decimator warheads
      mar = copy.deepcopy(gv.MarList[11])
      mar.strValue1 = 'Beam'
      mar.marName = mar.marName.replace('Weapon', mar.strValue1)
      ship.currentMAR.append(mar)
    elif (upgrade.index == 1057): # primaries gain corrosive
      mar = copy.deepcopy(gv.MarList[9])
      mar.strValue1 = 'Primary'
      mar.marName = mar.marName.replace('Weapon', mar.strValue1)
      ship.currentMAR.append(mar)
    elif (upgrade.index == 1058): # stealth systems mar
      mar = copy.deepcopy(gv.MarList[47])
      ship.currentMAR.append(mar)
    elif (upgrade.index == 1059):  #energy transfer 1 to energy transfer 2
      for mar in ship.currentMAR:
        if (mar.marIndex == 20):
          mar.intValue1 = 2
          break
    elif (upgrade.index == 1060):  #command +2
      ship.commandDistance += 2
    elif (upgrade.index == 1061):  #move +1
      ship.MV += 1
    elif (upgrade.index == 1062):  #command +3
      ship.commandDistance += 3
    elif (upgrade.index == 1063):  #scatter to kinetic
      for weapon in ship.currentWeapons:
        if (weapon.weaponType == 'Scatter'):
          weapon.weaponType = 'Kinetic'
    elif (upgrade.index == 1064):  # PD +1
      ship.PD += 1
    elif (upgrade.index == 1065): # remove weapons shielding mar, gain durable mar
      for mar in ship.currentMAR:
        if (mar.marIndex == 57):
          ship.currentMAR.remove(mar)
          break
      mar = copy.deepcopy(gv.MarList[54])
      ship.currentMAR.append(mar)
    elif (upgrade.index == 1066):  # sr +1
      ship.SR += 1
    elif (upgrade.index == 1067): #torpedoes to cyberwarfare, add assault robot torp mar
      mar = copy.deepcopy(gv.MarList[5])
      ship.currentMAR.append(mar)
      for weapon in ship.currentWeapons:
        if (weapon.weaponType == 'Torpedo'):
          weapon.weaponType = 'Cyberwarfare'
    elif (upgrade.index == 1065): # gain precision strike mar
      mar = copy.deepcopy(gv.MarList[54])
      mar.strValue1 = 'Beam'
      ship.currentMAR.append(mar)

    ship.currentUpgrades.append(upgrade)
    self.UpdateFleetCost()
    self.UpdateShipDetail(ship, 'fleet')
    self.CheckFleet()
    return

  def OnViewEscortShip(self, event):
    ship = dataStruct.ShipRecord()
    treeData = self.trFleetDisplay.GetItemData(self.selectedFleetShipID)
    ship = treeData.GetData()
    index = self.rbxAccomp.GetSelection()
    if (index == 0):
      return
    else:
      self.UpdateShipDetail(ship.accompanimentIncluded[index-1].ship, 'escort')
    return
  
  def OnAddAccompaniment(self, event):
    ship = dataStruct.ShipRecord()
    treeData = self.trFleetDisplay.GetItemData(self.selectedFleetShipID)
    ship = treeData.GetData()
    index = self.rbxAccomp.GetSelection()
    if (index == 0):
      ship.currentAccompaniment = None
      self.bViewEscort.Enable(False)
      self.spNumAccomp.Enable(False)
    else:
      self.bViewEscort.Enable(True)
      self.spNumAccomp.Enable(True)
      #ship.currentAccompaniment = dataStruct.AccompanimentRecord()
      ship.currentAccompaniment = copy.deepcopy(ship.accompanimentIncluded[index-1])
      ship.currentAccompaniment.numSelected = self.spNumAccomp.GetValue()
      ship.currentAccompaniment.accompIndex = index-1
      self.spNumAccomp.SetRange(0,ship.currentAccompaniment.numAllowed)
      
    self.UpdateFleetList()
    self.UpdateFleetCost()
    self.UpdateShipDetail(ship, 'fleet')
    self.CheckFleet()
    return
  
  def OnChangeSRS(self, event):
    ship = dataStruct.ShipRecord()
    treeData = self.trFleetDisplay.GetItemData(self.selectedFleetShipID)
    ship = treeData.GetData()
  
    wingsUsed = 0
    wingsCost = 0
    ship.currentSRS[0].srsType = self.cxTokenType1.GetStringSelection()
    self.stTokenInfo1.SetLabel(dataStruct.srsTypes[ship.currentSRS[0].srsType])
    if (self.cxTokenType1.GetStringSelection() != 'None'):
      self.spTokenWings1.Enable(True)
      ship.currentSRS[0].numWings = self.spTokenWings1.GetValue()
      ship.currentSRS[0].cost = ship.currentSRS[0].numWings * 5
      wingsCost += ship.currentSRS[0].cost
      wingsUsed += ship.currentSRS[0].numWings
    else:
      self.spTokenWings1.Enable(False)
      
    ship.currentSRS[1].srsType = self.cxTokenType2.GetStringSelection()
    self.stTokenInfo2.SetLabel(dataStruct.srsTypes[ship.currentSRS[1].srsType])
    if (self.cxTokenType2.GetStringSelection() != 'None'):
      self.spTokenWings2.Enable(True)
      ship.currentSRS[1].numWings = self.spTokenWings2.GetValue()
      ship.currentSRS[1].cost = ship.currentSRS[1].numWings * 5
      wingsCost += ship.currentSRS[1].cost
      wingsUsed += ship.currentSRS[1].numWings
    else:
      self.spTokenWings2.Enable(False)

    self.stWingCost.SetLabel(str(wingsCost))
    self.stRemainingWC.SetLabel(str(ship.WC - wingsUsed))
    self.UpdateFleetCost()
    self.UpdateShipDetail(ship, 'fleet')
    self.CheckFleet()
    return
  
  def SetSRS(self, ship):
    srs = dataStruct.srsRecord()
    if (not isinstance(ship, dataStruct.ShipRecord) or (ship.WC == 0)):
      self.stPromptWC.SetLabel("Selected ship has no wing capacity (WC = 0)")
      self.stWingCost.Hide()
      self.stWingCostLabel.Hide()
      self.stRemainingWC.Hide()
      self.cxTokenType1.Hide()
      self.cxTokenType2.Hide()
      self.spTokenWings1.Hide()
      self.spTokenWings2.Hide()
      self.stTokenInfo1.Hide()
      self.stTokenInfo2.Hide()
      self.stTokenLabel1.Hide()
      self.stTokenLabel2.Hide()
      self.stWingLabel1.Hide()
      self.stWingLabel2.Hide()
      return
    else:
      self.stPromptWC.SetLabel('Remaining Wing Capacity (WC):')
      self.stRemainingWC.Show()
      self.stWingCost.Show()
      self.stWingCostLabel.Show()
      self.cxTokenType1.Show()
      self.cxTokenType2.Show()
      self.spTokenWings1.Show()
      self.spTokenWings2.Show()
      self.stTokenInfo1.Show()
      self.stTokenInfo2.Show()
      self.stTokenLabel1.Show()
      self.stTokenLabel2.Show()
      self.stWingLabel1.Show()
      self.stWingLabel2.Show()
      
    
    wingsUsed = 0
    wingsCost = 0
    
    if (ship.currentSRS[0].srsType == 'None'):
      self.spTokenWings1.Enable(False)
    else:
      self.spTokenWings1.SetValue(ship.currentSRS[0].numWings)
      self.spTokenWings1.Enable(True)
      self.cxTokenType1.SetStringSelection(ship.currentSRS[0].srsType)
      self.stTokenInfo1.SetLabel(dataStruct.srsTypes[ship.currentSRS[0].srsType])
      ship.currentSRS[0].cost = 5 * ship.currentSRS[0].numWings
      wingsCost += ship.currentSRS[0].cost
      wingsUsed += ship.currentSRS[0].numWings
      
    if (ship.currentSRS[1].srsType == 'None'):
      self.spTokenWings2.Enable(False)
    else:
      self.spTokenWings2.SetValue(ship.currentSRS[1].numWings)
      self.spTokenWings2.Enable(True)
      self.cxTokenType2.SetStringSelection(ship.currentSRS[1].srsType)
      self.stTokenInfo2.SetLabel(dataStruct.srsTypes[ship.currentSRS[1].srsType])
      ship.currentSRS[1].cost = 5 * ship.currentSRS[1].numWings
      wingsCost += ship.currentSRS[1].cost
      wingsUsed += ship.currentSRS[1].numWings
      
    self.stRemainingWC.SetLabel(str(ship.WC-wingsUsed))
    self.stWingCost.SetLabel(str(wingsCost))
    return
  
  def SetAccompaniment(self, ship):
    if (not isinstance(ship, dataStruct.ShipRecord)) or (ship.accompanimentIncluded == []):
      self.stAccompaniment.SetLabel("Selected ship has no accompaniment available")
      self.spNumAccomp.Enable(False)
      self.bViewEscort.Enable(False)
      self.stQuantity.Enable(False)
      self.bAddAccomp.Enable(False)
      for index in range(18):
        self.rbxAccomp.SetItemLabel(index, "---------------------------------------")
        self.rbxAccomp.EnableItem(index, False)
      self.rbxAccomp.SetItemLabel(0, "No Accompaniment")
      return
    else:
      self.spNumAccomp.Enable(True)
      self.stQuantity.Enable(True)
      self.bViewEscort.Enable(True)
      self.bAddAccomp.Enable(True)
      self.stAccompaniment.SetLabel("Choose from the following accompaniment:")
      index = 1  #index 0 is reserved for the No Accompaniment entry
      self.rbxAccomp.EnableItem(0, True)
      for accomp in ship.accompanimentIncluded:
        self.rbxAccomp.EnableItem(index, True)
        self.rbxAccomp.SetItemLabel(index, ("0-{0} {1} Class {2} ({3} pts each)".format(accomp.numAllowed, accomp.ship.className, accomp.ship.designation, accomp.ship.baseCost)))
        index += 1
        if (index > 18):  #too many entries!
          logging.critical("{0} assigned too many escorts".format(accomp.ship.className))
          break
      if (ship.currentAccompaniment):
        self.rbxAccomp.SetSelection(ship.currentAccompaniment.accompIndex+1)  #offset +1 to avoid the 'No Accompaniment' entry
        self.spNumAccomp.SetValue(ship.currentAccompaniment.numSelected)
      else:
        self.rbxAccomp.SetSelection(0)
        self.spNumAccomp.SetValue(0)
    return
  
  def OnHelp(self, event):
    helpText = "This is the main editor for fleets.\n\nThe current inventory of ships for the selected faction is shown in the light green window.  The ships available in each tier will change based on the Maximum Fleet Value currently selected.\n\n"
    helpText += "To select a squadron for the fleet, double click on the Current Inventory window.  If the squadron can have more than one ship, you will be prompted to enter the number of ships to add.  The squadron will then be added to the Current Fleet window (light orange below the inventory window) in the appropriate tier.\n\n"
    helpText += "The Current Fleet window will show the squadron tiers, the type of squadron, and then the number of ships in the squadron.  Double click on a squadron to remove it from the fleet.  To change the number of ships in the squadron, select the number of ships shown in the field to the right of the fleet window and select 'Change' to modify the number of ships.\n\n"
    helpText += "The ship details are shown in the upper right text window.  Depending on the last selection made (either in the inventory or fleet window) the display will change color to match the area selected.\n   If selected from the inventory screen, the ship details will show the base configuration, and all available options.\n   If selected from the fleet screen, the statistics and options shown will be the current selected hardpoints, upgrades, etc.\n\n"
    helpText += "The lower right side of the window shows selection pages for Hardpoints, Upgrades,  Accompaniment, and SRS.\n"
    helpText += "For selecting the Hardpoints and Upgrades, simply select the option shown to add it to the current ship.\n"
    helpText += "For selecting accompaniment, choose the escort from the list of available options, choose the number of ships to add, and then click the 'Add Accompaniment' button.  "
    helpText += "To remove the escort ships, choose the 'No Accompaniment' option and click the 'Clear Accompaniment' button.\n"
    helpText += "To add SRS to the squadron, click on the checkbox to add the type of SRS, and then set the number of tokens and wings to be used.\n\n"
    
    dlg = HelpDialog(self, helpText)
    dlg.ShowModal()
    return
  
  def OnPrintFleet(self, event):
    dlg = PrintPreviewBase(self, self.fleet)
    dlg.ShowModal()
    return
  
  def OnViewMarDetails(self, event):
    """Display the MAR's selected for this ship in the help dialog."""
    if (not self.MarDisplay) or (self.MarDisplay == ""):
      txtBlock = "No MAR's available for the selected ship."
    else:
      txtBlock = self.MarDisplay
    dlg = HelpDialog(self, txtBlock)
    dlg.ShowModal()
    dlg.Destroy()
    return
  
  def OnRadioSelection(self, event):
    index = self.rbxAccomp.GetSelection()
    ship = dataStruct.ShipRecord()
    treeData = self.trFleetDisplay.GetItemData(self.selectedFleetShipID)
    ship = treeData.GetData()

    if (index == 0):
      self.spNumAccomp.Enable(False)
      self.bViewEscort.Enable(False)
      self.stQuantity.Enable(False)
      self.bAddAccomp.Enable(False)
      if (ship.currentAccompaniment):
        self.bAddAccomp.Enable(True)
        self.bAddAccomp.SetLabel('Clear Accompaniment')
      
    else:
      self.spNumAccomp.Enable(True)
      self.stQuantity.Enable(True)
      self.bViewEscort.Enable(True)
      self.bAddAccomp.Enable(True)
      self.bAddAccomp.SetLabel('Add Accompaniment')
      self.spNumAccomp.SetRange(1,ship.accompanimentIncluded[index-1].numAllowed)
    return

  def SetUpgrades(self, ship):
    self.cbUpgrade0.SetValue(False)
    self.cbUpgrade1.SetValue(False)
    self.cbUpgrade2.SetValue(False)
    self.cbUpgrade3.SetValue(False)
    self.cbUpgrade4.SetValue(False)
    self.cbUpgrade5.SetValue(False)
    self.cbUpgrade6.SetValue(False)
    upgradeBackgroundColor = wx.Colour(255,215,174)
    self.cbUpgrade0.SetBackgroundColour(upgradeBackgroundColor)
    self.cbUpgrade1.SetBackgroundColour(upgradeBackgroundColor)
    self.cbUpgrade2.SetBackgroundColour(upgradeBackgroundColor)
    self.cbUpgrade3.SetBackgroundColour(upgradeBackgroundColor)
    self.cbUpgrade4.SetBackgroundColour(upgradeBackgroundColor)
    self.cbUpgrade5.SetBackgroundColour(upgradeBackgroundColor)
    self.cbUpgrade6.SetBackgroundColour(upgradeBackgroundColor)
    if (not isinstance(ship, dataStruct.ShipRecord)) or (ship.upgrades == []):
      self.stUpgrades.SetLabel("Selected ship has no upgrades available")
      self.cbUpgrade0.Hide()
      self.cbUpgrade1.Hide()
      self.cbUpgrade2.Hide()
      self.cbUpgrade3.Hide()
      self.cbUpgrade4.Hide()
      self.cbUpgrade5.Hide()
      self.cbUpgrade6.Hide()
      return
    else:
      self.stUpgrades.SetLabel("Choose from the following available upgrades.  Select only one upgrade in each color group, if any.")
      try:
        self.cbUpgrade0.SetLabel('{0} points: {1}'.format(ship.upgrades[0].cost, ship.upgrades[0].text))
        self.cbUpgrade0.Show()
        if (ship.upgrades[0].color):
          self.cbUpgrade0.SetBackgroundColour(ship.upgrades[0].color)
        self.cbUpgrade0.SetValue(False)
        for selectUpgrade in ship.currentUpgrades:
          if (selectUpgrade.text == ship.upgrades[0].text):
            self.cbUpgrade0.SetValue(True)
            break
      except IndexError:
        self.cbUpgrade0.Hide()
      try:
        self.cbUpgrade1.SetLabel('{0} points: {1}'.format(ship.upgrades[1].cost, ship.upgrades[1].text))
        self.cbUpgrade1.Show()
        if (ship.upgrades[1].color):
          self.cbUpgrade1.SetBackgroundColour(ship.upgrades[1].color)
        self.cbUpgrade1.SetValue(False)
        for selectUpgrade in ship.currentUpgrades:
          if (selectUpgrade.text == ship.upgrades[1].text):
            self.cbUpgrade1.SetValue(True)
            break
      except IndexError:
        self.cbUpgrade1.Hide()
      try:
        self.cbUpgrade2.SetLabel('{0} points: {1}'.format(ship.upgrades[2].cost, ship.upgrades[2].text))
        self.cbUpgrade2.Show()
        if (ship.upgrades[2].color):
          self.cbUpgrade2.SetBackgroundColour(ship.upgrades[2].color)
        self.cbUpgrade2.SetValue(False)
        for selectUpgrade in ship.currentUpgrades:
          if (selectUpgrade.text == ship.upgrades[2].text):
            self.cbUpgrade2.SetValue(True)
            break
      except IndexError:
        self.cbUpgrade2.Hide()
      try:
        self.cbUpgrade3.SetLabel('{0} points: {1}'.format(ship.upgrades[3].cost, ship.upgrades[3].text))
        self.cbUpgrade3.Show()
        if (ship.upgrades[3].color):
          self.cbUpgrade3.SetBackgroundColour(ship.upgrades[3].color)
        self.cbUpgrade3.SetValue(False)
        for selectUpgrade in ship.currentUpgrades:
          if (selectUpgrade.text == ship.upgrades[3].text):
            self.cbUpgrade3.SetValue(True)
            break
      except IndexError:
        self.cbUpgrade3.Hide()
      try:
        self.cbUpgrade4.SetLabel('{0} points: {1}'.format(ship.upgrades[4].cost, ship.upgrades[4].text))
        self.cbUpgrade4.Show()
        if (ship.upgrades[4].color):
          self.cbUpgrade4.SetBackgroundColour(ship.upgrades[4].color)
        self.cbUpgrade4.SetValue(False)
        for selectUpgrade in ship.currentUpgrades:
          if (selectUpgrade.text == ship.upgrades[4].text):
            self.cbUpgrade4.SetValue(True)
            break
      except IndexError:
        self.cbUpgrade4.Hide()
      try:
        self.cbUpgrade5.SetLabel('{0} points: {1}'.format(ship.upgrades[5].cost, ship.upgrades[5].text))
        self.cbUpgrade5.Show()
        if (ship.upgrades[5].color):
          self.cbUpgrade5.SetBackgroundColour(ship.upgrades[5].color)
        self.cbUpgrade5.SetValue(False)
        for selectUpgrade in ship.currentUpgrades:
          if (selectUpgrade.text == ship.upgrades[5].text):
            self.cbUpgrade5.SetValue(True)
            break
      except IndexError:
        self.cbUpgrade5.Hide()
      try:
        self.cbUpgrade6.SetLabel('{0} points: {1}'.format(ship.upgrades[6].cost, ship.upgrades[6].text))
        self.cbUpgrade6.Show()
        if (ship.upgrades[6].color):
          self.cbUpgrade6.SetBackgroundColour(ship.upgrades[6].color)
        self.cbUpgrade6.SetValue(False)
        for selectUpgrade in ship.currentUpgrades:
          if (selectUpgrade.text == ship.upgrades[6].text):
            self.cbUpgrade6.SetValue(True)
            break
      except IndexError:
        self.cbUpgrade6.Hide()
    
    return

  def SetHardpoints(self, ship):
    """Called when the ship selection has changed, to show/hide the options, and update values"""
    if (not isinstance(ship, dataStruct.ShipRecord)) or (ship.availableHardPoints == []):
      self.stHardpoints.SetLabel("Selected ship has no hardpoints available")
      self.cbHardpoint0.Hide()
      self.cbHardpoint1.Hide()
      self.cbHardpoint2.Hide()
      self.cbHardpoint3.Hide()
      self.cbHardpoint4.Hide()
      self.cbHardpoint5.Hide()
      self.cbHardpoint6.Hide()
      
      self.spHardpoint0.Hide()
      self.spHardpoint1.Hide()
      self.spHardpoint2.Hide()
      self.spHardpoint3.Hide()
      self.spHardpoint4.Hide()
      self.spHardpoint5.Hide()
      self.spHardpoint6.Hide()
      return
    else:
      #reset the checkboxes;  only set to true if the current ship selection has them on.
      self.cbHardpoint0.SetValue(False)
      self.cbHardpoint1.SetValue(False)
      self.cbHardpoint2.SetValue(False)
      self.cbHardpoint3.SetValue(False)
      self.cbHardpoint4.SetValue(False)
      self.cbHardpoint5.SetValue(False)
      self.cbHardpoint6.SetValue(False)
      self.spHardpoint0.SetValue(1)
      self.spHardpoint1.SetValue(1)
      self.spHardpoint2.SetValue(1)
      self.spHardpoint3.SetValue(1)
      self.spHardpoint4.SetValue(1)
      self.spHardpoint5.SetValue(1)
      self.spHardpoint6.SetValue(1)
      
      if (ship.maxHardPoints == 1):  #set prompt text
        self.stHardpoints.SetLabel("Choose one hardpoint from this list:")
      else:
        self.stHardpoints.SetLabel("Choose up to {0} hardpoints from this list:".format(ship.maxHardPoints))
      try:
        self.cbHardpoint0.SetLabel('{0} points: {1}'.format(ship.availableHardPoints[0].cost, ship.availableHardPoints[0].description))
        self.cbHardpoint0.Show()
        self.cbHardpoint0.SetValue(False)
        if (ship.availableHardPoints[0].numAvailable > 1):
          self.spHardpoint0.Show()
          self.spHardpoint0.SetValue(ship.availableHardPoints[0].numSelected)
          self.spHardpoint0.SetRange(1,ship.availableHardPoints[0].numAvailable)
        else:
          self.spHardpoint0.Hide()
        for selectHardpoint in ship.currentHardpoints:
          if (selectHardpoint.index == ship.availableHardPoints[0].index):
            self.cbHardpoint0.SetValue(True)
            self.spHardpoint0.SetValue(selectHardpoint.numSelected)
            break
      except IndexError:
        self.cbHardpoint0.Hide()
        self.spHardpoint0.Hide()
      try:
        self.cbHardpoint1.SetLabel('{0} points: {1}'.format(ship.availableHardPoints[1].cost, ship.availableHardPoints[1].description))
        self.cbHardpoint1.Show()
        if (ship.availableHardPoints[1].numAvailable > 1):
          self.spHardpoint1.Show()
          self.spHardpoint1.SetValue(ship.availableHardPoints[1].numSelected)
          self.spHardpoint1.SetRange(1,ship.availableHardPoints[1].numAvailable)
        else:
          self.spHardpoint1.Hide()
        for selectHardpoint in ship.currentHardpoints:
          if (selectHardpoint.index == ship.availableHardPoints[1].index):
            self.cbHardpoint1.SetValue(True)
            self.spHardpoint1.SetValue(selectHardpoint.numSelected)
            break
      except IndexError:
        self.cbHardpoint1.Hide()
        self.spHardpoint1.Hide()
      try:
        self.cbHardpoint2.SetLabel('{0} points: {1}'.format(ship.availableHardPoints[2].cost, ship.availableHardPoints[2].description))
        self.cbHardpoint2.Show()
        if (ship.availableHardPoints[2].numAvailable > 1):
          self.spHardpoint2.Show()
          self.spHardpoint2.SetValue(ship.availableHardPoints[2].numSelected)
          self.spHardpoint2.SetRange(1,ship.availableHardPoints[2].numAvailable)
        else:
          self.spHardpoint2.Hide()
        for selectHardpoint in ship.currentHardpoints:
          if (selectHardpoint.index == ship.availableHardPoints[2].index):
            self.cbHardpoint2.SetValue(True)
            self.spHardpoint2.SetValue(selectHardpoint.numSelected)
            break
      except IndexError:
        self.cbHardpoint2.Hide()
        self.spHardpoint2.Hide()
      try:
        self.cbHardpoint3.SetLabel('{0} points: {1}'.format(ship.availableHardPoints[3].cost, ship.availableHardPoints[3].description))
        self.cbHardpoint3.Show()
        if (ship.availableHardPoints[3].numAvailable > 1):
          self.spHardpoint3.Show()
          self.spHardpoint3.SetValue(ship.availableHardPoints[3].numSelected)
          self.spHardpoint3.SetRange(1,ship.availableHardPoints[3].numAvailable)
        else:
          self.spHardpoint3.Hide()
        for selectHardpoint in ship.currentHardpoints:
          if (selectHardpoint.index == ship.availableHardPoints[3].index):
            self.cbHardpoint3.SetValue(True)
            self.spHardpoint3.SetValue(selectHardpoint.numSelected)
            break
      except IndexError:
        self.cbHardpoint3.Hide()
        self.spHardpoint3.Hide()
      try:
        self.cbHardpoint4.SetLabel('{0} points: {1}'.format(ship.availableHardPoints[4].cost, ship.availableHardPoints[4].description))
        self.cbHardpoint4.Show()
        if (ship.availableHardPoints[4].numAvailable > 1):
          self.spHardpoint4.Show()
          self.spHardpoint4.SetValue(ship.availableHardPoints[4].numSelected)
          self.spHardpoint4.SetRange(1,ship.availableHardPoints[4].numAvailable)
        else:
          self.spHardpoint4.Hide()
        for selectHardpoint in ship.currentHardpoints:
          if (selectHardpoint.index == ship.availableHardPoints[4].index):
            self.cbHardpoint4.SetValue(True)
            self.spHardpoint4.SetValue(selectHardpoint.numSelected)
            break          
      except IndexError:
        self.cbHardpoint4.Hide()
        self.spHardpoint4.Hide()
      try:
        self.cbHardpoint5.SetLabel('{0} points: {1}'.format(ship.availableHardPoints[5].cost, ship.availableHardPoints[5].description))
        self.cbHardpoint5.Show()
        if (ship.availableHardPoints[5].numAvailable > 1):
          self.spHardpoint5.Show()
          self.spHardpoint5.SetValue(ship.availableHardPoints[5].numSelected)
          self.spHardpoint5.SetRange(1,ship.availableHardPoints[5].numAvailable)
        else:
          self.spHardpoint5.Hide()
        for selectHardpoint in ship.currentHardpoints:
          if (selectHardpoint.index == ship.availableHardPoints[5].index):
            self.cbHardpoint5.SetValue(True)
            self.spHardpoint5.SetValue(selectHardpoint.numSelected)
            break
      except IndexError:
        self.cbHardpoint5.Hide()
        self.spHardpoint5.Hide()
      try:
        self.cbHardpoint6.SetLabel('{0} points: {1}'.format(ship.availableHardPoints[6].cost, ship.availableHardPoints[6].description))
        self.cbHardpoint6.Show()
        if (ship.availableHardPoints[0].numAvailable > 1):
          self.spHardpoint6.Show()
          self.spHardpoint6.SetValue(ship.availableHardPoints[6].numSelected)
          self.spHardpoint6.SetRange(1,ship.availableHardPoints[6].numAvailable)
        else:
          self.spHardpoint6.Hide()
        for selectHardpoint in ship.currentHardpoints:
          if (selectHardpoint.index == ship.availableHardPoints[6].index):
            self.cbHardpoint6.SetValue(True)
            self.spHardpoint6.SetValue(selectHardpoint.numSelected)
            break
      except IndexError:
        self.cbHardpoint6.Hide()
        self.spHardpoint6.Hide()
      self.Layout()
    return

  
########################################################################
class ShipCatalogBase(FirestormDialogs.CatalogCore):
  """Shows a list of all available ships"""
  #----------------------------------------------------------------------
  def __init__(self, parent, selection=False, inventory=False, faction=None):
    FirestormDialogs.CatalogCore.__init__(self, parent)
    """Constructor"""
    self.inventoryMode = inventory
    self.lbFactionFilter.AppendItems(dataStruct.factions)
    self.lbDesignationFilter.AppendItems(dataStruct.designations)
    
    self.lcCatalog.InsertColumn(0, "#")
    self.lcCatalog.InsertColumn(1, "Faction")
    self.lcCatalog.InsertColumn(2, "Designation")
    self.lcCatalog.InsertColumn(3, "Class Name")
    self.lcCatalog.InsertColumn(4, "# Owned")
    self.lcCatalog.InsertColumn(5, "Cost")
    self.lcCatalog.InsertColumn(6, "DR")
    self.lcCatalog.InsertColumn(7, "CR")
    self.lcCatalog.InsertColumn(8, "HP")
    self.lcCatalog.InsertColumn(9, "MV")
    self.lcCatalog.InsertColumn(10, "TL")
    self.lcCatalog.InsertColumn(11, "AP")
    self.lcCatalog.InsertColumn(12, "PD")
    self.lcCatalog.InsertColumn(13, "SR")
    self.lcCatalog.InsertColumn(14, "MN")
    self.lcCatalog.InsertColumn(15, "WC")
    self.lcCatalog.InsertColumn(16, "Source")
    
    if (not self.inventoryMode):
      self.bAddInventory.Hide()
      self.bRemoveInventory.Hide()
      self.bViewSelected.Hide()
      self.bAddShip.Show()
      self.bEditShip.Show()
      self.bCopyShip.Show()
      self.bDeleteShip.Show()
    else:
      self.bAddInventory.Show()
      self.bRemoveInventory.Show()
      self.bViewSelected.Show()
      self.bAddShip.Hide()
      self.bEditShip.Hide()
      self.bDeleteShip.Hide()
      self.bCopyShip.Hide()
      
    if (selection):
      self.bAddShip.Hide()
      self.bEditShip.Hide()
      self.bCopyShip.Hide()
      self.bDeleteShip.Hide()
      self.bClose.Hide()
      self.bCancel.Show()
      self.selection = True
    else:
      self.selection = False
      self.bCancel.Hide()
      self.bChooseShip.Hide()
      self.bEditShip.Enable(False)
      self.bDeleteShip.Enable(False)
      self.bCopyShip.Enable(False)
      
    self.selectedShip = None
    self.selectionIndex = None
    self.filterOutShipListByInventory = False    #set to true if only owned ships are to be shown
    self.filterOutCustomShips = False
    self.filterOutOfficialShips = False
    
    self.lbDesignationFilter.SetSelection(0)
    if (faction):
      self.lbFactionFilter.SetStringSelection(faction)
    else:
      self.lbFactionFilter.SetSelection(0)
    self.UpdateCatalogList()
    return
  
  def OnActivateShip(self, event):
    """double clicked list item"""
    self.EditShip(event)
    return
  
  def OnHelp(self, event):
    helpText = "Data for individual ship classes is entered in this screen.  The data for each ship is based on the faction fleet guides published by Spartan Games.  You may also add custom ships for any faction, which will be marked as custom ships in the database.\n\nIf you find any errors on the official ships, please notify bugs@pymapper.com so that the main database can be updated.\nNote: The statistics for official and custom ships are stored in separate databases; if you change an official ship, and then later update files from pymapper.com, your original changes will be lost.\n\nTo view the statistics of a ship class, select it from the list and click on the 'View Ship Statistics' button, or double-click the entry.\n\nThe selection lists on the right side of the window can be used to filter ships by faction and ship class.  Hold the CTRL key down to select multiple filters"
    dlg = HelpDialog(self, helpText)
    dlg.ShowModal()
    return
  
  def FilterByInventory(self, event):
    self.filterOutShipListByInventory = self.cbShowInventoryOnly.IsChecked()
    self.UpdateCatalogList()
    return
  
  def FilterCustomShips(self, event):
    self.filterOutCustomShips = not self.cbShowCustomShips.IsChecked()
    self.UpdateCatalogList()
    return
  
  def FilterOfficialShips(self, event):
    self.filterOutOfficialShips = not self.cbShowOfficialShips.IsChecked()
    self.UpdateCatalogList()
    return
  
  def OnAddInventory(self, event):
    self.ChangeQuantity('add')
    return
  
  def OnRemoveInventory(self, event):
    self.ChangeQuantity('remove')
    return
  
  def OnChangeInventory(self, event):
    """
    Press 'a' to add a ship to the inventory.  (keycode = 65)
    Press 'd' to remove a ship from the inventory  (keycode = 68)
    Press 'v' to view the ship stats (keycode = 86)
    """
    if (event.GetKeyCode() == 65):
      self.ChangeQuantity('add')
    elif (event.GetKeyCode() == 68):
      self.ChangeQuantity('remove')
    elif (event.GetKeyCode() == 86):
      pass
    event.Skip()
    return
  
  def ChangeQuantity(self, action):
    if (action == 'add'):  #add a ship to the inventory
      gv.ShipClassList[self.selectedShip].numOwned += 1
      self.bRemoveInventory.Enable(True)
    elif (action == 'remove'):
      gv.ShipClassList[self.selectedShip].numOwned = max(gv.ShipClassList[self.selectedShip].numOwned-1, 0)
      if (gv.ShipClassList[self.selectedShip].numOwned == 0):
        self.bRemoveInventory.Enable(False)
    self.UpdateCatalogList(event=None, updateColumns=False)
    self.lcCatalog.SetItemState(self.selectionIndex, wx.LIST_STATE_SELECTED,wx.LIST_STATE_SELECTED)
    return
  
  def EditShip(self, event):
    #first, match the selected ship to the correct record in the master list
    listIndex = self.selectedShip

    if (self.selection):
      #exit; return the shipIndex value
      self.EndModal(self.selectedShip)
      return
    else:
      #edit the selected ship
      ship = gv.ShipClassList[listIndex]
      dlg = ShipEditorBase(self, ship)
      dlg.bAddShip.SetLabel("Apply Changes")
      changeShip = dlg.ShowModal()
      if (changeShip):
        #update to the new values
        gv.ShipClassList[listIndex].faction = dlg.cxFaction.GetStringSelection()
        gv.ShipClassList[listIndex].className = dlg.txClassName.GetValue()
        gv.ShipClassList[listIndex].designation = dlg.cxShipDesignation.GetStringSelection()  # 0:Leviathan, 1:Dreadnought, 2:Battleship, 3:Battle Carrier, 4:Carrier, 5:Battlecruiser, 6:Heavy Cruiser, 7:Gunship, 8:Destroyer, 9:Frigate, 10:Corvette, 11:Escort
        gv.ShipClassList[listIndex].baseCost = dlg.spBaseCost.GetValue()
        if (dlg.cbSecondaryCost.IsChecked()):
          gv.ShipClassList[listIndex].secondaryCost = dlg.spSecondaryCost.GetValue()
        else:
          gv.ShipClassList[listIndex].secondaryCost = 0
        if (dlg.cbCloakingField.IsChecked()):
          gv.ShipClassList[listIndex].cloakingField = True
        else:
          gv.ShipClassList[listIndex].cloakingField = False
        gv.ShipClassList[listIndex].squadronSizeMin = dlg.spSquadronSizeMin.GetValue()
        gv.ShipClassList[listIndex].squadronSizeMax = dlg.spSquadronSizeMax.GetValue()
        gv.ShipClassList[listIndex].classSize = dlg.cxSizeClass.GetStringSelection()    # 0:Large Capital, 1:Medium Capital, 2:Small
        if (dlg.rbCustomShip.GetValue()):
          gv.ShipClassList[listIndex].source = 'Custom'
        else:
          gv.ShipClassList[listIndex].source = 'Official'
        gv.ShipClassList[listIndex].DR = dlg.spDR.GetValue()
        gv.ShipClassList[listIndex].CR = dlg.spCR.GetValue()
        gv.ShipClassList[listIndex].HP = dlg.spHP.GetValue()
        gv.ShipClassList[listIndex].MV = dlg.spMV.GetValue()
        gv.ShipClassList[listIndex].TL = dlg.spTL.GetValue()
        gv.ShipClassList[listIndex].CP = dlg.spCP.GetValue()
        gv.ShipClassList[listIndex].AP = dlg.spAP.GetValue()
        gv.ShipClassList[listIndex].PD = dlg.spPD.GetValue()
        gv.ShipClassList[listIndex].SR = dlg.spSR.GetValue()
        gv.ShipClassList[listIndex].MN = dlg.spMN.GetValue()
        gv.ShipClassList[listIndex].WC = dlg.spWC.GetValue()
        gv.ShipClassList[listIndex].imageFilename = dlg.imageFilename
        gv.ShipClassList[listIndex].weapons = copy.deepcopy(dlg.weaponsArray)
        gv.ShipClassList[listIndex].maxHardPoints = dlg.spHardpoints.GetValue()
        gv.ShipClassList[listIndex].availableHardPoints = copy.deepcopy(dlg.hardpointArray)
        gv.ShipClassList[listIndex].accompanimentIncluded = copy.deepcopy(dlg.accompanimentArray)
        gv.ShipClassList[listIndex].MAR = copy.deepcopy(dlg.marArray)
        gv.ShipClassList[listIndex].upgrades = copy.deepcopy(dlg.upgradeArray)
        gv.ShipClassList[listIndex].notes = dlg.txNotes.GetValue()
        self.UpdateCatalogList()
      return

  def OnDeselectShip(self, event):
    self.bEditShip.Enable(False)
    self.bCopyShip.Enable(False)
    self.bDeleteShip.Enable(False)
    self.bAddInventory.Enable(False)
    self.bRemoveInventory.Enable(False)
    self.bViewSelected.Enable(False)
    self.selectedShip = None
    return
  
  def OnSelectShip(self, event):
    self.selectedShip = int(self.lcCatalog.GetItemText(event.m_itemIndex))
    self.selectionIndex = event.GetIndex()
    if (not self.selection):  #do not show these if we are making a ship selection
      self.bEditShip.Enable(True)
      self.bCopyShip.Enable(True)
      self.bDeleteShip.Enable(True)
    
    if (self.inventoryMode):
      self.bAddInventory.Enable(True)
      if (gv.ShipClassList[self.selectedShip].numOwned > 0):
        self.bRemoveInventory.Enable(True)
      else:
        self.bRemoveInventory.Enable(False)
      self.bViewSelected.Enable(True)
    #try to display the image of the ship
    if (gv.ShipClassList[self.selectedShip].imageFilename):
      imagePath = os.path.join(gv.ship_images_folder, gv.ShipClassList[self.selectedShip].imageFilename)
    else:
      imagePath = os.path.join(gv.ship_images_folder, gv.defaultImage)
    image = wx.Image(imagePath)
    if (image.IsOk()):
      (width, height) = self.bmpShipImage.GetSize()
      scaledBitmap = wx.BitmapFromImage(image.Scale(width, height, wx.IMAGE_QUALITY_HIGH))
      self.bmpShipImage.SetBitmap(scaledBitmap)
    return
  
  def OnCancel(self, event):
    self.EndModal(False)
    return
  
  def OnChooseShip(self, event):
    self.EndModal(self.selectedShip)
    return
    
  
  def OnCopyShip(self, event):
    copyShip = copy.deepcopy(gv.ShipClassList[self.selectedShip])
    if (gv.pymapperUser):
      copyShip.index = gv.nextMasterIndex
      gv.nextMasterIndex += 1
    else:
      copyShip.index = gv.nextUserIndex
      gv.nextUserIndex += 1
      
    gv.ShipClassList.append(copyShip)
    self.UpdateCatalogList()
    return
  
  def ApplyFilters(self, event=None):
    factionFilter = self.lbFactionFilter.GetSelections()
    designationFilter = self.lbDesignationFilter.GetSelections()
    
    #filter by faction
    factionList = []
    if (self.rbShowKurakOnly.GetValue()):
      factionList = dataStruct.kurak
      self.lbFactionFilter.Enable(False)
    elif (self.rbShowZenianOnly.GetValue()):
      factionList = dataStruct.zenian
      self.lbFactionFilter.Enable(False)
    else:
      self.lbFactionFilter.Enable(True)
      for index in factionFilter:
        if (index == 0):
          #show all factions
          factionList = dataStruct.factions
        else:
          factionList.append(dataStruct.factions[index-1])
    
    #filter by designation
    designationList = []
    for index in designationFilter:
      if (index == 0):
        #show all designations
        designationList = copy.deepcopy(dataStruct.designations)
      else:
        designationList.append(dataStruct.designations[index-1])
        
    for ship in gv.ShipClassList:
      if (ship.faction not in factionList):
        ship.showShip = False
        continue
      if (ship.designation not in designationList):
        ship.showShip = False
      else:
        ship.showShip = True
    return
  
  def UpdateCatalogList(self, event=None, updateColumns=True):
    """updateColumns: updates the column widths"""
    self.ApplyFilters()
    self.lcCatalog.DeleteAllItems()
    ship = dataStruct.ShipRecord()
    gv.ShipClassList.reverse()
    for ship in gv.ShipClassList:
      if (ship.deleted_ship):  #do not show deleted ship
        continue
      if (ship.showShip == False):
        continue
      if (self.filterOutShipListByInventory) and (ship.numOwned == 0):
        continue
      if (self.filterOutCustomShips) and (ship.source == 'Custom'):
        continue
      if (self.filterOutOfficialShips) and (ship.source == 'Official'):
        continue
      li = wx.ListItem()
      li.SetText(str(ship.index))
      index = self.lcCatalog.InsertItem(li)
      self.lcCatalog.SetStringItem(index, 0, str(ship.index))
      self.lcCatalog.SetStringItem(index, 1, ship.faction)
      self.lcCatalog.SetStringItem(index, 2, ship.designation)
      self.lcCatalog.SetStringItem(index, 3, ship.className)
      self.lcCatalog.SetStringItem(index, 4, str(ship.numOwned))
      self.lcCatalog.SetStringItem(index, 5, str(ship.baseCost))
      self.lcCatalog.SetStringItem(index, 6, str(ship.DR))
      self.lcCatalog.SetStringItem(index, 7, str(ship.CR))
      self.lcCatalog.SetStringItem(index, 8, str(ship.HP))
      self.lcCatalog.SetStringItem(index, 9, str(ship.MV))
      self.lcCatalog.SetStringItem(index, 10, str(ship.TL))
      self.lcCatalog.SetStringItem(index, 11, str(ship.AP))
      self.lcCatalog.SetStringItem(index, 12,str(ship.PD))
      self.lcCatalog.SetStringItem(index, 13,str(ship.SR))
      self.lcCatalog.SetStringItem(index, 14,str(ship.MN))
      self.lcCatalog.SetStringItem(index, 15,str(ship.WC))
      self.lcCatalog.SetStringItem(index, 16,ship.source)
    if (updateColumns):
      self.lcCatalog.SetColumnWidth(0, wx.LIST_AUTOSIZE)
      self.lcCatalog.SetColumnWidth(1, wx.LIST_AUTOSIZE)
      self.lcCatalog.SetColumnWidth(2, wx.LIST_AUTOSIZE_USEHEADER)
      self.lcCatalog.SetColumnWidth(3, wx.LIST_AUTOSIZE_USEHEADER)
      self.lcCatalog.SetColumnWidth(4, wx.LIST_AUTOSIZE_USEHEADER)
      self.lcCatalog.SetColumnWidth(5, wx.LIST_AUTOSIZE_USEHEADER)
      self.lcCatalog.SetColumnWidth(6, wx.LIST_AUTOSIZE_USEHEADER)
      self.lcCatalog.SetColumnWidth(7, wx.LIST_AUTOSIZE_USEHEADER)
      self.lcCatalog.SetColumnWidth(8, wx.LIST_AUTOSIZE_USEHEADER)
      self.lcCatalog.SetColumnWidth(9, wx.LIST_AUTOSIZE_USEHEADER)
      self.lcCatalog.SetColumnWidth(10, wx.LIST_AUTOSIZE_USEHEADER)
      self.lcCatalog.SetColumnWidth(11, wx.LIST_AUTOSIZE_USEHEADER)
      self.lcCatalog.SetColumnWidth(12, wx.LIST_AUTOSIZE_USEHEADER)
      self.lcCatalog.SetColumnWidth(13, wx.LIST_AUTOSIZE_USEHEADER)
      self.lcCatalog.SetColumnWidth(14, wx.LIST_AUTOSIZE_USEHEADER)
      self.lcCatalog.SetColumnWidth(15, wx.LIST_AUTOSIZE_USEHEADER)
      self.lcCatalog.SetColumnWidth(16, wx.LIST_AUTOSIZE_USEHEADER)
    gv.ShipClassList.reverse()
    if (self.lcCatalog.GetColumnWidth(1) < 100):
      self.lcCatalog.SetColumnWidth(1,100)
    if (self.lcCatalog.GetColumnWidth(2) < 100):
      self.lcCatalog.SetColumnWidth(2,100)
    return
  
  def OnDeleteShip(self, event):
    result = wx.MessageDialog(self,"Are you certain you want to delete this ship?", "Confirm deletion", wx.YES_NO |wx.ICON_EXCLAMATION).ShowModal()
    if (result == wx.ID_YES):
      gv.ShipClassList[self.selectedShip].deleted_ship = True
    self.UpdateCatalogList()
    return

  def OnAddShip(self, event):
    dlg = ShipEditorBase(self)
    add_ship = dlg.ShowModal()
    if (add_ship):
      ship = dataStruct.ShipRecord()
      if (gv.pymapperUser):
        ship.index = gv.nextMasterIndex
        gv.nextMasterIndex += 1
      else:
        ship.index = gv.nextUserIndex
        gv.nextUserIndex += 1
      ship.faction = dlg.cxFaction.GetStringSelection()
      ship.className = dlg.txClassName.GetValue()
      ship.designation = dlg.cxShipDesignation.GetStringSelection()  # 0:Leviathan, 1:Dreadnought, 2:Battleship, 3:Battle Carrier, 4:Carrier, 5:Battlecruiser, 6:Heavy Cruiser, 7:Gunship, 8:Destroyer, 9:Frigate, 10:Corvette, 11:Escort
      ship.baseCost = dlg.spBaseCost.GetValue()
      if (dlg.cbSecondaryCost.IsChecked()):
        ship.secondaryCost = dlg.spSecondaryCost.GetValue()
      else:
        ship.secondaryCost = 0
      ship.squadronSizeMin = dlg.spSquadronSizeMin.GetValue()
      ship.squadronSizeMax = dlg.spSquadronSizeMax.GetValue()
      ship.classSize = dlg.cxSizeClass.GetStringSelection()    # 0:Large Capital, 1:Medium Capital, 2:Small
      if (dlg.rbOfficialShip.GetValue()):
        ship.source = 'Official'
      else:
        ship.source = 'Custom'
      ship.DR = dlg.spDR.GetValue()
      ship.CR = dlg.spCR.GetValue()
      ship.HP = dlg.spHP.GetValue()
      ship.MV = dlg.spMV.GetValue()
      ship.TL = dlg.spTL.GetValue()
      ship.CP = dlg.spCP.GetValue()
      ship.AP = dlg.spAP.GetValue()
      ship.PD = dlg.spPD.GetValue()
      ship.SR = dlg.spSR.GetValue()
      ship.MN = dlg.spMN.GetValue()
      ship.WC = dlg.spWC.GetValue()
      ship.cloakingField = dlg.cbCloakingField.GetValue()
      ship.imageFilename = dlg.imageFilename
      ship.weapons = copy.deepcopy(dlg.weaponsArray)
      ship.maxHardPoints = dlg.spHardpoints.GetValue()
      ship.availableHardPoints = copy.deepcopy(dlg.hardpointArray)
      ship.accompanimentIncluded = copy.deepcopy(dlg.accompanimentArray)
      ship.MAR = copy.deepcopy(dlg.marArray)
      ship.upgrades = copy.deepcopy(dlg.upgradeArray)
      ship.ApplyUpgrades()
      ship.notes = dlg.txNotes.GetValue()
      gv.ShipClassList.append(ship)
      self.UpdateCatalogList()
    dlg.Destroy()
    return
    
  def OnClose(self, event):
    self.EndModal(True)
    gv.ShipClassFile = os.path.join(gv.data_folder, "catalog.xml")
    export.SaveShipClassData(gv.ShipClassFile)
    
    gv.ShipInventoryFile = os.path.join(gv.data_folder, "inventory.xml")
    export.SaveShipInventoryData(gv.ShipInventoryFile)
    return

########################################################################
class AddAccompanimentBase(FirestormDialogs.AddAccompanimentCore):
  """"""
  #----------------------------------------------------------------------
  def __init__(self, parent, sourceAccomp=None, faction=None):
    """Constructor"""
    FirestormDialogs.AddAccompanimentCore.__init__(self, parent)
    self.accompClassList = []
    if (sourceAccomp):
      targetAccomp = copy.deepcopy(sourceAccomp)
      self.accompClassList.append(targetAccomp)
      self.spMaxShips.SetValue(targetAccomp.numAllowed)
      self.bClose.SetLabel('OK')
      self.bAddClass.Hide()
      self.bDeleteClass.Hide()

    self.faction = faction
    self.UpdateAccompList()
    return
    
  def OnClose(self, event):
    self.EndModal(True)
    return
  
  def OnCancel(self, event):
    self.EndModal(False)
    return
  
  def OnAddShipClass(self, event):
    dlg = ShipCatalogBase(self, selection=True, faction=self.faction)
    addShip = dlg.ShowModal()
    
    if (addShip != False):
      escort = dataStruct.AccompanimentRecord()
      escort.numAllowed = self.spMaxShips.GetValue()
      escort.shipIndex = addShip
      self.accompClassList.append(escort)
      self.UpdateAccompList()
    return
  
  def UpdateAccompList(self):
    selection = self.lbShipClasses.GetSelection()
    self.lbShipClasses.Clear()
    ship = dataStruct.AccompanimentRecord()
    if (self.accompClassList == []):  #nothing to update
      self.bDeleteClass.Enable(False)
      return
    else:
      for ship in self.accompClassList:
        for masterShip in gv.ShipClassList:
          if (masterShip.index == ship.shipIndex):
            break
        textline = "0-"+str(ship.numAllowed)+"   "+masterShip.faction+" "+masterShip.className+" "+masterShip.designation
        self.lbShipClasses.Append(textline)
      if (selection != wx.NOT_FOUND) and (selection < len(self.lbShipClasses.GetStrings())):
        self.lbShipClasses.SetSelection(selection)
    return
  
  def OnChangeMax(self, event):
    shipSelection = self.lbShipClasses.GetSelection()
    if (shipSelection == wx.NOT_FOUND):
      return
    else:
      self.accompClassList[shipSelection].numAllowed = self.spMaxShips.GetValue()
      self.UpdateAccompList()
    return
  
  def OnSelectShipClass(self, event):
    self.bDeleteClass.Enable(True)
    self.spMaxShips.Enable(True)
    return
  
  def OnDeleteShipClass(self, event):
    shipSelection = self.lbShipClasses.GetSelection()
    self.accompClassList.pop(shipSelection)
    self.bDeleteClass.Enable(False)
    self.UpdateAccompList()
    return
    
    
  
########################################################################

class AddNewMarBase(FirestormDialogs.AddNewMarCore):
  """Add MAR data to database"""

  #----------------------------------------------------------------------
  def __init__(self, parent):
    """Constructor"""
    FirestormDialogs.AddNewMarCore.__init__(self, parent)
    return
  
  def OnCancel(self, event):
    self.EndModal(False)
    return
  
  def OnHelp(self, event):
    text = "This dialog will add new Model Assigned Rules (MAR) to the database.\n\nThe MAR number, name, and description are all mandatory; the values are optional.\n\nUse the MAR value fields when there is a variable to be included in the description.  To have the values inserted into the text, use the tag MV1, MV2, and/or MV3 wherever you want the value to go.  These values will then show up in the ship details and printouts."
    dlg = HelpDialog(self, text)
    dlg.ShowModal()
    return
  
  def OnAddMar(self, event):
    allowClose = True
    prompt = ''
    if (self.txMarNumber.GetValue() == ''):
      prompt += "Add MAR number\n"
      allowClose = False
    if (self.txMarDescription.GetValue() == ''):
      prompt += "Add MAR description\n"
      allowClose = False
    if (self.txMarName.GetValue() == ''):
      prompt += 'Add MAR name\n'
      allowClose = False
    
    
    if (allowClose):
      gv.SaveMarData = True
      self.EndModal(True)
    else:
      wx.MessageBox(prompt)
    
    return
  
#--------------------------------------------------------------------------------------------------

class FleetManagerMain(wx.App):
  def OnInit(self):
    logging.basicConfig(filename="FleetManager.log", filemode='w',
                        format='%(asctime)s %(levelname)s %(funcName)s %(lineno)d %(message)s\n',
                        level=logging.DEBUG)
    logging.debug(str(sys.version_info))
    logging.debug(str(sys.platform))
    logging.info("Fleet Manager Version: " + str(VERSION))
    
    #initialize frame
    gv.frame = FleetManagerFrame()
    gv.frame.Show()
    
    gv.InventoryBackgroundColor = wx.Colour(187,217,217,255)
    gv.FleetBackgroundColor = wx.Colour(255,215,174,255)
    gv.EscortBackgroundColor = wx.Colour(97,241,0,255)
    
    return True
  
    #--------------------------------------------------------------------------------------------------


if __name__ == '__main__':
  app = FleetManagerMain(0)
  #wx.lib.inspection.InspectionTool().Show()
  app.MainLoop()
  